/**
 * @fileOverview Responsible for determining which committees need to review the study.
 * 
 * <pre>
 * &lt;script type=&quot;text/javascript&quot; src=&quot;rams-environment.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;dynamicGeneration.js&quot;&gt;&lt;/script&gt; *
 * </pre>
 * 
 * In some cases the jshint errors are suppressed with jshint -W### codes, jshint globals, or jshint
 *               exported comments
 * @version 2.3.0
 */

/* globals SP, spAppData, study, rollupAnswers, dynamicSharepointURL, IACUC, SRS, IRB, RDC, SC1, SC2, SC3 */
/* globals consoleLoggingEnabled */

// CONSTANTS START
var YES = "Yes";
var NO = "No";
// CONSTANTS END

var iacucOversightRequired = NO;
var question1Answer = "";
var question2Answer = "";
var question3Answer = "";
var question4Answer = "";

var answer1Matched = NO;
var answer2Matched = NO;
var answer3Matched = NO;
var answer4Matched = NO;
var iacucReviewRequired = false;
var srsReviewRequired = false;
var sc1ReviewRequired = false;
var sc2ReviewRequired = false;
var sc3ReviewRequired = false;

// -------------------------- IACUC Evaluation methods start ------------------------------------------------

/**
 * retrieve the IACUC Committee Rules to determine if IACUC review is required
 * 
 * @param callback {function} async callback
 */
function retrieveCommitteeRuleItems(callback) {
   if (typeof callback === "function") {
      var iacucComRulesUrl = dynamicSharepointURL + "_api/web/lists/getbytitle('Committee Rules')/items"
         + "?$filter=Title eq '" + IACUC + "' " + "&$select=Id,Title,Answer1,Answer2,Answer3,Answer4";
      $.ajax({
         url : iacucComRulesUrl,
         method : "GET",
         async : false,
         headers : {
            "Accept" : "application/json; odata=verbose"
         },
         success : function(data) {
            callback(data);
         },
         error : function(XMLHttpRequest, textStatus, errorThrown) {
            /* jshint unused: false */
            if (consoleLoggingEnabled) {
               console.log("Unable to read Committee Rules list. " + JSON.stringify(errorThrown));
            }
            throw "Unable to read Committee Rules list. ";
         }
      });
   } else {
      throw "Invalid callback function. (committeeBroker.retrieveCommitteeRuleItems)";
   }
}

/**
 * If the answers to the study questions match the pattern shown in the Committee Rules list an IACUC review will be
 * required. For example, assume the following table: <table>
 * <th>
 * <td>Committee</td>
 * <td>Answer1</td>
 * <td>Answer2</td>
 * <td>Answer3</td>
 * <td>Answer4</td>
 * </th>
 * <tr>
 * <td>IACUC</td>
 * <td>Yes</td>
 * <td>Yes</td>
 * <td>Yes</td>
 * <td>No</td>
 * </tr>
 * <tr>
 * <td>IACUC</td>
 * <td>Yes</td>
 * <td>Yes</td>
 * <td>Yes</td>
 * <td>Yes</td>
 * </tr>
 * <tr>
 * <td>IACUC</td>
 * <td>Yes</td>
 * <td>Yes</td>
 * <td>No</td>
 * <td></td>
 * </tr>
 * </table> The way to read the first row is to look at the answers to section M questions 1-4. Answer 1 - 3 must be set
 * to Yes and question 4 must be set to No.
 * 
 * @param answer1 {string} the yes or no answer to the study first study question
 * @param answer2 {string} the yes or no answer to the study second study question
 * @param answer3 {string} the yes or no answer to the study third study question
 * @param answer4 {string} the yes or no answer to the study forth study question
 */
function matchRule(committeeRuleAnswer1, committeeRuleAnswer2, committeeRuleAnswer3, committeeRuleAnswer4) {
   if (committeeRuleAnswer1 == YES) {
      if (question1Answer == YES) {
         answer1Matched = YES;
      }
   } else {
      if (question1Answer == NO) {
         answer1Matched = YES;
      }
   }

   if (committeeRuleAnswer2 == YES) {
      if (question2Answer == YES) {
         answer2Matched = YES;
      }
   } else {
      if (question2Answer == NO) {
         answer2Matched = YES;
      }
   }

   if (committeeRuleAnswer3 == YES) {
      if (question3Answer == YES) {
         answer3Matched = YES;
      }
   } else {
      if (question3Answer == NO) {
         answer3Matched = YES;
      }
   }

   if (committeeRuleAnswer4 == YES) {
      if (question4Answer == YES) {
         answer4Matched = YES;
      }
   } else {
      if (question4Answer == NO) {
         answer4Matched = YES;
      }
   }

   if (iacucOversightRequired == NO && answer1Matched && answer2Matched && answer3Matched && answer4Matched) {
      iacucOversightRequired = YES;
   }

   return (iacucOversightRequired == YES ? true : false);
}

/**
 * determine if an IACUC review is required by looking at the trigger questions associated with animals. In forms 53 and
 * 59, these include question B19 & M1-M4
 * 
 * @returns iacucReviewRequired {boolean} true if IACUC review is required, false otherwise.
 */
function isIACUCReviewRequired(callback) {
   var questionIACUCYesID, question1YesID, question1NoID, question2YesID, question2NoID, question3YesID, question3NoID, question4YesID, question4NoID;
   var triggerIACUC = false;
   iacucReviewRequired = false;

   // Question B19:
   questionIACUCYesID = study.containedFormCollection[1].containedForm.formQuestionCollection[18].question.answerCollection[0].answersID;

   // Question M01: Does the project involve the use of vertebrate animals?
   question1YesID = study.containedFormCollection[12].containedForm.formQuestionCollection[0].question.answerCollection[0].answersID;
   question1NoID = study.containedFormCollection[12].containedForm.formQuestionCollection[0].question.answerCollection[1].answersID;

   // Question M02: Will any of the vertebrate animals be alive for any part of the project?
   question2YesID = study.containedFormCollection[12].containedForm.formQuestionCollection[1].question.answerCollection[0].answersID;
   question2NoID = study.containedFormCollection[12].containedForm.formQuestionCollection[1].question.answerCollection[1].answersID;

   // Question M03: Will any of the live vertebrate animals used undergo any procedure(s) that they would not undergo if
   // they were not involved in the project?
   question3YesID = study.containedFormCollection[12].containedForm.formQuestionCollection[2].question.answerCollection[0].answersID;
   question3NoID = study.containedFormCollection[12].containedForm.formQuestionCollection[2].question.answerCollection[1].answersID;

   // Question M04: Are the live vertebrate animals to be used in this project already on an active protocol with IACUC
   // approval?
   question4YesID = study.containedFormCollection[12].containedForm.formQuestionCollection[3].question.answerCollection[0].answersID;
   question4NoID = study.containedFormCollection[12].containedForm.formQuestionCollection[3].question.answerCollection[1].answersID;

   if (rollupAnswers.contains(questionIACUCYesID)) {
      triggerIACUC = true;
   }

   if (triggerIACUC) {
      if (rollupAnswers.contains(question1YesID)) {
         question1Answer = YES;
      } else if (rollupAnswers.contains(question1NoID)) {
         question1Answer = NO;
      } else {
         question1Answer = NO;
      }

      if (rollupAnswers.contains(question2YesID)) {
         question2Answer = YES;
      } else if (rollupAnswers.contains(question2NoID)) {
         question2Answer = NO;
      } else {
         question2Answer = NO;
      }

      if (rollupAnswers.contains(question3YesID)) {
         question3Answer = YES;
      } else if (rollupAnswers.contains(question3NoID)) {
         question3Answer = NO;
      } else {
         question3Answer = NO;
      }

      if (rollupAnswers.contains(question4YesID)) {
         question4Answer = YES;
      } else if (rollupAnswers.contains(question4NoID)) {
         question4Answer = NO;
      } else {
         question4Answer = NO;
      }

      retrieveCommitteeRuleItems(function(data) {
         var iacucRules = data.d.results;
         for (var i = 0; i < iacucRules.length; i++) {
            var committeeRuleAnswer1 = iacucRules[i].Answer1;
            var committeeRuleAnswer2 = iacucRules[i].Answer2;
            var committeeRuleAnswer3 = iacucRules[i].Answer3;
            var committeeRuleAnswer4 = iacucRules[i].Answer4;
            if (matchRule(committeeRuleAnswer1, committeeRuleAnswer2, committeeRuleAnswer3, committeeRuleAnswer4)) {
               iacucReviewRequired = true;
               break;
            }
         }
         callback(iacucReviewRequired);
      });
   } else {
      callback(iacucReviewRequired);
   }
}
// -------------------------- IACUC Evaluation methods end ------------------------------------------------

// -------------------------- SRS Evaluation methods end ------------------------------------------------
/**
 * determine if an SRS review is required by looking at the trigger questions associated with safety. In forms 53 and
 * 59, these include question B11, 14, 15, & 16
 * 
 * @returns srsReviewRequired {boolean} true if SRS review is required, false otherwise.
 */
function isSRSReviewRequired() {
   var question1YesID, question1NoID, question2YesID, question2NoID, question3YesID, question3NoID, question4YesID, question4NoID, question21YesID, question21NoID;
   srsReviewRequired = false;

   // Question B11: Does the research involve a collection of specimens from human subjects
   question1YesID = study.containedFormCollection[1].containedForm.formQuestionCollection[10].question.answerCollection[0].answersID;
   question1NoID = study.containedFormCollection[1].containedForm.formQuestionCollection[10].question.answerCollection[1].answersID;

   // Question B14: Does the research involve the use of biohazards?
   question2YesID = study.containedFormCollection[1].containedForm.formQuestionCollection[13].question.answerCollection[0].answersID;
   question2NoID = study.containedFormCollection[1].containedForm.formQuestionCollection[13].question.answerCollection[1].answersID;

   // Question B15: Does the research involve the use of X-rays, radioisotopes, or Ionizing Radiation Source?
   question3YesID = study.containedFormCollection[1].containedForm.formQuestionCollection[14].question.answerCollection[0].answersID;
   question3NoID = study.containedFormCollection[1].containedForm.formQuestionCollection[14].question.answerCollection[1].answersID;

   // Question B16: Does the research involve the use of recombinant DNA, human gene transfer, xenotransplantation, or
   // investigational use of Ionizing Radiation Source?
   question4YesID = study.containedFormCollection[1].containedForm.formQuestionCollection[15].question.answerCollection[0].answersID;
   question4NoID = study.containedFormCollection[1].containedForm.formQuestionCollection[15].question.answerCollection[1].answersID;
   
   
   // Question B21: Would you like to upload Research Protocol Safety Survey (RPSS) (VA Form 10-0398)?
   question21YesID = study.containedFormCollection[1].containedForm.formQuestionCollection[20].question.answerCollection[0].answersID;
   question21NoID = study.containedFormCollection[1].containedForm.formQuestionCollection[20].question.answerCollection[1].answersID;


   if (rollupAnswers.contains(question1YesID) || rollupAnswers.contains(question2YesID)
      || rollupAnswers.contains(question3YesID) || rollupAnswers.contains(question4YesID) || rollupAnswers.contains(question21YesID)) {
      srsReviewRequired = true;
   }

   return srsReviewRequired;
}
// -------------------------- SRS Evaluation methods end ------------------------------------------------

// -------------------------- SC1 Evaluation methods end ------------------------------------------------
/**
 * determine if an SC1 review is required by looking at the trigger questions associated with SC1. In forms 59, these
 * include question B23
 * 
 * @returns sc1ReviewRequired {boolean} true if SC1 review is required, false otherwise.
 */
function isSC1ReviewRequired() {
   var question1SC1;
   sc1ReviewRequired = false;

   // Question B25: ???
   if (typeof study.containedFormCollection[1].containedForm.formQuestionCollection[22] !== "undefined") {
      question1SC1 = study.containedFormCollection[1].containedForm.formQuestionCollection[22].question.answerCollection[0].answersID;
      if (rollupAnswers.contains(question1SC1)) {
         sc1ReviewRequired = true;
      }
   }

   return sc1ReviewRequired;
}
// -------------------------- SC1 Evaluation methods end ------------------------------------------------

// -------------------------- SC2 Evaluation methods end ------------------------------------------------
/**
 * determine if an SC2 review is required by looking at the trigger questions associated with SC2. In forms 59, these
 * include question B24
 * 
 * @returns sc2ReviewRequired {boolean} true if SC2 review is required, false otherwise.
 */
function isSC2ReviewRequired() {
   var question1SC2;
   sc2ReviewRequired = false;

   // Question B25: ???
   if (typeof study.containedFormCollection[1].containedForm.formQuestionCollection[22] !== "undefined") {
      question1SC2 = study.containedFormCollection[1].containedForm.formQuestionCollection[22].question.answerCollection[1].answersID;
      if (rollupAnswers.contains(question1SC2)) {
         sc2ReviewRequired = true;
      }
   }
   return sc2ReviewRequired;
}
// -------------------------- SC2 Evaluation methods end ------------------------------------------------

// -------------------------- SC3 Evaluation methods end ------------------------------------------------
/**
 * determine if an SC3 review is required by looking at the trigger questions associated with SC3. In forms 59, these
 * include question B25
 * 
 * @returns sc3ReviewRequired {boolean} true if SC3 review is required, false otherwise.
 */
function isSC3ReviewRequired() {
   var question1SC3;
   sc3ReviewRequired = false;

   // Question B25: ???
   if (typeof study.containedFormCollection[1].containedForm.formQuestionCollection[22] !== "undefined") {
      question1SC3 = study.containedFormCollection[1].containedForm.formQuestionCollection[22].question.answerCollection[2].answersID;
      if (rollupAnswers.contains(question1SC3)) {
         sc3ReviewRequired = true;
      }
   }
   return sc3ReviewRequired;
}
// -------------------------- SC3 Evaluation methods end ------------------------------------------------

/* exported determineCommittees */
/**
 * Evaluates the study (in the study global variable and spAppData) to determine what committees need to review the
 * study.
 * 
 * @returns reviewsNeeded {Array} will always have RDC and IRB, may also have IACUC, SRS, SC1, SC2, SC3
 */
function determineCommittees(callback) {
   var reviewsNeeded = [];

   if (spAppData.ParentStudyID === null && ((typeof study !== "undefined") && (study !== null))) {
      reviewsNeeded.push(RDC);
      reviewsNeeded.push(IRB);
      if (isSRSReviewRequired()) {
         reviewsNeeded.push(SRS);
      }
      if (isSC1ReviewRequired()) {
         reviewsNeeded.push(SC1);
      }
      if (isSC2ReviewRequired()) {
         reviewsNeeded.push(SC2);
      }
      if (isSC3ReviewRequired()) {
         reviewsNeeded.push(SC3);
      }
      isIACUCReviewRequired(function(iacucReviewRequired) {
         if (iacucReviewRequired) {
            reviewsNeeded.push(IACUC);
         }
         callback(reviewsNeeded);
      });
   } else {
      callback(spAppData.ReviewRequired);
   }
}
