#!/bin/bash
function myHelp () {
# Using a here doc with standard out.
cat <<-END

Help for $0

Stages or copies the application files from the scp location, which is typically /tmp/RAMS-2.x.0, to the $CATALINA_HOME 
deployment location. It accepts the name of the application and the environment as parameters. The application name must 
match the name of the war file and the prefix of the xml configuration file. The script validates that it can find the 
files and their destination directories, creating them if they do not exist, copies Java Cryptography Environment (JCE)
files, set permissions and ownership of the scripts, copies the files to the deploy directory, and verifies that the 
stage is successful.

Usage:
------
   $0 -h | --help
     Display this help
   $0 <RAMS-2.x.0> <env>
     Run $0 to stage the RAMS-2.x.0 application and configuration. 
     Replace the 2.x.0 with the actual name of the war file without the extension, e.g. RAMS-2.3.0.
     Replace <env> with the environment name. This is case sensitive and must match the suffix on the 
     RAMS-2.x.0-<env>.xml file name. 
     Valid values include idev, itst, SQA, PPD, and PROD.
     
Example: 
--------
     $0 RAMS-2.3.0 idev
    
END
}
function copyfiles () {
   SRCDIR=$1
   SRCFILE=$2
   DSTDIR=$3
   DSTFILEEXT=$4
   if [ ! -z $5 ]
   then
      SUFFIX=.$5
   else
      SUFFIX=
   fi
   
   #echo "INFO: COPYFILES SRCDIR=$SRCDIR"
   #echo "INFO:          SRCFILE=$SRCFILE"
   #echo "INFO:           DSTDIR=$DSTDIR"
   #echo "INFO:       DSTFILEEXT=$DSTFILEEXT"
   #echo "INFO:           SUFFIX=$SUFFIX" 
   for F in $SRCDIR/$SRCFILE
   do 
      echo "INFO: COPYFILES cp $F" 
      echo "INFO:        TO    $F.$DSTFILEEXT$SUFFIX" 
      cp "$F" "$F.$DSTFILEEXT$SUFFIX"
   done
}

if [[ $# -le 2 ]] && [[ $# -gt 0 ]]
then
   if [[ "$1" == "-h" ]] || [[ "$1" == "--help" ]] || [[ "$1" == "-help" ]] || [[ "$2" == "-h" ]] || [[ "$2" == "--help" ]] || [[ "$2" == "-help" ]]
   then
      myHelp
      exit 0
   elif [ -z "$1" ]
      then 
         NEWAPPNAME=RAMS-2.3.0
         TMPDIR=/tmp/$NEWAPPNAME
         echo "INFO: Beginning staging by defaulting to stage from /tmp/$TMPDIR"
      else
         NEWAPPNAME=$1
         TMPDIR=/tmp/$NEWAPPNAME
         echo "INFO: Beginning staging from $TMPDIR"
   fi
else 
   echo "ERROR: Received $# parameters when expecting two, the release to deploy and the environment."
   myHelp
   exit 1
fi

TODAY=$(date +%Y%m%d%H%M%S)
echo "INFO: TODAY=$TODAY"

# if [ -z "$1" ]
# then 
   # NEWAPPNAME=RAMS-2.2.0
   # TMPDIR=/tmp/$NEWAPPNAME
   # echo "INFO: Beginning staging by defaulting to stage from /tmp/$TMPDIR"
# else
   # NEWAPPNAME=$1
   # TMPDIR=/tmp/$NEWAPPNAME
   # echo "INFO: Beginning staging from $TMPDIR"
# fi
if [ -z "$2" ]
then 
   ENVIRON=idev
   echo "INFO: defaulting to idev environment"
else
   ENVIRON=$2
   echo "INFO: environment set to $ENVIRON"
fi
if [ ! -d "$TMPDIR" ]
then 
   echo "ERROR: $TMPDIR is not found."
   myHelp
   #echo "INFO: Command Syntax is $0 [APPNAME] [ENVIRON] where APPNAME is the application name to be deployed and is also a directory name in the /tmp directory. "
   #echo "      ENVIRON is the environment being deployed and must match the environment on the RAMS-*-ENVIRON.xml file to be deployed. "
   #echo "INFO: Example: sudo $0 RAMS-2.2.0 idev"
   exit 1
fi

#See if Tomcat home directory, $CATALINA_HOME, is defined and if it exists
if [ -z ${CATALINA_HOME+x} ] 
then
   echo "INFO: \$CATALINA_HOME is not set - defaulting to /usr/share/apache-tomcat-7.0.57"
   CATALINA_HOME="/usr/share/apache-tomcat-7.0.57"
else 
   echo "INFO: using $CATALINA_HOME environment variable value $CATALINA_HOME" 
fi

if [ -d $CATALINA_HOME ]
then
   echo "INFO: \$CATALINA_HOME found"
else 
   echo "INFO: \$CATALINA_HOME not found at $CATALINA_HOME"
fi

# stage the Java Cryptography Extension (JCE) needed for RAMS-2.2.0. 
# This may not be needed in future releases.
if [ -z "$JAVA_HOME" ]
then 
   echo "WARNING: Unable to find \$JAVA_HOME environment variable" 
else 
   echo "INFO: \$JAVA_HOME found at $JAVA_HOME"
fi

# from https://stackoverflow.com/questions/7334754/correct-way-to-check-java-version-from-bash-script
if [[ -n "$JAVA_HOME" ]] && [[ -x "$JAVA_HOME/bin/java" ]]; then
    echo "INFO: found java executable in JAVA_HOME"
    _java="$JAVA_HOME/bin/java"
elif type -p java;  then
    echo "INFO: found java executable in PATH"
    _java=java
    JAVAPATH="$(readlink -f $(which java))"
    JAVA_HOME="$(dirname ${JAVAPATH})/../.."
    echo "INFO: set \$JAVA_HOME to $JAVA_HOME"
else
    echo "ERROR: no java"
    exit 1
fi
if [[ "$_java" ]]; then
    version=$("$_java" -version 2>&1 | awk -F '"' '/version/ {print $2}')
    version1=$(echo "$version" | awk -F. '{printf("%03d%03d",$1,$2);}')
    echo "INFO: Java version $version $version1 found"
    JRELIBSECURITYDIR=$JAVA_HOME/jre/lib/security
    LOCALPOLICYFILE=local_policy.jar
    EXPORTPOLICYFILE=US_export_policy.jar
    if [ -f $JRELIBSECURITYDIR/$LOCALPOLICYFILE ]
    then
       #cp $LOCALPOLICY $LOCALPOLICY.$NEWAPPNAME
       #echo "INFO: copying files using: $JRELIBSECURITYDIR $LOCALPOLICYFILE $JRELIBSECURITYDIR $NEWAPPNAME $TODAY"
       copyfiles $JRELIBSECURITYDIR $LOCALPOLICYFILE $JRELIBSECURITYDIR $NEWAPPNAME $TODAY
    fi
    if [ -f $JRELIBSECURITYDIR/$EXPORTPOLICYFILE ]
    then
       #cp $EXPORTPOLICY $EXPORTPOLICY.$NEWAPPNAME
       #echo "INFO: copying files using: $JRELIBSECURITYDIR $EXPORTPOLICYFILE $JRELIBSECURITYDIR $NEWAPPNAME $TODAY"
       copyfiles $JRELIBSECURITYDIR $EXPORTPOLICYFILE $JRELIBSECURITYDIR $NEWAPPNAME $TODAY
    fi
    if [ $version1 -eq 001006 ] ; then
    unzip -jo "$TMPDIR/jce_policy-6.zip" "jce/*.jar" -d "$JAVA_HOME/jre/lib/security"
    elif [ $version1 -eq 001007 ] ; then
        unzip -jo "$TMPDIR/jce_policy-7.zip" "UnlimitedJCEPolicy/*.jar" -d "$JAVA_HOME/jre/lib/security"
    elif [ $version1 -eq 001008 ] ; then
        unzip -jo "$TMPDIR/jce_policy-8.zip" "UnlimitedJCEPolicyJDK8/*.jar" -d "$JAVA_HOME/jre/lib/security"
    else
        echo "ERROR: unknown JAVA version $version $version1"
        exit 1
    fi
fi
# end JCE staging

# create backup and deploy directories if they do not exist
cd $CATALINA_HOME
BACKUPDIR=$CATALINA_HOME/backup
DEPLOYDIR=$CATALINA_HOME/deploy
if [ ! -d $BACKUPDIR ]
then 
   echo "INFO: creating $BACKUPDIR" 
   mkdir -p $BACKUPDIR
else 
   echo "INFO: $BACKUPDIR found"
fi

if [ ! -d $DEPLOYDIR ]
then 
   echo "INFO: creating $DEPLOYDIR" 
   mkdir -p $DEPLOYDIR
else 
   echo "INFO: $DEPLOYDIR found"
fi

#copy from the /tmp upload location to the deploy directory and set ownership and permissions
echo "INFO: staging the shell scripts from /tmp/$NEWAPPNAME to '$CATALINA_HOME' directory"
cp $TMPDIR/backup.sh $CATALINA_HOME/.
cp $TMPDIR/deploy.sh $CATALINA_HOME/.
cp $TMPDIR/stage.sh $CATALINA_HOME/.
echo "INFO: ... setting permissions"
chmod 774 backup.sh 
chmod 774 deploy.sh
chmod 774 stage.sh
echo "INFO: ... setting ownership"
chown apache:apache backup.sh
chown apache:apache deploy.sh
chown apache:apache stage.sh

echo "INFO: staging the war and xml files from /tmp/$NEWAPPNAME to '$CATALINA_HOME' directory"
cp $TMPDIR/$NEWAPPNAME.war $CATALINA_HOME/deploy/.
cp $TMPDIR/logback.xml $CATALINA_HOME/deploy/.
cp $TMPDIR/logback*.jar $CATALINA_HOME/deploy/.
cp $TMPDIR/$NEWAPPNAME-$ENVIRON.xml $CATALINA_HOME/deploy/$NEWAPPNAME.xml
CODEC=commons-codec-1.9.jar
cp $TMPDIR/$CODEC $CATALINA_HOME/deploy/$CODEC

rc=0
if [ ! -f $DEPLOYDIR/$NEWAPPNAME.xml ]
then 
   echo "ERROR: $NEWAPPNAME.xml not found in '$DEPLOYDIR' directory"
   rc=1
fi
if [ ! -f $DEPLOYDIR/$NEWAPPNAME.war ]
then 
   echo "ERROR: $NEWAPPNAME.war not found in '$DEPLOYDIR' directory"
   rc=2
fi
if [ ! -f $DEPLOYDIR/logback.xml ]
then 
   echo "ERROR: $logback.xml not found in '$DEPLOYDIR' directory"
   rc=3
fi
if [ ! -f $DEPLOYDIR/logback-core-*.jar ]
then 
   echo "ERROR: $logback-core-*.jar not found in '$DEPLOYDIR' directory"
   rc=4
fi
if [ ! -f $DEPLOYDIR/logback-classic-*.jar ]
then 
   echo "ERROR: $logback.-classic-*.jar not found in '$DEPLOYDIR' directory"
   rc=5
fi
if [ ! -f $CATALINA_HOME/backup.sh ]
then 
   echo "ERROR: '$CATALINA_HOME/backup.sh' not found in '$CATALINA_HOME' directory"
   rc=6
fi
if [ ! -f $CATALINA_HOME/deploy.sh ]
then 
   echo "ERROR: '$CATALINA_HOME/deploy.sh' not found in '$CATALINA_HOME' directory"
   rc=7
fi
if [ ! -f $CATALINA_HOME/stage.sh ]
then 
   echo "ERROR: '$CATALINA_HOME/stage.sh' not found in '$CATALINA_HOME' directory"
   rc=8
fi
if [ ! -f $DEPLOYDIR/$CODEC ]
then 
   echo "ERROR: $CODEC not found in '$CATALINA_HOME/deploy' directory"
   rc=9
fi

echo "INFO: Files Staged - exit code=$rc"
if [[ $rc -eq 0 ]]
then
   echo
   echo "INFO: The next step is to run the following commands: "
   echo "      cd \$CATALINA_HOME"
   echo "      sudo -E ./backup.sh $NEWAPPNAME"
   echo
fi
exit $rc

