function Create-SSISAgentJob { 
    <#
    
    .SYNOPSIS
        Create SSIS agent jobs
    
    .DESCRIPTION
        
    
    .PARAMETER webServiceUrl
        [string] The SSIS web services URL.
    
    .PARAMETER reportFolder
        [string] The name of the folder on the SSRS server where the reports will be imported. "RAMS Reports" is the 
        default. 
    
    .PARAMETER AgentJobName
        [string] The name of the folder on the SSRS server where the data sources will be imported. "Data Sources" is the 
        default. 
    
    .PARAMETER dtsxPackage
        [string] The base directory of the reports directory in the deployment package. Built using the RAMS release 
        number, e.g. RAMS-2.3.0, and adding the reports directory. For example, D:\RAMS-2.3.0\Reports.
    
    .NOTES
        Modified from http://microsoft-ssis.blogspot.com/2016/10/using-powershell-to-create-sql-agent.html
    
    .EXAMPLE
        Create-SSISAgentJob -ssisServer "DNS  S" -dbServer "DNS" -reportFolder "RAMS" -dtsxPackage "AnnualReviewNotification.dtsx" -projectName "EmailNotification" -contentDBName "WSS_Content" -ramsDBName "RAMS" -jobStartTime (New-Timespan -hours 23 -minutes 00)

    .EXAMPLE
        Create-SSISAgentJob -ssisServer "DNS  " -dbServer "DNS" -reportFolder "RAMS" -dtsxPackage "TrainingExpirationNotification.dtsx" -projectName "EmailNotification" -contentDBName "WSS_Content" -ramsDBName "RAMS" -jobStartTime (New-Timespan -hours 23 -minutes 30)
    #>
    Param(
        [string]$ssisServer,
        [string]$dbServer,
        [string]$reportFolder = "RAMS",
        [string]$dtsxPackage,
        [string]$projectName,
        [string]$contentDBName,
        [string]$ramsDBName,
        [Timespan]$jobStartTime
    )
    #PowerShell SSIS JobStep
    ################################
    ########## PARAMETERS ##########
    ################################ 
    # Destination
    #$SsisServer = $webServiceUrl
    #$reportFolder = $reportFolder
    #$ProjectName = $ProjectName
     
    # Job
    #$JobName = $agentJobName
    #$dtsxPackage = $dtsxPackage
    #$JobStartTime = New-TimeSpan -hours 23 -minutes 00
     
    #clear
    write-log -message "=================================" -consoleForeground White
    write-log -message "==  Used parameters            ==" -consoleForeground White
    write-log -message "=================================" -consoleForeground White
    write-log -message "SSIS Server   : $SsisServer" -consoleForeground White
    write-log -message "DB Server     : $dbServer" -consoleForeground White
    write-log -message "reportFolder  : $reportFolder" -consoleForeground White
    write-log -message "ProjectName   : $projectName" -consoleForeground White
    write-log -message "Job name      : $JobName" -consoleForeground White
    write-log -message "dtsxPackage   : $dtsxPackage" -consoleForeground White
    write-log -message "contentDBName : $contentDBName" -consoleForeground White
    write-log -message "ramsDBName    : $ramsDBName" -consoleForeground White
    write-log -message "ScheduleTime  : $JobStartTime" -consoleForeground White
    write-log -message "=================================" -consoleForeground White
    write-log -message " " -consoleForeground White
    
    # Reference SMO assembly and connect to the SQL Sever Instance 
    # Check the number in the path which is different for each version
    Add-Type -Path 'C:\Program Files\Microsoft SQL Server\130\SDK\Assemblies\Microsoft.SqlServer.Smo.dll'
    $SQLSvr = New-Object -TypeName  Microsoft.SQLServer.Management.Smo.Server($SsisServer) 
     
    # Check if job already exists. Then fail, rename or drop
    $SQLJob = $SQLSvr.JobServer.Jobs[$JobName]
    if ($SQLJob){
      # Use one of these 3 options to handle existing jobs
     
      # Fail:
      #Throw [System.Exception] "Job with name '$JobName' already exists."
     
      # Rename:
      write-log -message "Job with name '$JobName' found, renaming and disabling it" -consoleForeground White
      $SQLJob.Rename($SQLJob.Name +"_OLD_" + (Get-Date -f MM-dd-yyyy_HH_mm_ss))
      $SQLJob.IsEnabled = $false
      $SQLJob.Alter()
     
      # Drop:
      #write-log -message "Job with name $JobName found, removing it" -consoleForeground Yellow
      #$SQLJob.Drop()
    }
     
     
    #Create new (empty) job 
    $SQLJob = New-Object -TypeName Microsoft.SqlServer.Management.SMO.Agent.Job -argumentlist $SQLSvr.JobServer, $JobName
    $SQLJob.OwnerLoginName = "SA"
    $SQLJob.Create() 
    write-log -message "Job '$JobName' created" -consoleForeground Green
     
     
    # Command of jobstep
    # This string is copied from T-SQL, by scripting a job(step) in SSMS
    # Then replace the hardcode strings with [NAME] to replace them with variables
    $Command = @'
    /ISSERVER "\"\SSISDB\[FOLDER]\[PROJECT]\[PACKAGE]\"" /SERVER [SSISSERVER] /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=[CNTDB];Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
'@
    
    #                    FOLDER     PROJECT   PACKAGE              SSIS SERVER                                                                DB SERVER                  SP CNT DB                                                                                                                               DB SERVER                 RAMS DB                                                                                                                                                                                                            
    #                    |-------| |-------| |-------|            |----------|                                                               |--------|                 |-----|                                                                                                                                  |--------|                 |--|                                                                                                                                                                                                               
    #/ISSERVER "\"\SSISDB\[FOLDER]\[PROJECT]\[PACKAGE]\"" /SERVER [SSISSERVER] /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=[CNTDB];Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
    #/ISSERVER "\"\SSISDB\[FOLDER]\[PROJECT]\[PACKAGE]\"" /SERVER [SSISSERVER] /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=[CNTDB];Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
    #/ISSERVER "\"\SSISDB\RAMS\EmailNotification\TrainingExpirationNotification.dtsx\"" /SERVER DNS   /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=DNS;Initial Catalog=WSS_Content;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=DNS;Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
    #/ISSERVER "\"\SSISDB\RAMS\EmailNotification\AnnualReviewNotification.dtsx\"" /SERVER DNS  /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=DNS;Initial Catalog=SP_RAMS_WSS_Content_IRB_Application_DB_2_1;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=DNS;Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
    $Command = $Command.Replace("[FOLDER]", $reportFolder)
    $Command = $Command.Replace("[PROJECT]", $ProjectName)
    $Command = $Command.Replace("[PACKAGE]", $dtsxPackage)
    $Command = $Command.Replace("[SSISSERVER]", $SsisServer)
    $Command = $Command.Replace("[DBSERVER]", $dbServer)
    $Command = $Command.Replace("[CNTDB]", $contentDBName)
    $Command = $Command.Replace("[RAMSDB]", $ramsDBName)
    $Command = $Command.Replace("[DBSERVER]", $SsisServer)
     
     
    # Create new SSIS job step with command from previous block 
    $SQLJobStep = New-Object -TypeName Microsoft.SqlServer.Management.SMO.Agent.JobStep -argumentlist $SQLJob, "$ProjectName - $dtsxPackage"
    $SQLJobStep.OnSuccessAction = [Microsoft.SqlServer.Management.Smo.Agent.StepCompletionAction]::QuitWithSuccess
    $SQLJobStep.OnFailAction = [Microsoft.SqlServer.Management.Smo.Agent.StepCompletionAction]::QuitWithFailure
    $SQLJobStep.SubSystem = "SSIS"
    $SQLJobStep.DatabaseName = $SsisServer
    $SQLJobStep.Command = $Command
    $SQLJobStep.Create() 
    write-log -message "Jobstep $SQLJobStep created" -consoleForeground Green
     
     
    # Create a daily schedule
    $SQLJobSchedule = New-Object -TypeName Microsoft.SqlServer.Management.Smo.Agent.JobSchedule -ArgumentList $SQLJob, "Daily $JobStartTime"
    $SQLJobSchedule.IsEnabled = $true
    $SQLJobSchedule.FrequencyTypes = [Microsoft.SqlServer.Management.SMO.Agent.FrequencyTypes]::Daily
    $SQLJobSchedule.FrequencyInterval = 1 # Recurs Every Day
    $SQLJobSchedule.ActiveStartDate = Get-Date
    $SQLJobSchedule.ActiveStartTimeofDay = $JobStartTime
    $SQLJobSchedule.Create()
    write-log -message "Jobschedule $SQLJobSchedule created" -consoleForeground Green
     
     
    # Apply to target server which can only be done after the job is created
    $SQLJob.ApplyToTargetServer("(local)")
    $SQLJob.Alter()
    write-log -message "$($JobName) job saved" -consoleForeground Green
}