<#

.SYNOPSIS
    Deploy All script presents a menu for the deployment of RAMS components.

.DESCRIPTION
    Imports the necessary modules, sets up the environment, and presents the deployment menu that steps the 
    SharePoint Administrator through the deployment in conjunction with the Installation Guide.
    
.NOTES
    Each function in the script should have its own help.

.EXAMPLE
    ./DeployAll.ps1

#>

if ((Get-PSSnapin "Microsoft.SharePoint.PowerShell" -ErrorAction SilentlyContinue) -eq $null) {
    Add-PSSnapin "Microsoft.SharePoint.PowerShell"
}

$Global:PSVersion = $PSVersionTable
$ConfirmPreference = "None"
$Global:RAMSVersion = "RAMS-2.3.0"
      

function RemoveNonNzero-RAMS {
    <#
    
    .SYNOPSIS
        Deletes all non NMEA accounts from the site collection administrators group.
    
    .DESCRIPTION
        Retrieves all users from the site, checks to see if they are an administrator and if the account contains
        a zero, is a service account, or a super user; if they are they will be kept. Otherwise the administrator is 
        added to a list for removal. Then the removal list is iterated over and deleted.
        
    .EXAMPLE
        RemoveNonNzero-RAMS
    
    #>

    $siteUsers = (Get-SiteWeb).SiteUsers 
    $usersTobeRemoved = New-Object System.Collections.ArrayList

    foreach($user in $siteUsers) {
        if($user.IsSiteAdmin -eq $true) { 
            if(($user.Displayname -imatch "0") -or ($user.Displayname -imatch "super") -or ($user.Displayname -imatch "service") -or ($user.Displayname -imatch "account")) {
                write-log -message "Kept: Admin: $($user.Displayname)" -consoleForeground Green
            } else {
                $usersTobeRemoved.Add($user.ID)
                write-log -message "Removed: Non-Admin: $($user.DisplayName)" -consoleForeground Yellow
            }
        } 
    }
    foreach ($id in $usersTobeRemoved) {
        $siteUsers.RemoveByID($id)
    }
}

function Remove-AOGroup {
    <#
    
    .SYNOPSIS
        Remove the Security Group named AO Group from the site collection.
    
    .DESCRIPTION
        If the AO group is found, displays the list of users and then deletes the AO Group. The users may need to be 
        manually added to the Facility Administrative Officers security group. 
        
    .EXAMPLE
        Remove-AOGroup
    
    #>

    $groups = (Get-SiteWeb).SiteGroups
    $aoGroup = $groups | Where-Object {$_.Name -match "AO Group"}
    write-log -message "$($aoGroup.Name) Users:" -consoleForeground Cyan
    $aoGroup.Users | Select-Object -Property UserLogin, DisplayName | Format-Table -Wrap 
    
    if($aoGroup -eq $null) {
        write-log -message "No AO Group Found" -consoleForeground Yellow
    } else {
        $groups.RemoveByID($aoGroup.ID)
        write-log -message "$($aoGroup.Name) has been removed" -consoleForeground Yellow
    }
}

function Reset-Accounts {
    <#
    
    .SYNOPSIS
        Reset the Farm and Superreader accounts in the database
    
    .DESCRIPTION
        Get the Farm and the SuperReader UserId and Password from the SharePoint Administrator running the 
        installation. Run SQL queries to update the databases with their permissions. Also update tomcatUser account.  
        
    .EXAMPLE
        Reset-Accounts
    
    #>
    #Farm account
    $Global:cred = $Null
    #ask if the server info is missing.
    if($Global:farmUserID -eq $null -or $Global:farmUserID -eq "") {
        $Global:farmUserID = $Global:hostInfo.farmUser.GetValue(0)
        $Global:farmPwd = $Global:hostInfo.farmUser.GetValue(1)

        $Global:cred = Get-Credential -Message "Please Enter Valid UserID/PwD for Farm Account" 
        $Global:farmUserID = $cred.UserName 
        $Global:farmPwd = ($Global:cred.GetNetworkCredential()).Password
    }
    #Super Reader account
    #ask if the serve info is missing.
    if ($Global:superReaderUserID -eq $null -or $Global:superReaderUserID -eq "") {
        $Global:superReaderUserID = $Global:hostInfo.superReader.GetValue(0)
        $Global:superReaderPwd = $Global:hostInfo.superReader.GetValue(1)

        $Global:cred = Get-Credential -Message "Please Enter Valid UserID/PwD for superReader Account" 
        $Global:superReaderUserID = $cred.UserName
        $Global:superReaderPwd = ($Global:cred.GetNetworkCredential()).Password
    }
    
    $Global:tomcatUserID = "TomcatUser"
    $Global:tomcatPwd = "unknown"
    
    #Region 
    #assign R/W/X permission to SuperReader account on RAMS user database.
    write-log -message "Assign R/W/X permission to SuperReader and TomcatUser account on RAMS user database" -consoleForeground Green
    #Get-ADUser -Filter 'Name -like "*ram*SuperRead*"' 
    $query = "
        USE [MASTER]
        GO
        If not Exists (select loginname from master.dbo.syslogins where name = '$Global:superReaderUserID')
        Begin
            CREATE LOGIN [$Global:superReaderUserID] FROM WINDOWS WITH DEFAULT_DATABASE=[RAMS]
        End
        
        USE [RAMS]
        GO
        IF EXISTS (SELECT name
                     FROM [sys].[database_principals]
                    WHERE [type] = 'U' AND name = N'$Global:superReaderUserID')
        Begin
            DROP USER [$Global:superReaderUserID] 
        end

        IF NOT EXISTS (SELECT name
                         FROM [sys].[database_principals]
                        WHERE [type] = 'U' AND name = N'$Global:superReaderUserID')
        Begin
            CREATE USER [$Global:superReaderUserID] 
            FOR LOGIN [$Global:superReaderUserID] WITH DEFAULT_SCHEMA=[dbo]
        end
        ALTER ROLE [db_datareader] ADD MEMBER [$Global:superReaderUserID]
        GO
        ALTER ROLE [db_datawriter] ADD MEMBER [$Global:superReaderUserID]
        GO
        ALTER ROLE [db_executor] ADD MEMBER [$Global:superReaderUserID]
        GO
        
        IF EXISTS (SELECT name
                     FROM [sys].[database_principals]
                    WHERE [type] = 'S' AND name = N'$Global:tomcatUserID')
        Begin
            DROP USER [$Global:tomcatUserID] 
        end

        IF NOT EXISTS (SELECT name
                         FROM [sys].[database_principals]
                        WHERE [type] = 'S' AND name = N'$Global:tomcatUserID')
        Begin
            CREATE USER [$Global:tomcatUserID] 
            FOR LOGIN [$Global:tomcatUserID] WITH DEFAULT_SCHEMA=[dbo]
        end
        ALTER ROLE [db_datareader] ADD MEMBER [$Global:tomcatUserID]
        GO
        ALTER ROLE [db_datawriter] ADD MEMBER [$Global:tomcatUserID]
        GO
        ALTER ROLE [db_executor] ADD MEMBER [$Global:tomcatUserID]
        GO

    "
    Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -Query $query -Verbose | write-log

    #get service account for ETL server and add that account to RAMS userdb and config db.
    $sqlServiceAccount = (Get-WmiObject -ComputerName $Global:hostInfo.ssis -Class  win32_service | ? {$_.Name -match 'SQLSERVERAGENT'}).StartName
    if($sqlServiceAccount.ToLower().contains("farm")) {
        write-log -message "Service account not reset. The SQLSERVERAGENT on the SSIS server is running with the farm account and should be changed to run with a different service account." -consoleForeground Red
    } else {
        write-log -message "Adding service account $sqlServiceAccount for ETL server to the RAMS userdb and content db." -consoleForeground Green
        $query = "
            USE [MASTER]
            GO
            If not Exists (select loginname from master.dbo.syslogins where name = '$sqlServiceAccount')
            Begin
                CREATE LOGIN [$sqlServiceAccount] FROM WINDOWS WITH DEFAULT_DATABASE=[RAMS]
            End
        
            USE [RAMS]
            GO
            IF EXISTS (SELECT name
                         FROM [sys].[database_principals]
                        WHERE [type] = 'U' AND name = N'$sqlServiceAccount')
            Begin
                DROP USER [$sqlServiceAccount] 
            end

            IF NOT EXISTS (SELECT name
                             FROM [sys].[database_principals]
                            WHERE [type] = 'U' AND name = N'$sqlServiceAccount')
            Begin
                CREATE USER [$sqlServiceAccount] 
                FOR LOGIN [$sqlServiceAccount] WITH DEFAULT_SCHEMA=[dbo]
            end
            GO
            ALTER ROLE [db_datareader] ADD MEMBER [$sqlServiceAccount]
            GO
            ALTER ROLE [db_datawriter] ADD MEMBER [$sqlServiceAccount]
            GO
            ALTER ROLE [db_executor] ADD MEMBER [$sqlServiceAccount]
            GO
            ALTER ROLE [db_ddladmin] ADD MEMBER [$sqlServiceAccount]
            GO
            
            USE [MSDB]
            GO
            --IF EXISTS (SELECT name
            --             FROM [sys].[database_principals]
            --            WHERE [type] = 'U' AND name = N'$sqlServiceAccount')
            --Begin
            --    DROP USER [$sqlServiceAccount] 
            --end

            IF NOT EXISTS (SELECT name
                             FROM [sys].[database_principals]
                            WHERE [type] = 'U' AND name = N'$sqlServiceAccount')
            Begin
               CREATE USER [$sqlServiceAccount] FOR LOGIN [$sqlServiceAccount] WITH DEFAULT_SCHEMA=[dbo]
            end
            ALTER ROLE [DatabaseMailUserRole] ADD MEMBER [$sqlServiceAccount]
            GO
        "
        Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -Query $query -Verbose | write-log
    
        #content DB
        #$contentDbs = (Get-SPContentDatabase | ? {($_.WebApplication -match 'RAMS') -or ($_.WebApplication -match 'My Site') -or ($_.WebApplication -match 'MySite')}).Name
        $contentDbs = @((Get-SPContentDatabase -Site (get-siteUrl)).Name, (Get-SPContentDatabase -Site (get-mysiteUrl)).Name)
        $contentDbs | % {
            $contentDb = $_
            $query = "
                USE [$contentDb]
                GO
            
                IF NOT EXISTS (SELECT name
                                 FROM [sys].[database_principals]
                                WHERE [type] = 'U' AND name = N'$sqlServiceAccount')
                Begin
                    CREATE USER [$sqlServiceAccount] 
                    FOR LOGIN [$sqlServiceAccount] WITH DEFAULT_SCHEMA=[dbo]
                end
            GO
            ALTER ROLE [db_datareader] ADD MEMBER [$sqlServiceAccount]
            GO
            ALTER ROLE [db_datawriter] ADD MEMBER [$sqlServiceAccount]
            GO
            "
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -Query $query -Verbose | write-log
        }
    }
}

function Install-RAMS {
    <#
    
    .SYNOPSIS
        Setup the environment and run the main RAMS Deployment Driver
    
    .DESCRIPTION
        Setup the environment variable and run the deploymentDriver, passing the required variables, redirecting 
        the output to the log file in the log directory. When complete, run Remove-AOGroup, remove the AgendaReport 
        and generateagendareport items.
        
    .EXAMPLE
        Install-RAMS
    
    #>
    
    Reset-Accounts
    ###################################################
    $x = Get-SiteWeb
    $Global:EnvironmentDomainName = $x.Url.Substring(0, $x.Url.IndexOf("sites") - 1)
    # Data Sources connectionstring table
    $Global:rdsConnStringTable = @{
        "RAMS_DB"     = "Data Source=$sqlServerName;Initial Catalog=RAMS" ;
        "RAMS_SP"     = $siteUrl ;
        "DataSource1" = $siteUrl
    }
    ;

    # DataSource Credentials: Username and Password if needed
    $Global:rdsCredentialsTable = @{
        "RAMS_DB" = @($superReaderUserID, $superReaderPwd);
        "RAMS_SP" = @($superReaderUserID, $superReaderPwd)
    }
    ;
    #Schedule task user id and password (farm admin) 
    $Global:STUserName = $farmUserID
    $Global:STPassword = $farmPwd


    $Global:groupList = ((Get-SiteWeb).Webs).Title

    $Global:scriptInc = Join-Path $fileLocationPS DeploymentDriver.ps1
    write-log -message "$Global:scriptInc" -consoleForeground Magenta
    . $Global:scriptInc -siteUrl $siteUrl -fileLocationPS $Global:fileLocationPS -fileLocationTemplate $Global:fileLocationTemplate -fileLocationSiteAsset $Global:fileLocationSiteAsset -reportTargetServerName $Global:reportTargetServerName -reportSourceDirectory $Global:reportSourceDirectory -wfTempFileLocation $Global:wfTempFileLocation -fileLocationMySitesPages $Global:fileLocationMySitesPages -rdsConnStringTable $Global:rdsConnStringTable -rdsCredentialsTable $Global:rdsCredentialsTable

    #remove old AO and Groups
    Remove-AOGroup 
    #remove couple of pages 
    foreach ($subWeb in (Get-SiteWeb).Webs) {
        $sitepages = $subWeb.Lists["Site Pages"]
        $agendaPage = $sitepages.Items | ? {$_.Name -eq 'AgendaReport.aspx'} 
        if($agendaPage -ne $Null) {
            $agendaPage.Delete() 
            write-log -message "$($agendaPage.Name) has been deleted from $subWeb" -consoleForeground Yellow
        }
    }
    $siteAsset = (Get-SiteWeb).Lists["Site Assets"]
    $genAgendaReportJS = $siteAsset.Items | ?{$_.Name -eq 'generateagendareport.js'}
    if($genAgendaReportJS -ne $Null) {
        $genAgendaReportJS.Delete()
        write-log -message "$($genAgendaReportJS.Name) has been deleted from $(Get-SiteWeb)" -consoleForeground Yellow
    }
} 

function Deploy-RAMS ([string] $Version, [string]$drive) {
    <#
    
    .SYNOPSIS
        Initialization routine for the DeployAll process.
    
    .DESCRIPTION
        Loads the required powershell modules
        
    .EXAMPLE
        if ($env:COMPUTERNAME -match "VAPH") {
            Deploy-RAMS -Version "RAMS-2.3.0" -drive "D"
        } else {
            Deploy-RAMS -Version "RAMS-2.3.0" -drive "C"
        }
    
    #>
    if ($env:COMPUTERNAME -match "VAPH" ) {
        $Global:fileLocationPS = $drive + ":\$Global:RAMSVersion\PS\" #PowerShell Script Location
        Set-Location $Global:fileLocationPS
        Copy-Item -path profile.ps1 -Destination C:\Windows\System32\WindowsPowerShell\v1.0
    }            
    
    
    $Global:fileLocationTemplate = "D:\$RAMSVersion\Templates\" #Template file location
    $Global:fileLocationSiteAsset = "D:\$RAMSVersion\SiteAssets" #Site Asset location, note: no "/" at the end
    $Global:reportSourceDirectory = "D:\$RAMSVersion\Reports" #Report files location
    $Global:fileLocationSitePages = "D:\$RAMSVersion\SitePages"#Site Pages file location
    $Global:fileLocationOnlineHelp = "D:\$RAMSVersion\OnlineHelp"#Online Help file location
    $Global:fileLocationCatalog = "D:\$RAMSVersion\_catalogs"#Catalog file location
    $Global:fileLocationMySitesPages = "D:\$RAMSVersion\mysites"#Catalog file location

    $Global:fileLocationISAC = "D:\$RAMSVersion\SQL"#ISAC file location
    $Global:wfTempFileLocation = "D:\$RAMSVersion\Temp\NotificationWF.csv" #Temp file location
    $Global:diretoryName = "D:\$RAMSVersion\Logfiles"
    $rundate = Get-Date -UFormat "%Y%m%d%H%M"
    $Global:logFileName = "Deploy_All_$env:COMPUTERNAME.$rundate.log"
    
    #import module, ignore error related to SP snap-in
    if ((Get-Module -Name RAMS_Module_CleanHistory) -ne $null) {
        Remove-Module -Name RAMS_Module_CleanHistory
    }
    if ((Get-Module -Name RAMS_Module_Util) -ne $null) {
        Remove-Module -Name RAMS_Module_Util
    }
    
    try {
        if ($env:COMPUTERNAME -match "VAPH") {
            Import-Module .\module\RAMS_Module_Util.psd1 -DisableNameChecking -Global
            Import-Module .\module\RAMS_Module_CleanHistory.psd1 -DisableNameChecking -Global
        }
        else {
            Import-Module ..\UtilityScripts\RAMS_Module_Util.psd1 -DisableNameChecking -Global
            Import-Module ..\UtilityScripts\RAMS_Module_CleanHistory.psd1 -DisableNameChecking -Global
        }        
    write-log -message "Deploy-RAMS - Modules Loaded"  -consoleForeground Gray
    $Global:lastBuild = (ls "$Global:fileLocationPS\Env.json" | Select-Object -Property LastWriteTime).LastWriteTime
    $Global:hostInfo = Get-RAMSEnv -host $env:COMPUTERNAME
    if($Global:hostInfo -eq $Null) {
        write-log -message "No Envrionment match for Host: $env:COMPUTERNAME" -consoleForeground Yellow
    }
    $Global:reportTargetServerName = $Global:hostInfo.ssrs #Target SSRS Server
    $Global:sqlServerName = $Global:hostInfo.sql #SQL Server

    }
    catch {
        write-log -message "Host: $($env:COMPUTERNAME) is not a Server, SharePoint is not installed on this computer...ignore, information only." -consoleForeground Gray
        #do nothing.
    }
    $Global:siteUrl = Get-SiteURL #SharePoint Site Collection URL
}

function Backup-RAMS {
    <#
    
    .SYNOPSIS
        Backs-up the RAMS Site Collection database
    
    .DESCRIPTION
        Backs-up the RAMS Site Collection database by calling Backup-SPSite. The file is put in 
        D:\spbackup\ + $env:COMPUTERNAME + \SiteCollection\rams_sitecollection.bak
        
    .EXAMPLE
        Backup-RAMS
    
    #>
    
    $scFile = 'D:\spbackup\' + $env:COMPUTERNAME + '\SiteCollection\rams_sitecollection.bak'
    $msFile = 'D:\spbackup\' + $env:COMPUTERNAME + '\SiteCollection\ramsMySites_sitecollection.bak'
    if((Test-Path D:\spbackup\$env:COMPUTERNAME\SiteCollection) -eq $False) {
        New-Item D:\spbackup\$env:COMPUTERNAME\SiteCollection -ItemType Directory -Verbose
        write-log -message "Created destination folder for back-up" -consoleForeground Yellow
    } 
    if((Test-Path $scFile) -eq $True) {
        write-log -message "Backing-up RAMS SharePoint Site Collection DB - Overwriting $scFile" -consoleForeground Yellow
        Backup-SPSite -Identity (Get-SPSite -Identity (Get-SiteURL)) -Path ($scFile) -Force -Verbose 
        Write-log -message "RAMS SharePoint Site Collection DB backed up to $scFile" -consoleForeground Green
    }else {
        write-log -message "Backing-up RAMS SharePoint Site Collection DB - Creating $scFile" -consoleForeground Yellow
        Backup-SPSite -Identity (Get-SPSite -Identity (Get-SiteURL)) -Path ($scFile) -Force -Verbose 
        Write-log -message "RAMS SharePoint Site Collection DB backed up to $scFile" -consoleForeground Green
    }
    if((Test-Path $msFile) -eq $True) {
        write-log -message "Backing-up RAMS My Sites SharePoint Site Collection DB - Overwriting $msFile" -consoleForeground Yellow
        Backup-SPSite -Identity (Get-SPSite -Identity (Get-MySiteURL)) -Path ($msFile) -Force -Verbose 
        Write-log -message "RAMS My Sites SharePoint Site Collection DB backed up to $msFile" -consoleForeground Green
    }else {
        write-log -message "Backing-up RAMS My Sites SharePoint Site Collection DB - Creating $msFile" -consoleForeground Yellow
        Backup-SPSite -Identity (Get-SPSite -Identity (Get-MySiteURL)) -Path ($msFile) -Force -Verbose 
        Write-log -message "RAMS My Sites SharePoint Site Collection DB backed up to $msFile" -consoleForeground Green
    }
}

function Restore-RAMS {
    <#
    
    .SYNOPSIS
        Restores the RAMS Site Collection database
    
    .DESCRIPTION
        Restores the RAMS Site Collection database by calling Restore-SPSite. The file is read from 
        D:\spbackup\ + $env:COMPUTERNAME + \SiteCollection\rams_sitecollection.bak
        
    .EXAMPLE
        Restore-RAMS
    
    #>

    $scFile = 'D:\spbackup\' + $env:COMPUTERNAME + '\SiteCollection\rams_sitecollection.bak'
    $msFile = 'D:\spbackup\' + $env:COMPUTERNAME + '\SiteCollection\ramsMySites_sitecollection.bak'
    if((Test-Path $scFile) -eq $True) {
        $spcdb = Get-SPContentDatabase -Site (Get-SiteURL)
        $spcdbName = $spcdb.Name
        write-log -message "Restoring RAMS SharePoint Site Collection DB: $spcdbName from file: $scFile" -consoleForeground Cyan
        #Restore-SPSite -Identity (Get-SiteURL) -DatabaseName "$($spcdbName)" -Path "$($scFile)" -Force -Verbose
        Restore-SPSite -Identity (Get-SiteURL) -Path "$($scFile)" -Force -Verbose
        write-log -message "Restored RAMS SharePoint Site Collection DB: $($spcdbName) restored from file: $scFile" -consoleForeground Green
    } else {
        write-log -message "Unable to locate RAMS Site Collection back-up $scFile" -consoleForeground Red
    }
    if((Test-Path $scFile) -eq $True) {
        $spcdb = Get-SPContentDatabase -Site (Get-MySiteURL)
        $spcdbName = $spcdb.Name
        write-log -message "Restoring RAMS SharePoint Site Collection DB: $spcdbName from file: $msFile" -consoleForeground Cyan
        #Restore-SPSite -Identity (Get-MySiteURL) -DatabaseName "$($spcdbName)" -Path ($msFile) -Force -Verbose
        Restore-SPSite -Identity (Get-MySiteURL) -Path ($msFile) -Force -Verbose
        write-log -message "Restored RAMS SharePoint Site Collection DB: $($spcdbName) restored from file: $msFile" -consoleForeground Green
    } else {
        write-log -message "Unable to locate RAMS My Sites Site Collection back-up $msFile" -consoleForeground Red
    }
    Reset-Accounts
}

function Check-PSVersion {
    <#
    
    .SYNOPSIS
        Reports the version of PowerShell
    
    .DESCRIPTION
        Displays the version of powershell on the computer.
        
    .EXAMPLE
        Check-PSVersion
    
    #>

    write-log -message "$($Global:PSVersion.PSVersion.ToString())" -consoleForeground Green
}

function Check-RAMSVersion {
    <#
    
    .SYNOPSIS
        Reports the version of RAMS
    
    .DESCRIPTION
        Displays the version of RAMS by reading the version number in the footer of the Home.aspx page.
        
    .NOTES
        Does not work for RAMS 2.3.0 because the value is no longer just text on the page. it is placed on the 
        page dynamically using jQuery. Need to figure out how to render the page or read it from RAMS-environment.js.

    .EXAMPLE
        Check-RAMSVersion
    
    #>
    
    #$_uri = Get-SiteURL
    $_uri = "$(Get-SiteURL)/SiteAssets/js/rams-environment.js"
    $_js = Invoke-WebRequest -Uri $_uri -UseDefaultCredentials
    $_rv = $_js.Content | Select-String 'var\s+RAMSVersion\s*=\s*"([^"]+)"' -AllMatches | % { $_.Matches } | % { $_.Groups[1].Value }
    if ($_rv) {
       write-log -message "$_rv" -consoleForeground Green
       sleep -Seconds 2
    } else {
        $_uri = "$(Get-SiteURL)/SitePages/Home.aspx"
        write-log -message "Retrieving the version from the page at $_uri" -consoleForeground Cyan
        $_html = Invoke-WebRequest -Uri $_uri -UseDefaultCredentials
        $_strongs = $_html.ParsedHtml.getElementsByTagName("strong")
        foreach ($_strong in $_strongs) {
            if ($_strong.innerText.IndexOfAny("v2") -gt 0) {
                write-log -message "$($_strong.innerText)" -consoleForeground Green
                sleep -Seconds 2
            }
        }
    }
}   
   
if ($env:COMPUTERNAME -match "VAPH") {
    Deploy-RAMS -Version "RAMS-2.3.0" -drive "D"
} else {
    Deploy-RAMS -Version "RAMS-2.3.0" -drive "C"
}

# Start code block for RAMS Release
$Menu = {
    write-log -message "    ************************************************************************************************" -consoleForeground White
    write-log -message "      $RAMSVersion - Installation Script" -consoleForeground Cyan
    write-log -message "      LastBuild: $Global:lastBuild         " -consoleForeground DarkCyan 
    write-log -message "      Log file location: $Global:diretoryName\$Global:logFileName " -consoleForeground DarkCyan 
    write-log -message "    ************************************************************************************************" -consoleForeground White
    write-log -message "    RAMS Installation for SharePoint, SQL Server, SSRS Reports, and SSIS Packages on Windows Servers" -consoleForeground Yellow
    write-log -message "    " -consoleForeground White
    write-log -message "    Please select an option from the list below:" -consoleForeground White
    write-log -message "    " -consoleForeground White
    write-log -message "    0) Check PowerShell Version" -consoleForeground Gray
    write-log -message "    1) Install PowerShell Version 5.1 - Run Only if PowerShell version is less than v5.1" -consoleForeground DarkGray
    write-log -message "    2) Check RAMS Version" -consoleForeground Gray
    write-log -message "    3) Am I in Sysadmin group" -consoleForeground Gray
    write-log -message "    4) Check Service Config on SSIS Server" -consoleForeground Gray
    write-log -message "    " -consoleForeground White
    write-log -message "    10) Backup RAMS - SharePoint " -consoleForeground Yellow
    write-log -message "    11) Restore RAMS - SharePoint " -consoleForeground Red
    write-log -message "    " -consoleForeground White
    write-log -message "    12) Backup RAMS - SQL User Database [RAMS]" -consoleForeground Yellow
    write-log -message "    13) Restore RAMS - SQL User Database [RAMS]" -consoleForeground Red
    write-log -message "    " -consoleForeground White
    write-log -message "    14) Backup RAMS - Reports Databases  [ReportServer, ReportServerTempDB]" -consoleForeground Yellow
    write-log -message "    15) Restore RAMS - Reports Databases [ReportServer, ReportServerTempDB]" -consoleForeground Red
    write-log -message "    " -consoleForeground White
    write-log -message "    16) Backup RAMS - SSIS Database [SSISDB]" -consoleForeground Yellow
    write-log -message "    17) Restore RAMS - SSIS Database [SSISDB]" -consoleForeground Red
    write-log -message "    " -consoleForeground White
    write-log -message "    20) Deploy New $RAMSVersion SQL Scripts" -consoleForeground Green
    write-log -message "    21) Deploy New $RAMSVersion SSIS Packages" -consoleForeground Green
    write-log -message "    22) Install New $RAMSVersion SharePoint Objects" -consoleForeground Green
    write-log -message "    23) Remove (Non-Zero)Accounts from Site Collection Admin" -consoleForeground Green
    write-log -message "    " -consoleForeground White
    write-log -message "    30) Clean RAMS History" -consoleForeground Magenta
    write-log -message "    " -consoleForeground White
    write-log -message "    100) Restart the Server" -consoleForeground Red
    write-log -message "    200) Exit Script" -consoleForeground Green
    write-log -message "    " -consoleForeground White
    write-log -message "    ************************************************************************************************" -consoleForeground White
    write-host "    Select an option.. [0-200]? " -ForegroundColor White -nonewline
}

Do {
    #Show-Menu
    invoke-command -scriptblock $Menu 
    $Choice = Read-Host
    switch ($Choice) {
        0 {
            #Check PS current version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            Check-PSVersion
            write-log -message " " -consoleForeground White
            start-sleep 2
        }
        1 {
            #Install New PS Version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            Install-PS
            write-log -message "Make sure to reboot the server!" -consoleForeground Magenta
            write-log -message " " -consoleForeground White
            start-sleep 2
        } 
        2 {
            #Check RAMS current version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            Check-RAMSVersion
            write-log -message " " -consoleForeground White
            start-sleep 2
        }
        3 {
            #Am I in Sysadmin group
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            @($Global:hostInfo.sql,$Global:hostInfo.ssrs,$Global:hostInfo.ssis) | % {
                $dbServ = $_
                $Error.Clear()
                $isExist = 0
                try{
                    $isExist = Invoke-Sqlcmd -ServerInstance $dbServ -Query "SELECT IS_SRVROLEMEMBER ('sysadmin') as Role" -ErrorAction Stop
                    if($isExist.Role -eq 1) {
                        write-log -message "Yes, $($env:USERNAME) is in sysadmin group on SERVER: $dbServ" -consoleForeground Green
                    } else {
                        write-log -message "Sorry! $($env:USERNAME) not in sysadmin group on SERVER: $dbServ" -consoleForeground Magenta  
                    }
                } catch {
                    write-log -message "$Error on SERVER:$dbServ" -consoleForeground Red
                }
            }
            try {
                $currentUser = [Security.Principal.WindowsIdentity]::GetCurrent().Name.toLower()
                $web = Get-SiteWeb
                $siteUsers = $web.SiteUsers
                $found = $false
                foreach($user in $siteUsers) {
                    if($user.IsSiteAdmin -eq $true) {
                        $userLoginName = $user.LoginName.ToLower()
                        #write-host "Current User: $currentUser SiteUser: $($user.LoginName)" -ForegroundColor Gray
                        if($userLoginName.endsWith($currentUser)) {
                            $found = $true
                            break
                        } 
                    }
                }
                
                if ($found) {
                    write-host "Yes, $currentUser found as administrator for $($web.Url)" -ForegroundColor Green
                } else {
                    write-host "Sorry! $currentUser NOT found as administrator for $($web.Url)" -ForegroundColor Red
                }
            } catch {
                write-log -message "Unable to access Site Web ore retrieve the site collection administrators on web: $($web.Url)" -consoleForeground Red
            } 
            
            try {
                $currentUser = [Security.Principal.WindowsIdentity]::GetCurrent().Name.toLower()
                $web = Get-MySiteWeb
                $siteUsers = $web.SiteUsers
                $found = $false
                foreach($user in $siteUsers) {
                    if($user.IsSiteAdmin -eq $true) {
                        $userLoginName = $user.LoginName.ToLower()
                        #write-host "Current User: $currentUser SiteUser: $($user.LoginName)" -ForegroundColor Gray
                        if($userLoginName.endsWith($currentUser)) {
                            $found = $true
                            break
                        } 
                    }
                }
                
                if ($found) {
                    write-host "Yes, $currentUser found as administrator for $($web.Url)" -ForegroundColor Green
                } else {
                    write-host "Sorry! $currentUser NOT found as administrator for $($web.Url)" -ForegroundColor Red
                }
            } catch {
                write-log -message "Unable to access Site Web ore retrieve the site collection administrators on web: $($web.Url)" -consoleForeground Red
            } 

            start-sleep 2
        }
        4 {
            #Check SSIS Service Accounts
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            Get-WmiObject -ComputerName ($Global:hostInfo.ssis) -Class win32_service | ? {$_.Caption -match 'SQL'} | select Name, StartName | Out-GridView
            start-sleep 2
        }
        10 {
            #backup  - current version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            Backup-RAMS
            start-sleep 2
        }
        11 {
            #restore  - current version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            Restore-RAMS
            start-sleep 2
        }
        12 {
            #Backup RAMS SQL database  - current version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            $query = "EXECUTE [master].dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE', N'SOFTWARE\Microsoft\MSSQLServer\MSSQLServer', N'BackupDirectory'"
            $defaultPath = (Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -Query $query -Verbose).Data
            write-log -message "Backing-up RAMS DB from $defaultPath\$($RAMSVersion).bak" -consoleForeground Cyan
            Backup-SqlDatabase -CopyOnly -ServerInstance $Global:hostInfo.sql -Database RAMS -BackupAction Database -Verbose -BackupFile "$($RAMSVersion).bak"
            start-sleep 2
        }
        13 {
            #Restore RAMS SQL database  - current version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            $query = "EXECUTE [master].dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE', N'SOFTWARE\Microsoft\MSSQLServer\MSSQLServer', N'BackupDirectory'"
            $defaultPath = (Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -Query $query -Verbose).Data
            $restoreFile =  "$defaultPath\$($RAMSVersion).bak"
            write-log -message "Restoring RAMS DB from $defaultPath\$($RAMSVersion).bak" -consoleForeground Cyan    
            #set single user mode
            $query = "
                USE [master]
                ALTER DATABASE [RAMS] SET SINGLE_USER WITH ROLLBACK IMMEDIATE
                RESTORE DATABASE [RAMS] FROM  DISK = N'$restoreFile' WITH  FILE = 1,  NOUNLOAD,  REPLACE,  STATS = 5
                ALTER DATABASE [RAMS] SET MULTI_USER
                GO
             "
#            RESTORE DATABASE [RAMS] FROM  DISK = N'$restoreFile' WITH  FILE = 2,  NOUNLOAD,  REPLACE,  STATS = 5

            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -Query $query -Verbose | write-log
            start-sleep 2
        }
        14 {
            #backup rams reports dbs
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            $query = "EXECUTE [master].dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE', N'SOFTWARE\Microsoft\MSSQLServer\MSSQLServer', N'BackupDirectory'"
            $defaultPath = (Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssrs -Query $query -Verbose).Data
            $rFile = "ReportServer_$($RAMSVersion).bak"
            $rTFile = "ReportServerTempDB_$($RAMSVersion).bak"
            write-log -message "Backing-up RAMS Reports DB to $defaultPath\$rFile" -consoleForeground Cyan
            write-log -message "Backing-up RAMS ReportsTemp DB to $defaultPath\$rTFile" -consoleForeground Cyan
            $query = "
                BACKUP DATABASE [ReportServer] TO  DISK = N'$($rFile)' WITH  COPY_ONLY, FORMAT, INIT,  MEDIANAME = N'Report Server ',  
                NAME = N'ReportServer-Full Database Backup', SKIP, NOREWIND, NOUNLOAD, COMPRESSION,  STATS = 10
                GO
                declare @backupSetId as int
                select @backupSetId = position from msdb..backupset where database_name=N'ReportServer' and backup_set_id=(select max(backup_set_id) 
                from msdb..backupset where database_name=N'ReportServer' )
                if @backupSetId is null begin raiserror(N'Verify failed. Backup information for database ''ReportServer'' not found.', 16, 1) end
                RESTORE VERIFYONLY FROM  DISK = N'$($rFile)' WITH  FILE = @backupSetId,  NOUNLOAD,  NOREWIND
                GO
            "
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssrs -Query $query -Verbose | write-log
            start-sleep 2
            $query = "
                BACKUP DATABASE [ReportServerTempDB] TO  DISK = N'$($rTFile)' WITH  COPY_ONLY, FORMAT, INIT,  MEDIANAME = N'Report Server ',  
                NAME = N'ReportServerTempDB-Full Database Backup', SKIP, NOREWIND, NOUNLOAD, COMPRESSION,  STATS = 10
                GO
                declare @backupSetId as int
                select @backupSetId = position from msdb..backupset where database_name=N'ReportServerTempDB' and backup_set_id=(select max(backup_set_id) 
                from msdb..backupset where database_name=N'ReportServerTempDB' )
                if @backupSetId is null begin raiserror(N'Verify failed. Backup information for database ''ReportServerTempDB'' not found.', 16, 1) end
                RESTORE VERIFYONLY FROM  DISK = N'$($rTFile)' WITH  FILE = @backupSetId,  NOUNLOAD,  NOREWIND
                GO
            "
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssrs -Query $query -Verbose | write-log
            Reset-Accounts
            start-sleep 2
        }
        15 {
            #Restore RAMS SQL database  - current version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            #restore report server
            $query = "EXECUTE [master].dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE', N'SOFTWARE\Microsoft\MSSQLServer\MSSQLServer', N'BackupDirectory'"
            $defaultPath = (Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssrs -Query $query -Verbose).Data
            $rFile = "ReportServer_$($RAMSVersion).bak"
            $rTFile = "ReportServerTempDB_$($RAMSVersion).bak"
            write-log -message "Restoring RAMS Reports DB from $defaultPath\$rFile" -consoleForeground Cyan
            write-log -message "Restoring RAMS ReportsTemp DB from $defaultPath\$rTFile" -consoleForeground Cyan
            $query = "
                USE [master]
                ALTER DATABASE [ReportServer] SET SINGLE_USER WITH ROLLBACK IMMEDIATE
                RESTORE DATABASE [ReportServer] FROM  DISK = N'$($rFile)' WITH  FILE = 1,  
                NOUNLOAD,  REPLACE,  STATS = 5
                ALTER DATABASE [ReportServer] SET MULTI_USER
                GO
            "
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssrs -Query $query -Verbose | write-log
            start-sleep 2
            #restore temp db
            $query = "
                USE [master]
                ALTER DATABASE [ReportServerTempDB] SET SINGLE_USER WITH ROLLBACK IMMEDIATE
                RESTORE DATABASE [ReportServerTempDB] FROM  DISK = N'$($rTFile)' WITH  FILE = 1,  
                NOUNLOAD,  REPLACE,  STATS = 5
                ALTER DATABASE [ReportServerTempDB] SET MULTI_USER
                GO
            "
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssrs -Query $query -Verbose | write-log
            Reset-Accounts
            start-sleep 2
        }
        16 {
            #backup rams SSIS dbs
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            $query = "EXECUTE [master].dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE', N'SOFTWARE\Microsoft\MSSQLServer\MSSQLServer', N'BackupDirectory'"
            $defaultPath = (Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssis -Query $query -Verbose).Data
            $rFile = "SSISDB_$($RAMSVersion).bak"
            write-log -message "Backing-up RAMS SSIS DB to $defaultPath\$rFile" -consoleForeground Cyan
            $query = "
                BACKUP DATABASE [SSISDB] TO  DISK = N'$($rFile)' WITH  COPY_ONLY, FORMAT, INIT,  MEDIANAME = N'SSIS DB ',
                NAME = N'SSISDB-Full Database Backup', SKIP, NOREWIND, NOUNLOAD, COMPRESSION,  STATS = 10
                GO
                declare @backupSetId as int
                select @backupSetId = position from msdb..backupset where database_name=N'SSISDB' and backup_set_id=(select max(backup_set_id) 
                from msdb..backupset where database_name=N'SSISDB' )
                if @backupSetId is null begin raiserror(N'Verify failed. Backup information for database ''SSISDB'' not found.', 16, 1) end
                RESTORE VERIFYONLY FROM  DISK = N'$($rFile)' WITH  FILE = @backupSetId,  NOUNLOAD,  NOREWIND
                GO
            "
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssis -Query $query -Verbose | write-log
            start-sleep 2
            Reset-Accounts
            start-sleep 2
        }
        17 {
            #Restore RAMS SSIS database  - current version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            #restore report server
            $query = "EXECUTE [master].dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE', N'SOFTWARE\Microsoft\MSSQLServer\MSSQLServer', N'BackupDirectory'"
            $defaultPath = (Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssis -Query $query -Verbose).Data
            $rFile = "SSISDB_$($RAMSVersion).bak"
            write-log -message "Restoring RAMS Reports DB from $defaultPath\$rFile" -consoleForeground Cyan
            $query = "
                USE [master]
                ALTER DATABASE [SSISDB] SET SINGLE_USER WITH ROLLBACK IMMEDIATE
                RESTORE DATABASE [SSISDB] FROM  DISK = N'$($rFile)' WITH  FILE = 1,  
                NOUNLOAD,  REPLACE,  STATS = 5
                ALTER DATABASE [SSISDB] SET MULTI_USER
                GO
            "
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.ssis -Query $query -Verbose | write-log
            Reset-Accounts
            start-sleep 2
        }
        20 {
            #Updating DB table structures, proceedures, and data
            write-log -message "Option $Choice chosen..." -consoleForeground White
            write-log -message "Deploying new SQL Scripts..." -consoleForeground Yellow
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -InputFile "..\SQL\Release2_3.sql" -Verbose | write-log
            #Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -InputFile "..\SQL\CreateRAMSStoredProc.sql" -Verbose | write-log
            Invoke-Sqlcmd -ServerInstance $Global:hostInfo.sql -InputFile "..\SQL\Setup_DatabaseMail_Profile.sql" -Verbose | write-log
        }
        21 {
            #Install SSIS Adapter, Copy ispac, build catalog, and create jobs
            write-log -message "Option $Choice chosen..." -consoleForeground White
            write-log -message "Installing Sharepoint SSIS Adapter..." -consoleForeground Yellow
            Install-SSISAdapter -dbServer ($Global:hostInfo.ssis) | Write-Log

            write-log -message "Installing new SSISDB..." -consoleForeground Yellow
            $envName = Get-RAMSEnvName -host ($env:COMPUTERNAME)
            write-log -message "Deploying new SSIS Packages..." -consoleForeground Yellow 
            $Global:fileLocationISAC = "D:\$RAMSVersion\sql"
            Copy-Item "$Global:fileLocationISAC\EmailNotification_$($envName).ispac" -Destination "\\$($Global:hostInfo.ssis)\D$\" -Verbose | Write-Log
            New-SSISDB -dbServer ($Global:hostInfo.ssis) -ProjectName "EmailNotification" -ProjectFilePath "D:\EmailNotification_$($envName).ispac"
         }
        22 {
            #Install RAMS Latest version
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            #grant R/W/X permission to farm account on RAMS database

            Install-RAMS
            #Add 
            write-log -message "Make sure to check the Tomcat!" -consoleForeground Magenta
            write-log -message " " -consoleForeground White
            start-sleep 2
        }
        23 {
            #remove non-zero accounts from site collection admins group
            write-log -message "Option $Choice chosen..." -consoleForeground White
            RemoveNonNzero-RAMS
        }
        30 {
            #clean history
            write-log -message "Option $Choice chosen..." -consoleForeground White
            start-sleep 1
            $_historyDate = Read-Host -Prompt "Please enter history date..(MM/DD/YYYY)" -Verbose
            Remove-RAMSHistoryData -cleanDate $_historyDate -dbServer $Global:hostInfo.sql
            start-sleep 2
        }
        100 {
            #Exit and restart
            write-log -message "Option $Choice chosen..." -consoleForeground White
            Stop-Transcript
            restart-computer -computername localhost -force
        }
        200 {
            #   Exit
            write-log -message "Option $Choice chosen..." -consoleForeground White
            write-log -message "Exiting..." -consoleForeground White
        }
        default {write-log -message "You haven't selected any of the available options. " -consoleForeground White}
    }
} while ($Choice -ne 200)