<#

.SYNOPSIS
    Uploads files to SharePoint folders

.DESCRIPTION
    Upload the files in the specified folder to the destination in SharePoint. This is for recursively uploading the 
    HTML, ASPX, js, Help, etc files to Site Pages, Site Assets, _catalogs, PublishingImages, and mySites pages.
    
.EXAMPLE
    ./DeployFiles.ps1

#>


#check to ensure Microsoft.SharePoint.PowerShell is loaded
$snapin = Get-PSSnapin | Where-Object {$_.Name -eq 'Microsoft.SharePoint.Powershell'}
if ($snapin -eq $null) {
   Add-PSSnapin "Microsoft.SharePoint.Powershell"
}

############################################# UploadFiles ###########################################
function UploadFiles($siteUrl, $DestFolderUrl, $LocalFileOrFolderPath) {
    <#
    
    .SYNOPSIS
        Uploads files to SharePoint folders
    
    .DESCRIPTION
        Upload the files in the specified folder to the destination in SharePoint. This is for recursively uploading the
        HTML, ASPX, js, Help, etc files to Site Pages, Site Assets, _catalogs, PublishingImages, and mySites pages. Once
        uploaded, call UploadAndApprove. May call createSPFolder if the folder does not exist.
        
    .PARAMETER siteUrl
        [string] The site collection URL

    .PARAMETER DestFolderUrl
        [string] Where in the SharePoint folder structure to upload to.

    .PARAMETER LocalFileOrFolderPath
        [string] The source folder location to upload from.

    .EXAMPLE
        UploadFiles -siteUrl $siteUrl  -DestFolderUrl ($siteUrl + "/_catalogs") -LocalFileOrFolderPath $fileLocationCatalog
    
    #>
    
    if((get-item $LocalFileOrFolderPath).PSIsContainer -eq $true) {
        write-log -message "Looping Items in Folder: $LocalFileOrFolderPath" -consoleForeground Cyan
        Get-ChildItem $LocalFileOrFolderPath | Foreach-Object{
            $name = $_.Name
            $fullName = $_.FullName

            #only files, not folders
            if((get-item $fullName).PSIsContainer -eq $false){
                UploadandApprove -siteUrl $siteUrl -DestFolderUrl $DestFolderUrl -LocalFilePath $fullName
            } else {
                createSPFolder -siteUrl $siteUrl -DestFolderUrl $DestFolderUrl -LocalFolderName $name | Out-Null
                #recursively iterate 
                UploadFiles -siteUrl $siteUrl -DestFolderUrl ($DestFolderUrl + "/" +$name) -LocalFileOrFolderPath $fullName 
            }
        }
        write-log -message "Loop completed" -consoleForeground White
    } else {
        UploadandApprove -siteUrl $siteUrl -DestFolderUrl $DestFolderUrl -LocalFilePath $_.FullName
    }
}

############################################# createSPFolder ###########################################
function createSPFolder($siteUrl,  $DestFolderUrl, $LocalFolderName) {
    <#
    
    .SYNOPSIS
        Create SharePoint Folders
    
    .DESCRIPTION
        Retreives the parent folder $DestFolderUrl and adds the folder $LocalFolderName. If it fails an error will be 
        displayed.
        
    .PARAMETER siteUrl
        [string] The site collection URL
    
    .PARAMETER DestFolderUrl
        [string] Where in the SharePoint folder structure to upload to.
    
    .PARAMETER LocalFolderName
        [string] The folder to create
    
    .EXAMPLE
        createSPFolder -siteUrl $siteUrl -DestFolderUrl $DestFolderUrl -LocalFolderName $name
    
    #>
    
    write-log -message "Creating SPFolder:  $LocalFolderName" -consoleForeground Cyan
    $spSite = New-Object Microsoft.SharePoint.SPSite($siteUrl)
    $spWeb = $spSite.OpenWeb()  
    try {  
        $destfolder= $spWeb.GetFolder($DestFolderUrl)
        $destfolder.SubFolders.Add($LocalFolderName)
        $destfolder.update();
    } catch {
        write-log -message "Exception Occurred:" -consoleForeground Red
        if ($($_.Exception -ne $null)) {
            $exception = $_.Exception
            write-log -message "$($exception.GetType().FullName)" -consoleForeground Red
            if (([pscustomobject]$exception.psobject.properties.match('Message')) -and $($exception.Message) -ne $null) { write-log -message "$($exception.Message)" -consoleForeground Red }
            if (([pscustomobject]$exception.psobject.properties.match('ErrorRecord')) -and $($exception.ErrorRecord) -ne $null) { write-log -message "$($exception.ErrorRecord)" -consoleForeground Red }
            if (([pscustomobject]$exception.psobject.properties.match('stacktrace')) -and $($exception.stacktrace) -ne $null) { write-log -message "$($exception.stacktrace)" -consoleForeground Red }
        } else {
            write-log -message "$exception" -consoleForeground Red
        }
        if (([pscustomobject]$_.psobject.properties.match('ScriptStackTrace')) -and $($_.ScriptStackTrace -ne $null)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
    } finally {
        $spSite.Dispose()
        $spWeb.Dispose()
    }
}
############################################# UploadandApprove ###########################################

function UploadandApprove($siteUrl, $DestFolderUrl, $LocalFilePath) {
    <#
    
    .SYNOPSIS
        Checkout, upload, and approve a file
    
    .DESCRIPTION
        Checks out the file from the SharePoint destination folder, uploads the new file, checks in the file, 
        and approves the change.
        
    .PARAMETER siteUrl
        [string] The site collection URL
    
    .PARAMETER DestFolderUrl
        [string] Where in the SharePoint folder structure to upload to.
    
    .PARAMETER LocalFilePath
        [string] The file to upload
    
    .EXAMPLE
        UploadandApprove -siteUrl $siteUrl -DestFolderUrl $DestFolderUrl -LocalFilePath $_.FullName
    
    #>
    write-log -message "Upload And Approve $LocalFilePath to $DestFolderUrl" -consoleForeground Cyan
    $spSite = New-Object Microsoft.SharePoint.SPSite($siteUrl)
    $spWeb = $spSite.OpenWeb()  
    try {  
        $destfolder= $spWeb.GetFolder($DestFolderUrl) 
        $destfolderCheckoutRequired = $destfolder.RequiresCheckout
        $spFiles = $destfolder.Files;  
        write-log -message "Upload..." -consoleForeground Yellow
        if($($destfolder.Item) -ne $null) {
            $spList = $destfolder.Item.ParentList
            write-log -message "  $($destfolder.Item) found so the Parent list is $spList" -consoleForeground Yellow
        } else {
            $spList = $false
            write-log -message "  $($destfolder.Item) NOT found so the Parent list is $spList" -consoleForeground Yellow
        }
        
        $fileInfo = Get-Item $LocalFilePath
        $fileContent = Get-Content $fileInfo -encoding byte
        $fileName = $fileInfo.Name
        #$ExistingFile=$spFiles[$fileName]
        $ExistingFile=$spWeb.GetFile("$DestFolderUrl/$fileName")
        #if(($ExistingFile.Exists) -and ($spList) -and ($spList.EnableVersioning) -eq $true) {
        #if($($ExistingFile -ne $null) -and $($ExistingFile.Exists) -and $($spList -ne $null) -and ($($spList.EnableVersioning) -eq $true)) {
        if ($($ExistingFile -ne $null) -and $($spList -ne $null)) {
            write-log -message "  Attempting to checkout existing file $fileName in list $spList" -consoleForeground Gray
            if ([pscustomobject]$spList.psobject.properties.match('EnableVersioning') -and ($spList -ne $false)){
                if ($($spList.EnableVersioning) -eq $true) {
                    if ([pscustomobject]$ExistingFile.psobject.properties.match('Exists')) {
                        if ($($ExistingFile.Exists)) {
	                       #check out
	                       $ExistingFile.CheckOut()
                        } else {
                            write-log -message "    File $fileName either does not exist or doesn't have the Exists property" -consoleForeground Gray
                        }
                    }
                }
            } else {
                write-log -message "    $spList either does not have the EnableVersioning property or the property is false" -consoleForeground Gray
            }
        } else {
            write-log -message "    File $fileName in list $spList does not exist - no need to checkout" -consoleForeground Yellow
        }
 
        #update file
        write-log -message "Calling SPFile.Add with $fileName" -consoleForeground Cyan
        $spFile = $spFiles.Add($fileName,$fileContent,$true);
         
        if($spFile.CheckOutType  -ne [Microsoft.SharePoint.SPFile+SPCheckOutType]::None) {
            $spFile.CheckIn("")
        }

        if(($spList -ne $null) -and $([pscustomobject]$spList.psobject.properties.match('EnableVersioning')) -and $($spList.EnableVersioning) -and $([pscustomobject]$spList.psobject.properties.match('EnableMinorVersions')) -and $($spList.EnableMinorVersions -eq $true)) {
            $spFile.publish("");
        }
        <#
        $spSite.Dispose();
        #Dispose of Web object
        $spWeb.Dispose() 

        ########################### Approve Items ############################################

        $spSite = New-Object Microsoft.SharePoint.SPSite($siteUrl)
        $spWeb = $spSite.OpenWeb() 

        $destfolder= $spWeb.GetFolder($DestFolderUrl) 
        $spFiles = $destfolder.Files
        write-log -message "Approve..." -consoleForeground Yellow
        if($($destfolder.Item) -ne $null) {
            $spList = $destfolder.Item.ParentList
            write-log -message "  $($destfolder.Item) found so the Parent list is $spList" -consoleForeground Yellow
        } else {
            $spList = $false
            write-log -message "  $($destfolder.Item) NOT found so the Parent list is $spList" -consoleForeground Yellow
        }
 
        $fileInfo = Get-Item $LocalFilePath
        $fileContent = Get-Content $fileInfo -encoding byte
        $fileName = $fileInfo.Name
        $ExistingFile=$spFiles[$fileName]
        #>
        #if($($ExistingFile -ne $null) -and ($ExistingFile.Exists) -and ($spList -ne $null) -and ($spList.EnableVersioning) -eq $true) {
        #if(($ExistingFile.Exists) -and ($spList) -and ($spList.EnableModeration) -eq $true) {
        if ($($ExistingFile -ne $null) -and $($spList -ne $null)) {
            if ($([pscustomobject]$spList.psobject.properties.match('EnableModeration')) -and ($($spList.EnableModeration) -eq $true)) {
                $ExistingFile.Approve('')
            }
        } 
        #Dispose of Web object
        $spWeb.Dispose()
    } catch {
        write-log -message "Exception Occurred:" -consoleForeground Red
        if ($($_.Exception -ne $null)) {
            $exception = $_.Exception
            write-log -message "$($exception.GetType().FullName)" -consoleForeground Red
            if (([pscustomobject]$exception.psobject.properties.match('Message')) -and $($exception.Message) -ne $null) { write-log -message "$($exception.Message)" -consoleForeground Red }
            if (([pscustomobject]$exception.psobject.properties.match('ErrorRecord')) -and $($exception.ErrorRecord) -ne $null) { write-log -message "$($exception.ErrorRecord)" -consoleForeground Red }
            if (([pscustomobject]$exception.psobject.properties.match('stacktrace')) -and $($exception.stacktrace) -ne $null) { write-log -message "$($exception.stacktrace)" -consoleForeground Red }
        } else {
            write-log -message "$exception" -consoleForeground Red
        }
        if (([pscustomobject]$_.psobject.properties.match('ScriptStackTrace')) -and $($_.ScriptStackTrace -ne $null)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
    } finally {
        $spSite.Dispose()
        $spWeb.Dispose()
    }
}

############# Begin here ############################################

write-log -message "DeployFiles.." -consoleForeground Cyan

#$siteUrl = "http://DNS.URL/sites/IRBApplications"

# The local path can be file or folder
# If folder, then all files will be iterated and uploaded.

$siteUrl = Get-SiteURL

$destFolderUrl = "$siteUrl/_catalogs"
UploadFiles -siteUrl $siteUrl -DestFolderUrl $destFolderUrl -LocalFileOrFolderPath $fileLocationCatalog

$destFolderUrl = "$siteUrl/SiteAssets"
UploadFiles -siteUrl $siteUrl -DestFolderUrl $destFolderUrl -LocalFileOrFolderPath $fileLocationSiteAsset 

$destFolderUrl = "$siteUrl/SitePages"
UploadFiles -siteUrl $siteUrl -DestFolderUrl $destFolderUrl -LocalFileOrFolderPath $fileLocationSitePages 

$destFolderUrl = "$siteUrl/PublishingImages"
UploadFiles -siteUrl $siteUrl -DestFolderUrl $destFolderUrl -LocalFileOrFolderPath $fileLocationOnlineHelp 


$mySiteUrl = get-MySiteUrl
UploadFiles -siteUrl $mySiteUrl -DestFolderUrl ($mySiteUrl) -LocalFileOrFolderPath $fileLocationMySitesPages
