<#

.SYNOPSIS
    DeploymentDriver controls the flow and sequence of the deployment steps.

.DESCRIPTION
    DeploymentDriver controls the flow and sequence of the deployment steps by calling the other specialty scripts
    in the sequence required for the update. It also tracks and provides progress until the process is complete. When 
    complete, a final progress is written to inform the user.

.PARAMETER siteUrl
    [String] The URL for the site collection.

.PARAMETER fileLocationPS
    [String] The base directory of the powershell directory in the deployment package. Built using the RAMS release 
    number, e.g. RAMS-2.3.0, and adding the ps directory. For example, D:\RAMS-2.3.0\ps. 

.PARAMETER fileLocationTemplate
    [String] The base directory of the templates directory in the deployment package. Built using the RAMS release 
    number, e.g. RAMS-2.3.0, and adding the Templates directory. For example, D:\RAMS-2.3.0\Templates.

.PARAMETER fileLocationSiteAsset
    [String] The base directory of the site assets directory in the deployment package. Built using the RAMS release 
    number, e.g. RAMS-2.3.0, and adding the site assets directory. For example, D:\RAMS-2.3.0\SiteAssets.

.PARAMETER reportTargetServerName
    [String] The host information for the SSRS server.

.PARAMETER reportSourceDirectory
    [String] The base directory of the reports directory in the deployment package. Built using the RAMS release 
    number, e.g. RAMS-2.3.0, and adding the reports directory. For example, D:\RAMS-2.3.0\Reports.

.PARAMETER wfTempFileLocation
    [String] The base directory of the Temp\NotificationWF.csv file in the deployment package. Built using the RAMS 
    release number, e.g. RAMS-2.3.0, and adding the directory and file name. For example, 
    D:\RAMS-2.3.0\Temp\NotificationWF.csv. Provided, but not used in RAMS-2.3.0.

.PARAMETER rdsConnStringTable
    [hashtable] The connection strings for the RAMS_DB, RAMS_SP, and DataSource1. RAMS_DB is set to 
    "Data Source=$sqlServerName;Initial Catalog=RAMS". RAMS_SP and DataSource1 are set to $siteUrl.

.PARAMETER rdsCredentialsTable
    [hashtable] The login credentials for the RAMS_DB and RAMS_SP connections.contains the SUPERREADER userid and 
    password

.NOTES
    RAMS Release: 2.3.0
    Author: By Light
    Assumptions: The user executing the script must be part of the Site Collection Administration group.

.EXAMPLE
    #See DeployAll.ps1 for a full, running example in the Install-RAMS function.
    $Global:scriptInc = Join-Path $fileLocationPS DeploymentDriver.ps1
    write-log -message "$Global:scriptInc" -consoleForeground Magenta
    . $Global:scriptInc -siteUrl $siteUrl -fileLocationPS $fileLocationPS -fileLocationTemplate $fileLocationTemplate -fileLocationSiteAsset $fileLocationSiteAsset -reportTargetServerName $reportTargetServerName -reportSourceDirectory $reportSourceDirectory -wfTempFileLocation $wfTempFileLocation -rdsConnStringTable $rdsConnStringTable -rdsCredentialsTable $rdsCredentialsTable

#>

#SPSite
Param( [Parameter(Mandatory=$True)]
    [string]$siteUrl,
    
    [Parameter(Mandatory=$True)]
    [string]$fileLocationPS,
    
    [Parameter(Mandatory=$True)]
    [string]$fileLocationTemplate,
    
    [Parameter(Mandatory=$True)]
    [string]$fileLocationSiteAsset,
    
    [Parameter(Mandatory=$True)]
    [string]$reportTargetServerName,
    
    [Parameter(Mandatory=$True)]
    [string]$reportSourceDirectory,
    
    [Parameter(Mandatory=$True)]
    [string]$wfTempFileLocation,
    
    [Parameter(Mandatory=$True)]
    [string]$fileLocationMySitesPages,
    
    # DataSources Connection string Table
    [Parameter(Mandatory=$True)]
    [hashtable]$rdsConnStringTable,
    
    # DataSource Credentials: Username and Password if needed
    [Parameter(Mandatory=$True)]
    [hashtable]$rdsCredentialsTable

)

$Global:perComplete = 0 
$Global:count = 2
$ConfirmPreference = "None"

Run-Script 005 DisableEventReceiver.ps1
Run-Script 010 UpdateEnvironmentList.ps1
Run-Script 020 NewCommittteeStatus.ps1
Run-Script 030 NewFacilityInfo.ps1
Run-Script 040 CreateAnimalFacilityManagerGroups.ps1
Run-Script 050 NewCommittteeInfo.ps1
Run-Script 060 UpdateApplicationList.ps1
Run-Script 070 AddOffsiteWaiverSiteColumn.ps1
Run-Script 080 CreateOffsiteWaiverRequestsView.ps1
Run-Script 090 MigrateMigrateOffsiteWaiverData.ps1
Run-Script 100 AddAdverseReportableSiteColumns.ps1
Run-Script 110 CreateAFDocumentLibrary.ps1
Run-Script 120 CreateInvestigatorDocumentLibrary.ps1
Run-Script 130 EnableVersioningOnAFDocumentLibrary.ps1 
Run-Script 140 CreateFacilitiesEditView.ps1
Run-Script 150 UpdateTMSList.ps1
Run-Script 160 CreateMyTMSView.ps1 
Run-Script 170 UpdateTMSAllItemsView.ps1
Run-Script 180 AddApplicationSiteColumnsForCommittees.ps1
Run-Script 190 CreateAdverseReportableView.ps1
Run-Script 200 CreateFacilityCalendars.ps1
Run-Script 210 CreateNewCalendarViews.ps1
Run-Script 220 MigrateCIRBReportComments.ps1
Run-Script 230 NewSSRSList.ps1
Run-Script 240 MigrateNotificationRules.ps1
Run-Script 250 UpdateCalendarLookups.ps1
Run-Script 260 FixCalendarOrder.ps1
Run-Script 270 TurnOffApplicationListThrottling.ps1
Run-Script 280 UpdateGroupViewPermission.ps1
Run-Script 290 EnableVersioningOnAllDocLib.ps1
Run-Script 300 MigrateCommitteeRoster.ps1
Run-Script 310 CreateCommitteeRosterView.ps1
Run-Script 320 UpdateSubmittedColumn.ps1
Run-Script 330 MigrateFWAMOU.ps1
Run-Script 340 MigrateDataSharingRepository.ps1
Run-Script 350 UpdateLSAGroupPermission.ps1
Run-Script 360 UpdateCOIDisclosureView.ps1
Run-Script 370 DeployFiles.ps1
Run-Script 380 ScheduleTask.ps1

$Global:perComplete = $Global:perComplete + $Global:count
Write-Progress -Activity "RAMS 2.3 Deployment - Step: 390" -PercentComplete $Global:perComplete
write-log -message "Run Reports deployment script" -consoleForeground Cyan
$scriptInc = Join-Path $fileLocationPS Reports_Import.ps1
write-log -message "$scriptInc" -consoleForeground Cyan
. $scriptInc -webServiceUrl "http://$reportTargetServerName" -reportFolder "RAMS Reports" -DataSourcesFolder "Data Sources" -SourceDirectory $reportSourceDirectory -overwrite $True -rdsConnStringTable $rdsConnStringTable -rdsCredentialsTable $rdsCredentialsTable

#Run-Script 400 Reports_Import.ps1

Run-Script 410 MasterPageSetter.ps1

Write-Progress -Activity "RAMS 2.3 Deployment - Step: 420" -PercentComplete $Global:perComplete
write-log -message "Remove old sitepages" -consoleForeground Cyan
#clean old site pages and site assets.
Remove-SitePages -sitePages manageCommittee.html
Remove-SitePages -sitePages Untitled_1.aspx
Remove-SitePages -sitePages Version.aspx

Run-Script 425 Disable-ListAttachments.ps1

Run-Script 430 removeAllWorkflows.ps1

Write-Progress -Activity "RAMS 2.3 Deployment - Step: 440 Facility level workflows" -PercentComplete $Global:perComplete
[Microsoft.SharePoint.SPWeb]$rootWeb = (Get-SiteWeb)
Import-csv .\csv\WfList.csv | select * -Unique | % {
    #get wf name
    $newWfName = $_.NewWFName
    $newFeatureId = $_.NewFeatureID
    $oldFeatureId = $_.OldWFName
    $targetName =  $_.TargetName
    $wfType =  $_.wfType
    write-log -message " Adding Workflow $($newWfName) FeatureId: $newFeatureId TargetName: $($targetName) wfType: $($wfType)" -consoleForeground Gray
    
    #for facility level WFs
    if ($wfType -eq "FacilityList") {
        $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
        Add-FacilityWorkflow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -listName $targetName -unInstall $False
    } elseif (($wfType -eq "SiteList") -or (($wfType -eq "Site"))) {
        $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
        Add-SiteCollectionWorkFlow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -unInstall $False
    } elseif ($wfType -eq "SiteWFContentTypeAssociation") {
        $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
        Add-ReusableWorkFlowToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -contentTypeName $targetName -unInstall $True
    } elseif ($wfType -eq "SiteEventReceiver") {
        $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
        Add-EventReceiverToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -contentTypeName $targetName -unInstall $True
    } else {
        write-log -message "ERROR: Unknown wfType: wfType: $($wfType)" -consoleForeground Red -Level "ERROR"
    }
}

Run-Script 450 getWorkflows.V3.ps1

#all done
Write-Progress -Activity "RAMS 2.3 Deployment completed" -PercentComplete $Global:perComplete -Completed 