<#

.SYNOPSIS
    Periodically update the subsite ApplicationFacilityLookup list from the application list

.DESCRIPTION
    Run from the scheduled task every 10 minutes to update the ApplicationFacilityLookup from the Application list.
    Loops through all application items and adds an entry in the ApplicationFacilityLookup list for the site listed in 
    the Application List Facilities Lookup. The ApplicationFacilityLookup list is at the subsite level so the 
    application lookups on the calendars can show applications for the facility. 
    
.EXAMPLE
    #when run from command line
    .\MigrateFacilityDataScheduleTask.ps1 -localSiteUrl $siteUrl
#>
Param( [Parameter(Mandatory=$True)]
    [string]$localSiteUrl
)

Add-PSSnapin Microsoft.SharePoint.Powershell


$facilities = @{
}

$recordCounts = @{
}

$recordChangedCounts = @{
}

function GetAllWebs($url) {
    #try {
        write-host "$([String]::Format("Procesing web {0}",$w.Url))" -foregroundColor cyan
        $applicationlist =$w.Lists["Application"]
        $applicationitems = $applicationlist.items
        
        #Loop through List
        #Get Facilities Lookup
        #Copy record to correct list
        foreach ($item in $applicationitems) {
            $applicationStatusField = $item["Application Status"] 
            $applicationStudyTitleField = $item["Study Title"] 
            $applicationStudyIDField = $item["Study_ID"] 
            $applicationTitleField = $item["Title"];
            $idField = $item["ID"] 
            $applicationFacilityLookup=$item["Facilities Lookup"]
            if ($applicationFacilityLookup -ne $null) {
                $applicationFacilityLookup = $applicationFacilityLookup.Substring(3)    
            }
            if($w.Webs.Count -gt 0 -and $applicationFacilityLookup) {
                foreach($web in $w.Webs){
                    $facilityNameKey=$web.Url;
                    $facilityNameLookup=$facilities.Item($web.Url);
                    if($facilityNameLookup -eq $applicationFacilityLookup -and $applicationStatusField -ne 'Draft') {
                        MigrateToNewList $web $applicationStatusField $applicationStudyTitleField $idField $applicationStudyIDField $applicationTitleField
                    }
                }
            }
        }
    #} catch {
    #    write-host "$([String]::Format("Error processing web at $url, with Exception: {0}", $_.Exception.Message))" -foregroundColor Red
    #    if ($($_.Exception.ErrorRecord)) { write-host "$($_.Exception.ErrorRecord)" -foregroundColor Red }
    #    if ($($_.Exception.stacktrace)) { write-host "$($_.Exception.stacktrace)" -foregroundColor Red }
    #    if ($($_.ScriptStackTrace)) { write-host "$($_.ScriptStackTrace)" -foregroundColor Red }
    #}   
} 
    
function getAllFacilities ($rootSiteCollectionUrl) {
    #Get the target SP Web
    $scWeb = Get-SPWeb -identity $rootSiteCollectionUrl
    
    #Get the Source List
    $facilityList = $scWeb.Lists["Facilities"]
    $facilityitems = $facilityList.items
    
    foreach($facilityItem in $facilityitems) {
        $facilityName=$facilityItem["Facility Name"]
        $fldurl=new-object Microsoft.SharePoint.SPFieldUrlValue($facilityItem["FacilitySubsite"])
        $facilitySubsite=$fldurl.URL
        $facilityId=$facilityItem["ID"]
        #write-host "Adding facilities Name=$facilityName ID=$facilityId URL=$facilitySubsite" -foregroundColor Green
        $facilities.add($facilitySubsite, $facilityName)
    }
}

function getRecordCounts ($rootSiteCollectionUrl) {
    $applicationlist =$w.Lists["Application"]
    $applicationitems = $applicationlist.items
    $count = 0
    
    #Loop through List
    #Get Facilities Lookup
    #Copy record to correct list
    foreach ($item in $applicationitems) {
        $count++
        $applicationFacilityLookup=$item["Facilities Lookup"]
        
        if ($applicationFacilityLookup -ne $null -and $applicationFacilityLookup -ne "") {
            $applicationFacilityLookup = $applicationFacilityLookup.Substring(3)
            $currentFacilityCount = $recordCounts.Item($applicationFacilityLookup)
            
            if ($currentFacilityCount -ne $null) {
                $currentFacilityCount++
                $recordCounts.Remove($applicationFacilityLookup)
                $recordCounts.add($applicationFacilityLookup, $currentFacilityCount)
            } else {
                $currentFacilityCount=1
                $recordCounts.add($applicationFacilityLookup, $currentFacilityCount)
            }
        }
    }
   $recordCounts.add("ApplicationTotalCount", $count)
}
    
function MigrateToNewList($subsite, $applicationStatusField, $applicationStudyTitleField, $idField, $studyIDField, $applicationTitleField)  {
    #Get the target SP Web
    $scWeb = Get-SPWeb -identity $subsite.URL
    
    #Get the Source List
    $applicationFacilityLookupList = $scWeb.Lists["ApplicationFacilityLookup"]
    $lookupRecorditems = $applicationFacilityLookupList.items
 
    $applicationRecordIDField="ApplicationRecordID"
    $facilityLookupItems = $applicationFacilityLookupList.Items | where {$_[$applicationRecordIDField] -eq $idField}
   
    $updatedKey = $subsite.Title + "-Updated"
    $updatedCounts = $recordChangedCounts.Item($updatedKey)

    $newKey = $subsite.Title + "-New"
    $newCounts = $recordChangedCounts.Item($newKey)

    $listName = "ApplicationFacilityLookup"
    if ($facilityLookupItems -eq $null) {
        $list = $web.Lists[$listname]
        if ($list -ne $null) {
            write-host "Migrate To New List $($subsite.Title)" -ForegroundColor White
            $newItem = $list.AddItem()
            $newItem["Application Status"] = $applicationStatusField
            $newItem["Study Title"] = $applicationStudyTitleField
            $newItem["ApplicationRecordID"] = $idField
            $newItem["Study_ID"] = $studyIDField
            $newItem["Title"] = $applicationTitleField
            $newItem.update()

            if ($recordChangedCounts.contains($newKey)) {
                $newCounts++
                $recordChangedCounts.Remove($newKey);
                $recordChangedCounts.Add($newKey, $newCounts)
            } else {
                $newCounts = 1
                $recordChangedCounts.Add($newKey, $newCounts)
            }
        }
    } else {
        foreach($oldItem in $facilityLookupItems) {
            $oldItem["Application Status"] = $applicationStatusField
            $oldItem["Study Title"] = $applicationStudyTitleField
            $oldItem["ApplicationRecordID"] = $idField
            $oldItem["Study_ID"] = $studyIDField
            $oldItem["Title"] = $applicationTitleField
            $oldItem.update()

            if ($recordChangedCounts.contains($updatedKey)) {
                $updatedCounts++
                $recordChangedCounts.Remove($updatedKey)
                $recordChangedCounts.Add($updatedKey, $updatedCounts)
            } else {
                $updatedCounts = 1
                $recordChangedCounts.Add($updatedKey, $updatedCounts)
            }
        }
    }
    write-host "Migrated for: $scWeb" -foregroundColor Green
}

function OutputStatsToLog($startTime, $endTime) {
    $time = Get-Date -Format "yyyyMMddHHmmss"

    $filePath = "C:\temp\" + "MigrateFacilityData-" + $time + ".log"

    $currentLine ="Start Time  $startTime End Time $endTime " 
    Write-Output $currentLine | Out-File -FilePath $filepath -Append
    
    $currentLine ="Summary Of Record Counts"
    Write-Output $currentLine | Out-File -FilePath $filepath -Append
    $recordCounts.GetEnumerator() | % { 
        $currentKey=$($_.key)
        $currentValue=$($_.value)
        $currentLine = "$currentKey Total Records=$currentValue"
        Write-Output $currentLine | Out-File -FilePath $filepath -Append
    }

    $currentLine ="Summary Of New/Updated Records"
    Write-Output $currentLine | Out-File -FilePath $filepath -Append
    $recordChangedCounts.GetEnumerator() | % { 
        $currentKey=$($_.key)
        $updatedName = $currentKey.replace("-New"," New Records")
        $updatedName = $updatedName.replace("-Updated"," Updated Records")
        $currentValue=$($_.value)
        $currentLine = "$updatedName    $currentValue"
        Write-Output $currentLine | Out-File -FilePath $filepath -Append
    }
}

#This is to test the driver contains a $localSiteUrl
$startTime = Get-Date -Format "yyyyMMddHHmmss"
#$localSiteUrl="https://DNS.URL/sites/IRBApplications/";
$w = Get-SPWeb $localSiteUrl
getAllFacilities $localSiteUrl
getRecordCounts $localSiteUrl
GetAllWebs $localSiteUrl
$endTime = Get-Date -Format "yyyyMMddHHmmss"
OutputStatsToLog $startTime $endTime
