<#

.SYNOPSIS
    Migrates the notification rules .

.DESCRIPTION
    Changes the facility lookup to use the new facilities list. 
    Migrates the data from the old column to the new values

.EXAMPLE
    #when run from deploymentDriver.ps1
    Run-Script 240 MigrateNotificationRules.ps1

.EXAMPLE
    #when run from command line
    .\MigrateNotificationRules.ps1
#>
Add-PSSnapin Microsoft.SharePoint.Powershell

function MigrationNotificationWorkflowConfigurationItems($url) {
    try {
        write-log -message "$([String]::Format("Procesing web {0}",$w.Url))" -consoleForeground Cyan
        $notificationlist =$w.Lists["Notification Workflow Configuration"]
        AddSiteColumnToList $notificationlist
        $notificationitems = $notificationlist.items
        #Loop through List
        #Get Facility Lookup
        #Copy record to correct field
        
        foreach ($item in $notificationitems) {
            $facilityNameLookup=$item["Facility Name"]
            if($facilityNameLookup) {
                $facilityNameIndex=$facilityNameLookup.IndexOf("#") + 1
                $facilityNameLookupTitle = $facilityNameLookup.substring($facilityNameIndex)
                $notificationsFacilityLookupItem =GetFacilitiesItem($facilityNameLookupTitle)
               
                # should add columns before update
                if ($notificationsFacilityLookupItem.ID -ne $null) {
                    $item["Site Name"] = $notificationsFacilityLookupItem.ID
                }
                $item.Update()
                $notificationlist.Update()
            }
        }

        if(($notificationlist.Fields | ?{$_.Title -match 'Facility Name'}) -eq $True) {
            #Delete Should Happen After Migration
            $deleteField=$notificationlist.Fields["Facility Name"];
            $notificationlist.Fields.Delete($deleteField)    
            $notificationlist.update()
        }
        
        AdjustContentTypeOnList $notificationlist
        ModifyAllItemsView $w
    } catch {
        write-log -message "$([String]::Format("Error processing web at $url, with Exception: {0}", $_.Exception.Message))" -consoleForeground Red
        if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
        if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
        if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
    } 
    # $w
} 
 
function GetFacilitiesItem ($title) {
    $facilitiesList = (Get-SiteWeb).Lists.TryGetList("Facilities")
    $spListItem =  $facilitiesList.Items | ? {$_["Title"] -match $title}
    return $spListItem;
}
    
function CreateNotificationsFacilityLookups () {
    try {  
        #CREATE SITE COLUMNS
        write-log -message "CREATING SITE COLUMNS" -consoleForeground White
        
        # GET THE LISTS
        $myList = $w.Lists["Facilities"];
        
        #Where the Source List for Lookup Exists
        
        $ParentWebURL=$siteURL
        $ParentListName="Facilities"
        $ParentLookupColumnName="Title"
        
        #Where the Lookup Site column Going to get created
        $ChildWebURL=$siteURL
        $ChildLookupColumnName="SiteName"
        
        #Get the Parent and Child Webs and List
        
        $ParentWeb = $w
        $ParentList = $w.Lists[$ParentListName]
        $ChildWeb = $w
        
        if(!$ChildWeb.Fields.ContainsField($ChildLookupColumnName)) {
            #Add Lookup Field
            $ChildLookupColumn = $ChildWeb.Fields.AddLookup($ChildLookupColumnName,$ParentList.id,$False)
            $ChildLookupColumn = $ChildWeb.Fields[$ChildLookupColumnName]

            #Setup lookup Field property
            $ChildLookupColumn.LookupWebId = $ParentWeb.ID
            $ChildLookupColumn.LookupField = $ParentList.Fields[$ParentLookupColumnName].InternalName
            #$ChildLookupColumn.AllowMultipleValues=$true
            $ChildLookupColumn.Group="RAMS IRB" 
            $ChildLookupColumn.Title="Site Name"
            
            $ChildLookupColumn.update()
        
            write-log -message "Lookup field added successfully!" -consoleForeground green
        } else {
            write-log -message "Field Exists already!" -consoleForeground red
        }
    } catch {
        write-log -message "Exception Occurred:" -consoleForeground Red
        write-log -message "$($_.Exception.Message)" -consoleForeground Red
        if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
        if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
        if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
    } 
 }

function AddSiteColumnToList ($list) {
    $sFieldInternalName = "SiteName"
    $siteNameField=$w.Fields.TryGetFieldByStaticName($sFieldInternalName) 
    if (($list.Fields.TryGetFieldByStaticName($sFieldInternalName)) -eq $null) {
    $list.Fields.Add($siteNameField)
    $list.update() 
        write-log -message "$sFieldInternalName added to $($list.Title)" -consoleForeground Green
    } else {
        write-log -message "$sFieldInternalName already on $($list.Title)" -consoleForeground Yellow
    }

}

function AdjustContentTypeOnList ($list) {     
    
    #ADJUST CONTENT TYPE ON LIST
    write-log -message "ADJUST CONTENT TYPE ON LIST" -consoleForeground White
    
    $ct = $list.ContentTypes["Item"] 
    
    $fldList = @(
        "Title",
        "SiteName",
        "Notified Users"
    )
    
    $ct.FieldLinks.Reorder($fldList)
    $ct.Update()
    
    #$list.Update()
}

function ModifyAllItemsView ($web) {
    $fldList = @(
        'LinkTitle'; 'SiteName';'Notified Users'
    )
    $viewQuery = "<Where></Where>"
    New-SpFacilityListView -web $web -fldList $fldList -viewTitle 'NotificationRulesView' -listName 'NotificationWorkflowConfiguration' -viewQuery $viewQuery -viewRowLimit '50'
}

   
#This is to test the driver contains a $siteUrl
#$siteUrl="https://DNS.URL/sites/IRBApplications/";
$w = Get-SiteWeb
CreateNotificationsFacilityLookups
$siteUrl = Get-SiteURL

#migrate data
MigrationNotificationWorkflowConfigurationItems $siteUrl

