<#

.SYNOPSIS
    Adds R2.3 fields to the new Committees list.

.DESCRIPTION
    Creates the Delegated and Facilities Lookup columns
    Creates the Hidden and Acronym site Columns
    Creates the Administratirs, Committee Coordinators, and Committee Members site columns
    Adds the fields above the the Committees CT Content Type
    Creates the new Committees list from the Committees CT
    Renames the old "Committee List" to "Old Comittee List"
    Builds the CommitteesView
    For each CommitteeType and Facility builds the committee entries.

.EXAMPLE
    #when run from deploymentDriver.ps1
    Run-Script 050 NewCommittteeInfo.ps1

.EXAMPLE
    #when run from command line
    .\NewCommittteeInfo.ps1
#>
$weburl = Get-SiteURL
$webSite = Get-SiteWeb

#add new lookup site column
#Set-SiteColumnLookup -web $webSite -colName "Committee Type" -colStaticName "CommitteeType" -lookupListName "Committee Types" -lookupColumnName "Committee's Name" 
Set-SiteColumnLookup -web $webSite -colName "Delegated" -colStaticName "Delegated" -lookupListName "Facilities" -lookupColumnName "Title" 
Set-SiteColumnLookup -web $webSite -colName "Facilities Lookup" -colStaticName "FacilitiesLookup" -lookupListName "Facilities" -lookupColumnName "Title" 

#add new Site-column
Set-SiteColumn -web $webSite -sFieldDisplayName "Hidden" -sFieldStaticName "Hidden" -sFieldInternalName "Hidden" -sFieldType Boolean -spFieldGroup "RAMS IRB" -isRequired -showInEditForm -showInNewForm
Set-SiteColumn -web $webSite -sFieldDisplayName "Acronym" -sFieldStaticName "Acronym" -sFieldInternalName "Acronym" -sFieldType Text -spFieldGroup "RAMS IRB" -showInEditForm -showInNewForm

$userFldList = @("Administrators",
    "Committee Coordinators",
    "Committee Members"    
)
$userFldList | % {
    Set-SiteColumn -web $webSite -sFieldDisplayName "$($_)" -sFieldStaticName "$($_.ToString().Replace(' ',''))" -sFieldInternalName "$($_.ToString().Replace(' ',''))" -sFieldType User -spFieldGroup "RAMS IRB" -isRequired -showInEditForm -showInNewForm
}

#Add a new content type.
$newCT = New-Object Microsoft.SharePoint.SPContentType($webSite.AvailableContentTypes["item"],$webSite.ContentTypes,"Committees CT")
$webSite.ContentTypes.Add($newCT) | write-log -consoleForeground Gray

#add set of columns w/ content type
$fldList = @("ID",
    "FacilitiesLookup",
    "CommitteesLookup",    
    "Acronym",
    "Administrators",
    "Committee Coordinators",
    "Committee Members",
    "Delegated",
    "Hidden"
)
$fldList | % {
    Add-SiteColumnToCT -web $webSite -ctName "Committees CT" -sFieldInternalName "$($_.ToString().Replace(' ',''))" -sFieldStaticName "$($_.ToString().Replace(' ',''))"
}

#create new list
$listTemplate = $webSite.ListTemplates | ? {$_.InternalName -match "custlist"}
$webSite.Lists.Add("Committees","Committees", $listTemplate)
$CommitteesList = $webSite.Lists.TryGetList("Committees")
$CommitteesCT = $webSite.ContentTypes["Committees CT"]
#order CT 
$CommitteesCT.FieldLinks.Reorder(($fldList | % {$_.ToString().Replace(" ","")}))
#add content type
$CommitteesList.ContentTypes.Add($CommitteesCT) | write-log -consoleForeground Gray
$CommitteesList.Update()

#rename current facility list
$oldList = $webSite.Lists.TryGetList("Committee List")
$oldList.Title = "Old Committee List"
$oldList.RootFolder.MoveTo("OldCommitteeList")

#create new view
$CommitteesList.Views.Add("CommitteesView",$fldList,"",100,$True,$True) | write-log -consoleForeground Gray
$CommitteesView = $CommitteesList.Views["CommitteesView"]
#set sort order
$orderBy = "
<OrderBy>
    <FieldRef Name='FacilitiesLookup' Ascending='True'/>
    <FieldRef Name='CommitteesLookup' Ascending='True'/>
</OrderBy>"
$CommitteesView.Query = $orderBy
$CommitteesView.Update()

#Change lookup fields 
$facilityList = $webSite.Lists.TryGetList("Facilities") 
$lookupFld = $webSite.Fields.TryGetFieldByStaticName("FacilitiesLookup")
$schema = [xml]$lookupFld.SchemaXml
$schema.Field.Attributes["List"].Value = $facilityList.ID
$schema.Field.Attributes["ShowField"].Value = "Title"
$lookupFld.SchemaXml = $schema.OuterXml.ToString()
$lookupFld.Update()

#populate premilinary committee list data
$committeeTypeListItems = ($website.Lists.TryGetList("Committee Types")).Items
$facilityListItems = ($website.Lists.TryGetList("Facilities")).Items
$facilities = $facilityListItems | Select-Object -Property Title
$CommitteesList = $webSite.Lists.TryGetList("Committees")

$committeeTypeListItems | % {    
    $committeeName = $_["Committee's Name"]
    $facilities | % {
        $title = $_.Title        
        $newItem = $CommitteesList.AddItem()                
        $newItem["Acronym"] = $committeeName
        $newItem["Title"] = $committeeName
        $newItem["Facilities Lookup"] = ($facilityListItems | ? {$_.Title -match $title}).ID
        $newItem["Committees Lookup"] = ($committeeTypeListItems | ? {$_["Committee's Name"] -match $committeeName}).ID 
        $newItem["Administrators"] = $website.Groups["$($_.Title) $committeeName Administrators"]
        $newItem["Committee Coordinators"] = $website.Groups["$($_.Title) $committeeName Committee Coordinators"]
        $newItem["Committee Members"] = $website.Groups["$($_.Title) $committeeName Committee Members"]
        $newItem["Hidden"] = [Boolean]::FalseString
        $newItem.Update()
    }
}


#Turn on versioning for the Facility List
$CommitteesList.EnableVersioning = $true
$CommitteesList.EnableMinorVersions = $false

$webSite.Update()

#clean
if($webSite -ne $null) {
    $webSite.Dispose();
}