<#

.SYNOPSIS
    Bulk import SSRS Reports from a folder and update the reports data source.

.DESCRIPTION
    Makes a connection to the SSRS server. Creates the Reports Folder, subfolders, and Data Source Folders on the 
    SSRS server, if they do not exist already. Loops through all the Data Source (RDS) files and configures them on 
    the SSRS server. Loops through the Report Source (RDL) files for each subfolder and parent folder, uploading 
    them to the SSRS server. 

.PARAMETER webServiceUrl
    [string] The SSRS web services URL.

.PARAMETER reportFolder
    [string] The name of the folder on the SSRS server where the reports will be imported. "RAMS Reports" is the 
    default. 

.PARAMETER DataSourcesFolder
    [string] The name of the folder on the SSRS server where the data sources will be imported. "Data Sources" is the 
    default. 

.PARAMETER SourceDirectory
    [string] The base directory of the reports directory in the deployment package. Built using the RAMS release 
    number, e.g. RAMS-2.3.0, and adding the reports directory. For example, D:\RAMS-2.3.0\Reports.

.PARAMETER overwrite
    [boolean] if $true, indicates that the report should be overwritten. 

.PARAMETER rdsConnStringTable
    [hashtable] The connection strings for the RAMS_DB, RAMS_SP, and DataSource1. RAMS_DB is set to 
    "Data Source=$sqlServerName;Initial Catalog=RAMS". RAMS_SP and DataSource1 are set to $siteUrl.

.PARAMETER rdsCredentialsTable
    [hashtable] The login credentials for the RAMS_DB and RAMS_SP connections.contains the SUPERREADER userid and 
    password

.EXAMPLE
    #when run from deploymentDriver.ps1
    . $scriptInc -webServiceUrl "http://$reportTargetServerName" -reportFolder "RAMS Reports" -DataSourcesFolder "Data Sources" -SourceDirectory $reportSourceDirectory -overwrite $True -rdsConnStringTable $rdsConnStringTable -rdsCredentialsTable $rdsCredentialsTable

#>

Param(
    [string]$webServiceUrl,
    [string]$reportFolder = "RAMS Reports",
    [string]$DataSourcesFolder = "Data Sources",
    [string]$SourceDirectory = $PSScriptRoot,
    [bool]$overwrite = $true,
    # DataSources Connection string Table
    [hashtable]$rdsConnStringTable,
    # DataSource Credentials: Username and Password if needed
    [hashtable]$rdsCredentialsTable
)

#$rdsConnStringTable = @{ "RAMS_SP" = "http://DNS.URL/sites/IRBApplications/" ; "DataSource1" = "http://DNS.URL/sites/IRBApplications/"; "test" = "Data Source=DNS;Initial Catalog=RAMS"};
#$rdsCredentialsTable = @{"RAMS_DB" = @("BLABLA","BLAHBLAH")};

#Connect to SSRS
write-log -message "Reportserver: $webServiceUrl" -consoleForeground Magenta
write-log -message "Estabishing Proxy connection, connecting to : $webServiceUrl/ReportServer/ReportService2010.asmx?WSDL" -consoleForeground Gray
write-log -message  " " -consoleForeground Gray

$ssrsProxy = New-WebServiceProxy -Uri $webServiceUrl'/ReportServer/ReportService2010.asmx?WSDL' -UseDefaultCredential
if ($ssrsProxy -eq $null) {
    write-log -message "SSRS Proxy not created for URL: $webServiceUrl/ReportServer/ReportService2010.asmx?WSDL" -consoleForeground Red
} else {
    write-log -message "SSRS Proxy created for URL: $webServiceUrl/ReportServer/ReportService2010.asmx?WSDL" -consoleForeground Green
}

function SSRSFolder ([string]$reportFolder,[string]$reportPath){
    <#
    
    .SYNOPSIS
        Creates the folder on the SSRS server
    
    .DESCRIPTION
        Creates the folder named $reportFolder at the location specified by $reportPath
    
    .PARAMETER reportFolder
        [string] The folder name.
    
    .PARAMETER reportPath
        [string] The path to the folder on the SSRS server. 
    
    .EXAMPLE
        SSRSFolder $reportFolder "/"
    
    .EXAMPLE
        SSRSFolder "CIRB Reports" "/$reportFolder"
    
    #>

    #Create Folder
    try {
        $ssrsProxy.CreateFolder($reportFolder, $reportPath, $null) | out-null
        if($reportPath -eq '/') {
            write-log -message "$reportpath$reportFolder Folder Created" -consoleForeground Gray
        } else {
            write-log -message "$reportpath/$reportFolder Folder Created" -consoleForeground Gray
        }
    } catch [System.Web.Services.Protocols.SoapException] {
        if ($_.Exception.Detail.InnerText -match "rsItemAlreadyExists400") {
            write-log -message "Folder: $reportFolder already exists." -consoleForeground Gray
        } else {
            $msg = "Error creating folder: $reportFolder. Msg: '{0}'" -f $_.Exception.Detail.InnerText
            Write-Error $msg
        }
    }
}

function SSRSItem ([string]$ItemType,$item,[string]$folder) {
    <#
    
    .SYNOPSIS
        Creates the report item on the SSRS server
    
    .DESCRIPTION
        Uploads the report rdl file to the server, creating the catalog item for the report.
    
    .PARAMETER ItemType
        [string] Type of item. In RAMS-2.3.0 it is always set to "Report" 
    
    .PARAMETER item
        [object] The report rdl file object
    
    .PARAMETER folder
        [string] The path and name of the folder on the SSRS server. 
    
    .EXAMPLE
        SSRSItem Report $rdlfile "/$reportFolder/CIRB Reports"
    
    #>

    write-log -message " " -consoleForeground Gray
    #ReportName
    if ($ItemType -ne "Resource") {
        $ItemName = [System.IO.Path]::GetFileNameWithoutExtension($item);
    } else {
        $ItemName = $item.Name
    }
    write-log -message "$ItemName" -consoleForeground Magenta 
   
    #Upload File
    try {
        #Get Report content in bytes
        write-log -message "Getting file content of: $item" -consoleForeground White
        $byteArray = Get-Content $item.FullName -encoding byte
        $msg = "Size: {0} KB" -f "{0:N0}" -f ($byteArray.Length/1KB) 
        write-log -message "$msg" -consoleForeground White
        
        write-log -message "Uploading $itemName to folder: $folder " -consoleForeground White
 
        #Sets property for images(only png)
        $type = $ssrsProxy.GetType().Namespace
        $datatype = ($type + '.Property')
        $property =New-Object ($datatype);
        if ($ItemType -eq "Resource" -and $item -like "*.png"){
            $property.Name = "MimeType"
            $property.Value = "image/png"
        } else {
            $property = $null
        }

        #Call Proxy to upload report  
        $warnings =@();
        $ssrsProxy.CreateCatalogItem($ItemType,$itemName,$folder,$overwrite,$byteArray,$property,[ref]$warnings) | out-null
        if($warnings.Length -le 1) { 
            write-log -message "Upload of $itemName Success." -consoleForeground Green 
        } else {
            foreach ($message in $warnings) {
                if ($message.Code -ne "rsDataSourceReferenceNotPublished"){
                    write-log -message "$($message.Severity) $($message.Code) $($message.Message)" -consoleForeground Yellow
                }
            }
        }
    } catch [System.IO.IOException] {
        $msg = "Error while reading rdl file : '{0}', Message: '{1}'" -f $rdlFile, $_.Exception.Message
        Write-Error $msg 
    } catch [System.Web.Services.Protocols.SoapException] {
        $caught = $false
        if ($_.Exception.Detail.InnerText -match 'rsItemAlreadyExists400') {
            $caught = $true
            write-log -message "Report: $itemName already exists." -consoleForeground Red
        }
        if ($_.Exception.Detail.InnerText -match 'CoretechSSRS') {
            $caught = $true
            write-log -message "Cant't find Reporting Extention File." -consoleForeground Red
        } elseif ($caught -eq $false) {
            $msg = "Error uploading report: $reportName. Msg: '{0}'" -f $_.Exception.Detail.InnerText
            Write-Error $msg 
        }
    }
}

Function SSRSDatasource ([string]$ReportPath,$DataSourceName,$DataSourcePath){
    <#
    
    .SYNOPSIS
        Creates the data source item on the SSRS server
    
    .DESCRIPTION
        Uploads the data source rds file to the server, creating the catalog item for the data source. Each datasource 
        in the report is looped through and adjusted for the current environment. 
    
    .PARAMETER ReportPath
        [string] Type path to the report on the server.
    
    .PARAMETER DataSourceName
        [object] "Dummy" because it doesn't matter, the script loops through all data sources in the report. 
    
    .PARAMETER DataSourcePath
        [string] The path to the data source on the server.
    
    .EXAMPLE
        #Change Report Datasource
	    $ReportPath = "/" + $reportFolder + "/CIRB Reports/" +  [System.IO.Path]::GetFileNameWithoutExtension($rdlfile)
	    $DataSourcePath = "/" + $DataSourcesFolder;
	    $DatasourceName = "dummy"
	    write-log -message "ReportPath: $ReportPath DataSourcePath: $DataSourcePath" -consoleForeground Gray
	    SSRSDatasource $ReportPath $DataSourceName $DataSourcePath
    
    #>

    $report = $ssrsProxy.GetItemDataSources($ReportPath)
    ForEach ($Source in $report) {
        if($DataSourcePath -match "RAMS_SP") {
            $DataSourcePath= $DataSourcePath.Replace("RAMS_SP","RAMS_DB")
        } else {
            $DataSourcePath= $DataSourcePath +"/"+ $Source.Name;
        }
        $DataSourceName=$Source.Name;
        #write-log -message "SSRSDataSource for DataSourcePath: $DataSourcePath DataSourceName: $DataSourceName" -consoleForeground White
    
        $proxyNamespace = $Source.GetType().Namespace
        
        $constDatasource = New-Object ("$proxyNamespace.DataSource")
        $constDatasource.Name = $DataSourceName
        $constDatasource.Item = New-Object ("$proxyNamespace.DataSourceReference")
        $constDatasource.Item.Reference = $DataSourcePath
        write-log -message "SSRSDataSource for datasource: $DataSourceName DataSourcePath: $DataSourcePath" -consoleForeground White

        $Source.item = $constDatasource.Item
        $ssrsProxy.SetItemDataSources($ReportPath, $Source)
        write-log -message "SSRSDataSource for Changing datasource: $($Source.Name) to $($Source.Item.Reference)" -consoleForeground White
    }
}

##Create Folder Structure
#Create Base Folder
SSRSFolder $reportFolder "/"
SSRSFolder "CIRB Reports" "/$reportFolder"
SSRSFolder $DataSourcesFolder "/"

ForEach($Folder in Get-ChildItem $SourceDirectory -Directory){
    #Add each folder in the sourcefolder to the reporting service 
    SSRSFolder $Folder.Name /$reportFolder
}

#Create the Data Sources
ForEach ($rdsfile in Get-ChildItem $SourceDirectory -Filter *.rds ) { 
    #Get the datasource file name without extension
    $rdsFileName = [System.IO.Path]::GetFileNameWithoutExtension($rdsfile)
    #Source file
    $src = $SourceDirectory +"/"+ $rdsfile

    #Target folder
    $tar = "/"+ $DataSourcesFolder

    write-log -message "FOR: $rdsFileName" -consoleForeground Gray

    # CREATE A NEW DATASOURCE OBJECT WITH SOURCE FILE DATA
    [xml]$XmlDataSourceDefinition = Get-Content $src;
    

    write-log -message "Template Definition" -consoleForeground Gray
    $XmlDataSourceDefinition.DataSourceDefinition

    try { 
        $type = $ssrsproxy.GetType().Namespace;
    } catch { 
        throw $_.Exception;
    }
    
    $dataSourceDefinitionType = ($type + '.DataSourceDefinition');
    $dataSourceDefinition = New-Object ($dataSourceDefinitionType);
    $dataSourceDefinition.Extension = $XmlDataSourceDefinition.DataSourceDefinition.Extension;

    #Set the connectionstring from the $rdsConnStringTable provided

    if($rdsConnStringTable.ContainsKey($rdsFileName)){
        if($rdsConnStringTable.Item($rdsFileName) -ne $null -and $rdsConnStringTable.Item($rdsFileName) -ne "" ){
        $rdsConnStringTable.Item($rdsFileName);
            $dataSourceDefinition.ConnectString = $rdsConnStringTable.Item($rdsFileName);
        } else {
            write-log -message "The Connection string provided against Datasource $rdsFileName is Null or empty" -consoleForeground Red
        }
    } else {
        write-log -message "The Connection string against Datasource $rdsFileName has not been provided, It will be set as blank/empty going further" -consoleForeground Red
        $dataSourceDefinition.ConnectString = "";
    }

    $dataSourceDefinition.CredentialRetrieval = $XmlDataSourceDefinition.DataSourceDefinition.CredentialRetrieval;
    $dataSourceDefinition.WindowsCredentials = $XmlDataSourceDefinition.DataSourceDefinition.WindowsCredentials;
    $dataSourceDefinition.Enabled = $XmlDataSourceDefinition.DataSourceDefinition.Enabled;
    $dataSourceDefinition.ImpersonateUser = $XmlDataSourceDefinition.DataSourceDefinition.ImpersonateUser;

    # Set the authentication Credentials from the $rdsCredentialsTable
    if($rdsCredentialsTable.ContainsKey($rdsFileName)){
        if($rdsCredentialsTable.Item($rdsFileName) -ne $null -and $rdsCredentialsTable.Item($rdsFileName).count -eq 2){
            $dataSourceDefinition.CredentialRetrieval = 'Store';
            $dataSourceDefinition.WindowsCredentials = $true;
            $dataSourceDefinition.UserName=$rdsCredentialsTable.Item($rdsFileName)[0];
            $dataSourceDefinition.Password=$rdsCredentialsTable.Item($rdsFileName)[1];
        } else {
            write-log -message "The Credentials provided against Datasource $rdsFileName is Null or empty" -consoleForeground Red
        }
    }
    
    write-log -message "New Definition" -consoleForeground Gray
    $dataSourceDefinition

    try { 
        #CREATE THE DATASOURCE
        $newDataSource = $ssrsproxy.CreateDataSource([System.IO.Path]::GetFileNameWithoutExtension($rdsfile),$tar,$true,$dataSourceDefinition,$null); 
    } catch { 
        throw $_.Exception; 
    }
}

#Upload Reports in root folder
ForEach ($rdlfile in Get-ChildItem "$SourceDirectory\CIRB Reports" -Filter *.rdl ) {
    SSRSItem Report $rdlfile "/$reportFolder/CIRB Reports"
    #Change Report Datasource
    $ReportPath = "/" + $reportFolder + "/CIRB Reports/" +  [System.IO.Path]::GetFileNameWithoutExtension($rdlfile)
    $DataSourcePath = "/" + $DataSourcesFolder;
    $DatasourceName= "dummy"
    write-log -message "ReportPath: $ReportPath DataSourcePath: $DataSourcePath" -consoleForeground Gray
    SSRSDatasource $ReportPath $DataSourceName $DataSourcePath
}

ForEach ($rdlfile in Get-ChildItem $SourceDirectory -Filter *.rdl ) {
    SSRSItem Report $rdlfile /$reportFolder
    #Change Report Datasource
    $ReportPath = "/" + $reportFolder + "/" +  [System.IO.Path]::GetFileNameWithoutExtension($rdlfile)
    $DataSourcePath = "/" + $DataSourcesFolder;
    $DatasourceName= "dummy"
    write-log -message "ReportPath: $ReportPath DataSourcePath: $DataSourcePath" -consoleForeground Gray
    SSRSDatasource $ReportPath $DataSourceName $DataSourcePath
    
}


##For Each folder
<#
ForEach($Folder in Get-ChildItem $SourceDirectory -Directory) {
    #Add each folder in the sourcefolder to the reporting service 
    #SSRSFolder $Folder.Name /$Basefolder 
    
    #Add reports in the folder
    ForEach ($rdlfile in Get-ChildItem $Folder -Filter *.rdl ) {
        SSRSItem Report $rdlfile /$reportFolder/$($folder.Name) 

        #Change Report Datasource
        $ReportPath = "/" + $reportFolder + "/" + $folder.Name + "/" + $rdlfile.BaseName 
        #SSRSDatasource $ReportPath $DataSourceName $DataSourcePath
    }
}
#>