<#
.SYNOPSIS
    Update the Committee Calendar Meeting Minute Lookups.
    
.DESCRIPTION
    This updated the Meeting Minute Lookups for the Committees.  
        
.EXAMPLE
    #when run from deploymentDriver.ps1
    Run-Script 110 UpdateCalendarLookups.ps1

.EXAMPLE
    #when run from the powershell command prompt.
    .\UpdateCalendarLookups.ps1
#>

Add-PSSnapin Microsoft.SharePoint.Powershell

function GetAllWebs($url) {
    <#
    
    .SYNOPSIS
        This function loops through all subsites.
        
    .DESCRIPTION
        This function loops through all subsites and calls Deletes and creates document library for each subsite.
            
    .EXAMPLE
        #when run from script
        GetAllWebs $url
     
     #>

    try {
        write-log -message  "$([String]::Format("Procesing web {0}",$w.Url))" -consoleForeground DarkCyan
        if($w.Webs.Count -gt 0) {
            foreach($web in $w.Webs) {
                write-log -message  "$([String]::Format("Procesing web {0}",$web.Url))" -consoleForeground Cyan
                <#
                $updNeeded = $false
            
                $ct = $web.ContentTypes["Event"]
            
                $rdcFieldLink = $web.Fields.TryGetFieldByStaticName("RDCCalendarMeetingMinuteLookup")
                $irbFieldLink = $web.Fields.TryGetFieldByStaticName("CommitteeCalendarMeetingMinuteLookup")
                $iacucFieldLink = $web.Fields.TryGetFieldByStaticName("IACUCCalendarMeetingMinuteLookup")
                $srsFieldLink = $web.Fields.TryGetFieldByStaticName("SRSCalendarMeetingMinuteLookup")
            
                #Remove the field from the ct and update
                if ($rdcFieldLink) {
                    $ct.FieldLinks.Delete($rdcFieldLink.Id)
                    $updNeeded = $true
                }
                if ($irbFieldLink) {
                    $ct.FieldLinks.Delete($irbFieldLink.Id)
                    $updNeeded = $true
                }
                if ($iacucFieldLink) {
                    $ct.FieldLinks.Delete($iacucFieldLink.Id)
                    $updNeeded = $true
                }
                if ($srsFieldLink) {
                    $ct.FieldLinks.Delete($srsFieldLink.Id)
                    $updNeeded = $true
                }
                write-log -message "Deleting Old Lookups (RDC/Committee/IACUC/SRS Meeting Minute Lookup) From Event Content Type" -consoleForeground Cyan
            
                if ($updNeeded) {
                    $ct.Update()
                }
                #>
                deleteSiteColumn "RDC" $web
                deleteSiteColumn "IRB" $web
                deleteSiteColumn "IACUC" $web
                deleteSiteColumn "SRS" $web
            
                fixCalendar "RDC" $web
                fixCalendar "IRB" $web
                fixCalendar "IACUC" $web
                fixCalendar "SRS" $web
            
                New-MeetingMinuteLookup $web "RDC"
                New-MeetingMinuteLookup $web "IRB"
                New-MeetingMinuteLookup $web "IACUC"
                New-MeetingMinuteLookup $web "SRS"
            
                #AddMeetingMinute to Event Content Type?
            
                AddLookupsToCalendar "RDC" $web
                AddLookupsToCalendar "IRB" $web
                AddLookupsToCalendar "IACUC" $web
                AddLookupsToCalendar "SRS" $web
            
                #dispose the web-object
                $web.Dispose()
            }
        }
    } catch {
        write-log -message "$([String]::Format("Error processing web at $web, with Exception: {0}", $_.Exception.Message))" -consoleForeground Red
        if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
        if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
        if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
    }   
}

function deleteSiteColumn($committee, $web) {
    if ($committee -eq "IRB") {
        $SiteColumnToRemove = "Committee Meeting Minute Lookup"
    } else {
        $SiteColumnToRemove = $committee + " Meeting Minute Lookup"
    }

    if ($web.Fields.ContainsField($SiteColumnToRemove)) {
        #delete site column powershell sharepoint
        $web.Fields.Delete($SiteColumnToRemove)
        write-log -message "$SiteColumnToRemove Site column has been Deleted Successfully on $($web.url)" -consoleForeground green
    }
}

function fixCalendar ($committee, $web) {
   write-log -message "Fixing Calendar for $committee Committee in $web" -consoleForeground Cyan
    if ($committee -eq "IRB") {
       $committeeList = $web.Lists.TryGetList("Committee Calendar")
       $ColumnName = "Committee Meeting Minute Lookup"
    } else {
       $committeeList = $web.Lists.TryGetList($committee +  " Calendar")
       $ColumnName = $committee + " Meeting Minute Lookup"
    }
    if($committeeList -ne $null) {
        #Get the column 
        if ($committeeList.Fields.ContainsField($ColumnName)) {
            $column = $committeeList.Fields.GetField($columnName)
            if($column -ne $null) {
                #Reset column properties to allow delete
                $column.Hidden = $false
                $column.ReadOnlyField = $false
                $column.AllowDeletion = $true
                $column.Update()
                
                #Delete the column from list
                $committeeList.Fields.Delete($column)
                write-log -message "Column '$ColumnName' deleted on $($web.Url)" -consoleForeground Green
            } else {
                write-log -message "Column '$ColumnName' not found on $($web.Url)" -consoleForeground Yellow
            }
        } else {
           write-log -message "Column $ColumnName Not on list $($Committeelist.Title)" -consoleForeground Yellow
        }
    } else {
        write-log -message "Specified List is not found!" -consoleForeground Red
    }
}

function AddLookupsToCalendar ($committee, $web) {
   write-log -message "AddLookupsToCalendar for $committee Committee in $web" -consoleForeground Cyan
    if ($committee -eq "IRB") {
       $committeeList = $web.Lists.TryGetList("Committee Calendar")
       $ColumnName = "Committee Meeting Minute Lookup"
    } else {
       $committeeList = $web.Lists.TryGetList($committee +  " Calendar")
       $ColumnName = $committee + " Meeting Minute Lookup"
    }

    if($committeeList -ne $null) {
        $lookupField = $committeeList.Fields[$ColumnName]
        if ($lookupField -eq $null) {
            $meetingMinuteLookupName = $ColumnName.Replace(" ","")
            $newField = $web.Fields.TryGetFieldByStaticName($meetingMinuteLookupName)
            $committeeList.Fields.Add($newField);
            write-log -message "$columnName added to $committeeList on $web" -consoleForeground Green

        } else {
            write-log -message "$columnName found on $committeeList on $web" -consoleForeground Yellow
    }
    }
}



#This is to test the driver contains a $siteUrl
#$siteUrl="https://DNS.URL/sites/IRBApplications/";
$w = Get-SPWeb $siteUrl;
GetAllWebs($siteUrl)