<#
.SYNOPSIS
    Generates individual help files from the ps1 script help templates
    
.DESCRIPTION
    Expects the source to be in the d:\RAMS-2.3.0\ps directory.
    Loops through all ps1 files and executes Get-Help, redirecting the output to the d:\RAMS-2.3.0\ps\Documentation 
    directory. Then, for each function that is documented, also generates help information for the function and adds 
    it to the help file.
        
.EXAMPLE
    #when run from the powershell command prompt.
    .\get-helpLocal.ps1 
 #>
<#
[CmdletBinding()]
Param(
   [Parameter(Mandatory=$true,Position=1)]
   [string]$fileName
)
#>

#Modified  From https://stackoverflow.com/questions/45929043/get-all-functions-in-a-powershell-script#
#$rs = [runspacefactory]::CreateRunspace()
#$rs.Open()
#$fileName = "D:\RAMS-2.3.0\PS\DeploymentDriver.ps1"


get-childitem "D:\RAMS-2.3.0\PS" -Filter *.ps1 | % {
    $fileName = $_.FullName
    if(Test-Path -Path $fileName) { 
        $file = Split-Path -Path $fileName -Leaf
        $helpFileName = join-path -path "D:\RAMS-2.3.0\PS\Documentation" -childpath "$file.help.txt"
        Write-host "get-helplocal for $fileName" -ForegroundColor Gray
        Write-host "  output will be in $helpFileName" -ForegroundColor Gray

        Get-Help "$fileName" -Full > "$helpFileName"

        # Get the AST of the file
        $tokens = $errors = $null
        $ast = [System.Management.Automation.Language.Parser]::ParseFile(
            $fileName,
            [ref]$tokens,
            [ref]$errors)

        # Get only function definition ASTs
        $functionDefinitions = $ast.FindAll({
            param([System.Management.Automation.Language.Ast] $Ast)

            $Ast -is [System.Management.Automation.Language.FunctionDefinitionAst] -and
            # Class methods have a FunctionDefinitionAst under them as well, but we don't want them.
            ($PSVersionTable.PSVersion.Major -lt 5 -or
            $Ast.Parent -isnot [System.Management.Automation.Language.FunctionMemberAst])

        }, $true)
        <#
        # Add the functions into the runspace
        $functionDefinitions | ForEach-Object {
            $rs.SessionStateProxy.InvokeProvider.Item.Set(
                'function:\{0}' -f $_.Name,
                $_.Body.GetScriptBlock()) 
        }

        # Get help within the runspace.
        $ps = [powershell]::Create().AddScript('Get-Help MyFunction')
        try {
            $ps.Runspace = $rs
            $ps.Invoke()
        } finally {
            $ps.Dispose()
        }
        #>

        $helpContent = $functionDefinitions | ForEach-Object {

            $functionName = $_.Name
            # Get the plain string comment block from the AST.
            try {
                $commentBlock = $_.GetHelpContent().GetCommentBlock()

                # Create a scriptblock that defines a blank version of the
                # function with the CBH. You may lose some parameter info
                # here, if you need that replace param() with
                # $_.Body.ParamBlock.Extent.Text
                $scriptBlock = [scriptblock]::Create(('
                function {0} {{
                    {1}
                    param()
                }}' -f $_.Name, $commentBlock))

                # Dot source the scriptblock in a different scope so we can
                # get the help content but still not pollute the session.
                & {
                    . $scriptBlock

                    #Write-Host "Get-Help $($_.Name)" -ForegroundColor White
                    #Write-Host "$commentBlock" -ForegroundColor White
                    #Write-Host "$scriptBlock" -ForegroundColor yellow

                    Get-Help $_.Name >> "$helpFileName"
                }
            } catch {
                # probably a null Help Content
                write-log -message "Exception Occurred:" -consoleForeground Red
                write-log -message "$($_.Exception.Message)" -consoleForeground Red
                if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
                if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
                if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
                #Write-Host "Probably received a null on getHelpContent for $functionName" -ForegroundColor Red
            }
        }
    } else {
        write-log -message "File Not Found: $fileName" -consoleForeground Red
    }

}

    

