<#
.SYNOPSIS
    Retrieves the error information associated to a correlation id

.DESCRIPTION
    Retrieves the error information associated to a correlationId and writes the log information to a file in the 
    script's current direcory. The file will be named CorrelationIDLog.$sCorrelationID.log where the $sCorrelationID is 
    the CorrelationId used in the query. 

.PARAMETER CorrelationId
    [String] A Correlation Id that can be obtained from the browser error message or other locations.
    
.PARAMETER Minutes
    [Integer] OPTIONAL. A number of minutes to limit the search. For example, -30 implies that the error occurred in 
    the past 30 minutes. 
     
.EXAMPLE
    #when run from the powershell command prompt.
    .\GetCorrelationId.ps1 -CorrelationId "72767b9e-9c76-e0ce-1257-0a40dfeb50bf"

#>
[CmdletBinding()]
Param(
   [Parameter(Mandatory=$false)]
   [string]$CorrelationId = "72767b9e-9c76-e0ce-1257-0a40dfeb50bf"
   [Parameter(Mandatory=$false)]
   [Int]$Minutes
)
#Get-SPLogEvent | ?{$_.Correlation -eq "a17b0b9e-ec9a-e0b4-442f-0417e2a30c73"}
#Get-SPLogEvent -starttime (Get-Date).AddMinutes(-30) | ?{$_.Correlation -eq "225b729e-1ca4-e0ce-1257-006377c4f851"} | select Timestamp, Area, Category, Level, EventID, Message | Format-List

############################################################################################################################################ 
# Script that allows to find all the ocurrences for a Correlation ID in the SharePoint Logs 
# Required Parameters:  
#    -> $sCorrelationID: Correlation ID to look for 
#    -> $sLogFile: Name of the log file where the information obtained is written 
############################################################################################################################################ 
 
If ((Get-PSSnapIn -Name Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue) -eq $null ) { 
    Add-PSSnapIn -Name Microsoft.SharePoint.PowerShell 
} 
 
$host.Runspace.ThreadOptions = "ReuseThread" 
 
#Definition of the function that finds a specific correlation ID in SharePoint Log Files 
function Find-CorrelationID { 
    param ($sCorrelationID,$sLogFile,$sMinutes)  
    try { 
        Write-Host "Finding the correlation ID $sCorrelationID in SharePoint Logs" -foregroundcolor Gray 
        
        if ($sMinutes) {
            Write-Host "   limiting the time to $sMinutes " -foregroundcolor Gray 
            Get-SPLogEvent -starttime (Get-Date).AddMinutes($sMinutes) | ?{$_.Correlation -eq $sCorrelationID} | select Timestamp, Area, Category, Level, EventID, Message | Format-List > $sLogFile 
        } else {
            Write-Host "   searching all log files - this could take some time... " -foregroundcolor Yellow 
            Get-SPLogEvent | ?{$_.Correlation -eq $sCorrelationID} | select Timestamp, Area, Category, Level, EventID, Message | Format-List > $sLogFile 
        }
        Write-Host "Log entries with the correlation ID $sCorrelationID written to in $sLogFile" -foregroundcolor Green 
    } catch [System.Exception] { 
        write-host -f red $_.Exception.ToString() 
    } 
} 
 
Start-SPAssignment -Global 
#Calling the function 
$sCorrelationID=$CorrelationId 
$sMinutes=$Minutes
if (sMinutes -gt 0 ) {
    sMinutes = sMinutes * -1
)
$sScriptDir = Split-Path -parent $MyInvocation.MyCommand.Path 
$sLogFile=$sScriptDir + "\CorrelationIDLog.$sCorrelationID.log" 
Find-CorrelationID -sCorrelationID $sCorrelationID -sLogFile $sLogFile -sMinutes $sMinutes 
Stop-SPAssignment -Global 
 
Remove-PSSnapin Microsoft.SharePoint.PowerShell 