try {
Add-PSSnapin Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue
}
catch {
 write-log -message "PSSnapin Microsoft.SharePoint.PowerShell already loaded" -consoleForeground Yellow
}

Set-StrictMode -Version Latest

if($env:COMPUTERNAME -match "VAPH" ) {
    Add-Type -Path .\RAMSUtil.dll
}
else {
    Add-Type -Path ..\UtilityScripts\RAMSUtil.dll
}

function Get-FullHelp() { 
    <#

    .SYNOPSIS
        Display the help for all functions in this module. The RAMS_Module_Util contains reusable PowerShell helper functions.

    .DESCRIPTION
        Displays the Help for all functions in this module by retrieving a list of functions and calling get-help for each one. 
        The reusable helper functions in the rest of the module provide a consistent mechanism for building SharePoint 
        Site Columns, Content Types, Lists, Views, Security Groups, and more. It creates a repeatable process that provides 
        consistency in development.

    .NOTES
        Each function in the module should have its own help.

    .EXAMPLE
        # Shows how to retrieve the help
        Import-Module .\module\RAMS_Module_Util.psm1
        Get-FullHelp
    
    .EXAMPLE
        # Shows how to retrieve the help
        Import-Module .\module\RAMS_Module_Util.psm1
        get-command -Module RAMS_Module_Util | % {get-help $_.name -full }
    
    .EXAMPLE
        # Shows how to use other functions in the module
        $PSModuleAutoLoadingPreference
        Remove-Module -Name RAMS_Module_Util
        Import-Module .\module\RAMS_Module_Util.psm1
        New-RAMSEnv
        Update-RAMSEnv

        Set-Location .\UtilityScripts
        Set-Location .\module
        New-ModuleManifest -Path .\RAMS_Module_Util.psd1 -ModuleVersion 1.2.0 -Author "Chita Nilak, ByLight" -RootModule .\RAMS_Module_Util.psm1 -Description "RAMS Utility Functions" -CompanyName "DNS" -Verbose 

        if((Get-Module -Name RAMS_Module_Util) -ne $null) {
            Remove-Module -Name RAMS_Module_Util
        }
        try {
            Import-Module .\RAMS_Module_Util.psd1 
        } catch {
            Write-log -message "Host: $($env:COMPUTERNAME) is not a Server, SharePoint is not installed on this computer...ignore, information only." -ConsoleForeground Yellow
            #do nothing.
        }

    #>
    
    get-command -Module RAMS_Module_Util | % {get-help $_.name -full }
}

function Remove-CalendarColumns ([String[]]$colList, $calTitle) { 
    <#
    
    .SYNOPSIS
        Removes a list of columns from a calendar list on all subsites.

    .DESCRIPTION
        For a given calendar and set of columns, loop through the subsites removing the column from the calendar.

    .PARAMETER colList
        [String[]] A collection of site column static names

    .PARAMETER calTitle
        [String] the title of the Calendar List

    .EXAMPLE
        Remove-CalendarColumns -colList Attendees, Category -calTitle "SRS Calendar"

    #>

    foreach ($col in $colList) {
        foreach ($subweb in (Get-SiteWeb).Webs) {
             $calList = $subweb.Lists.TryGetList($calTitle)
             #remove column
             if($calList.Fields.TryGetFieldByStaticName($col) -ne $Null) {
                 $calList.Fields.Delete($col)
                 write-log -message "$col has been deleted from $subweb : $calList"  -ConsoleForeground Yellow
                 $calList.Update()
             }
        }
    }
}

function RunSQL-ReturnInt ($sqlQuery,$dbServer) {
    <#

    .SYNOPSIS
        Runs a SQL statement against the requested SQL Server database.

    .DESCRIPTION
        The function use Windows Authentication for access to the SQL database, it takes two arguments, both required, please refer the example to get more details.

    .PARAMETER sqlQuery
        [String] A SQL query.

    .PARAMETER dbServer
        [String] The URL of the SQL Server instance.

    .EXAMPLE
        $cleanUpDate = "'05/30/2017'"
        $dbServer = "DNS.URL"
        $query = "exec DataClenup $cleanUpDate"
        $result = RunSQL-ReturnInt -sqlQuery $query -dbServer $dbServer
        write-log -message "Number of rows deleted: $result" -consoleForeground Cyan

    #>

    $RAMSSQL = New-Object Gov.VA.RAMS.RAMSUtil.Util    
    $RAMSSQL.SQLConnectionString = "Server=$dbServer;Database=RAMS;Trusted_Connection=Yes;"
    $result = $RAMSSQL.RunSQL($sqlQuery)
    #close connection
    if($RAMSSQL.SQLConnection.State -ne "Closed") {
        $RAMSSQL.CloseSQLConnection()
    }
    return $result
}

function RunSQL-ReturnDR ($sqlQuery,$dbServer) {
    <#

    .SYNOPSIS
        Runs a SQL statement against the requested SQL Server database.

    .DESCRIPTION
        The function use Windows Authentication for access to the SQL database, it takes two arguments, both required, please refer the example to get more details.

    .PARAMETER sqlQuery
        [String] A SQL query.

    .PARAMETER dbServer
        [String] The URL of the SQL Server instance.

    .EXAMPLE
        $result = RunSQL-ReturnDR -sqlQuery "select * from Studies" -dbServer SERVERNAME

    #>

    $RAMSSQL = New-Object Gov.VA.RAMS.RAMSUtil.Util    
    $RAMSSQL.SQLConnectionString = "Server=$dbServer;Database=RAMS;Trusted_Connection=Yes;"
    $dR = $RAMSSQL.RunSQLResturnDS($sqlQuery)        
    return $dR
}

function Get-RAMSSharePointInstallInfo {
    <# 
    .SYNOPSIS
        Verify that the environment to make sure it meets expectations.

    .DESCRIPTION
        Verifies that the person running the script is an administrator; that SharePoint PowerShell is loaded; and that SharePoint and PowerShell are what is expected.

    .EXAMPLE
        Get-RAMSSharePointInstallInfo

    .EXAMPLE
        Get-RAMSSharePointInstallInfo | Format-Table -AutoSize

    .NOTES
        Requires PowerShell Version 2.0 or Version 3.0
        Requires to be executed as an Administrator

    .EXAMPLE
        Get-RAMSSharePointInstallInfo -Verbose

    #>
    [CmdletBinding()]
    Param()

    Begin {
        Write-Verbose "Entering Begin Block"

        if (-NOT ([Security.Principal.WindowsPrincipal][Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")) {
            Write-Warning "You do not have Administrator rights to run this script!`nPlease re-run this script as an Administrator!"
            Exit
        } else {
            Write-Verbose "You are an Administrator. Continue to Process Block."
        }


        try {
            $isSPPowerShellLoaded = $null
            $SPSnapin = "Microsoft.SharePoint.PowerShell"
            if (Get-PSSnapin $SPSnapin -ErrorAction SilentlyContinue) {
                $isSPPowerShellLoaded = $true
                Write-Verbose "Microsoft.SharePoint.PowerShell Snappin already registered and loaded"
            }
            elseif (Get-PSSnapin $SPSnapin -Registered -ErrorAction SilentlyContinue) {
                Add-PSSnapin $SPSnapin
                $isSPPowerShellLoaded = $true 
                Write-Verbose "Microsoft.SharePoint.PowerShell Snappin is registered and has been loaded for script operation"
            }
            else {
                $isSPowerShellLoaded = $false
                Write-Warning "Microsoft.SharePoint.PowerShell Snappin not found."
                Write-Warning "Will not be able to report SharePoint Build Version"
            }

        } catch {
            $isSPPowerShellLoaded = $false
            Write-Error "There was an issue loading the Microsoft.SharePoint.PowerShell Snappin. Script execution will continue"
            Write-Error $_
            Write-Warning "Will not be able to report SharePoint Build Version"            
        }

        Write-Verbose "Leaving Begin Block"
    }

    Process {
        Write-Verbose "Entering Process Block"
        
        Write-Verbose "Define SharePoint 2010 Products Registry Path variable"
        $RegPath2010 = "HKLM:\SOFTWARE\Microsoft\Shared Tools\Web Server Extensions\14.0\WSS\InstalledProducts\"

        Write-Verbose "Define SharePoint 2010 Products hash table"
        Write-Verbose "SharePoint 2010 SKU ID's found at http://msdn.microsoft.com/en-us/library/ff721969(v=office.14).aspx"
        $SP2010Products = @{
                            "BEED1F75-C398-4447-AEF1-E66E1F0DF91E" = "SharePoint Foundation 2010"
                            "1328E89E-7EC8-4F7E-809E-7E945796E511" = "Search Server Express 2010"
                            "B2C0B444-3914-4ACB-A0B8-7CF50A8F7AA0" = "SharePoint Server 2010 Standard Trial"
                            "3FDFBCC8-B3E4-4482-91FA-122C6432805C" = "SharePoint Server 2010 Standard"
                            "88BED06D-8C6B-4E62-AB01-546D6005FE97" = "SharePoint Server 2010 Enterprise Trial"
                            "D5595F62-449B-4061-B0B2-0CBAD410BB51" = "SharePoint Server 2010 Enterprise"
                            "BC4C1C97-9013-4033-A0DD-9DC9E6D6C887" = "Search Server 2010 Trial"
                            "08460AA2-A176-442C-BDCA-26928704D80B" = "Search Server 2010"   
                            "84902853-59F6-4B20-BC7C-DE4F419FEFAD" = "Project Server 2010 Trial"
                            "ED21638F-97FF-4A65-AD9B-6889B93065E2" = "Project Server 2010"
                            "926E4E17-087B-47D1-8BD7-91A394BC6196" = "Office Web Companions 2010"
                            }


        Write-Verbose "Define SharePoint 2013 Products Registry Path variable"
        $RegPath2013 = "HKLM:\SOFTWARE\Microsoft\Shared Tools\Web Server Extensions\15.0\WSS\InstalledProducts\"

        Write-Verbose "Define SharePoint 2013 Products hash table"
        Write-Verbose "SharePoint 2013 SKU ID's found at http://msdn.microsoft.com/en-us/library/jj659075.aspx"
        $SP2013Products =  @{
                            "35466B1A-B17B-4DFB-A703-F74E2A1F5F5E" = "Project Server 2013"
                            "BC7BAF08-4D97-462C-8411-341052402E71" = "Project Server 2013 Preview"
                            "C5D855EE-F32B-4A1C-97A8-F0A28CE02F9C" = "SharePoint Server 2013"
                            "CBF97833-C73A-4BAF-9ED3-D47B3CFF51BE" = "SharePoint Server 2013 Preview"
                            "B7D84C2B-0754-49E4-B7BE-7EE321DCE0A9" = "SharePoint Server 2013 Enterprise"
                            "298A586A-E3C1-42F0-AFE0-4BCFDC2E7CD0" = "SharePoint Server 2013 Enterprise Preview"
                            "D6B57A0D-AE69-4A3E-B031-1F993EE52EDC" = "Microsoft Office Web Apps Server 2013"
                            "9FF54EBC-8C12-47D7-854F-3865D4BE8118" = "SharePoint Foundation 2013"   
                            }

        Write-Verbose "Define function variables to store data based on current environment"
        $RegPath = $null
        $SPProducts = $null
        $SharePointVersion = $null
        $CanProcess = $false

        Write-Verbose "Test Registry Paths to determine if current environment has SharePoint 2010 or 2013 installed"
        $is2010 = Test-Path $RegPath2010
        $is2013 = Test-Path $RegPath2013

        Write-Verbose "Populate function variables with SharePoint version specific data, if the environment contains SharePoint 2010/2013"
        if($is2010) {
            Write-Verbose "SharePoint 2010 is installed"
            $RegPath = $RegPath2010
            $SPProducts = $SP2010Products
            $SPVersion = 2010
            $CanProcess = $true
        } elseif($is2013) {
            Write-Verbose "SharePoint 2013 is installed"
            $RegPath = $RegPath2013
            $SPProducts = $SP2013Products
            $SPVersion = 2013
            $CanProcess = $true
        } else {
            Write-Warning "SharePoint 2010 or 2013 is not installed within current environment"   
            Write-Warning "Hash table intended to store SharePoint Product information will be empty"    
        }
        
        Write-Verbose "Define hash table to store installed SharePoint Product information"
        $InstalledProducts = @{}

        Write-Verbose "Verify that current environment is hosting SharePoint 2010 or 2013"
        if($CanProcess) {

            Write-Verbose "Determine whether the Microsoft.SharePoint.PowerShell Snappin is loaded"
            if($isSPPowerShellLoaded) {
                try {
                    Write-Verbose "Get SharePoint Build Version"
                    $SPBuildVersion = (Get-SPFarm).BuildVersion.ToString()

                    Write-Verbose "Adding the SharePoint Build Version to the InstalledProducts hash table"
                    $InstalledProducts.Add("SharePoint $SPVersion Build Version",$SPBuildVersion)
                } catch {
                    Write-Error "Unable to get the SharePoint Build Version."
                    Write-Error $_
                }
            }

            Write-Verbose "SharePoint $SPVersion is installed within the current environment"
            Write-Verbose "Populate InstalledProducts hash table with SharePoint Products installed on current environment"
            Get-Item $RegPath | 
                Select-Object -ExpandProperty property | 
                ForEach-Object {(Get-ItemProperty -Path $RegPath -Name $_).$_}  | 
                    ForEach-Object {$InstalledProducts.Add($SPProducts.Get_Item($_),$_)}
        }

        Write-Verbose "Output the InstalledProducts hash table to pipeline."
        Write-Output $InstalledProducts 
        
        Write-Verbose "Leaving Process Block"       
    }
}

function Get-SiteURL {
    <#

    .SYNOPSIS
        Returns the URL of the site collection

    .DESCRIPTION
        Loops through the Site Collections looking for the one named IRBApplications.

    .EXAMPLE
        $siteUrl = Get-SiteURL

    .EXAMPLE
        $web = Get-SPWeb -Identity (Get-SiteURL)

    .EXAMPLE
        $Global:siteUrl = Get-SiteURL 

    #>

    $site = Get-SPSite -Limit All | Select-String IRBApplications
    $x = $site.ToString();
    $url = $x.Substring($x.IndexOf("=")+1)
    $w = Get-SPWeb $url
    return $url
}

function Get-MySiteURL {
    <#

    .SYNOPSIS
        Returns the URL of the My Sites site collection

    .DESCRIPTION
        Loops through the Site Collections looking for the one named My Sites.

    .EXAMPLE
        $mySiteUrl = Get-MySiteURL

    .EXAMPLE
        $web = Get-SPWeb -Identity (Get-MySiteURL)

    .EXAMPLE
        $Global:mySiteUrl = Get-MySiteURL 

    #>

    $site = get-spwebapplication | select DisplayName, Url | ? { ($_.DisplayName -ilike "*My Site*") -or ($_.DisplayName -ilike "*MySite*") }
    $url = $site.Url
    $w = Get-SPWeb $url
    return $url
}
function Get-SiteWeb {
    <#

    .SYNOPSIS
        Returns the SPWeb for the Site Collection

    .DESCRIPTION
        Gets the SiteURL and then returns a SPWeb object for the site Collection.

    .EXAMPLE
        $web = Get-SiteWeb

    .EXAMPLE
        foreach ($subWeb in (Get-SiteWeb).Webs) { ... }

    .EXAMPLE
        (Get-SiteWeb).Webs 

    #>

    $url = Get-SiteURL
    $xWeb = Get-Spweb $url
    return $xWeb
}

function Get-MySiteWeb {
    <#

    .SYNOPSIS
        Returns the SPWeb for the MySite Site Collection

    .DESCRIPTION
        Gets the SiteURL and then returns a SPWeb object for the MySite site Collection.

    .EXAMPLE
        $web = Get-MySiteWeb

    .EXAMPLE
        foreach ($subWeb in (Get-MySiteWeb).Webs) { ... }

    .EXAMPLE
        (Get-MySiteWeb).Webs 

    #>

    $url = Get-MySiteURL
    $xWeb = Get-Spweb $url
    return $xWeb
}
Function Deploy-Feature {
    <#

    .SYNOPSIS
        Deploys a Workflow Feature to the site collection.

    .DESCRIPTION
        Adds the SPUserSolution, Installs the SPUserSolution, and then enables the Feature. These steps essentially import 
        the workflow as feature and then publish it to the site collection or the subsite.

    .PARAMETER SiteCollection
        [String] The site collection URL.

    .PARAMETER site
        [String] The site the feature is to be published to.

    .PARAMETER FeatureName
        [String] The name of the workflow feature

    .PARAMETER OldFeatureId
        [String] The Name of the workflow feature to remove. Must be provided, but not currently used.

    .PARAMETER FeatureId
        [String] The workflow's feature id.

    .PARAMETER file
        [String] The file name of the workflow template, including the .wsp extension.

    .PARAMETER bolUninstall
        [bool] Indicates if the workflow should be uninstalled. Must be provided, but not currently used.

    .PARAMETER bolEnableFeatureOnly
        [bool] Once the workflow is installed, this indicator tells the script to enable the workflow on subsites.

    .EXAMPLE
        Deploy-Feature -SiteCollection $_siteURL -site $w.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $False

    #>


    Param(      
        [Parameter(Mandatory=$True)]
        [string]$SiteCollection,

        [Parameter(Mandatory=$True)]
        [string]$site,
        
        [Parameter(Mandatory=$True)]
        [string]$FeatureName,

        [Parameter(Mandatory=$True)]
        [string]$OldFeatureId,
        
        [Parameter(Mandatory=$True)]
        [string]$FeatureId,

        [Parameter(Mandatory=$True)]
        [string]$file,

        [Parameter(Mandatory=$True)]
        [bool] $bolUninstall,

        [Parameter(Mandatory=$False)]
        [bool] $bolEnableFeatureOnly   
    )
    

    Add-PSSnapin Microsoft.SharePoint.PowerShell

    # GET THE SITE COLLECTION
    $SiteCollectionweb = Get-SPWeb $SiteCollection

    # GET THE SITE
    $web = Get-SPWeb $site

    # CLEAN and old one first
    if($bolEnableFeatureOnly -eq $False) {
        <# 
        # All the old workflows were uninstalled in a previous step. This is not needed at this time,but may be 
        # needed in future releases
        if($bolUninstall) {
            write-log -message "  Uninstall old Feature $FeatureName" -consoleForeground Gray
            
            write-log -message "    Disabling feature Identity $OldFeatureId URL $($web.Url)" -consoleForeground Yellow
            Disable-SPFeature -Identity $OldFeatureId -Url $SiteCollectionweb.Url -Force -ErrorAction SilentlyContinue  
            
            write-log -message "    Uninstalling feature Feature $FeatureName Identity $FeatureName Site $($SiteCollectionweb.Url)" -consoleForeground Yellow
            Uninstall-SPUserSolution -Identity $FeatureName -Site $SiteCollectionweb.Site 
            
            write-log -message "    Removing feature Feature $FeatureName Identity $FeatureName Site $($SiteCollectionweb.Url)" -consoleForeground Yellow
            Remove-SPUserSolution -Identity $FeatureName -Site $SiteCollectionweb.Site 
            $web.Update()     
            
            #remove from workflow manager as well
            Remove-WorkFlow -wfMatchName ($FeatureName.Replace(".wsp","")) -wfWeb $web
        }
        #>
        $status = "Start"
        # ADD/UPLOAD TO THE SOLUTIONS
        write-log -message "  Adding new Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Gray
        try {
            Add-SPUSerSolution -LiteralPath $file -Site $SiteCollectionweb.Url -Verbose
            $web.Update()
            write-log -message "    Added Feature $FeatureName file $file Site $($SiteCollectionweb.Url)" -consoleForeground Green
            $status = "Added"
        } catch {
            write-log -message "    Add-SPUserSolution Failed to Add Feature $FeatureName file $file Site $($SiteCollectionweb.Url)" -ConsoleForeground Red
        }
        if ($status -eq "Added") {
            # ACTIVATE THE SOLUTION
            write-log -message "  Installing new Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Gray
            try {
                Install-SPUserSolution -Identity $FeatureName -Site $SiteCollectionweb.Url 
                $web.Update()
                write-log -message "    Install Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Green
                $status = "Installed"
            } catch {
               write-log -message "    Install-SPUserSolution Failed to Install Feature $FeatureName Site $($SiteCollectionweb.Url)" -ConsoleForeground Red
            }
        }
    }
    if (($bolEnableFeatureOnly -eq $True) -or ($status -eq "Installed")) {
        # ENABLE THE FEATURE FOR THE GIVEN SITE
        # feature id goes into -Identity
        write-log -message "  Enabling new Feature $FeatureName Identity $FeatureId Url $($web.Url)" -consoleForeground Gray
        try {
            Enable-SPFeature -Identity $FeatureId -Url $web.Url -Verbose -Force
            $web.Update()
            write-log -message "    Enabled Feature $FeatureName Identity $FeatureId Url $($web.Url)" -consoleForeground Green
        } catch {
            write-log -message "Enable-SPFeature failed for Feature $FeatureName Identity $FeatureId Url $($web.Url)" -ConsoleForeground Red
        }
    }
}

Function Deploy-FarmERFeature {
    <#

    .SYNOPSIS
        Deploys a Feature to the site collection.

    .DESCRIPTION
        Adds the SPSolution, Installs the SPSolution, and then enables the Feature. These steps essentially import 
        the workflow or event receiver as feature and then publish it to the site collection or the subsite.

    .PARAMETER SiteCollection
        [String] The site collection URL.

    .PARAMETER site
        [String] The site the feature is to be published to.

    .PARAMETER FeatureName
        [String] The name of the workflow feature

    .PARAMETER OldFeatureId
        [String] The Name of the workflow feature to remove. Must be provided, but not currently used.

    .PARAMETER FeatureId
        [String] The workflow's feature id.

    .PARAMETER file
        [String] The file name of the workflow template, including the .wsp extension.

    .PARAMETER bolUninstall
        [bool] Indicates if the workflow should be uninstalled. Must be provided, but not currently used.

    .PARAMETER bolEnableFeatureOnly
        [bool] Once the workflow is installed, this indicator tells the script to enable the workflow on subsites.

    .EXAMPLE
        Deploy-FarmERFeature -SiteCollection $_siteURL -site $w.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $False

    #>


    Param(      
        [Parameter(Mandatory=$True)]
        [string]$SiteCollection,

        [Parameter(Mandatory=$True)]
        [string]$site,
        
        [Parameter(Mandatory=$True)]
        [string]$FeatureName,

        [Parameter(Mandatory=$True)]
        [string]$OldFeatureId,
        
        [Parameter(Mandatory=$True)]
        [string]$FeatureId,

        [Parameter(Mandatory=$True)]
        [string]$file,

        [Parameter(Mandatory=$True)]
        [bool] $bolUninstall,

        [Parameter(Mandatory=$False)]
        [bool] $bolEnableFeatureOnly   
    )
    

    Add-PSSnapin Microsoft.SharePoint.PowerShell

    # GET THE SITE COLLECTION
    $SiteCollectionweb = Get-SPWeb $SiteCollection

    # GET THE SITE
    $web = Get-SPWeb $site

    # CLEAN and old one first
    if($bolEnableFeatureOnly -eq $False) {
         
        # All the old workflows were uninstalled in a previous step. This is not needed at this time,but may be 
        # needed in future releases
        if($bolUninstall) {
            write-log -message "  Uninstall old Feature $FeatureName" -consoleForeground Gray
            
            write-log -message "    Disabling feature Identity $OldFeatureId URL $($web.Url)" -consoleForeground Yellow
            Disable-SPFeature -Identity $OldFeatureId -Url $SiteCollectionweb.Url -Force -ErrorAction SilentlyContinue  
            Start-Sleep -seconds 10
            
            write-log -message "    Uninstalling feature Feature $FeatureName Identity $FeatureName Site $($SiteCollectionweb.Url)" -consoleForeground Yellow
            Uninstall-SPSolution -Identity $FeatureName 
            Start-Sleep -seconds 60
            
            write-log -message "    Removing feature Feature $FeatureName Identity $FeatureName Site $($SiteCollectionweb.Url)" -consoleForeground Yellow
            Remove-SPSolution -Identity $FeatureName -Force
            Start-Sleep -seconds 10
            $web.Update()
            
            #remove from workflow manager as well
            #Remove-WorkFlow -wfMatchName ($FeatureName.Replace(".wsp","")) -wfWeb $web
        }
        
        $status = "Start"
        # ADD/UPLOAD TO THE SOLUTIONS
        write-log -message "  Adding new Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Gray
        try {
            Add-SPSolution -LiteralPath $file -Verbose
            Start-Sleep -seconds 10
            $web.Update()
            write-log -message "    Added Feature $FeatureName file $file Site $($SiteCollectionweb.Url)" -consoleForeground Green
            $status = "Added"
        } catch {
            write-log -message "    Add-SPSolution Failed to Add Feature $FeatureName file $file Site $($SiteCollectionweb.Url)" -ConsoleForeground Red
            if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
            if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
            if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
        }
        if ($status -eq "Added") {
        # ACTIVATE THE SOLUTION
        write-log -message "  Installing new Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Gray
            try {
                Install-SPSolution -Identity $FeatureName -GACDeployment -Force
                Start-Sleep -seconds 60
                $web.Update()
                write-log -message "    Install Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Green
                $status = "Installed"
            } catch {
               write-log -message "    Install-SPSolution Failed to Install Feature $FeatureName Site $($SiteCollectionweb.Url)" -ConsoleForeground Red
                if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
                if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
                if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
            }
        }
    }
    
    if (($bolEnableFeatureOnly -eq $True) -or ($status -eq "Installed")) {
        # ENABLE THE FEATURE FOR THE GIVEN SITE
        # feature id goes into -Identity
        write-log -message "  Enabling new Feature $FeatureName Identity $FeatureId Url $($web.Url)" -consoleForeground Gray
        try {
            Enable-SPFeature -Identity $FeatureId -Verbose -Force -Url (get-SiteURL)
            Start-Sleep -seconds 60
            $web.Update()
            write-log -message "    Enabled Feature $FeatureName Identity $FeatureId Url $($web.Url)" -consoleForeground Green
        } catch {
            write-log -message "Enable-SPFeature failed for Feature $FeatureName Identity $FeatureId Url $($web.Url)" -ConsoleForeground Red
            if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
            if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
            if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
        }
    }
    
}
Function Deploy-FarmWFFeature {
    <#

    .SYNOPSIS
        Deploys a Feature to the site collection.

    .DESCRIPTION
        Adds the SPSolution, Installs the SPSolution, and then enables the Feature. These steps essentially import 
        the workflow or event receiver as feature and then publish it to the site collection or the subsite.

    .PARAMETER SiteCollection
        [String] The site collection URL.

    .PARAMETER site
        [String] The site the feature is to be published to.

    .PARAMETER FeatureName
        [String] The name of the workflow feature

    .PARAMETER OldFeatureId
        [String] The Name of the workflow feature to remove. Must be provided, but not currently used.

    .PARAMETER FeatureId
        [String] The workflow's feature id.

    .PARAMETER file
        [String] The file name of the workflow template, including the .wsp extension.

    .PARAMETER bolUninstall
        [bool] Indicates if the workflow should be uninstalled. Must be provided, but not currently used.

    .PARAMETER bolEnableFeatureOnly
        [bool] Once the workflow is installed, this indicator tells the script to enable the workflow on subsites.

    .EXAMPLE
        Deploy-FarmWFFeature -SiteCollection $_siteURL -site $w.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $False

    #>


    Param(      
        [Parameter(Mandatory=$True)]
        [string]$SiteCollection,

        [Parameter(Mandatory=$True)]
        [string]$site,
        
        [Parameter(Mandatory=$True)]
        [string]$FeatureName,

        [Parameter(Mandatory=$True)]
        [string]$OldFeatureId,
        
        [Parameter(Mandatory=$True)]
        [string]$FeatureId,

        [Parameter(Mandatory=$True)]
        [string]$file,

        [Parameter(Mandatory=$True)]
        [bool] $bolUninstall,

        [Parameter(Mandatory=$False)]
        [bool] $bolEnableFeatureOnly   
    )
    

    Add-PSSnapin Microsoft.SharePoint.PowerShell

    # GET THE SITE COLLECTION
    $SiteCollectionweb = Get-SPWeb $SiteCollection

    # GET THE SITE
    $web = Get-SPWeb $site

    # CLEAN and old one first
    if($bolEnableFeatureOnly -eq $False) {
         
        # All the old workflows were uninstalled in a previous step. This is not needed at this time,but may be 
        # needed in future releases
        if($bolUninstall) {
            write-log -message "  Uninstall old Feature $FeatureName" -consoleForeground Gray
            
            write-log -message "    Disabling feature Identity $OldFeatureId URL $($web.Url)" -consoleForeground Yellow
            Disable-SPFeature -Identity $OldFeatureId -Url $SiteCollectionweb.Url -Force -ErrorAction SilentlyContinue  
            Start-Sleep -seconds 10
            
            write-log -message "    Uninstalling feature Feature $FeatureName Identity $FeatureName Site $($SiteCollectionweb.Url)" -consoleForeground Yellow
            Uninstall-SPSolution -Identity $FeatureName 
            Start-Sleep -seconds 60
            
            write-log -message "    Removing feature Feature $FeatureName Identity $FeatureName Site $($SiteCollectionweb.Url)" -consoleForeground Yellow
            Remove-SPSolution -Identity $FeatureName -Force
            Start-Sleep -seconds 10
            $web.Update()     
            
            #remove from workflow manager as well
            #Remove-WorkFlow -wfMatchName ($FeatureName.Replace(".wsp","")) -wfWeb $web
        }
        
        $status = "Start"
        # ADD/UPLOAD TO THE SOLUTIONS
        write-log -message "  Adding new Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Gray
        try {
            Add-SPSolution -LiteralPath $file -Verbose
            Start-Sleep -seconds 10
            $web.Update()
            write-log -message "    Added Feature $FeatureName file $file Site $($SiteCollectionweb.Url)" -consoleForeground Green
            $status = "Added"
        } catch {
            write-log -message "    Add-SPSolution Failed to Add Feature $FeatureName file $file Site $($SiteCollectionweb.Url)" -ConsoleForeground Red
            if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
            if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
            if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
        }
        if ($status -eq "Added") {
        # ACTIVATE THE SOLUTION
        write-log -message "  Installing new Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Gray
            try {
                Install-SPSolution -Identity $FeatureName -GACDeployment -Force
                Start-Sleep -seconds 60
                $web.Update()
                write-log -message "    Install Feature $FeatureName Identity $FeatureId Site $($SiteCollectionweb.Url)" -consoleForeground Green
                $status = "Installed"
            } catch {
               write-log -message "    Install-SPSolution Failed to Install Feature $FeatureName Site $($SiteCollectionweb.Url)" -ConsoleForeground Red
                if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
                if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
                if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
            }
        }
    }
    
    if (($bolEnableFeatureOnly -eq $True) -or ($status -eq "Installed")) {
        # ENABLE THE FEATURE FOR THE GIVEN SITE
        # feature id goes into -Identity
        write-log -message "  Enabling new Feature $FeatureName Identity $FeatureId Url $($web.Url)" -consoleForeground Gray
        try {
            Enable-SPFeature -Identity $FeatureId -Verbose -Force -Url (Get-SiteURL)
            Start-Sleep -seconds 60
            $web.Update()
            write-log -message "    Enabled Feature $FeatureName Identity $FeatureId Url $($web.Url)" -consoleForeground Green

            $taskList=$web.Lists["Workflow Tasks"]
            $historyList=$web.Lists["Workflow History"]
            $ct=$web.ContentTypes["Document"]
            $culture=New-Object System.Globalization.CultureInfo("en-US")
            $FeatureBaseName = $FeatureName.replace(".wsp","")
            $wfTemplate=$web.WorkflowTemplates.GetTemplateByName($FeatureBaseName,$culture)
            
            if ($wfTemplate) {
                $associationWF=[Microsoft.SharePoint.Workflow.SPWorkflowAssociation]::CreateListContentTypeAssociation($wfTemplate, $FeatureName,$taskList,$historyList)
                $associationWF.AllowManual = $true;
                $associationWF.AutoStartChange = $true;
                $associationWF.AutoStartCreate = $true;
                $ct.WorkflowAssociations.Add($associationWF)
                $ct.UpdateWorkflowAssociationsOnChildren($true,$true,$true,$false)
            } else {
                write-log -Message " Unable to find workflow template for $FeatureName" -ConsoleForeground Red
            }
        } catch {
            write-log -message "Enable-SPFeature failed for Feature $FeatureName Identity $FeatureId Url $($web.Url)" -ConsoleForeground Red
            if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
            if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
            if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
        }
    }
}
    
function Add-SiteColumnToCTByTitle ([Microsoft.SharePoint.SPSecurableObject]$web, [String]$ctName,[String]$sFieldTitle) {
    <#

    .SYNOPSIS
        Adds a site column to a contentType.

    .DESCRIPTION
        Gets the content type from the spweb object. Looks up the site column by display name. If the content type 
        does not already have the site column it is added and pushed to all subsites and lists that use the content type.

    .PARAMETER web
        [Microsoft.SharePoint.SPSecurableObject] a web object that represents the site collection.

    .PARAMETER ctName
        [String] The name of the content type.

    .PARAMETER sFieldTitle
        [String] The display name of the site column.

    .EXAMPLE
        $fldTmp = $newDelegatedCommitee + ' Delegated'    
        Add-SiteColumnToCTByTitle -web $webSite -ctName "Committee CT" -sFieldTitle $fldTmp

    #>

    $ct = $web.ContentTypes[$ctName];
    $field=$web.Fields.GetField($sFieldTitle)
    $sFieldStaticName = $field.StaticName

    if(!$ct.Fields.ContainsFieldWithStaticName($sFieldStaticName)){
        $link = new-object Microsoft.SharePoint.SPFieldLink $field
        #check if comments fields exist or not.
        if(($ct.FieldLinks | Where-Object {$_.Name -eq $field.InternalName}) -EQ $null) {
            $ct.FieldLinks.Add($link)
            $ct.Update($true)
            write-log -message "$sFieldTitle Field has been added to $ct" -ConsoleForeground Green
        }
        else {
            write-log -message "$sFieldTitle Field already exist" -ConsoleForeground Yellow
        }
    }
}

function Add-SiteColumnToCT ([Microsoft.SharePoint.SPSecurableObject]$web, [String]$ctName,[String]$sFieldInternalName, [String]$sFieldStaticName) {
    <#

    .SYNOPSIS
        Adds a site column to a contentType.

    .DESCRIPTION
        Gets the content type from the spweb object. Looks up the site column by display name. If the content type 
        does not already have the site column it is added and pushed to all subsites and lists that use the content type.

    .PARAMETER web
        [Microsoft.SharePoint.SPSecurableObject] a web object that represents the site collection.

    .PARAMETER ctName
        [String] The name of the content type.

    .PARAMETER sFieldInternalName
        [String] The internal name of the site column.

    .PARAMETER sFieldStaticName
        [String] The static name of the site column.

    .EXAMPLE
        Add-SiteColumnToCT -web $webSite -ctName "Application CT" -sFieldInternalName "CIRB_ID" -sFieldStaticName "CIRB_ID"

    #>

    $ct = $web.ContentTypes[$ctName];
    if(!$ct.Fields.ContainsFieldWithStaticName($sFieldStaticName)){
        $field=$web.Fields.TryGetFieldByStaticName($sFieldStaticName)
        $link = new-object Microsoft.SharePoint.SPFieldLink $field
        #check if comments fields exist or not.
        if(($ct.FieldLinks | Where-Object {$_.Name -eq $sFieldInternalName}) -EQ $null) {
            $ct.FieldLinks.Add($link)
            $ct.Update($true)
            write-log -message "$sFieldInternalName Field has been added to $($ct.Name) Content Type" -consoleForeground Green
        }
        else {
            write-log -message "$sFieldInternalName Field already exist in $($ct.Name) Content Type" -consoleForeground Yellow
        }
    }
}

function Set-SiteColumn([Microsoft.SharePoint.SPSecurableObject]$web, [String]$sFieldDisplayName,[String]$sFieldStaticName, [String]$sFieldInternalName, [Microsoft.SharePoint.SPFieldType]$sFieldType, [System.Collections.Specialized.StringCollection] $sChoice, [String]$spFieldGroup, [switch] $isRequired, [switch] $showInEditForm, [switch] $showInNewForm, [switch] $allowMultipleValues, [String]$defaultValue) {
    <#

    .SYNOPSIS
        Adds a site column to a site collection.

    .DESCRIPTION
        Gets the content type from the spweb object. Looks up the site column by display name. If the content type 
        does not already have the site column it is added and pushed to all subsites and lists that use the content type.

    .PARAMETER web
        [Microsoft.SharePoint.SPSecurableObject] a web object that represents the site collection.

    .PARAMETER sFieldDisplayName
        [String] The display name of the site column. This should be as it should be displayed and it can contain spaces.

    .PARAMETER sFieldStaticName
        [String] The static name of the site column. This should be the same as the internal name for consistency. This should be the same as the display name, but without spaces.

    .PARAMETER sFieldInternalName
        [String] The internal name of the site column. This should be the same as the internal name for consistency. This should be the same as the display name, but without spaces.

    .PARAMETER $sFieldType, 
        [Microsoft.SharePoint.SPFieldType] The type of field to create. https://msdn.microsoft.com/en-us/library/microsoft.sharepoint.spfieldtype.aspx
    
    .PARAMETER $sChoice, 
        [System.Collections.Specialized.StringCollection] Provides the Choice values that will be set in the new choice site column.
    
    .PARAMETER $spFieldGroup, 
        [String] Specifies which Field Group the site column should be added to.
    
    .PARAMETER $isRequired, 
        [switch] Indicates if the site column is required.
    
    .PARAMETER $showInEditForm, 
        [switch] Indicates if the site column should be added to the Edit Item Form
    
    .PARAMETER $showInNewForm, 
        [switch] Indicates if the site column should be added to the New Item Form
    
    .PARAMETER $allowMultipleValues, 
        [switch] For People or Group fields indicates if multiple people can be selected or only one.
    
    .PARAMETER $defaultValue
        [String] Optional. Sets the default value for the new field. Do not use for Person or Group Fields.
        
    .EXAMPLE
        Set-SiteColumn -web (Get-SiteWeb) -sFieldDisplayName "Triennial Review  Date" -sFieldStaticName "TriennialReviewDate" -sFieldInternalName "TriennialReviewDate" -sFieldType DateTime  -spFieldGroup "RAMS IRB" -showInEditForm -showInNewForm

    .EXAMPLE
        Set-SiteColumn -web (Get-SiteWeb) -sFieldDisplayName $scDN -sFieldStaticName $scIN -sFieldInternalName $scIN -sFieldType $scType  -spFieldGroup "RAMS IRB" -showInEditForm -showInNewForm -allowMultipleValues

    .EXAMPLE
        Set-SiteColumn -web $webSite -sFieldDisplayName "Adverse Reportable Event" -sFieldStaticName "AdverseReportableEvent" -sFieldInternalName "AdverseReportableEvent" -sFieldType "Text"  -spFieldGroup "RAMS IRB"

    #>

    $web = (Get-SiteWeb)
    if (!$web.Fields.ContainsFieldWithStaticName($sFieldStaticName)){
        $sFieldDisplayName=$sFieldDisplayName 
            write-log -message "Adding the field $sFieldDisplayName to the Site" -consoleForeground cyan   
            $sFieldStaticName = $web.Fields.Add($sFieldInternalName,$sFieldType, $false, $false, $sChoice);
            $spField=$web.Fields.TryGetFieldByStaticName($sFieldStaticName)
            $spField.Group= $spFieldGroup 
            $spField.Title=$sFieldDisplayName 
            if($allowMultipleValues) {
                $spField.AllowMultipleValues = $True
            }
            if($defaultValue) {
                if($defaultValue -eq "true" -or $defaultValue -eq "$true" -or $defaultValue -eq "1") {
                    $spField.DefaultValue = "1"
                } elseif($defaultValue -eq "false" -or $defaultValue -eq "$false" -or $defaultValue -eq "0") {
                    $spField.DefaultValue = "0"
                } else {
                    $spField.DefaultValue = $defaultValue
                }
            }
            if($isRequired) {$spField.Required = $True}
            if($showInEditForm) {$spField.ShowInEditForm = $True}
            if($showInNewForm) {$spField.ShowInNewForm = $True}

            $spField.Update()  
    }
    else {
         write-log -message "$sFieldInternalName Field already exist" -ConsoleForeground Yellow
    }
}

function Set-SiteColumnLookup([Microsoft.SharePoint.SPSecurableObject]$web, [String]$sFieldStaticName, [String]$sFieldDisplayName, [String]$lookupListTitle, [String]$spFieldGroup) {
    <#

    .SYNOPSIS
        Adds a lookup site column to a site collection.

    .DESCRIPTION
        If the lookup column does not already exist it is added using Fields.AddLookup.

    .PARAMETER web
        [Microsoft.SharePoint.SPSecurableObject] a web object that represents the site collection.

    .PARAMETER sFieldStaticName
        [String] The static name of the site column. This should be the same as the internal name for consistency. This should be the same as the display name, but without spaces.

    .PARAMETER sFieldDisplayName
        [String] The display name of the site column. This should be as it should be displayed and it can contain spaces.

    .PARAMETER lookupListTitle
        [String] The name of the list that is the source for the lookup.

    .PARAMETER $spFieldGroup, 
        [String] Specifies which Field Group the site column should be added to.
    
    .EXAMPLE
        Set-SiteColumnLookup -web $web -colName "Facilities Lookup" -colStaticName "FacilitiesLookup" -lookupListName "Facilities" -lookupColumnName "Title" 

    .EXAMPLE
        Set-SiteColumnLookup -web $web -colName "Delegated" -colStaticName "Delegated" -lookupListName "Facilities" -lookupColumnName "Title"

    #>

    if (!$web.Fields.ContainsFieldWithStaticName($sFieldStaticName)){       
            write-log -message "Adding the field $sFieldDisplayName to the Site" -consoleForeground cyan
            $sFieldStaticName = $web.Fields.AddLookup($sFieldDisplayName,(($web.Lists.TryGetList($lookupListTitle)).Id), $false)
            $spField=$web.Fields.TryGetFieldByStaticName($sFieldStaticName)
            $spField.Group= $spFieldGroup 
            $spField.Title=$sFieldDisplayName 
            $spField.Update()  
    }
    else {
         write-log -message "$sFieldInternalName Field already exist" -ConsoleForeground Yellow
    }
}

function Update-SiteColumnLookup ([Microsoft.SharePoint.SPSecurableObject]$web,$siteColumnTitle,$lookupListTitle,$lookupListFieldTitle){
    <#

    .SYNOPSIS
        Changes the source of the Lookup site column.

    .DESCRIPTION
        Finds the lookup field, gets the new lookup table, and sets the lookup field site columns to the Id and lookupListFieldTitle. All errors are displayed to the user.

    .PARAMETER web
        [Microsoft.SharePoint.SPSecurableObject] a web object that represents the site collection.

    .PARAMETER siteColumnTitle
        [String] the site column display name.

    .PARAMETER lookupListTitle
        [String] The static name of the site column. This should be the same as the internal name for consistency. This should be the same as the display name, but without spaces.

    .PARAMETER lookupListFieldTitle
        [String] The display name of the site column. This should be as it should be displayed and it can contain spaces.

   .EXAMPLE
        NOT USED in RAMS-2.3.0

    #>

    #$web = Get-SPWeb $webURL
    #$siteColumnFld = ($web.Fields | ? {$_.Title -match $siteColumnTitle})
    $siteColumnFld = $web.Fields.TryGetFieldByStaticName($siteColumnTitle)
    $listLookup = $web.Lists.TryGetList($lookupListTitle)
    # unfortunately, you can't change the source list of a lookup site column. If the list is the same and you are 
    # changing the field, that is ok.
    #$siteColumnFld.LookupList = $listLookup.ID
    $siteColumnFld.LookupField = $lookupListFieldTitle
    $siteColumnFld.Update()
    $web.dispose()
}

function Add-FacilityWorkflow ($featureTemplateName, $newFeatureId, $oldFeatureId, $searchDisplay, $listName, $unInstall){
     <#

    .SYNOPSIS
        Adds a workflow at the facility level, for each facility subweb to the list configured in the workflow.

    .DESCRIPTION
        Loops through each subsite spweb and calls DeployFeature passing the incoming parameters.

    .PARAMETER featureTemplateName
        [String] The name of the WSP workflow template file with the .wsp extension added.

    .PARAMETER newFeatureId
        [String] The featureID GUID from inside the wsp file.

    .PARAMETER oldFeatureId
        [String] The name or the old Feature Id (guid) of the workflow that should be deleted or replaced by the new Workflow.

    .PARAMETER searchDisplay
        [String] The name of the WSP workflow template file without the .wsp extension.

    .PARAMETER listName
        [String] The List Name that the workflow will be associated to.

    .PARAMETER unInstall
        [boolean] Indicates if the workflow should be uninstalled.

   .EXAMPLE
        # This example reads a list of workflows from WfList.csv file that contain the New Workflow Name, New Feature Id, Old Workflow Name, the List Name, and IsFacilityLevel
        # and based on the IsFacility Level calls Add-FacilityWorkflow or Add-SiteCollectionWorkFlow
        if ($wfType -eq "FacilityList") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-FacilityWorkflow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -listName $targetName -unInstall $False
        } elseif (($wfType -eq "SiteList") -or (($wfType -eq "Site"))) {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-SiteCollectionWorkFlow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -unInstall $False
        } elseif ($wfType -eq "SiteWFContentTypeAssociation") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-ReusableWorkFlowToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -contentTypeName $targetName -unInstall $False
        } elseif ($wfType -eq "SiteEventReceiver") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-EventReceiverToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $OldWFName -contentTypeName $targetName -unInstall $False
        } else {
            write-log -message "ERROR: Unknown wfType: wfType: $($wfType)" -consoleForeground Red -Level "ERROR"
        }
    #>

    $bolEnableFeatureOnly = $False #initialize
    # GET THE SITE
    $_siteURL = Get-SiteURL
    $w = Get-SPWeb $_siteURL

    write-log -message "$([String]::Format("Procesing web {0}",$w.Url))" -ConsoleForeground Cyan

    if($w.Webs.Count -gt 0)
    {
        foreach($web in $w.Webs)
        {   
            #TAR Remove Features in previous steps
            #Enable old workflows from 2.1 or before
            #if($oldFeatureName -contains "*IACUC*" -or $oldFeatureName -contains "*RDC*") {
            #    Enable-SPFeature -Identity $oldFeatureName -Url $web.Url -Verbose -Force
            #    Enable-SPFeature -Identity $oldFeatureName -Url $siteUrl -Verbose -Force
            #}
            #$web.Update()
            #get above old workflows
            $Oldfeature = $null #initialize

            if($bolEnableFeatureOnly -eq $False) {
                try {
                    $Oldfeature = Get-SPFeature -Identity $oldFeatureId -Web $_siteURL -ErrorAction Ignore -Verbose
                }
                catch {
                    write-log -message "Feature: $oldFeatureId does not exist for Site: $web.Url" -ConsoleForeground Red
                }
            }
            $tempFeatureId = $null
            if($Oldfeature -ne $null) {
                $tempFeatureId = $Oldfeature.Id
            }
            if ($tempFeatureId -eq $null)
            {
                $tempFeatureId = $newFeatureId
            }
            write-log -message "Calling Deploy-Feature on $web with $featureTemplateName OldFeatureId $tempFeatureId FeatureId $newFeatureId " -ConsoleForeground Gray
            if($bolEnableFeatureOnly -eq $False) {
                Deploy-Feature -SiteCollection $_siteURL -site $web.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $bolEnableFeatureOnly
                $bolEnableFeatureOnly = $True
            }
            else {
                Deploy-Feature -SiteCollection $_siteURL -site $web.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $bolEnableFeatureOnly
            }
        }
    }
}

function Add-SiteCollectionWorkFlow ($featureTemplateName, $newFeatureId, $oldFeatureId, $searchDisplay, $unInstall) {
     <#

    .SYNOPSIS
        Adds a workflow at the site collection level to the site itself or the list configured in the workflow.

    .DESCRIPTION
        Calls DeployFeature passing the incoming parameters.

    .PARAMETER featureTemplateName
        [String] The name of the WSP workflow template file with the .wsp extension added.

    .PARAMETER newFeatureId
        [String] The featureID GUID from inside the wsp file.

    .PARAMETER oldFeatureName
        [String] The name of the workflow that should be deleted or replaced by the new Workflow.

    .PARAMETER searchDisplay
        [String] The name of the WSP workflow template file without the .wsp extension.

    .PARAMETER unInstall
        [boolean] Indicates if the workflow should be uninstalled.

   .EXAMPLE
        # This example reads a list of workflows from WfList.csv file that contain the New Workflow Name, New Feature Id, Old Workflow Name, the List Name, and IsFacilityLevel
        # and based on the IsFacility Level calls Add-FacilityWorkflow or Add-SiteCollectionWorkFlow
        if ($wfType -eq "FacilityList") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-FacilityWorkflow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -listName $targetName -unInstall $False
        } elseif (($wfType -eq "SiteList") -or (($wfType -eq "Site"))) {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-SiteCollectionWorkFlow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -unInstall $False
        } elseif ($wfType -eq "SiteWFContentTypeAssociation") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-ReusableWorkFlowToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -contentTypeName $targetName -unInstall $False
        } elseif ($wfType -eq "SiteEventReceiver") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-EventReceiverToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $OldWFName -contentTypeName $targetName -unInstall $False
        } else {
            write-log -message "ERROR: Unknown wfType: wfType: $($wfType)" -consoleForeground Red -Level "ERROR"
        }

    #>

    try {    
        # GET THE SITE
        $_siteURL = Get-SiteURL
        $w = Get-SPWeb $_siteURL
        write-log -message "$([String]::Format("Add-SiteCollectionWorkflow Procesing web {0}",$w.Url))" -ConsoleForeground Yellow
     
        #$featureTemplateName="Notification Workflow Configuration Workflow.wsp"
        #$newFeatureId="df439192-e32f-4760-9bef-d239d8519593"
        
        $tempFeatureId = $Null
        write-log -message " Trying to find Feature $searchDisplay when adding Feature $featureTemplateName" -consoleForeground Yellow
        $Oldfeature = Get-SPFeature -Limit ALL  -Web $w.Url | Where-Object {($_.DisplayName -like $searchDisplay) -or ($_.DisplayName -like $featureTemplateName)}

        if($Oldfeature -ne $Null) {
            write-log -message "Found Old Feature $Oldfeature" -ConsoleForeground Yellow
            $tempFeatureId = $Oldfeature.Id
        }
        if ($tempFeatureId -eq $null)
        {
            $tempFeatureId = $oldFeatureId
        }
        write-log -message "Calling Deploy-Feature on $w with $featureTemplateName OldFeatureId $tempFeatureId FeatureId $newFeatureId "
        Deploy-Feature -SiteCollection $_siteURL -site $w.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $script -bolUninstall $unInstall -bolEnableFeatureOnly $False    

    } catch {
        write-log -message "$([String]::Format("Error processing web at $_siteURL, with Exception: {0}", $_.Exception.Message))" -ConsoleForeground Red
        if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
        if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
        if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
    }  
}

function Add-ReusableWorkFlowToContentType ($featureTemplateName, $newFeatureId, $oldFeatureId, $searchDisplay, $contentTypeName, $unInstall) {
     <#

    .SYNOPSIS
        Adds a reusable workflow to a content type at the site collection level 

    .DESCRIPTION
        Calls DeployFeature passing the incoming parameters then adds the workflow feature to the content type.

    .PARAMETER featureTemplateName
        [String] The name of the WSP workflow template file with the .wsp extension added.

    .PARAMETER newFeatureId
        [String] The featureID GUID from inside the wsp file.

    .PARAMETER oldFeatureName
        [String] The name or Old Feature Id (guid) of the workflow that should be deleted or replaced by the new Workflow.

    .PARAMETER searchDisplay
        [String] The name of the WSP workflow template file without the .wsp extension.

    .PARAMETER contentTypeName
        [String] The name of the content type.

    .PARAMETER unInstall
        [boolean] Indicates if the workflow should be uninstalled.

   .EXAMPLE
        # This example reads a list of workflows from WfList.csv file that contain the New Workflow Name, New Feature Id, Old Workflow Name, the List Name, and IsFacilityLevel
        # and based on the IsFacility Level calls Add-FacilityWorkflow or Add-SiteCollectionWorkFlow
        if ($wfType -eq "FacilityList") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-FacilityWorkflow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -listName $targetName -unInstall $False
        } elseif (($wfType -eq "SiteList") -or (($wfType -eq "Site"))) {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-SiteCollectionWorkFlow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -unInstall $False
        } elseif ($wfType -eq "SiteWFContentTypeAssociation") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-ReusableWorkFlowToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -contentTypeName $targetName -unInstall $False
        } elseif ($wfType -eq "SiteEventReceiver") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-EventReceiverToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $OldWFName -contentTypeName $targetName -unInstall $False
        } else {
            write-log -message "ERROR: Unknown wfType: wfType: $($wfType)" -consoleForeground Red -Level "ERROR"
        }

    #>

    try {
        # GET THE SITE
        $_siteURL = Get-SiteURL
        $w = Get-SPWeb $_siteURL
        write-log -message "$([String]::Format("Add-ReusableWorkFlowToContentType Procesing web {0}",$w.Url))" -ConsoleForeground Yellow
        
        $tempFeatureId = $Null
        write-log -message " Trying to find Feature $searchDisplay when adding Feature $featureTemplateName" -consoleForeground Yellow
        $Oldfeature = Get-SPFeature -Limit ALL  -Web $w.Url | Where-Object {($_.DisplayName -like $searchDisplay) -or ($_.DisplayName -like $featureTemplateName)}

        if($Oldfeature -ne $Null) {
            write-log -message "Found Old Feature $Oldfeature" -ConsoleForeground Yellow
            $tempFeatureId = $Oldfeature.Id
        }
        if ($tempFeatureId -eq $null)
        {
            $tempFeatureId = $oldFeatureId
        }
        write-log -message "Calling Deploy-FarmWFFeature on $w with $featureTemplateName OldFeatureId $tempFeatureId FeatureId $newFeatureId "
        Deploy-FarmWFFeature -SiteCollection $_siteURL -site $w.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $Global:script -bolUninstall $unInstall -bolEnableFeatureOnly $False
    } catch {
        write-log -message "$([String]::Format("Error processing web at $_siteURL, with Exception: {0}", $_.Exception.Message))" -ConsoleForeground Red
        if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
        if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
        if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
    }
}

function Add-EventReceiverToContentType ($featureTemplateName, $newFeatureId, $oldFeatureId, $searchDisplay, $contentTypeName, $unInstall) {
     <#

    .SYNOPSIS
        Adds a reusable workflow to a content type at the site collection level 

    .DESCRIPTION
        Calls DeployFeature passing the incoming parameters then adds the workflow feature to the content type.

    .PARAMETER featureTemplateName
        [String] The name of the WSP workflow template file with the .wsp extension added.

    .PARAMETER newFeatureId
        [String] The featureID GUID from inside the wsp file.

    .PARAMETER oldFeatureName
        [String] The name or Old Feature Id (guid) of the workflow that should be deleted or replaced by the new Workflow.

    .PARAMETER searchDisplay
        [String] The name of the WSP workflow template file without the .wsp extension.

    .PARAMETER contentTypeName
        [String] The name of the content type.

    .PARAMETER unInstall
        [boolean] Indicates if the workflow should be uninstalled.

   .EXAMPLE
        # This example reads a list of workflows from WfList.csv file that contain the New Workflow Name, New Feature Id, Old Workflow Name, the List Name, and IsFacilityLevel
        # and based on the IsFacility Level calls Add-FacilityWorkflow or Add-SiteCollectionWorkFlow
        if ($wfType -eq "FacilityList") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-FacilityWorkflow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -listName $targetName -unInstall $False
        } elseif (($wfType -eq "SiteList") -or (($wfType -eq "Site"))) {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-SiteCollectionWorkFlow -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -unInstall $False
        } elseif ($wfType -eq "SiteWFContentTypeAssociation") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-ReusableWorkFlowToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $oldFeatureId -contentTypeName $targetName -unInstall $False
        } elseif ($wfType -eq "SiteEventReceiver") {
            $Global:script = Join-Path $fileLocationTemplate "$($newWfName).wsp"
            Add-EventReceiverToContentType -featureTemplateName "$($newWfName).wsp" -newFeatureId $newFeatureId -searchDisplay $newWfName -oldFeatureId $OldWFName -contentTypeName $targetName -unInstall $False
        } else {
            write-log -message "ERROR: Unknown wfType: wfType: $($wfType)" -consoleForeground Red -Level "ERROR"
        }
    
    #>
    try {
        # GET THE SITE
        $_siteURL = Get-SiteURL
        $w = Get-SPWeb $_siteURL
        write-log -message "$([String]::Format("Add-EventReceiverToContentType Procesing web {0}",$w.Url))" -ConsoleForeground Yellow
        
        $tempFeatureId = $Null
        write-log -message " Trying to find Feature $searchDisplay when adding Feature $featureTemplateName" -consoleForeground Yellow
        $Oldfeature = Get-SPFeature -Limit ALL  -Web $w.Url | Where-Object {($_.DisplayName -like $searchDisplay) -or ($_.DisplayName -like $featureTemplateName)}
    
        if($Oldfeature -ne $Null) {
            write-log -message "Found Old Feature $Oldfeature" -ConsoleForeground Yellow
            $tempFeatureId = $Oldfeature.Id
        }
        if ($tempFeatureId -eq $null)
        {
            $tempFeatureId = $oldFeatureId
}
        write-log -message "Calling Deploy-FarmERFeature on $w with $featureTemplateName OldFeatureId $tempFeatureId FeatureId $newFeatureId "
        Deploy-FarmERFeature -SiteCollection $_siteURL -site $w.Url -FeatureName $featureTemplateName -OldFeatureId $tempFeatureId -FeatureId $newFeatureId -file $Global:script -bolUninstall $unInstall -bolEnableFeatureOnly $False
    } catch {
        write-log -message "$([String]::Format("Error processing web at $_siteURL, with Exception: {0}", $_.Exception.Message))" -ConsoleForeground Red
        if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
        if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
        if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
    }
}

#region utility functions Get-DateFromDays, Format-ListOutput, Get-ListInternalName
function Get-DateFromDays ($minusdays) {
    <#

    .SYNOPSIS
        Adds the number of days parameter to today

    .DESCRIPTION
        Adds the number of days parameter to today and returns the result. If minusdays is negative the result will be before today.

    .PARAMETER minusdays
        [integer] the number of days to add to today's date. to subtract days, pass a negative number 

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        $targetDate = Get-DateFromDays -10

    #>

    $days = $minusdays 
    $count = $collListItems.Count;
    $date = get-date;
    return $date.AddDays($days);

}

function Format-ListOutput($listTable) {
    <#

    .SYNOPSIS
        Format a list output as a list.

    .DESCRIPTION
        wrapper function to foromat a list to the console as a list.

    .PARAMETER listTable
        [Object] a list object that represents a SharePoint List

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        $facilityListItems = ((Get-SiteWeb).Lists.TryGetList("Facilities")).Items
        Format-ListOutput $facilityListItems

    #>

  $listTable | Format-List
}

function Get-ListInternalName ($staticName, $list) {
    <#

    .SYNOPSIS
        Return the Internal Name of a column on a list.

    .DESCRIPTION
        For the request staticname and list, lookup and return the internal name.

    .PARAMETER staticName
        [string] the static name of a site column on a list

    .PARAMETER list
        [object] a list object that represents a SharePoint List

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        $facilitiesList = ((Get-SiteWeb).Lists.TryGetList("Facilities"))
        $locationStaticName = "Location"
        $locationInternalName = Get-ListInternalName $locationStaticName $facilitiesList

    #>

    $list.Fields.TryGetFieldByStaticName($staticName).InternalName
    return;
}

function Get-SPList($web, $Name, $Url, $subWeb) {
    <#

    .SYNOPSIS
        Return the requested list.

    .DESCRIPTION
        Attempts to return the requested list by URL and if that fails, tries by name. Returns null list if nothing is found.

    .PARAMETER web
        [Microsoft.SharePoint.SPSecurableObject] a web object that represents the site collection.

    .PARAMETER Name
        [string] a list object that represents a SharePoint List

    .PARAMETER Url
        [string] The URL of the list.

    .PARAMETER subWeb
        [Microsoft.SharePoint.SPSecurableObject] a web object that represents the subsite spweb.

    .EXAMPLE
        $facilityList = Get-SPList -web $webSite -Name "Facility" -Url ($webSite.ServerRelativeUrl + "/Lists/Facility/AllItems.aspx")

    .EXAMPLE
        $list = Get-SPList -web $Site -Name $listName -Url $listUrl -subWeb $subSite
    #>

    $list = $null
    try {  
        if($subWeb -eq $null) {  
            $list = $web.GetListFromURL($Url) 
         } else {
            $list = $subWeb.GetListFromURL($Url) 
         }
         return $list
    } catch {
        try {
            if($subWeb -eq $null) {  
                $list = $web.GetList($Name) 
             } else {
                $list = $subWeb.GetList($Name) 
             }
             return $list
        } catch {  
            return $list          
        }
    }   
}

function Remove-DataFromAList ($Site, $subSite, $filterDate, $listUrl, $listName) {
    <#

    .SYNOPSIS
        Deletes Data from the specified list created before the filterDate.

    .DESCRIPTION
        Removes data from the specified list where the created date occurs before the filterDate.

    .PARAMETER Site
        [Microsoft.SharePoint.SPSecurableObject] a web object that represents the site collection.

    .PARAMETER subSite
        [Microsoft.SharePoint.SPSecurableObject] Optional a web object that represents the subsite spweb.

    .PARAMETER filterDate
        [date] The date considered when determining to delete the records.

    .PARAMETER listUrl
        [string] The URL of the list to delete the data from.

    .PARAMETER listName
        [string] The name of the list.

    .EXAMPLE
        #get site collection
        $site= Get-SiteURL
        $x = $site.ToString();
        $url = $x.Substring($x.IndexOf("=")+1)
        $w = Get-SiteWeb

        #filter by days
        $filterPriorDate = $cleanDate
        
        #Delete Application list for older studies.
        $listUrl = $url + "/Lists/Application/AllItems.aspx"
        $listOfStudies = Get-ColumnValueFromAList -Site $w -filterDate $filterPriorDate -listUrl $listUrl -listColumnName "Study_ID"

        Remove-DataFromAList -Site $w -filterDate $filterPriorDate -listUrl $listUrl

    #>

    #get a collection of ids
    $itemsToDelete = New-Object System.Collections.ArrayList
    $list = Get-SPList -web $Site -Name $listName -Url $listUrl -subWeb $subSite
    #get application list
    if($list -ne $null) {
        #Get all items in this list and save them to a variable
        $items = $list.items
        $listTitle = $list.Title
        $itemCount = 0
        #Go through all items
        write-log -message "Total Items in the $($list.EntityTypeName) : $($items.Count)" -ConsoleForeground Green
        foreach($item in $items)
        {
            if($item["Created"] -lt $filterDate)
            { 
                if ($item.Title -ne $null) {
                    $itemDescription = $item.Title
                } elseif ($item.Name -ne $null ) {
                    $itemDescription = $item.Name
                } else {
                    $itemDescription = "Unknown Title or Name"
                }
                #add each item id to the delete list  
                write-log -message "$itemCount. Add item $($item.ID) from the list $($listTitle) for Study: $($item["Title"]) on $($item["Created"])" -ConsoleForeground Cyan                                 
                $itemsToDelete.Add($item.ID)                
                $itemCount ++;
            }            
        }        
        #now start deleting those items
        if($itemCount -eq 0) {
            write-log -message "Nothing to delete, count: $itemCount" -ConsoleForeground Yellow
         return
        }
        write-log -message "Total items to be deleted $itemCount" -ConsoleForeground Yellow
        foreach($itemToDelete in $itemsToDelete) {
            write-log -message "Deleting item.Id $itemToDelete with Title $($item.Title) from $listTitle" -ConsoleForeground Cyan  
            $items.DeleteItemById($itemToDelete);
        }
        $items.Update();
        $list.Update();
        write-log -message "Total Items After $($list.items.Count)" -ConsoleForeground Green
    } else {
        write-log -message No list found by the name: $listUrl
    }
}
#endregion

#region function Update-UtilMF
function Update-UtilMF {
    <#

    .SYNOPSIS
        Update this module's manifest.

    .DESCRIPTION
        Generate the manifest for this module.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Update-UtilMF

    #>
    New-ModuleManifest -Path .\RAMS_Module_Util.psd1 -ModuleVersion 1.0.0 -Author "Chita Nilak, ByLight" -RootModule .\RAMS_Module_Util.psm1 -Description "RAMS Utility Functions" -CompanyName "DNS" -Verbose 
}
#endregion

function Get-RAMSEnv ($host) {
    <#

    .SYNOPSIS
        Retrieve the environment information from the Env.json file

    .DESCRIPTION
        Read the Env.json file and return the information for the specified host. Reads the Env.host from the current directory and searches for the environment information for the given host.

    .PARAMETER host
        [String] The name of the computer to lookup in the env.host file

    .EXAMPLE
        $Global:hostInfo = Get-RAMSEnv -host $env:COMPUTERNAME

    #>

    $_host = $host
    $_env = Get-Content .\Env.json | ConvertFrom-Json
    $_server = $null
    switch ($_host) {
        $_env.DEV.sp {$_server = $_env.DEV}    
        $_env.TEST.sp {$_server = $_env.TEST}    
        $_env.PPD.sp {$_server = $_env.PPD}    
        $_env.SQA.sp {$_server = $_env.SQA}    
        $_env.PROD.sp {$_server = $_env.PROD}    
    }
    return $_server
}

function Get-RAMSEnvName ($host) {
    <#

    .SYNOPSIS
        Retrieve the environment information from the Env.json file

    .DESCRIPTION
        Read the Env.json file and return the information for the specified host. Reads the Env.host from the current directory and searches for the environment information for the given host.

    .PARAMETER host
        [String] The name of the computer to lookup in the env.host file

    .EXAMPLE
        $envName = Get-RAMSEnvName -host ($env:COMPUTERNAME)

    #>

    $_host = $host
    $_env = Get-Content .\Env.json | ConvertFrom-Json
    $_server = $null
    switch ($_host) {
        $_env.DEV.sp {$_server = 'DEV'}    
        $_env.TEST.sp {$_server = 'TEST'}    
        $_env.PPD.sp {$_server = 'PPD'}    
        $_env.SQA.sp {$_server = 'SQA'}    
        $_env.PROD.sp {$_server = 'PROD'}    
    }
    return $_server
}

function New-RAMSEnv {
    <#

    .SYNOPSIS
        Creates a new, empty Env.json file.

    .DESCRIPTION
        Creates a new, empty Env.json file with entries for each environment and saves the results in the current directory.

    .EXAMPLE
        New-RAMSEnv

    #>

    $_serverEnv = @{
        DEV = @{
                            "sp" = "DNS";                            
                            "sql" = "DNS";
                            "ssrs" = "DNS";
                            "ssis" = "DNS";
                            "superReader" = @("",""); #READ-ONLY access to RAMS-DB
                            "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
                        }; #DEV
        TEST = @{
                            "sp" = "DNS";                            
                            "sql" = "DNS";
                            "ssrs" = "DNS";
                            "ssis" = "DNS";
                            "superReader" = @("",""); #READ-ONLY access to RAMS-DB
                            "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
                        }; #TEST
        SQA = @{
                            "sp" = "DNS";                         
                            "sql" = "DNS";
                            "ssrs" = "DNS";
                            "ssis" = "DNS";
                            "superReader" = @("",""); #READ-ONLY access to RAMS-DB
                            "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
                        }; #SQA DNS";                         
                            "sql" = "DNS";
                            "ssrs" = "DNS";
                            "ssis" = "vDNS";
                            "superReader" = @("",""); #READ-ONLY access to RAMS-DB
                            "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
                        }; #PPD
        PROD = @{
                            "sp" = "DNS  ";                         
                            "sql" = "DNS";
                            "ssrs" = "DNS";
                            "ssis" = "DNS";
                            "superReader" = @("",""); #READ-ONLY access to RAMS-DB
                            "farmUser" = @("",""); #SharePoint Farm Account w/ local Admin permission                                                                 
                        }; #PROD
    }
    if((Get-ChildItem ".\" -Filter "ENV.json") -ne $null) {                     
        Remove-Item .\ENV.json -Verbose
    }
    ConvertTo-json $_serverEnv -Compress | New-Item -Name ENV.json -Path .\ -ItemType file -Force
    return Get-Content .\Env.json | ConvertFrom-Json
}

function Get-FacilitySites {
    <#

    .SYNOPSIS
        Return the list of Subsite names.

    .DESCRIPTION
        Loop through the subsites and return an arraylist of the subsite names.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Get-FacilitySites

    #>

    $webs = Get-SiteWeb 
    $_facilities = New-Object System.Collections.ArrayList    
    foreach($web in $webs.Webs) {        
       $_facilities.Add($web.Name)       
    }
    #return
    $_facilities
}

function New-RAMSGroup ($facility, $newGroupName, $newGroupPermission, $groupOwnerSuffix, $newGroupDescSuffix) {
    <#

    .SYNOPSIS
        Creates a new security group for a given facility.

    .DESCRIPTION
        Builds a new facility security group with the provided permissions. If it already exists, it is removed and added again.

    .PARAMETER facility
        [string] the name of the facility as it exists on the Facility list. In the case of Miami Animal Facility Managers, the facility is the Miami part.

    .PARAMETER newGroupName
        [String] The name of the security group. In the case of Miami Animal Facility Managers, the newGroupName is the Animal Facility Managers part.

    .PARAMETER newGroupPermission
        [String] The standard SharePoint role to be given to the security group.

    .PARAMETER groupOwnerSuffix
        [String] The name of the group owner. Typicall "Local Site Administrators". The script will prepend the facility to the suffix.

    .PARAMETER newGroupDescSuffix
        [String] The security group description. Informational only.

    .EXAMPLE
        New-RAMSGroup -facility $_.Title -newGroupName "Animal Facility Managers" -newGroupPermission "Edit" -groupOwnerSuffix "Local Site Administrators" -newGroupDescSuffix "Animal Facility Managers"    

    #>

    #Get site and Group objects  
    $_url = Get-SiteURL 
    $_Site = Get-SPSite -Identity $_url
    
    #add new security group
    $_siteGroups = $_Site.RootWeb.SiteGroups
    $_ownerGroup = $_siteGroups["$facility $groupOwnerSuffix"]
    
    #check if the group already exist
    try {
        $_siteGroups.Add("$facility $newGroupName",$_ownerGroup,$null,"$facility $newGroupDescSuffix")
    }    
    catch {
        $_siteGroups.Remove("$facility $newGroupName")
        $_siteGroups.Add("$facility $newGroupName",$_ownerGroup,$null,"$facility $newGroupDescSuffix")
    }
    $_Site.RootWeb.Update()

    #Change AO group permission at site collection level to EDIT
    $role = $_Site.RootWeb.RoleDefinitions[$newGroupPermission]
    $_newGroup = $_Site.RootWeb.SiteGroups["$facility $newGroupName"]
    $RoleAssignment = New-Object Microsoft.SharePoint.SPRoleAssignment($_newGroup)
    $RoleAssignment.RoleDefinitionBindings.Add($role)

    # the following could be for a web, list or item object
    $_Site.RootWeb.RoleAssignments.Add($RoleAssignment)
    $_Site.RootWeb.Update()
    write-log -message "$facility $newGroupName has been added w/ $newGroupPermission permission on $_Site" -consoleForeground yellow
    
    #dispose site object   
    $_Site.Dispose();  
}

function Add-FacilityGroup ($newGroupName, $newGroupPermission) {
    <#

    .SYNOPSIS
        Grants the permission to the facility subsite.

    .DESCRIPTION
        For each facility in the Facility List, loop through each granting the newGroupName the newGroupPermission.

    .PARAMETER newGroupName
        [String] The name of the security group. In the case of Miami Animal Facility Managers, the newGroupName is the Animal Facility Managers part.

    .PARAMETER newGroupPermission
        [String] The standard SharePoint role to be given to the security group.

    .EXAMPLE
        Add-FacilityGroup -newGroupName "Animal Facility Managers" -newGroupPermission "Edit" 

    #>
    
    $_facilityWebs = Get-SiteWeb
    $_facilityList = $_facilityWebs.GetListFromUrl("Lists/Facility/AllItems.aspx")

    #add each new groups to the respective facility sub-site.
    foreach ($item in $_facilityList.Items) {
        #facility sub-site
        $facilitySubsite = $item["Facility Subsite"]
        #facility name
        $facilityName = $item["Facility Name"]
        #foreach
        foreach ($facilityWeb in $_facilityWebs.Webs) {
            if($facilitySubsite.IndexOf($facilityWeb.Name) -gt 0) {
                #get role
                $role = $facilityWeb.RoleDefinitions[$newGroupPermission]
                $_newGroup = $_facilityWebs.SiteGroups["$facilityName $newGroupName"]
                $RoleAssignment = New-Object Microsoft.SharePoint.SPRoleAssignment($_newGroup)
                $RoleAssignment.RoleDefinitionBindings.Add($role)

                # the following could be for a web, list or item object
                $facilityWeb.RoleAssignments.Add($RoleAssignment)
                $facilityWeb.Update()
                write-log -message "$_newGroup has been added w/ $newGroupPermission permission on $facilityWeb" -consoleForeground yellow
            
                #$item["Research Compliance Officers"]        
            }
            $facilityWeb.Update()
        }    
    }
}

function Remove-WorkFlow {
    <#

    .SYNOPSIS
        Removes the requested workflow from the site collection and the feature from the Farm

    .DESCRIPTION
        Removes the workflow from the site collection and the feature from the farm. Starts by disabling the SPFeature, then Uninstalls the 
        SPUserSolution, then Removes the SPUserSolution. Next, it loops through the definitions and subscriptions, deleting them all.

    .PARAMETER wfMatchName
        [String] The name of the workflow.

    .PARAMETER wfWeb
        [Microsoft.SharePoint.SPWeb] The SPWeb of the site collection or facility subsite.

    .PARAMETER SiteCollection
        [String] The URL of the site collection.

    .NOTES
        Not used in RAMS-2.3.0
        Does not work because it may not completely remove the workflow.

    .EXAMPLE
        Remove-WorkFlow -wfMatchName ($FeatureName.Replace(".wsp","")) -wfWeb $web -SiteCollection (Get-SiteURL)

    #>
    
    Param(
        [Parameter(Mandatory=$True)]
        [string]$wfMatchName,

        [Parameter(Mandatory=$True)]
        [Microsoft.SharePoint.SPWeb]$wfWeb,

        [Parameter(Mandatory=$True)]
        [string]$SiteCollection

    )
    $SiteCollectionweb = Get-SPWeb $SiteCollection
    $wfm = New-Object Microsoft.SharePoint.WorkflowServices.WorkflowServicesManager($wfWeb);
    $wfss =$wfm.GetWorkflowSubscriptionService()
    $wfds = $wfm.GetWorkflowDeploymentService()
    $wfis=$wfm.GetWorkflowInstanceService()
    #$wfss.EnumerateSubscriptions()

    write-log -message "Remove-WorkFlow Feature: $wfMatchName on site: $($wfWeb.Url)" -ConsoleForeground Yellow
    write-log -message "  Disable-SPFeature $wfMatchName" -ConsoleForeground Gray
    #Disable-SPFeature -Identity $wfMatchName -Url $SiteCollectionweb.Url -Confirm:$false -Force -ErrorAction SilentlyContinue  
    Disable-SPFeature -Identity $wfMatchName -Url $SiteCollectionweb.Url -Confirm:$false -Force   
    $checkSPFeature = Get-SPFeature -Identity $wfMatchName -Site $SiteCollectionweb.Url
    if($checkSPFeature -ne $null){
        write-log -message "    The feature: $wfMatchName is not disabled" -consoleForeground Red
    } else {
        write-log -message "    The feature: $wfMatchName is not found" -consoleForeground Green
    }
        
    write-log -message "  Uninstall-SPUserSolution $wfMatchName" -ConsoleForeground Gray
    Uninstall-SPUserSolution -Identity $wfMatchName -Site $SiteCollectionweb.Site 
    $checkSPUserSolution = Get-SPUserSolution -Identity $wfMatchName -Site $SiteCollectionweb.Url
    if($checkSPUserSolution -ne $null){
        write-log -message "    The SPUserSolution: $wfMatchName is not disabled" -consoleForeground Red
    } else {
        write-log -message "    The SPUserSolution: $wfMatchName is not found" -consoleForeground Green
    }
        
    write-log -message "  Remove-SPUserSolution $wfMatchName" -ConsoleForeground Gray
    Remove-SPUserSolution -Identity $wfMatchName -Site $SiteCollectionweb.Site 

    foreach($wf in $wfds.EnumerateDefinitions($false)){        
        if($wf.properties['Definition.DisplayName'] -match $wfMatchName){
            $wfDefId = $wf.properties['Definition.Id']
            $wfSubId = $wf.properties['SubscriptionId']
            write-log -message "$($wf.properties['Definition.DisplayName']) wfDeployment Deleted from web $($wfWeb)" -consoleForeground Gray
            $wfss.DeleteDefinition($wfDefId)
            write-log -message " - Success" -ConsoleForeground Green
            $wfSub = $wfss.GetSubscription( $wfSubId)
            write-log -message "$($wf.properties['Definition.DisplayName']) wfSubscription Deleted from web $($wfWeb)" -consoleForeground Gray
            if($wfSub -ne $null) {
                $wfds.DeleteSubscription($wfSubId)
                write-log -message " - Success" -ConsoleForeground Green
            } else {
                write-log -message " - NOT FOUND" -ConsoleForeground RED
            }
        } 
    }
    
    #$wfDefs = $wfds.EnumerateDefinitions($false)
    #foreach ($wfDef in $wfDefs){
    #    if($wfDef.Name -match $wfMatchName){
    #        $wfds.DeleteDefinition($wfDef.Id)
    #    }
    #}
    #$wfWeb.Update()


    # TAR  Adding complete removal of Workflow
    $wfWeb.Update()     
           
}

function Rename-List($spWeb,$listName,$newTitle) {
    <#

    .SYNOPSIS
        Renames a list to a new name and location.

    .DESCRIPTION
        Renames a list by moving it to a new name and location.

    .PARAMETER spWeb
        [Microsoft.SharePoint.SPWeb] The SPWeb of the site collection or facility subsite.

    .PARAMETER listName
        [String] The original name of the list.

    .PARAMETER newTitle
        [String] The new name of the list.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Rename-List -spWeb $subWeb -listName "RDC Calendar" -newTitle "RDC Calendar_bak"

    #>

    ForEach($culture in $spWeb.SupportedUICultures) {
        [System.Threading.Thread]::CurrentThread.CurrentUICulture=$culture;
        #check if the list has already been named or not
        if(($spWeb.Lists.TryGetList($newTitle)) -eq $null) {
            $list = $spWeb.Lists.TryGetList($listName) 
            $list.Title = $newtitle;                    
            $list.RootFolder.MoveTo($newtitle)
            $list.Update();
        }        
    }
    $spWeb.Dispose()
}

function Clone-Calendar($spWeb, $calName,$OldCalName, $calTitle) {
    <#

    .SYNOPSIS
        Copies a calendar template to create a new calendar.

    .DESCRIPTION
        Checks to see if a calendar with calTitle exists and deletes it. Creates a new calendar base on the Calendar List Template. Adds the same columns from the old calendar 
        to the new calendar. Builds the views for display and editing. Finally, migrates the data from the old calendar list to the new calendar list.

    .PARAMETER spWeb
        [Microsoft.SharePoint.SPWeb] The SPWeb of the site collection or facility subsite.

    .PARAMETER calName
        [String] The new name of the calendar list.

    .PARAMETER OldCalName
        [String] The name of the backup list to copy data from. Probably created using Rename-List.

    .PARAMETER calTitle
        [String] Title of the new list.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Clone-Calendar -spWeb $subWeb -calName "RDCCalendar" -OldCalName "RDC Calendar_bak" -calTitle "RDC Calendar"

    #>
  
    $SPTemplate = (Get-SiteWeb).ListTemplates["Calendar"]
    
    $newCal = $spWeb.Lists.TryGetList($calTitle)
    #remove if the clone list already exist
    if($newCal -ne $null) {    
        $spWeb.Lists.Delete($newCal.ID)
        $spWeb.Update()
    }
    #create a new clone list
    $spWeb.Lists.Add($calTitle,$calTitle,$SPTemplate)
    $spWeb.Update()
    
    #get old list (after rename)    
    $oldList = $spWeb.Lists.TryGetList($OldCalName)

    #get new list 
    $newList = $spWeb.Lists.TryGetList($calTitle)
               
    if($oldList -ne $null) {
        
        foreach($oldFld in $oldList.Fields) {            
            if(($newList.Fields.ContainsFieldWithStaticName($oldFld.StaticName) -eq $False) -and 
                   ($newList.Fields.ContainsField($oldFld.Title) -eq $False)) {          
                $newList.Fields.Add($oldFld);
                $newList.Update()
                write-log -message "$oldFld has been added to $newList" -ConsoleForeground Green
            } else {
                write-log -message "$oldFld already exist in $newList" -ConsoleForeground Yellow
            }
        }
        
        $spWeb.Update()
        #set default view
        $newView = $newList.Views | ? { $_.Title -eq 'All Events' };

        <#add old view
        $oldView = $oldList.Views | ? { $_.Title -eq 'All Events' };
        #set default view
        $newView = $newList.Views | ? { $_.Title -eq 'All Events' };
        #clone view fields
        foreach($oldViewFld in $oldView.ViewFields){
            if($newView.ViewFields.exists($oldViewFld)) {
                write-log -message "$($oldViewFld) already exist in $newView" -ConsoleForeground Yellow
            }
            else {
                $newView.ViewFields.Add($oldViewFld)
                write-log -message "$($oldViewFld) has been added to $newView" -ConsoleForeground Green
            }
        }
        #>
        $contentType = "Event"
        $newFieldOrder = @("Title", "EventDate", "EndDate", "Location", "fAllDayEvent", "fRecurrence","EventAttendees", "EventCommitteeAttendees",  "ParticipantsPicker","Administrative_x0020_Items", "TaxKeyword", "ApprovedApplicationLookup", "IACUCMeetingMinuteLookup", "RDCMeetingMinuteLookup", "Decision", 
        "Deliberation"
        )
        $ct = $newList.ContentTypes[$contentType]       
        $ct.FieldLinks.Reorder($newFieldOrder)
        $ct.Update() 
        $newList.Update()
        #set as default view
        $newView.DefaultView = $true
        $newView.Update()
        #migrate data from old list to new list
        $newItem = $null
        foreach($oldItem in $oldList.Items) {
            $newItem = $newList.AddItem()
            foreach($fld in $oldItem.Fields) {
              if(($newList.Fields.ContainsField($fld.Title) -eq $True)) {  
                if("ID","WF","Link","_","Agenda","Attachments","Facilities" -notcontains $fld.StaticName){
                    if($fld.ReadOnlyField -eq $False) {
                        $newItem[$fld.StaticName] = $oldItem[$fld.StaticName]
                        write-log -message "Item: $($newItem[$fld.StaticName]) has been added to the $newList" -ConsoleForeground Cyan
                    }
                 } 
               } 
             }
            $newItem.Update()
        }
        $newList.Update()
    }
}

function Remove-SitePages ([string[]]$sitePages) {
    <#

    .SYNOPSIS
        Removes items from the SitePages folder in SharePoint.

    .DESCRIPTION
        Loops through the pages in site pages and if the requested page is found, deletes it.

    .PARAMETER sitePages
        [String[]] The names of the page to remove.

    .EXAMPLE
        Remove-SitePages -sitePages manageCommittee.html

    .EXAMPLE
        Remove-SitePages -sitePages manageCommittee.html, anotherPage.aspx

    #>

    foreach($page in $sitepages) {
        $targetPage = ((Get-SiteWeb).Lists["Site Pages"]).Items | ? {$_.Name -eq $page} 
        if($targetPage -ne $Null) {
            $targetPage.Delete() 
            write-log -message "$page has been deleted from Site Collection: $(Get-SiteWeb)" -ConsoleForeground Yellow
        }
    }
}

function Remove-SiteAssets ([string[]]$siteAssets) {
    <#

    .SYNOPSIS
        Removes items from the SiteAssets folder in SharePoint.

    .DESCRIPTION
        Loops through the items in site assets and if the requested item is found, deletes it.

    .PARAMETER siteAssets
        [String[]] The names of the item to remove.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Remove-SitePages -sitePages manageCommittee.html

    .EXAMPLE
        Remove-SitePages -sitePages manageCommittee.html, anotherPage.aspx

    #>

    foreach($asset in $siteassets) {
        $targetAsset = ((Get-SiteWeb).Lists["Site Assets"]).Items | ? {$_.Name -eq $asset} 
        if($targetAsset -ne $Null) {
            $targetAsset.Delete() 
            write-log -message "$asset has been deleted from Site Collection: $(Get-SiteWeb)" -ConsoleForeground Yellow
        }
    }
}

function Remove-FacilitySitePages ([string[]]$sitePages) {
    <#

    .SYNOPSIS
        Removes items from the SitePages folder each facility subsite.

    .DESCRIPTION
        Loops through the pages in site pages and if the requested page is found, deletes it.

    .PARAMETER sitePages
        [String[]] The names of the page to remove.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Remove-FacilitySitePages -sitePages manageCommittee.html

    .EXAMPLE
        Remove-FacilitySitePages -sitePages manageCommittee.html

    #>

    foreach ($subWeb in (Get-SiteWeb).Webs) {        
        foreach($page in $sitepages) {
            $targetPage = ($subWeb.Lists["Site Pages"]).Items | ? {$_.Name -eq $page} 
            if($targetPage -ne $Null) {
                $targetPage.Delete() 
                write-log -message "$($targetPage.Name) has been deleted from $subWeb" -ConsoleForeground Yellow
            }
        }
    }
}

function Remove-FacilitySiteAssets ([string[]]$siteAssets) {
    <#

    .SYNOPSIS
        Removes items from the SiteAssets folder each facility subsite.

    .DESCRIPTION
        Loops through the items in site assets and if the requested item is found, deletes it.

    .PARAMETER siteAssets
        [String[]] The names of the item to remove.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Remove-FacilitySiteAssets -siteAssets manageCommittee.html

    .EXAMPLE
        Remove-FacilitySitePages -sitePages manageCommittee.html, anotherPage.aspx

    #>

    foreach ($subWeb in (Get-SiteWeb).Webs) {
        foreach($asset in $siteassets) {
            $targetAsset = ($subWeb.Lists["Site Assets"]).Items | ? {$_.Name -eq $asset} 
            if($targetAsset -ne $Null) {
                $targetAsset.Delete() 
                write-log -message "$($targetAsset.Name) has been deleted from $subWeb" -ConsoleForeground Yellow
            }
        }
    }
}

function New-SpListView ([System.Collections.Specialized.StringCollection] $fldList, $viewTitle, $listName,$viewQuery,$viewRowLimit) {
    <#

    .SYNOPSIS
        Create a List view for a given list.

    .DESCRIPTION
        Builds a list view by retrieving the list, creating a view with the provided name, adding the provided fields and updating the list.

    .PARAMETER fldList
        [String[]] The list of fields.

    .PARAMETER viewTitle
        [String] The names of the view.

    .PARAMETER listName
        [String] The names of the list.

    .PARAMETER viewQuery
        [String] The query.

    .PARAMETER viewRowLimit
        [String[]] The number of rows to include in the view.

    .EXAMPLE
        $fldList = @('StudyTitle';
            'StudyIDLink';
            'PrincipalInvestigator';
            'IRBAdministrator';
            'ApplicationStatus';
            'Modified';
            'ApplicationSiteType';
            'FacilitiesLookup';
            'AdverseReportableEvent';
            'DeviationReportableEvent'
        )
        $viewQuery ="<Where><And><Contains><FieldRef Name='StudyTitle'/><Value Type='Text'>Adverse/Reportable Event</Value></Contains>"
            +"<Neq><FieldRef Name='ApplicationStatus'/><Value Type='Text'>Draft</Value></Neq></And></Where>"
        New-SpListView -fldList $fldList -viewTitle 'AdverseReportableEvent' -listName 'Application' -viewQuery $viewQuery -viewRowLimit '50'

    #>


    #Get destination site and list
    $web = Get-SiteWeb   
    $list = $web.GetList($web.ServerRelativeUrl.TrimEnd("/") + "/Lists/$listName")
    
    #Title property
    $viewTitle = $viewTitle
    $View = $list.Views | ? {$_.Title -match $viewTitle}
    if ($View -ne $null) {
           $list.Views.Delete($View.ID)
           $list.Update()
    }
        #Add the column names from the ViewField property to a string collection
    $viewFields = New-Object System.Collections.Specialized.StringCollection
    foreach ($fldName in $fldList) {
        $viewFields.Add($fldName) > $null
    }

    #Paged property
    $viewPaged = $true

    #DefaultView property
    $viewDefaultView = $false

    #Create the view in the destination list
    $newview = $list.Views.Add($viewTitle, $viewFields, $viewQuery, $viewRowLimit, $viewPaged, $viewDefaultView)
    write-log -message "$($newview.Title) view has been added to the $($list.Title) list on site $($web.Url)" -consoleForeground Green

    #update view and list
    $newview.Update()
    $list.Update()    
}

function New-SpFacilityListView ($web, [System.Collections.Specialized.StringCollection] $fldList, $viewTitle, $listName,$viewQuery,$viewRowLimit) {
    <#

    .SYNOPSIS
        Create a List view for a given list on each facility subsite.

    .DESCRIPTION
        Builds a list view for each facility list by retrieving the list, creating a view with the provided name, adding the provided fields and updating the list.

    .PARAMETER fldList
        [String[]] The list of fields.

    .PARAMETER viewTitle
        [String] The names of the view.

    .PARAMETER listName
        [String] The names of the list.

    .PARAMETER viewQuery
        [String] The query.

    .PARAMETER viewRowLimit
        [String[]] The number of rows to include in the view.

    .EXAMPLE
        $fldList = @('LinkTitle'; 
            'EventDate';
            'EndDate';
            'Location'; 
            'Attendees'; 
            'CommitteeAttendees'
        )
        $viewQuery = "<Where></Where>"
        New-SpFacilityListView  -web $web -fldList $fldList -viewTitle 'StandardView' -listName 'FullBoardReviewCalendar' -viewQuery $viewQuery -viewRowLimit '50'


    #>

    $list = $web.GetList($web.ServerRelativeUrl.TrimEnd("/") + "/Lists/$listName")
    
    #Title property
    $viewTitle = $viewTitle
    $View = $list.Views | ? {$_.Title -match $viewTitle}
    if ($View -ne $null) {
           $list.Views.Delete($View.ID)
           $list.Update()
    }
        #Add the column names from the ViewField property to a string collection
    $viewFields = New-Object System.Collections.Specialized.StringCollection
    foreach ($fldName in $fldList) {
        $viewFields.Add($fldName) > $null
    }

    #Paged property
    $viewPaged = $true

    #DefaultView property
    $viewDefaultView = $false

    #Create the view in the destination list
    $newview = $list.Views.Add($viewTitle, $viewFields, $viewQuery, $viewRowLimit, $viewPaged, $viewDefaultView) 
    write-log -message "$viewTitle View has been added to the $listName list on site $($web.Url)" -ConsoleForeground Green

    #update view and list
    $newview.Update()
    $list.Update()    
}

function Update-SpListView ([System.Collections.Specialized.StringCollection] $fldList, $listName) {
    <#

    .SYNOPSIS
        Adds fields to each view on a given list

    .DESCRIPTION
        Loops through each view on a given list and adds the fields specified in fldList

    .PARAMETER fldList
        [String[]] The list of fields.

    .PARAMETER listName
        [String] The names of the list.

    .EXAMPLE
        $fldList = @('LocalSiteLiaisons';'LocalSiteAdministrators')
        Update-SpListView -fldList $fldList -listName 'Facility'

    #>

    #Get destination site and list
    $web = Get-SiteWeb    
    $list = $web.GetList($web.ServerRelativeUrl.TrimEnd("/") + "/Lists/$listName")
    
    #Update Views
    try {
        foreach($View in $list.Views) {
            #Add the column names from the ViewField property to a string collection
            $viewFields = $View.ViewFields
            foreach ($fldName in $fldList) {
                $ViewFields.Add($fldName) > $null
            }    
            #update view and list        
            $View.Update()
            write-log -message "$($View.Title) View has been updated for the $($list.Title) list on site $($web.Url)" -ConsoleForeground Green
       }
   } catch {
       #do nothing.
   } finally {
       $list.Update()
   }
}

Function Remove-ContentTypeFromList() {
    <#

    .SYNOPSIS
        Removes a content type from a given list.

    .DESCRIPTION
        NOT USED because it does not work as written. When trying to remove the columns, the data needs to be deleted first??? See notes in UpdateFacilityInfo.ps1.

    .PARAMETER SiteURL
        [String] The site URL.

    .PARAMETER ListName
        [String] The names of the list.

    .PARAMETER ContentTypeName
        [String] The names of the content type to remove .

    .NOTES
        Not used in RAMS-2.3.0
        Does not work. 

    .EXAMPLE
        Remove-ContentTypeFromList -SiteURL (Get-SiteURL) -ListName "Facility" -ContentTypeName "Facility CT" -Verbose

    #>

    param(
        [Parameter(Mandatory=$true)] [string] $SiteURL,
        [Parameter(Mandatory=$true)] [string] $ListName,
        [Parameter(Mandatory=$true)] [string] $ContentTypeName
    )
 
    try {

        Add-Type -Path "C:\Program Files\Common Files\Microsoft Shared\Web Server Extensions\15\ISAPI\Microsoft.SharePoint.Client.Runtime.dll"
        Add-Type -Path "C:\Program Files\Common Files\Microsoft Shared\Web Server Extensions\15\ISAPI\Microsoft.SharePoint.Client.dll"

        $Credentials = New-Object System.Net.NetworkCredential("RAMINTSPFARM","&35Dkc>Av+KX","DNS      ")
         
        #Setup the context
        $Ctx = New-Object Microsoft.SharePoint.Client.ClientContext($SiteURL)
        $Ctx.Credentials = $Credentials
     
        #Get the List
        $List = $Ctx.Web.Lists.GetByTitle($ListName)
        $Ctx.Load($List)
 
        #Get the content type from list
        $ContentTypeColl = $List.ContentTypes
        $Ctx.Load($ContentTypeColl)
        $Ctx.ExecuteQuery()
 
        #Get the content type to remove
        $CTypeToRemove = $ContentTypeColl | Where {$_.Name -eq $ContentTypeName}
        If($CTypeToRemove -ne $Null)
        {
            #Remove content type from list
            $CTypeToRemove.DeleteObject()
            $Ctx.ExecuteQuery()
 
            write-log -message "Content Type '$ContentTypeName' Removed From '$ListName'" -consoleForeground  Green
        }
        else
        {
            write-log -message "Content Type '$ContentTypeName' doesn't exists in '$ListName'" -consoleForeground  Yellow
            Return
        }
   } catch {
        write-log -message "Error Removing Content Type from List! $($_.Exception.Message)" -consoleForeground Red
        if ($($_.Exception.ErrorRecord)) { write-log -message "$($_.Exception.ErrorRecord)" -consoleForeground Red }
        if ($($_.Exception.stacktrace)) { write-log -message "$($_.Exception.stacktrace)" -consoleForeground Red }
        if ($($_.ScriptStackTrace)) { write-log -message "$($_.ScriptStackTrace)" -consoleForeground Red }
   }
}

function Copy-SpList($spWeb, $calName,$OldCalName, $calTitle) {  
    <#

    .SYNOPSIS
        Copies a calendar template to create a new calendar.

    .DESCRIPTION
        Checks to see if a calendar with calTitle exists and deletes it. Creates a new calendar base on the Calendar List Template. Adds the same columns from the old calendar 
        to the new calendar. Builds the views for display and editing. Finally, migrates the data from the old calendar list to the new calendar list.

    .PARAMETER spWeb
        [Microsoft.SharePoint.SPWeb] The SPWeb of the site collection or facility subsite.

    .PARAMETER calName
        [String] The new name of the calendar list.

    .PARAMETER OldCalName
        [String] The name of the backup list to copy data from. Probably created using Rename-List.

    .PARAMETER calTitle
        [String] Title of the new list.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Clone-Calendar -spWeb $subWeb -calName "RDCCalendar" -OldCalName "RDC Calendar_bak" -calTitle "RDC Calendar"

    #>

    $SPTemplate = (Get-SiteWeb).ListTemplates["Calendar"]
    
    $newCal = $spWeb.Lists.TryGetList($calTitle)
    #remove if the clone list already exist
    if($newCal -ne $null) {    
        $spWeb.Lists.Delete($newCal.ID)
        $spWeb.Update()
    }
    #create a new clone list
    $spWeb.Lists.Add($calTitle,$calTitle,$SPTemplate)
    $spWeb.Update()
    
    #get old list (after rename)    
    $oldList = $spWeb.Lists.TryGetList($OldCalName)

    #get new list 
    $newList = $spWeb.Lists.TryGetList($calTitle)
               
    if($oldList -ne $null) {
        
        foreach($oldFld in $oldList.Fields) {            
            if(($newList.Fields.ContainsFieldWithStaticName($oldFld.StaticName) -eq $False) -and 
                   ($newList.Fields.ContainsField($oldFld.Title) -eq $False))
            {          
                $newList.Fields.Add($oldFld);
                $newList.Update()
                write-log -message "$oldFld has been added to $newList" -ConsoleForeground Green
            }
            else {
                write-log -message "$oldFld already exist in $newList" -ConsoleForeground Yellow
            }
        }
        
        $spWeb.Update()
        #set default view
        $newView = $newList.Views | ? { $_.Title -eq 'All Events' };

        <#add old view
        $oldView = $oldList.Views | ? { $_.Title -eq 'All Events' };
        #set default view
        $newView = $newList.Views | ? { $_.Title -eq 'All Events' };
        #clone view fields
        foreach($oldViewFld in $oldView.ViewFields){
            if($newView.ViewFields.exists($oldViewFld)) {
                write-log -message "$oldViewFld already exist in $newView" -ConsoleForeground Yellow
            }
            else {
                $newView.ViewFields.Add($oldViewFld)
                write-log -message "$oldViewFld has been added to $newView" -ConsoleForeground Green
            }
        }
        #>
        $contentType = "Event"
        $newFieldOrder = @("Title", "EventDate", "EndDate", "Location", "fAllDayEvent", "fRecurrence","EventAttendees", "EventCommitteeAttendees",  "ParticipantsPicker","Administrative_x0020_Items", "TaxKeyword", "ApprovedApplicationLookup", "IACUCMeetingMinuteLookup", "RDCMeetingMinuteLookup", "Decision", 
        "Deliberation"
        )
        $ct = $newList.ContentTypes[$contentType]       
        $ct.FieldLinks.Reorder($newFieldOrder)
        $ct.Update() 
        $newList.Update()
        #set as default view
        $newView.DefaultView = $true
        $newView.Update()
        #migrate data from old list to new list
        $newItem = $null
        foreach($oldItem in $oldList.Items) {
            $newItem = $newList.AddItem()
            foreach($fld in $oldItem.Fields) {
              if(($newList.Fields.ContainsField($fld.Title) -eq $True)) {  
                if("ID","WF","Link","_","Agenda","Attachments","Facilities" -notcontains $fld.StaticName){
                    if($fld.ReadOnlyField -eq $False) {
                        $newItem[$fld.StaticName] = $oldItem[$fld.StaticName]
                        write-log -message "Item: $($newItem[$fld.StaticName]) has been added to the $newList" -ConsoleForeground Cyan            
                    }              
                 } 
               } 
             }
            $newItem.Update()            
        }
        $newList.Update()
    }
}

function Remove-ColumnFromList ($WebURL, $ListName, $ColumnName) {
    <#

    .SYNOPSIS
        Removes a column from a given list.

    .DESCRIPTION
        Gets the web and list, then loops through to find the field to remove. When found, reset the attributes to allow deletion and then delete the field.

    .PARAMETER WebURL
        [String] The site URL.

    .PARAMETER ListName
        [String] The names of the list.

    .PARAMETER ColumnName
        [String] The names of the column to remove .

    .EXAMPLE
        Remove-ColumnFromList -WebURL (Get-SiteURL) -ListName "Facility" -ColumnName $_.Title

    #>

    #Get Internal Name of the columns
    $web = Get-SPWeb $WebURL
 
    #Get the list
    $list = $web.Lists.TryGetList($ListName)
 
    if($List -ne $null) {
        #Get the column
        $list.Fields | ? {$_.Title -match $ColumnName} | % {
            #Reset column properties to allow delete
            #$_.Hidden = $False
            $_.ReadOnlyField = $False
            $_.AllowDeletion = $True
            $_.Update()
            #Delete the column from list
            $list.Fields.Delete($_)
            write-log -message ("Column " + $_.Title + " has been deleted!") -f Magenta
        }
    } else {
        write-log -message "Specified List is not found!" -consoleForeground Yellow
    }
}

function Set-SiteColumnLookup ($web, $colName,$colStaticName, $lookupListName,$lookupColumnName) {
    <#

    .SYNOPSIS
        Creates a lookup column on a list.

    .DESCRIPTION
        Creates a lookup column from the lookupListName using the Id and lookupColumnTitle.

    .PARAMETER colName
        [String] The display name of the new Lookup column.

    .PARAMETER colStaticName
        [String] The static name of the new Lookup column. Typically the same as the display name without spaces. Will also be used as the internal name.

    .PARAMETER lookupListName
        [String] The list that is the source of the data to be looked up.

    .PARAMETER lookupColumnName
        [String] The column to be displayed in the lookup column dropdown list.

    .EXAMPLE
        Set-SiteColumnLookup -web $web -colName "Facilities Lookup" -colStaticName "FacilitiesLookup" -lookupListName "Facilities" -lookupColumnName "Title" 

    #>

    #$web = Get-SiteWeb
    $lookupList = $web.Lists[$lookupListName]  
    $web.Fields.AddLookup($colStaticName, $lookupList.ID,$false);
    $field = $web.Fields[$colStaticName]
    $field.LookupField = $lookupColumnName
    $field.Title = $colName
    $field.Update()
}

function Set-FacilityListPermissionAndVersioning ($listTitle,[Microsoft.SharePoint.SPMember]$securityMember,[Microsoft.SharePoint.SPRights]$permissionMask, [switch] $enableVersioning) {
    <#

    .SYNOPSIS
        For a given list set the permissions and turn versioning on

    .DESCRIPTION
        Loop through all webs and find the given list. When found set the permissions and turn on versioning

    .PARAMETER listTitle
        [String] The name of the list.

    .PARAMETER securityMember
        [[Microsoft.SharePoint.SPMember] The security group that will be granted the permission

    .PARAMETER permissionMask
        [Microsoft.SharePoint.SPRights] The rights and permissions to be granted.

    .PARAMETER enableVersioning
        [switch] The column to be displayed in the lookup column dropdown list.

    .NOTES
        Not used in RAMS-2.3.0

    .EXAMPLE
        Set-FacilityListPermissionAndVersioning -listTitle "Facilities" -securityMember "Miami Local Site Administrator" -permissionMask "Contribute" -enableVersioning 

    #>

    (Get-SiteWeb).webs | % {
        $listPerm = $_.Lists[$listTitle]
        $listPerm.Permissions.Add($securityMember,$permissionMask)
        #versioning
        if($enableVersioning) {
            $listPerm.EnableVersioning = $True            
        }
        $listPerm.Update()
    }
}

function Import-RAMSUsers ($inputFilePath) {
    <#

    .SYNOPSIS
        Add and remove users from the RAMS security groups

    .DESCRIPTION
        Using a csv file with 3 columns, Security Group, Person, and Action; update the specified security group. This can be used to remove all members 
        from a security group and then add new users using their full name from the Active Directory.

    .PARAMETER inputFilePath
        [String] The filename with the path of the csv file.

    .NOTES
        The file must have the 3 columns Security Group, Person, and Action.
        Security Group is the RAMS security Group Name, e.g. New Orleans Administrative Officers.
        Person is the person's name to remove or add to the group. it must be exactly like SharePoint is expecting it, including all organizational suffixes, e.g. LName, FName (CTR-ORG) or LName, FName (OIFO)
        The action column should only have the values Add or Remove. Add puts the person in the security group. Remove takes the person out of the security group.
        See GetSecurityGroupUsers.ps1 in ps\misc to generate a list of the current users. This can then be edited to produce the security groups needed.

    .EXAMPLE
        try {
            Add-PSSnapin Microsoft.SharePoint.PowerShell -ErrorAction SilentlyContinue
        } catch {
            Write-Host "PSSnapin Microsoft.SharePoint.PowerShell already loaded" -ForegroundColor Yellow
        }

        if ((get-module | Where-Object({$_.Name -eq 'RAMS_Module_Util'}) ) -eq $null) {
            Import-Module D:\RAMS-2.3.0\ps\module\RAMS_Module_Util.psm1 -Verbose
        } else {
            Remove-Module RAMS_Module_Util -Verbose
            Import-Module D:\RAMS-2.3.0\ps\module\RAMS_Module_Util.psm1 -Verbose
        }
        $inputFilePath = "D:\RAMS-2.3.0\ps\misc\RAMS-itstUsers-remove.csv"
        $inputFileName = Split-Path $inputFilePath -leaf
        $Global:diretoryName = $PSScriptRoot
        $Global:logFileName = "$($inputFileName).log"
        Import-RAMSUsers -inputFilePath $inputFilePath

    #>

    $weburl = Get-SiteURL
    $webSite = Get-SiteWeb
    $userList = import-csv -Path $inputFilePath -Delimiter "," | Select-Object -Property "Security Group", Person, Action

    $userList | % {    
        $group = $_."Security Group"
        $person = $_.Person
        $userGroup = $webSite.SiteGroups | ? {$_.Name -eq ($group)}
        $matchUser = $webSite.SiteUsers | ? {$_.DisplayName -eq $person}
        $action = $_.Action
        if($matchUser) {
            $matchUser | % {
                if ($action -eq "Add") {
                    $userGroup.AddUser($_)
                    write-log -message "$($matchUser.DisplayName) has been added to Security Group: $($userGroup.Name)" -consoleForeground Cyan
                } elseif ($action -eq "Remove") {
                    $userGroup.RemoveUser($_)
                    write-log -message "$($matchUser.DisplayName) has been removed to Security Group: $($userGroup.Name)" -consoleForeground Yellow
                } else {
                    write-log -message "$($action) is not a known action 'Add or Remove' for user Group: $($userGroup.Name) and user $($matchUser.DisplayName)" -consoleForeground Red
                }
            }
        }
    }
}

function New-MeetingMinuteLookup ([Parameter (Mandatory=$True)][Microsoft.SharePoint.SPWeb]$web, [Parameter (Mandatory=$True)][string]$committee) {
    <#
    
    .SYNOPSIS
        Creates a meeting minute lookup column on for the calendar list.
    
    .DESCRIPTION
        Creates a lookup column for the committee acronym + MeetingMinuteLookup, i.e. SC1MeetingMinuteLookup. First checks to see if the lookup already exists, if not, it is created
        using the committee acronym + Documents document list as the lookup target. once created, the column is added to the calendar list.
    
    .PARAMETER web
        [Microsoft.SharePoint.SPWeb] The SPWeb of the site collection or facility subsite.
    
    .PARAMETER committee
        [String] The acronym for the committee
    
    .EXAMPLE
        New-MeetingMinuteLookup -web $facilitySite -committee $newCommiteeName 
    
    #>

    # GET THE SITE
    #$web = Get-SPWeb $siteURL    

    #CREATE SITE COLUMNS
    write-log -message "CREATING $committee Meeting Minute Lookup SITE COLUMNS for site $($web.Url)" -consoleForeground Cyan
    
    if ($committee -ne "IRB") {
        $calendarName=$committee + " Calendar"
    } else {
        $calendarName= "Committee Calendar"
    }
    # GET THE LISTS
    $myList = $web.Lists[$calendarName];

    #Where the Source List for Lookup Exists
    
    $ParentWebURL=$web.Url
    
    if ($committee -ne "IRB") {
        $ParentListName=$committee + " Documents"
    } else {
        $ParentListName="Documents"
    }

    $ParentLookupColumnName="Title"
    
    #Where the Lookup Site column Going to get created
    $ChildWebURL=$web.Url
    
    if ($committee -ne "IRB") {
        $ChildLookupColumnName= $committee + "MeetingMinuteLookup"
    } else {
       $ChildLookupColumnName= "CommitteeMeetingMinuteLookup"
    }

    #Get the Parent and Child Webs and ListQ
    $ParentWeb = Get-SPWeb $ParentWebURL
    $ParentList = $ParentWeb.Lists[$ParentListName]
    $ChildWeb = Get-SPWeb $ChildWebURL
    
    write-log -message "Creating Lookup $ChildLookupColumnName for $calendarName $($ChildWeb.Url)" -consoleForeground Gray
    write-log -message "   from $ParentLookupColumnName on $ParentListName on $($ParentWeb.Url)" -consoleForeground Gray
    if (!$ChildWeb.Fields.ContainsField($ChildLookupColumnName)) {
       #Add Lookup Field
       $ChildLookupColumn = $ChildWeb.Fields.AddLookup($ChildLookupColumnName,$ParentList.id,$False)
       $ChildLookupColumn = $ChildWeb.Fields[$ChildLookupColumnName]
    
       #Setup lookup Field property
       $ChildLookupColumn.LookupWebId = $ParentWeb.ID
       $ChildLookupColumn.LookupField = $ParentList.Fields.GetFieldByInternalName($ParentLookupColumnName)

       $ChildLookupColumn.AllowMultipleValues=$true
       $ChildLookupColumn.Group="RAMS IRB" 
       if ($committee -ne "IRB") {
           $ChildLookupColumn.Title=$committee + " Meeting Minute Lookup"
       }
       else {
            $ChildLookupColumn.Title="Committee Meeting Minute Lookup"
       }

       $ChildLookupColumn.update() 

       write-log -message "$($ChildLookupColumn.Title) Lookup field added successfully to the web $($web.url)" -consoleForeground green
       if ($myList.Fields.ContainsFieldWithStaticName($ChildLookupColumnName) -eq $False) {    
          $myList.Fields.Add($ChildLookupColumn)
          $myList.Update()
          write-log -message "$($ChildLookupColumn.Title) Lookup field added successfully to the Calendar List!" -consoleForeground green
       }
    } else {
       $ChildLookupColumn = $ChildWeb.Fields.TryGetFieldByStaticName($ChildLookupColumnName)
       write-log -message "$($ChildLookupColumn.Title) Lookup Field Exists already on the web $($web.url)" -consoleForeground yellow
       #Add column to the Calendar List, if missing
       if ($myList.Fields.ContainsFieldWithStaticName($ChildLookupColumnName) -eq $False) {    
          $myList.Fields.Add($ChildLookupColumn)
          $myList.Update()
          write-log -message "$($ChildLookupColumn.Title) Lookup field added successfully to the Calendar List!" -consoleForeground green
       }
    } 
}

function Update-CalendarView([Parameter (Mandatory=$True)][Microsoft.SharePoint.SPWeb]$web, [Parameter (Mandatory=$True)][string]$committee) {
    <#

    .SYNOPSIS
        Updates the All Events view for the calendar.

    .DESCRIPTION
        Gets the All Events View and adds the standard calendar fields to the view. 

    .PARAMETER web
        [Microsoft.SharePoint.SPWeb] The SPWeb of the site collection or facility subsite.

    .PARAMETER committee
        [String] The acronym for the committee

    .EXAMPLE
        Update-CalendarView -web $facilitySite -committee $newCommiteeName

    #>
 
    $list = $web.Lists.TryGetList("$committee Calendar")
    if ($list -ne $null) {
       #UPDATE THE VIEW
       write-log -message "UPDATING THE $committee Calendar All Events VIEW " -consoleForeground Cyan
       $view = $list.Views | ? { $_.Title -eq 'All Events' };
    
       if ($view.ViewFields.exists("Location")){
          $view.ViewFields.Delete("Location");
       }

     
       if ( $view.ViewFields.exists("EventDate")){
          $view.ViewFields.Delete("EventDate");
       }

       if ($view.ViewFields.exists("EndDate")){
          $view.ViewFields.Delete("EndDate");
       }
       
       if ($view.ViewFields.exists("fAllDayEvent")){
          $view.ViewFields.Delete("fAllDayEvent");
       }

       #Create customized view
       if (!$view.ViewFields.exists("EventDate")){
          $view.ViewFields.Add("EventDate"); 
       }
       
       if (!$view.ViewFields.exists("EndDate")){
          $view.ViewFields.Add("EndDate"); 
       }
       
       if (!$view.ViewFields.exists("Location")){
          $view.ViewFields.Add("Location"); 
       }
    
       if (!$view.ViewFields.exists("EventAttendees")){
          $view.ViewFields.Add("EventAttendees"); 
       }
       
       if (!$view.ViewFields.exists("Committee Attendees")){
          $view.ViewFields.Add("Committee Attendees"); 
       }
  
       if (!$view.ViewFields.exists("EventAdministrativeItems")){
          $view.ViewFields.Add("EventAdministrativeItems"); 
       }
  
       $view.DefaultView = $true
       $view.Update()

       if ($list -ne $null) {
     
          $contentType = "Event"
          $newFieldOrder = @("Title", 
            "EventDate", 
            "EndDate", 
            "Location", 
            "fAllDayEvent", 
            "fRecurrence",
            "EventCommitteeAttendees",  
            "EventAttendees", 
            "ParticipantsPicker",
            "EventAdministrativeItems", 
            "TaxKeyword", 
            "SubmittedApplicationLookup", 
            "$($committee)MeetingMinuteLookup", 
            "Deliberation",
            "Decision"
          )
          if ($list.Fields.ContainsField("Description")) {
             $ct = $list.ContentTypes[$contentType] 
             $ct.FieldLinks["Description"].Hidden = $True
             $ct.FieldLinks.Reorder($newFieldOrder)
             $ct.Update() 
             $list.Update()
          }

          if ($list.Fields.ContainsField("Category")) {
             $categoryfield=$list.Fields["Category"]
             $categoryfield.ShowInNewForm=$false 
             $categoryfield.ShowInEditForm=$false
             $categoryfield.Update($true)
             $list.Update()
          }
          
          if ($list.Fields.ContainsField("ApplicationsLookup")) {
             $applicationsField=$list.Fields["ApplicationsLookup"]
             $applicationsField.ShowInNewForm=$false 
             $applicationsField.ShowInEditForm=$false
             $applicationsField.Update($true)
             $list.Update()
          }
          
          if ($list.Fields.ContainsField("RDCMinuteLookup")) {
             $approvedRDCApplicationLookup=$list.Fields["RDCMinuteLookup"]
             $approvedRDCApplicationLookup.ShowInNewForm=$false 
             $approvedRDCApplicationLookup.ShowInEditForm=$false
             $approvedRDCApplicationLookup.Update($true)
             $list.Update()
          }

          if ($list.Fields.ContainsField("Approved Application Lookup")) {
             $applicationsField=$list.Fields["Approved Application Lookup"]
             $applicationsField.ShowInNewForm=$false 
             $applicationsField.ShowInEditForm=$false
             $applicationsField.Update($true)
             $list.Update()
          }

          if ($list.Fields.ContainsField("Attendees")) {
             $applicationsField=$list.Fields["Attendees"]
             $applicationsField.ShowInNewForm=$false 
             $applicationsField.ShowInEditForm=$false
             $applicationsField.Update($true)
             $list.Update()
          }

          if ($list.Fields.ContainsField("Administrative Items")) {
             $applicationsField=$list.Fields["Administrative Items"]
             $applicationsField.ShowInNewForm=$true 
             $applicationsField.ShowInEditForm=$true
             $applicationsField.Update($true)
             $list.Update()
          }
       }
    }
}

function New-SPList ([Parameter (Mandatory=$True)][Microsoft.SharePoint.SPWeb]$web, [Parameter (Mandatory=$True)][string]$newCommiteeName,[Parameter (Mandatory=$True)][ValidateSet("Documents", "Calendar", "Form Library")][string]$templateName ) {
    <#
    
    .SYNOPSIS
        Creates a new SharePoint List.
    
    .DESCRIPTION
        Create a new sharepoint list of type documents, calendar, or Form Library. First checks to see if the list is 
        present then names it by the committee name + the template name. finally it turns on versioning.
    
    .PARAMETER web
        [Microsoft.SharePoint.SPWeb] The SPWeb of the site collection or facility subsite.
    
    .PARAMETER newCommiteeName
        [String] The acronym for the committee
    
    .PARAMETER templateName
        [String] The type of list to create. Can only be of type "Documents", "Calendar", or "Form Library".
    
    .EXAMPLE
        $newDoc = New-SPList -web $facilitySite -newCommiteeName $newCommiteeName -template Documents -Verbose
    
    .EXAMPLE
        $newCalendar = New-SPList -web $facilitySite -newCommiteeName $newCommiteeName -template Calendar -VerboseUpdate-CalendarView -web $facilitySite -committee $newCommiteeName
    
    #>
     
    $website = (Get-SiteWeb)
    
    if($templateName -eq "Documents") {
        $template = $website.ListTemplates["Document Library"]
    } else {
        $template = $website.ListTemplates["$templateName"]
    } 
    
    $facilitySite = $web
    
    #add new list
    if(($facilitySite.Lists.TryGetList("$($newCommiteeName)$templateName")) -eq $Null) {
        $facilitySite.Lists.Add("$($newCommiteeName)$templateName","$($newCommiteeName) $templateName",$template)
    }
    $newSPList = $facilitySite.Lists.TryGetList("$($newCommiteeName)$templateName")
              
    #change title
    $newSPList.Title="$($newCommiteeName) $templateName";
        
    #enable versioning
    $newSPList.EnableVersioning = $True;
        
    #update calendar
    $newSPList.Update() 
    write-log -message "$($newSPList.Title) added to site $($web.Url)" -consoleForeground Green 
    #return
    return $newSPList
}

function New-SSISDB ($dbServer,$ProjectName, $ProjectFilePath) {
    <#
    
    .SYNOPSIS
        Install and update the SSIS Database with the release components.
    
    .DESCRIPTION
        Builds a script and executes on the SSISDB Server. The script connects to the SSIS instance, creates the catalog, 
        creates a project folder, and then deploys the project ispac file. 
    
    .PARAMETER dbServer
        [String] The name of the SSIS server.
    
    .PARAMETER ProjectName
        [String] Name of the project without extensions. An ispac file will have 3 parts, the project name + _ + Environment + 
        .ispac, e.g EmailNotification_TEST.ispac or EmailNotification_PROD.ispac. This parameter is only the first part of the 
        file name. 
    
    .PARAMETER ProjectFilePath
        [String] The location of the ispac file. An ispac file will have 3 parts, the project name + _ + Environment + 
        .ispac, e.g EmailNotification_TEST.ispac or EmailNotification_PROD.ispac. This parameter is the full name of the file with 
        the full path on the destination server.
    
    .EXAMPLE
        $Global:fileLocationISAC = "D:\$RAMSVersion\sql"
        Copy-Item "$Global:fileLocationISAC\EmailNotification_$($envName).ispac" -Destination "\\$($Global:hostInfo.ssis)\D$\" -Verbose
        New-SSISDB -dbServer ($Global:hostInfo.ssis) -ProjectName "EmailNotification" -ProjectFilePath "D:\EmailNotification_$($envName).ispac"
    
    #>
    $sbSSISServer = $Global:hostInfo.ssis
    $sbDBServer = $Global:hostInfo.sql
    $sbContentDBName = Get-SPContentDatabase -Site (Get-SiteURL)
    $sbRAMSDBName = "RAMS"
    $scriptBlock = {
        $sbSSISServer = $args[0]
        $sbDBServer = $args[1]
        $sbContentDBName = $args[2]
        $sbRAMSDBName = $args[3]

        function Create-SSISAgentJob { 
            <#
            
            .SYNOPSIS
                Create SSIS agent jobs
            
            .DESCRIPTION
                
            
            .PARAMETER webServiceUrl
                [string] The SSIS web services URL.
            
            .PARAMETER reportFolder
                [string] The name of the folder on the SSRS server where the reports will be imported. "RAMS Reports" is the 
                default. 
            
            .PARAMETER AgentJobName
                [string] The name of the folder on the SSRS server where the data sources will be imported. "Data Sources" is the 
                default. 
            
            .PARAMETER dtsxPackage
                [string] The base directory of the reports directory in the deployment package. Built using the RAMS release 
                number, e.g. RAMS-2.3.0, and adding the reports directory. For example, D:\RAMS-2.3.0\Reports.
            
            .NOTES
                Modified from http://microsoft-ssis.blogspot.com/2016/10/using-powershell-to-create-sql-agent.html
            
            .EXAMPLE
                Create-SSISAgentJob -ssisServer $sbSSISServer -dbServer $sbDBServer -reportFolder "RAMS" -dtsxPackage "AnnualReviewNotification.dtsx" -projectName "EmailNotification" -contentDBName $sbContentDBName -ramsDBName $sbRAMSDBName -jobStartTime (New-Timespan -hours 23 -minutes 00)
        
            .EXAMPLE
                Create-SSISAgentJob -ssisServer $sbSSISServer -dbServer $sbDBServer -reportFolder "RAMS" -dtsxPackage "TrainingExpirationNotification.dtsx" -projectName "EmailNotification" -contentDBName $sbContentDBName -ramsDBName $sbRAMSDBName -jobStartTime (New-Timespan -hours 23 -minutes 30)
            #>
            Param(
                [string]$ssisServer,
                [string]$dbServer,
                [string]$reportFolder = "RAMS",
                [string]$dtsxPackage,
                [string]$jobName,
                [string]$projectName,
                [string]$contentDBName,
                [string]$ramsDBName,
                [Timespan]$jobStartTime
            )
            #PowerShell SSIS JobStep
            ################################
            ########## PARAMETERS ##########
            ################################ 
            # Destination
            #$SsisServer = $webServiceUrl
            #$reportFolder = $reportFolder
            #$ProjectName = $ProjectName
             
            # Job
            #$JobName = $agentJobName
            #$dtsxPackage = $dtsxPackage
            #$JobStartTime = New-TimeSpan -hours 23 -minutes 00
             
            #clear
            write-host "==========================================================" -ForegroundColor white
            write-host "==  Used parameters                                     ==" -ForegroundColor white
            write-host "==========================================================" -ForegroundColor white
            write-host "SSIS Server            : $SsisServer"  -ForegroundColor white
            write-host "DB Server              : $dbServer" -ForegroundColor white
            write-host "reportFolder           : $reportFolder" -ForegroundColor white
            write-host "ProjectName            : $projectName" -ForegroundColor white
            write-host "Job name               : $jobName" -ForegroundColor white
            write-host "dtsxPackage            : $dtsxPackage" -ForegroundColor white
            write-host "contentDBName          : $contentDBName" -ForegroundColor white
            write-host "ramsDBName             : $ramsDBName" -ForegroundColor white
            write-host "ScheduleTime           : $JobStartTime" -ForegroundColor white
            write-host "==========================================================" -ForegroundColor white
            write-host " " -ForegroundColor white
             
             
            # Reference SMO assembly and connect to the SQL Sever Instance 
            # Check the number in the path which is different for each version
            Add-Type -Path 'C:\Program Files (x86)\Microsoft SQL Server\100\SDK\Assemblies\Microsoft.SqlServer.Smo.dll'
            $SQLSvr = New-Object -TypeName  Microsoft.SQLServer.Management.Smo.Server($SsisServer) 
             
            # Check if job already exists. Then fail, rename or drop
            $SQLJob = $SQLSvr.JobServer.Jobs[$JobName]
            if ($SQLJob){
              # Use one of these 3 options to handle existing jobs
             
              # Fail:
              #Throw [System.Exception] "Job with name '$JobName' already exists."
             
              # Rename:
              write-host  "Job with name '$JobName' found, renaming and disabling it" -ForegroundColor white
              $SQLJob.Rename($SQLJob.Name +"_OLD_" + (Get-Date -f MM-dd-yyyy_HH_mm_ss))
              $SQLJob.IsEnabled = $false
              $SQLJob.Alter()
             
              # Drop:
              #write-host  "Job with name $JobName found, removing it"
              #$SQLJob.Drop()
            }
             
             
            #Create new (empty) job 
            $SQLJob = New-Object -TypeName Microsoft.SqlServer.Management.SMO.Agent.Job -argumentlist $SQLSvr.JobServer, $JobName
            $SQLJob.OwnerLoginName = "SA"
            $SQLJob.Create() 
            write-host  "Job '$JobName' created" -ForegroundColor Green
             
             
            # Command of jobstep
            # This string is copied from T-SQL, by scripting a job(step) in SSMS
            # Then replace the hardcode strings with [NAME] to replace them with variables
            $Command = @'
            /ISSERVER "\"\SSISDB\[FOLDER]\[PROJECT]\[PACKAGE]\"" /SERVER [SSISSERVER] /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=[CNTDB];Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
'@
            
            #                    FOLDER     PROJECT   PACKAGE              SSIS SERVER                                                                DB SERVER                  SP CNT DB                                                                                                                               DB SERVER                 RAMS DB                                                                                                                                                                                                            
            #                    |-------| |-------| |-------|            |----------|                                                               |--------|                 |-----|                                                                                                                                  |--------|                 |--|                                                                                                                                                                                                               
            #/ISSERVER "\"\SSISDB\[FOLDER]\[PROJECT]\[PACKAGE]\"" /SERVER [SSISSERVER] /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=[CNTDB];Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
            #/ISSERVER "\"\SSISDB\[FOLDER]\[PROJECT]\[PACKAGE]\"" /SERVER [SSISSERVER] /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=[CNTDB];Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=[DBSERVER];Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
            #/ISSERVER "\"\SSISDB\RAMS\EmailNotification\TrainingExpirationNotification.dtsx\"" /SERVER DNS  SQLRAM906 /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=DNS;Initial Catalog=WSS_Content;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=DNS;Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
            #/ISSERVER "\"\SSISDB\RAMS\EmailNotification\AnnualReviewNotification.dtsx\"" /SERVER DNS  SQLRAM906 /Par "\"$Project::ConfigDB_ConnectionString\"";"\"Data Source=DNS;Initial Catalog=SP_RAMS_WSS_Content_IRB_Application_DB_2_1;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$Project::RAMS_ConnectionString\"";"\"Data Source=DNS;Initial Catalog=RAMS;Provider=SQLNCLI11.1;Integrated Security=SSPI;Auto Translate=False;\"" /Par "\"$ServerOption::LOGGING_LEVEL(Int16)\"";1 /Par "\"$ServerOption::SYNCHRONIZED(Boolean)\"";True /CALLERINFO SQLAGENT /REPORTING E
            $Command = $Command.Replace("[FOLDER]", $reportFolder)
            $Command = $Command.Replace("[PROJECT]", $ProjectName)
            $Command = $Command.Replace("[PACKAGE]", $dtsxPackage)
            $Command = $Command.Replace("[SSISSERVER]", $SsisServer)
            $Command = $Command.Replace("[DBSERVER]", $dbServer)
            $Command = $Command.Replace("[CNTDB]", $contentDBName)
            $Command = $Command.Replace("[RAMSDB]", $ramsDBName)
            $Command = $Command.Replace("[DBSERVER]", $SsisServer)
             
             
            # Create new SSIS job step with command from previous block 
            $SQLJobStep = New-Object -TypeName Microsoft.SqlServer.Management.SMO.Agent.JobStep -argumentlist $SQLJob, "$ProjectName - $dtsxPackage"
            $SQLJobStep.OnSuccessAction = [Microsoft.SqlServer.Management.Smo.Agent.StepCompletionAction]::QuitWithSuccess
            $SQLJobStep.OnFailAction = [Microsoft.SqlServer.Management.Smo.Agent.StepCompletionAction]::QuitWithFailure
            $SQLJobStep.SubSystem = "SSIS"
            $SQLJobStep.DatabaseName = $SsisServer
            $SQLJobStep.Command = $Command
            $SQLJobStep.Create() 
            write-host  "Jobstep $SQLJobStep created" -ForegroundColor green
             
             
            # Create a daily schedule
            $SQLJobSchedule = New-Object -TypeName Microsoft.SqlServer.Management.Smo.Agent.JobSchedule -ArgumentList $SQLJob, "Daily $JobStartTime"
            $SQLJobSchedule.IsEnabled = $true
            $SQLJobSchedule.FrequencyTypes = [Microsoft.SqlServer.Management.SMO.Agent.FrequencyTypes]::Daily
            $SQLJobSchedule.FrequencyInterval = 1 # Recurs Every Day
            $SQLJobSchedule.ActiveStartDate = Get-Date
            $SQLJobSchedule.ActiveStartTimeofDay = $JobStartTime
            $SQLJobSchedule.Create()
            write-host  "Jobschedule $SQLJobSchedule created" -ForegroundColor green
             
             
            # Apply to target server which can only be done after the job is created
            $SQLJob.ApplyToTargetServer("(local)")
            $SQLJob.Alter()
            write-host  "Job '$JobName' saved" -ForegroundColor green
        }
        
        
        # Load the IntegrationServices Assembly  
        [Reflection.Assembly]::LoadWithPartialName("Microsoft.SqlServer.Management.IntegrationServices")  
        # Store the IntegrationServices Assembly namespace to avoid typing it every time  
        $ISNamespace = "Microsoft.SqlServer.Management.IntegrationServices"  

        write-host  "Connecting to server ..."  -ForegroundColor white 

        # Create a connection to the server  
        $sqlConnectionString = "Data Source=$Using:dbServer;Initial Catalog=master;Integrated Security=SSPI;"  
        $sqlConnection = New-Object System.Data.SqlClient.SqlConnection $sqlConnectionString  

        # Create the Integration Services object  
        $integrationServices = New-Object $ISNamespace".IntegrationServices" $sqlConnection  

        #enable CLR
        $query = "exec sp_configure 'clr enabled', 1 
            reconfigure
            go
        "
        Invoke-Sqlcmd -ServerInstance $Using:dbServer -Query $query -Verbose 

        if ($integrationServices.Catalogs.Count -gt 0) { 
            $catalog = $integrationServices.Catalogs["SSISDB"]
            $folder = $catalog.Folders["RAMS"]
            if(!$folder) {
                $folder = New-Object $ISNamespace".CatalogFolder" ($catalog, "RAMS","RAMS SSIS Folder")
                $folder.Create()
            }
        } else {
            # Provision a new SSIS Catalog  
            $catalog = New-Object $ISNamespace".Catalog" ($integrationServices, "SSISDB", "'7G>n3Rr=4wV")  
            $catalog.Create() 
            
            # Create the target folder
            $folder = New-Object $ISNamespace".CatalogFolder" ($catalog, "RAMS","RAMS SSIS Folder")
            $folder.Create()
        }
        
        # Read the project file and deploy it
        [byte[]] $projectFile = [System.IO.File]::ReadAllBytes($Using:ProjectFilePath)
        $folder.DeployProject($Using:ProjectName, $projectFile)

        # Create the jobs
        
        
        Create-SSISAgentJob -ssisServer $sbSSISServer -dbServer $sbDBServer -reportFolder "RAMS" -dtsxPackage "AnnualReviewNotification.dtsx" -jobName "Annual Review Notification" -projectName "EmailNotification" -contentDBName $sbContentDBName -ramsDBName $sbRAMSDBName -jobStartTime (New-Timespan -hours 23 -minutes 00)
        Create-SSISAgentJob -ssisServer $sbSSISServer -dbServer $sbDBServer -reportFolder "RAMS" -dtsxPackage "TrainingExpirationNotification.dtsx" -jobName "Training Expiration Notification" -projectName "EmailNotification" -contentDBName $sbContentDBName -ramsDBName $sbRAMSDBName -jobStartTime (New-Timespan -hours 23 -minutes 30)
        write-host  "Done." -ForegroundColor Green
    }
    
    write-log -message "SSIS Install Results Start: NOTE -  Not all of the information between here and the SSIS Install Results End is written to the log." -consoleForeground Magenta
    $job = Invoke-Command -ComputerName $dbServer -ScriptBlock $scriptBlock -ArgumentList $sbSSISServer, $sbDBServer, $sbContentDBName.Name, $sbRAMSDBName -AsJob -Verbose
    Wait-job $job
    $results = Receive-Job $job
    $results | select PSComputerName, CreatedTime, ObjectName, Status, Parent, Completed  | Write-Log
    write-log -message "SSIS Install Results End: NOTE -  Not all of the information between here and the SSIS Install Results Start is written to the log." -consoleForeground Magenta
}

function Install-SSISAdapter ($dbServer) {
    <#

    .SYNOPSIS
        Install the SharePoint List Adapter on the SSIS Server.

    .DESCRIPTION
        Remotely install the SharePoint List Adapter on the SSIS Server. This script assumes success.

    .PARAMETER dbServer
        [String] The name of the SSIS server.

    .EXAMPLE
        Install-SSISAdapter -dbServer ($Global:hostInfo.ssis)

    #>
     
    Copy-Item .\SharePointListAdapterSetup_x64.msi -Destination \\$dbServer\D$\ -Verbose
    $SPAdapterInstall={
        Start-Process msiexec.exe -Wait -ArgumentList '/I D:\SharePointListAdapterSetup_x64.msi /quiet' -PassThru
    }
    Invoke-Command -ComputerName $dbServer -ScriptBlock $SPAdapterInstall -Verbose | write-log
    Start-Sleep -Seconds 60
    write-log -message "SharePoint SSIS Adapter is now installed on $dbServer" -consoleForeground Green
}

function Run-Script ([string] $step, [string]$scriptName) {
    <#
    
    .SYNOPSIS
        Executes the ScriptName
    
    .DESCRIPTION
        Calculates the percent complete to show progress. Tells the users what is happening, checks to 
        see if the requested script name is present, and if so, executes it.
    
    .PARAMETER step
        [String] The text to display the user specified step number. Suggest numbering by 10s so steps 
        can easily be inserted without having to renumber too often.
    
    .PARAMETER scriptName
        [String] The filename of the powershell script to execute. This will use the fileLocationPS to 
        look for the script.  
    
    .NOTES
        TODO: Need to figure out how to pass in additional parameters and pass them thru to the 
        designated script. For now, this function cannot run scripts that require additional parameters.
    
    .EXAMPLE
        Run-Script 010 UpdateEnvironmentList.ps1
        
    #>
    
    $Global:perComplete = $Global:perComplete + $Global:count
    Write-Progress -Activity "RAMS 2.3 Deployment - Step: $step [$scriptInc]" -PercentComplete $Global:perComplete
    $scriptInc = Join-Path $Global:fileLocationPS $scriptName
    write-log -message "$scriptInc" -consoleForeground Magenta
    if(Test-Path -Path $scriptInc) { 
        . $scriptInc 
    } else {
        write-log -message "File Not Found: $scriptInc" -consoleForeground Red
    }
}

function Write-Log {
    <#
    .SYNOPSIS
        Writes logging information to screen and log file simultaneously.
    
    .DESCRIPTION
        Writes logging information to screen and log file simultaneously. Supports multiple log levels.
    
    .PARAMETER Message - The Message to be logged.
    .PARAMETER Level - The type of Message to be logged, ERROR, WARN, INFO (default).
    .PARAMETER NoConsoleOut - Specifies to not display the Message to the console.
    .PARAMETER ConsoleForeground - Specifies what color the text should be be displayed on the console. Ignored when switch 'NoConsoleOut' is specified.
    .PARAMETER Indent - The number of spaces to indent the line in the log file.
    .PARAMETER Path - The log file path.  
    .PARAMETER Clobber - Existing log file is deleted when this is specified.
    .PARAMETER EventLogName - The name of the system event log, e.g. 'Application'.
    .PARAMETER EventSource - The name to appear as the source attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
    .PARAMETER EventID  - The ID to appear as the event ID attribute for the system event log entry. This is ignored unless 'EventLogName' is specified.
    .PARAMETER LogEncoding The text encoding for the log file. Default is ASCII.
    
    .EXAMPLE
        PS C:\> Write-Log -Message "It's all good!" -Path C:\MyLog.log -Clobber -EventLogName 'Application'
        PS C:\> Write-Log -Message "Oops, not so good!" -Level Warn -EventID 3 -Indent 2 -EventLogName 'Application' -EventSource "My Script" .INPUTS System.String
    
    .OUTPUTS
        No output
    
    .NOTES
        from https://gallery.technet.microsoft.com/scriptcenter/Powershell-function-to-5fde9091
        #Author - Vikram Bedi  
        #vikram.bedi.it@gmail.com   
        #Powershell v2.0 #v1.0 Initial Script
    #>

    #region Parameters
        [cmdletbinding()]
        Param(
            [Parameter(ValueFromPipeline=$True,Mandatory=$True)] [ValidateNotNullOrEmpty()]
            [string] $Message,
            [Parameter()] [ValidateSet("Error", "Warn", "Info")]
            [string] $Level = "Info",
            [Parameter()]
            [Switch] $NoConsoleOut,
            [Parameter()]
            [String] $ConsoleForeground = 'White',
            [Parameter()] [ValidateRange(1,30)]
            [Int16] $Indent = 0,
            [Parameter()]
            [IO.FileInfo] $Path = "$Global:diretoryName\$Global:logFileName",
            [Parameter()]
            [Switch] $Clobber,
            [Parameter()]
            [String] $EventLogName,
            [Parameter()]
            [String] $EventSource,
            [Parameter()]
            [Int32] $EventID = 1,
            [Parameter()]
            [String] $LogEncoding = "ASCII"
        ) #end Parameters
    #endregion

    Begin {}
    Process {
        try {
            $msg = '{0}{1,-20} : {2,-5} : {3,-8} : {4}' -f (" " * $Indent), (Get-Date -Format "yyyy-MM-dd HH:mm:ss"), $Level.ToUpper(), $ConsoleForeground.ToUpper(), $Message
            if ($NoConsoleOut -eq $False) {
                switch ($Level) {
                    "Error" { Write-Error $Message }
                    "Warn" { Write-Warning $Message }
                    "Info" { Write-Host ('{0}{1}' -f (" " * $Indent), $Message) -ForegroundColor $ConsoleForeground}
                }
            }
            if (-not $Path.Exists) {
                New-Item -Path $Path.FullName -ItemType File -Force | Out-Null
            }
            if ($Clobber) {
                $msg | Out-File -FilePath $Path -Encoding $LogEncoding -Force
                #$sw = [system.io.streamwriter]::new($Path)
                #$sw.writeline($msg)
                #$sw.close()
            } else {
                $msg | Out-File -FilePath $Path -Encoding $LogEncoding -Append
                #$sw = [system.io.streamwriter]::new($Path)
                #$sw.writeline($msg)
                #$sw.close()
            }
            if ($EventLogName) {
                if (-not $EventSource) {
                    $EventSource = ([IO.FileInfo] $MyInvocation.ScriptName).Name
                }
                if(-not [Diagnostics.EventLog]::SourceExists($EventSource)) { 
                    [Diagnostics.EventLog]::CreateEventSource($EventSource, $EventLogName) 
                } 
                $log = New-Object System.Diagnostics.EventLog  
                $log.set_log($EventLogName)  
                $log.set_source($EventSource) 
                switch ($Level) {
                    "Error" { $log.WriteEntry($Message, 'Error', $EventID) }
                    "Warn"  { $log.WriteEntry($Message, 'Warning', $EventID) }
                    "Info"  { $log.WriteEntry($Message, 'Information', $EventID) }
                }
            }
        } catch {
            throw "Failed to create log entry in: $Path. The error was: $_ ."
        }
    } #End Process

    End {}
}

Export-ModuleMember -function Get-SiteURL 
Export-ModuleMember -function Get-SiteWeb 
Export-ModuleMember -function Set-SiteColumn -Alias Add-SiteColumn, asc
Export-ModuleMember -function Add-SiteColumnToCT -Alias ascct

Export-ModuleMember -function Add-FacilityWorkflow -Alias afwf
Export-ModuleMember -function Add-SiteCollectionWorkFlow
Export-ModuleMember -function Get-DateFromDays
Export-ModuleMember -function Format-ListOutput
Export-ModuleMember -function Get-ListInternalName
Export-ModuleMember -function Get-SPList
Export-ModuleMember -function Remove-DataFromAList
Export-ModuleMember -function Deploy-Feature
Export-ModuleMember -function Update-UtilMF
Export-ModuleMember -function Get-RAMSEnv
Export-ModuleMember -function New-RAMSEnv
Export-ModuleMember -function Update-RAMSEnv
Export-ModuleMember -function Get-RAMSSharePointInstallInfo
Export-ModuleMember -function Get-FacilitySites
Export-ModuleMember -function New-RAMSGroup -Alias nrg
Export-ModuleMember -function Add-FacilityGroup -Alias afg
Export-ModuleMember -Function RunSQL-ReturnInt -Alias rsri
Export-ModuleMember -Function RunSQL-ReturnDR -Alias rsrdr
Export-ModuleMember -Function Remove-WorkFlow -Alias remwf
Export-ModuleMember -Function Rename-List -Alias renList
Export-ModuleMember -Function Clone-Calendar -Alias cloneCal
Export-ModuleMember -Function Remove-CalendarColumns -Alias RCC

Export-ModuleMember -Function Remove-FacilitySitePages 
Export-ModuleMember -Function Remove-FacilitySiteAssets 

#added on 03/16/2018
Export-ModuleMember -Function New-SpListView
Export-ModuleMember -Function Update-SpListView

#added on 03/17/2018
Export-ModuleMember -function Remove-ContentTypeFromList 
#Export-ModuleMember -function Set-SiteColumnLookup
Export-ModuleMember -function Add-SiteColumnToCTByTitle

#added on 03/19/2018
Export-ModuleMember -function Remove-ColumnFromList

#added on 04/10/2018
Export-ModuleMember -function Set-SiteColumnLookup

#added on 04/13/2018
Export-ModuleMember -function Update-SiteColumnLookup

#added on 4/15/2018
Export-ModuleMember -Function Remove-SitePages 
Export-ModuleMember -Function Remove-SiteAssets 

#added on 5/01/2018
Export-ModuleMember -Function Set-FacilityListPermissionAndVersioning

#added on 5/07/2018
Export-ModuleMember -Function Import-RAMSUsers

#added on 5/10/2018
Export-ModuleMember -Function New-MeetingMinuteLookup
Export-ModuleMember -Function Update-CalendarView
Export-ModuleMember -Function New-SPList

#added on 5/31/2018
Export-ModuleMember -Function New-SSISDB

#added on 6/01/2018
Export-ModuleMember -Function Install-SSISAdapter
Export-ModuleMember -Function New-SpFacilityListView

#added on 6/02/2018
Export-ModuleMember -Function Get-RAMSEnvName

#added on 7/12/2018
Export-ModuleMember -Function Get-FullHelp
Export-ModuleMember -Function Write-Log
Export-ModuleMember -Function Run-Script
#added on 7/12/2018
Export-ModuleMember -function Get-MySiteURL 
Export-ModuleMember -function Get-MySiteWeb 

#added on 7/19/2018
Export-ModuleMember -function Add-ReusableWorkFlowToContentType

#added on 8/8/2018
Export-ModuleMember -function Deploy-FarmERFeature

#added on 8/8/2018
Export-ModuleMember -function Deploy-FarmWFFeature
Export-ModuleMember -function Add-EventReceiverToContentType

