/**
 * @fileoverview RAMS_Tools manages the toolbar functions of a study application. Depends on dynamicGeneration for
 *               logError functions and much more.
 * 
 * <pre>
 * &lt;script type=&quot;text/javascript&quot; src=&quot;jshashtable-3.0.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;jshashset-3.0.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;rams-environment.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;dynamicGeneration.js&quot;&gt;&lt;/script&gt;
 * </pre>
 * 
 * In some cases the jshint errors are suppressed with jshint -W### codes, jshint globals, or jshint
 *               exported comments
 * @version 2.3.0
 */
// the following globals are either functions or variables defined on other required script files, like
// dynamicGeneration.js
/* globals assignedID, currentUser, currentFormType, spAppData, IACUC, IRB, RDC, SRS, SC1, SC2, SC3, RAMS */
/* globals sendREST, readOnlyState, siteUrl, buildUsers, targetStage, insertStudyMember, restPath, sendFile */
/* globals grabApplicationSiteType, validSection, continueSectionBuilder, getRAMSUsers, currentStudyStatus */
/* globals childrenStudies, generateAssignedId, logError, ramsUsers, removeStudyMember, currentUsers */
/* globals PRINCIPALINVESTIGATOR, COPRINCIPALINVESTIGATOR, LIAISONREVIEWER, PRIMARYREVIEWER */
/* globals ASSIGNEDREVIEWER, COMMITTEETYPE, IACUCSTUDYADMINISTRATOR, IRBSTUDYADMINISTRATOR, RDCSTUDYADMINISTRATOR */
/* globals SRSSTUDYADMINISTRATOR, SC1STUDYADMINISTRATOR, SC2STUDYADMINISTRATOR, SC3STUDYADMINISTRATOR */
/* globals LOCALSITEINVESTIGATOR, Roles, managePersonnelNotificationWorkflowSubscriptionId */
/* globals ficObj, users, confirm, startRAMSWorkflow, refreshDisplay */
/* globals targetStage, targetState, currentFormVersion, currentStudyStatus, studyTitle */

/**
 * Holds all of the comments for the application done in review
 * 
 * @type {Hashtable}
 */
var appReviewComments;

/**
 * NOT USED?? array of names of reviewers who commented - built but never read.
 * 
 * @type {string[]}
 */
var reviewersComments = [];

/**
 * Collection of administrators that can administer the study.
 * 
 * @type {Hashset}
 */
var adminCollection;

// var reviewerCollection = [];

/**
 * NOT USED OR DOESNT WORK BECAUSE OF THE WAY IT IS DETERMINED?? the url of the facility built in {@link selectToolbar}
 * 
 * @type {string}
 */
var facilityUrl = "";

/**
 * NOT USED OR DOESNT WORK BECAUSE OF THE WAY IT IS DETERMINED?? the url of the committee built in {@link selectToolbar}
 * 
 * @type {string}
 */
var committeeUrl = "";

/**
 * object to hold the studyMemberCollection with all the incumbent personnel, including the following additional
 * objects. The additional objects are not guaranteed to be present if there is nobody identified in the role.
 * <ul>
 * <li>assignedID {string} - the assignedID associated with this studyMemberCollection
 * <li>smc {StudyMember[]} - an associative array of the studyMembers indexed by the SPUserId
 * <li>currentPI {User} - the PI's studyMemberObject
 * <li>currentCoPI {User} - the Co-PI's studyMemberObject
 * <li>currentAdministrator {User} - the study administrator's studyMemberObject
 * <li>currentPrimaryReviewer {User} - the PrimaryReviewer's studyMemberObject
 * <li>currentAssignedReviewers {User[]} - an associative array of AssignedReviewer's studyMemberObjects indexed by
 * SPUserId
 * </ul>
 * The StudyMemberObject consist of the following attributes:
 * <ul>
 * <li>study_MembersID {number} the Study_Members id from the RAMS Study_Members table
 * <li>user {User} a user object representing the user to the application
 * <ul>
 * <li>SPUser {number} the SPUserId
 * <li>displayName {string} the display name of the user from active directory
 * <li>loginName {string} the domain\\userid of the user from active directory
 * <li>permanentLoginIdentifier {string} GUID to identify a user in active directory
 * <li>synchronizedTimestamp {string} timestamp the user object was last updated in YYYY-MM-DDTHH:mm:ss-TZH:tzm, e.g.
 * 2017-05-09T11:45:40-04:00
 * <li>usersID {number} the userId from the RAMS User table
 * </ul>
 * <li>coiRequired {boolean} indicates id a Conflict of Interest is required
 * <li>writePermissions {boolean} indicates if the user can update the study
 * <li>explaination {string} text annotating why the user was granted write permission
 * <li>notifications {boolean} indicates if the user is to receive notifications
 * <li>memberRole {string} what role the user is playing on the study. A user can only be in one role per study
 * <li>teamMember {boolean} indicating if the user is considered a team member
 * <li>activeMember {boolean} indicating if the member is active
 * <li>createdAt {string} timestamp that the StudyMember was created and added to the study in
 * YYYY-MM-DDTHH:mm:ss-TZH:tzm, e.g. 2017-05-09T11:45:40-04:00
 * </ul>
 * The smcPlusIncumbents object is populated by calling the getStudyMembers(assignedID, callback) function.
 * 
 * @type {Object}
 */
var smcPlusIncumbents = {};

var committeeTable = new Hashtable();

/**
 * selects the toolbar based on who the user is relative to the state and role
 * 
 * @param {number} targetState The next state of the application. Possible values are 1-4.
 */
/* exported selectToolbar */
function selectToolbar(targetState) {
   var previousUrl = document.referrer;
   var splitUrl = previousUrl.split(".aspx")[0];
   if (splitUrl !== "") {
      if (splitUrl.indexOf("/sites/IRBApplications/") != -1) {
         splitUrl = splitUrl.split("/sites/IRBApplications/")[1];
         if (splitUrl.indexOf("/SitePages/") != -1) {
            facilityUrl = splitUrl.split("/SitePages/")[0];
            committeeUrl = splitUrl.split("/SitePages/")[1];
         }
      }
   }

   var allReviewers = [];
   //
   // /**
   // * converts a list of ids in an array into a HashSet
   // *
   // * @param {Array}
   // * list
   // * @returns {HashSet} tempReviewers
   // */
   // function gatherReviewers(list) {
   // var i;
   // if (list != null) {
   // for (i = 0; i < list.results.length; i++) {
   // tempReviewers.add(list.results[i]);
   // }
   // }
   // }
   // gatherReviewers(spAppData[RAMS.Environment.SharePoint.List.Application.PrimaryReviewersId]);
   // gatherReviewers(spAppData.ReviewersId);
   if (!(smcPlusIncumbents)) {
      getStudyMembers(assignedID, function() {
         // guarantee that smcPlusIncumbents is populated
      });
   }

   // allReviewers = smcPlusIncumbents.currentAssignedReviewers;

   // if (car && car.length > 0) {
   // for (var ar = 0; ar < car.length; ar++) {
   // allReviewers[car[ar].user.SPUser] = car[ar];
   // }
   // }
   // if (smcPlusIncumbents.currentPrimaryReviewer) {
   // allReviewers[smcPlusIncumbents.currentPrimaryReviewer.user.SPUser] = smcPlusIncumbents.currentPrimaryReviewer;
   // }
   /********************************************************************************************************************
    * var tempIRB = tempIACUC = tempSRS = tempRDC = tempSC1 = tempSC2 = tempSC3 = []; for(var i=0; i <
    * COMMITTEETYPE.length; i++){ if(smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers){ if(COMMITTEETYPE[i]
    * === IRB){ tempIRB = smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers; }else if(COMMITTEETYPE[i] ===
    * IACUC){ tempIACUC = smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers; }else if(COMMITTEETYPE[i] ===
    * SRS){ tempSRS = smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers; }else if(COMMITTEETYPE[i] === RDC){
    * tempRDC = smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers; }else if(COMMITTEETYPE[i] === SC1){
    * tempSC1 = smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers; }else if(COMMITTEETYPE[i] === SC2){
    * tempSC2 = smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers; }else if(COMMITTEETYPE[i] === SC3){
    * tempSC3 = smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers; } } } allReviewers =
    * tempIRB.concat(tempIACUC,tempSRS,tempRDC,tempSC1,tempSC2,tempSC3);
    *******************************************************************************************************************/

   for (var i = 0; i < COMMITTEETYPE.length; i++) {
      if (smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers) {
         for ( var car in smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers) {
            if (smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers.hasOwnProperty(car)) {
               allReviewers[smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers[car].user.SPUser] = smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers[car];
            }
         }
      }
      if (smcPlusIncumbents[COMMITTEETYPE[i]].currentPrimaryReviewer) {
         allReviewers[smcPlusIncumbents[COMMITTEETYPE[i]].currentPrimaryReviewer.user.SPUser] = smcPlusIncumbents[COMMITTEETYPE[i]].currentPrimaryReviewer;
      }
   }

   groupCheck(function(irbAdmin) { // add liaison.
      // console.log("receiving: committee Member " + committeeMember + ",
      // irbAdmin " +
      // irbAdmin);
      /* initializing toolbars and r/w access */

      // JSHINT Read Only - the readOnlyState in multiple locations below
      // needs care when setting it
      // jshint -W020
      if (((smcPlusIncumbents.currentPI) && (currentUser.Id == smcPlusIncumbents.currentPI.user.SPUser))
         || ((smcPlusIncumbents.currentCoPI) && (currentUser.Id == smcPlusIncumbents.currentCoPI.user.SPUser))) {
         // console.log("you are the PI");
         if ((targetState == 1) || (targetState == 3)) {
            generatePITool();
            readOnlyState = false;
         } else if (((currentFormType == "Parent") || (currentFormType == "Local") || (currentFormType == "Amendment")) && (targetState >= 4)
            && (spAppData.ApplicationStatus != RAMS.Environment.SharePoint.List.CommitteeStatus.CLOSEDCHOICE)) {
            // Sustainment Defect #653367 Give LSI Toolbar on Approved Local
            // Site Application
            //5= Disapproved          //8 = Suspended
            if ((targetState !== 5) && (targetState !== 8)) {
              generateApprovedTools();
            }
         }
      } else if (allReviewers[currentUser.Id]) {
         // console.log("you are a reviewer or the primary reviewer");
         if ((targetState == 2) || (targetState == 4)) {
            generateReviewerTool();
         }
      } else if (((smcPlusIncumbents.IRB.currentAdministrator) && (currentUser.Id == smcPlusIncumbents.IRB.currentAdministrator.user.SPUser))
         || ((smcPlusIncumbents.IACUC.currentAdministrator) && (currentUser.Id == smcPlusIncumbents.IACUC.currentAdministrator.user.SPUser))
         || ((smcPlusIncumbents.RDC.currentAdministrator) && (currentUser.Id == smcPlusIncumbents.RDC.currentAdministrator.user.SPUser))
         || ((smcPlusIncumbents.SRS.currentAdministrator) && (currentUser.Id == smcPlusIncumbents.SRS.currentAdministrator.user.SPUser))
         || ((smcPlusIncumbents.SC1.currentAdministrator) && (currentUser.Id == smcPlusIncumbents.SC1.currentAdministrator.user.SPUser))
         || ((smcPlusIncumbents.SC2.currentAdministrator) && (currentUser.Id == smcPlusIncumbents.SC2.currentAdministrator.user.SPUser))
         || ((smcPlusIncumbents.SC3.currentAdministrator) && (currentUser.Id == smcPlusIncumbents.SC3.currentAdministrator.user.SPUser))) {
         // reviewers should always be in a read only state
         // console.log("you are the assigned IRB Admin");
         if ((targetState !== 1) && (targetState !== 3)) {
            generateIRBAdminTool();
         }
      } else if (smcPlusIncumbents.smc[currentUser.Id]) {
         // TODO TAR see if the studyMember is a not a RAMS Reviewer or
         // Administrator. if not a
         // reviewer or admin, you can give this toolbar given the other
         // rules below

         // console.log("you are a study member");
         if ((targetState === 1) || (targetState === 3)) {
            generateStudyTool();
            readOnlyState = false;
         }
      } else if (irbAdmin === true) {
         // should really check if the person is in the site's irb admin
         // group...
         // console.log("you are an IRB Admin");
         // TODO check with Neetu which toolbar to showup.
         if ((targetState != 1) && (targetState != 3)) {
            generateAssignTool();
            // generateLocalSiteTool();
         }
      } //else if (committeeMember === true) {
         // JSHINT Empty block - The empty blocks are intentional for
         // debugging purposes.
         // jshint -W035
         // console.log("you are a committeeMember");
      //}
   });

   // if(readOnlyState){
   // console.log("you should be in a read only state");
   // }
}

/**
 * encompases the basic toolbar structure and specialized buttons that an IRB Admin would see. Also pre-generates the
 * assignments tab since it's the first button.
 */
function generateIRBAdminTool() {
   // console.log("irb admin tool generation");

   // generateToolStructure("Assigned IRB Admin Toolbar");
   if (committeeUrl === "") {
      generateToolStructure("Assigned Admin Toolbar");
   }

   if (committeeUrl == IACUC) {
      generateToolStructure("Assigned IACUC Admin Toolbar");
   }
   if (committeeUrl == RDC) {
      generateToolStructure("Assigned R&D Admin Toolbar");
   }
   if (committeeUrl == IRB) {
      generateToolStructure("Assigned IRB Admin Toolbar");
   }
   if (committeeUrl == SRS) {
      generateToolStructure("Assigned SRS Admin Toolbar");
   }

   swapToolGenerator("assignApp");
}

/**
 * Creates the toolbar intended for the Study Team Member
 */
function generateStudyTool() {
   generateToolStructure("Study Member Toolbar");

   swapToolGenerator("studyComments");
}

/**
 * Creates the toolbar intended for the Principal Investigator
 */
function generatePITool() {
   generateToolStructure("PI Toolbar");

   swapToolGenerator("piComments");
}

/**
 * Creates the toolbar intended for the Principal Investigator once the study is approved.
 */
function generateApprovedTools() {
   generateToolStructure("Approved Toolbar");

   swapToolGenerator("postApproveEvents");
}

/**
 * Creates the toolbar intended for the assigning the study.
 */
function generateAssignTool() {
   // generateToolStructure("General IRB Admin Toolbar");
   if (committeeUrl === "") {
      generateToolStructure("General Admin Toolbar");
   }

   if (committeeUrl == IACUC) {
      generateToolStructure("General IACUC Admin Toolbar");
   }
   if (committeeUrl == RDC) {
      generateToolStructure("General R&D Admin Toolbar");
   }
   if (committeeUrl == IRB) {
      generateToolStructure("General IRB Admin Toolbar");
   }
   if (committeeUrl == SRS) {
      generateToolStructure("General SRS Admin Toolbar");
   }

   swapToolGenerator("assignIRBAdmin");
}

/**
 * Creates the toolbar intended for the reviewers.
 */
function generateReviewerTool() {
   generateToolStructure("Reviewer Toolbar");

   swapToolGenerator("reviewComments");
}

// ****************************************************************************
/**
 * NOT USED. Creates the toolbar intended for the Local Site Liaison.
 * 
 * @deprecated
 */
/* exported generateLocalSiteTool */
function generateLocalSiteTool() {
   generateToolStructure("Local Site Liaison Toolbar");

   swapToolGenerator("localSite");
}

/**
 * NOT USED. Creates the toolbar intended for the CIRB Administrator.
 * 
 * @deprecated
 */
/* exported generateCIRBAdminTool */
function generateCIRBAdminTool() {
   generateToolStructure("cIRB Admin Toolbar");

   swapToolGenerator("CIRBAdmin");
}
// ****************************************************************************

/**
 * Uses the passed in target tool to create and display the toolbar for the user.
 * 
 * @param {string} targetTool Valid values for TargetTool are:
 *           <ul>
 *           <li>assignApp
 *           <li>appReview
 *           <li>appChecklists
 *           <li>submitApp
 *           <li>appEvent
 *           <li>studyComments
 *           <li>postApproveEvents
 *           <li>piChecklists
 *           <li>piComments
 *           <li>piSubmitApp
 *           <li>assignIRBAdmin
 *           <li>reviewComments
 *           <li>reviewChecklists
 *           <li>reviewerSubmit
 *           <li>localSite - NOT USED {@link generateLocalSiteTool}
 *           <li>CIRBAdmin - NOT USED {@link generateCIRBAdminTool}
 *           </ul>
 */
function swapToolGenerator(targetTool) {
   var row;

   // var centralFilter = "";
   // var adminGroupIds = [];
   // var siteMemberGroupIds = [];
   var irbAdminsJson = [];
   var committeeList;
   // var siteMembersJson = [];
   // TODO TAR the following line is not in the tst environment
   var facilityName = getFacilityNameById(spAppData.FacilitiesLookupId);
   if ((typeof facilityName === "undefined") || (facilityName === null)) {
      return;
   }
   $('#built-tool').remove();

   row = document.createElement("row");
   row.setAttribute("class", "row");
   row.setAttribute("id", "built-tool");

   if (targetTool == "assignApp") {

      getStudyMembers(assignedID, function(smcPlusIncumbents) {
         /* jshint unused:false */
         row.appendChild(buildAdminSelecters());
         row.appendChild(buildAdminReviewers());
         row.appendChild(generateToolTabs(targetTool));
         var rows = document.getElementsByName("SPU");
         for (var i = 0; i < rows.length; i++) {
            rows[i].style.display = "none";
         }
         $('#insert-tool').append(row);
         
         //check button label
         var adminToolBarButtonText = $("#navToolButton").text();
         if(adminToolBarButtonText.indexOf("Assigned Admin Toolbar") === -1){
          if($("#navToolButton").val() === "expanded"){
             $("#navToolButton").text("Expand Assigned Admin Toolbar");
             }else{
                $("#navToolButton").text("Collapse Assigned Admin Toolbar");
             }
         }

         committeeList = committeeTable.get(assignedID);
         if (committeeList && committeeList.length === 1) {
            $('#selectCommittee').val(committeeList[0]).change();
            populateAdminAndReviewerDropDowns('selectCommittee', 'addAdmin', 'addReviewer');
         }
      });

      // TODO ADD FILTER IF spAppData.ApplicationSiteType
      // if (spAppData.ApplicationSiteType == "Central") {
      // centralFilter = "or startswith(Title, 'Central')";
      // sendREST("GET", siteUrl.concat("_api/Web/sitegroups" +
      // "?$filter=startswith(Title, '"
      // + filter + "') eq true"), null, function(centralJSON) {
      // var currentGroups = centralJSON.d.results;
      // var adminGroupId = -1;
      // var siteMembersGroupId = -1;
      // for (var i = 0; i < currentGroups.length; i++) { // find the
      // admin group and site
      // // users' group for the
      // if (currentGroups[i].Title.indexOf("Administrators") > -1) {
      // adminGroupId = currentGroups[i].Id;
      // adminGroupIds = buildGroupIds(adminGroupIds, adminGroupId);
      // }
      //
      // if ((currentGroups[i].Title.indexOf("Administrators") > -1)
      // || (currentGroups[i].Title.indexOf("Members") > -1)
      // || (currentGroups[i].Title.indexOf("Coordinators") > -1)) {
      // siteMembersGroupId = currentGroups[i].Id;
      // siteMemberGroupIds = buildGroupIds(siteMemberGroupIds,
      // siteMembersGroupId);
      // }
      // }
      // });
      // }

      // sendREST(
      // "GET",
      // siteUrl.concat("_api/Web/sitegroups"
      // + "?$filter=startswith(Title, '" + facilityName + "') "
      // + centralFilter + " eq true&?orderby=Title asc"),
      // null,
      // function(groupJSON) {
      // var currentGroups = groupJSON.d.results;
      // var adminGroupId = -1;
      // var siteMembersGroupId = -1;
      // for (var i = 0; i < currentGroups.length; i++) { // find
      // // the
      // // admin
      // // group
      // // and
      // // site
      // // users' group for
      // // the
      // // user's site
      // // Assums user is member of only 1 group. Should these
      // // read in the facilityanswer?
      // // TODO make adminGroups an array of Ids
      // // TODO make UsersGroups an array of Ids
      // if (currentGroups[i].Title.indexOf("Administrators") > -1) {
      // adminGroupId = currentGroups[i].Id;
      // adminGroupIds = buildGroupIds(adminGroupIds,
      // adminGroupId);
      // }
      // // TODO TAR Why is this not Members and Coordinators,
      // // like it was before?
      // // else if ((currentGroups[i].Title.indexOf("Members") >
      // // -1) ||
      // // (currentGroups[i].Title.indexOf("Coordinators") >
      // // -1)) {
      // if ((currentGroups[i].Title.indexOf("Administrators") > -1)
      // || (currentGroups[i].Title.indexOf("Members") > -1)
      // || (currentGroups[i].Title
      // .indexOf("Coordinators") > -1)) {
      // siteMembersGroupId = currentGroups[i].Id;
      // siteMemberGroupIds = buildGroupIds(
      // siteMemberGroupIds, siteMembersGroupId);
      // }
      // }
      // * getCurrentAdminUsers(function(adminGroupIds, siteMemberGroupIds) {
      // perform REST calls if we have values
      // TODO see if afminGroupIds.length and
      // siteUsersGroupIds.length > 0
      // * if ((adminGroupIds.length > 0)
      // * && (siteMemberGroupIds.length > 0)) { // assuming
      // we
      // only need the
      // admins from one site
      // TODO loop through the adminIds and add all the admins
      // to a new irbAdmins array
      // TODO Install JSHINT

      // for (var adminGroup = 0; i < adminGroupIds .length;
      // i++) {
      // sendREST("GET", siteUrl.concat("_api/Web/SiteGroups("
      // +
      // adminGroupIds[adminGroup] +
      // ")/Users"), null, function(irbAdmins){
      // irbAdminsJson= irbAdminJson.concat(irbAdmins);
      // * irbAdminsJson = getUsersFromSecurityUsingAjax(adminGroupIds);
      // irbAdminsJson=JSON.parse(irbAdminsJson);
      // });

      // }
      // for (var siteGroup = 0; i < siteUserGroupsIds
      // .length; i++) {
      // sendREST("GET", siteUrl.concat("_api/Web/SiteGroups("
      // +
      // siteUserGroupsIds[siteGroup] +
      // ")/Users"), null, function(siteGroups){
      // siteMembersJson= siteMembersJson.concat(siteGroups);
      // * siteMembersJson = getUsersFromSecurityUsingAjax(siteMemberGroupIds);
      // siteMembersJson=JSON.parse(siteMembersJson);

      // });

      // }
      // * getStudyMembers(
      // * assignedID,
      // * function(smcPlusIncumbents) {
      // * if (smcPlusIncumbents) {
      // * if (smcPlusIncumbents.smc) {
      // * var smc = smcPlusIncumbents.smc;
      // * var availableReviewers = [];

      // parse JSON and extract names
      // * var userCollection = buildUsers(
      // * siteMembersJson).values();

      // Only add people to t he
      // availableReviewers if they are
      // not in the
      // studyMemberCollection
      // * for (i = 0; i < userCollection.length; i++) {
      // * if (typeof smc[userCollection[i].Id] === "undefined") {
      // * availableReviewers
      // * .push(userCollection[i]);
      // * }
      // * }
      // * availableReviewers = availableReviewers
      // * .sort(function(a, b) {
      // * return (a.Title > b.Title) ? 1
      // * : ((b.Title > a.Title) ? -1
      // * : 0);
      // * });
      // TODO TAR the following was in the
      // test environment - Is it needed?
      // }
      // * row
      // * .appendChild(buildAdminSelecters(
      // * irbAdminsJson,
      // * availableReviewers));
      // * row
      // * .appendChild(buildAdminReviewers());
      // * row
      // * .appendChild(generateToolTabs(targetTool));

      // * if (typeof spAppData.IRBAdministratorId !== "undefined") {
      // * $('#selectCommittee').val(IRB);
      // * }else if (typeof spAppData.IACUCAdministratorId !== "undefined") {
      // * $('#selectCommittee').val(IACUC);
      // * }else if (typeof spAppData.RDCAdministratorId !== "undefined") {
      // * $('#selectCommittee').val(RDC);
      // * }else if (typeof spAppData.SRSAdministratorId !== "undefined") {
      // * $('#selectCommittee').val(SRS);
      // * }else if (typeof spAppData.SC1AdministratorId !== "undefined") {
      // * $('#selectCommittee').val(SC1);
      // * }else if (typeof spAppData.SC2AdministratorId !== "undefined") {
      // * $('#selectCommittee').val(SC2);
      // * }else if (typeof spAppData.SC3AdministratorId !== "undefined") {
      // * $('#selectCommittee').val(SC3);
      // * }
      // select CurrentAdmin in the dropdown
      // * var selectedCommittee = $("#selectCommittee").find(':selected').val();
      // * if(selectedCommittee){
      // * $('#addAdmin').val(smcPlusIncumbents[selectedCommittee].currentAdministrator.user.SPUser).change();
      // * }
      // if ((smcPlusIncumbents.currentAdministrator)
      // && (smcPlusIncumbents.currentAdministrator.user.SPUser)) {
      // $('#addAdmin')
      // .val(
      // smcPlusIncumbents.currentAdministrator.user.SPUser)
      // .change();
      // }
      // var rows = document
      // .getElementsByName("SPU");
      // for (i = 0; i < rows.length; i++) {
      // rows[i].style.display = "none";
      // }

      // }
      // }
      // });
      // }

      // });
      // });

   } else if (targetTool == "appReview") {
      /* review tab */
      // 681460: Toolbars creating infinite loop: Commented out the
      // validSection check and recursive call because it
      // seemed like an infinite loop. In reality it was an attempt to wait
      // for the buttons on the study page to load.
      // Since it was slow the toolbar comment sections and questions
      // dropdowns do not get loaded properly. The
      // population of the dropdowns was moved to toggleNavToolbar.
      // if (validSection.length == 0) {
      // window.setTimeout(function() {
      // swapToolGenerator(targetTool);
      // }, 2000);
      // }
      // sendREST("GET",
      // restPath.concat("study/"+assignedID+"/comments/"+targetStage), null,
      // function(comments){
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         // row.appendChild(buildAdminComments());
         row.appendChild(buildMultiCommitteesComments());
         row.appendChild(buildReviewersComments());
         row.appendChild(generateToolTabs(targetTool));
      });
      $('#insert-tool').append(row);
   } else if (targetTool == "appChecklists") {
      /* checklists tab */
      sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('SiteAssets/IRB Admin Checklists')/Files"),
         null, function(checklists) {
            sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID
               + "/Review%20Team')/Files"), null, function(returnedFiles) {
               row.appendChild(buildChecklistTemplates(checklists.d.results));
               row.appendChild(buildUploadedChecklists(returnedFiles.d.results, "Review%20Team"));
               row.appendChild(generateToolTabs(targetTool));
            });
         });
      $('#insert-tool').append(row);
   } else if (targetTool == "submitApp") {
      /* approve tab */
      // sendREST("GET",
      // restPath.concat("study/"+assignedID+"/comments/"+targetStage), null,
      // function(comments){
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         row.appendChild(buildSubmitComments([ "Re-Submit to Study Team", "Approve Application",
            "Approve with Minor Modifications", "Approve with Major Modifications", "Suspend", "Rejected"
         ], "committee"));
         row.appendChild(buildAppHistory());
         row.appendChild(generateToolTabs(targetTool));
         hideAndShowTriennialExpirationDate("triennialExpirationLabel", "triennialExpirationId");
      });
      $('#insert-tool').append(row);
      // hideAndShowTriennialExpirationDate("triennialExpirationLabel", "triennialExpirationId");
   } else if (targetTool == "appEvent") {
      /* events tab */
      // row.appendChild(buildAddEvent("Review Team", "1"));
      row.appendChild(buildPostApproveReview());
      // row.appendChild(buildEventHistory());
      row.appendChild(buildRenewalHistory());
      row.appendChild(generateToolTabs(targetTool));
      $('#insert-tool').append(row);
   } else if (targetTool == "studyComments") {
      /* study member's comments tab */
      // 681460: Toolbars creating infinite loop: Commented out the
      // validSection check and recursive call because it
      // seemed like an infinite loop. In reality it was an attempt to wait
      // for the buttons on the study page to load.
      // Since it was slow the toolbar comment sections and questions
      // dropdowns do not get loaded properly. The
      // population of the dropdowns was moved to toggleNavToolbar.
      // if (validSection.length == 0) {
      // window.setTimeout(function() {
      // swapToolGenerator(targetTool);
      // }, 2000);
      // }
      // sendREST("GET",
      // restPath.concat("study/"+assignedID+"/comments/"+targetStage), null,
      // function(comments){
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         row.appendChild(buildAdminComments());
         row.appendChild(buildReviewersComments());
         row.appendChild(generateStudyToolTabs(targetTool));
      });
      $('#insert-tool').append(row);
   } else if (targetTool == "postApproveEvents") {
      /* Possible events post-approval */
      row.appendChild(buildAddEvent());
      row.appendChild(buildEventHistory());
      $('#insert-tool').append(row);
   } else if (targetTool == "piChecklists") {
      sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('SiteAssets/PI Checklists')/Files"), null,
         function(checklists) {
            sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID
               + "/Study%20Team')/Files"), null, function(returnedFiles) {
               var studyChecklists = filterChecklists(returnedFiles.d.results, "-CHECKLIST-");
               row.appendChild(buildChecklistTemplates(checklists.d.results));
               row.appendChild(buildUploadedChecklists(studyChecklists, "Study%20Team"));
               row.appendChild(generatePIToolTabs(targetTool));
            });
         });
      $('#insert-tool').append(row);
   } else if (targetTool == "piComments") {
      // 681460: Toolbars creating infinite loop: Commented out the
      // validSection check and recursive call because it
      // seemed like an infinite loop. In reality it was an attempt to wait
      // for the buttons on the study page to load.
      // Since it was slow the toolbar comment sections and questions
      // dropdowns do not get loaded properly. The
      // population of the dropdowns was moved to toggleNavToolbar.

      // if (validSection.length == 0) {
      // window.setTimeout(function() {
      // swapToolGenerator(targetTool);
      // }, 2000);
      // }
      // sendREST("GET",
      // restPath.concat("study/"+assignedID+"/comments/"+targetStage), null,
      // function(comments){
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         row.appendChild(buildAdminComments());
         row.appendChild(buildReviewersComments());
         row.appendChild(generatePIToolTabs(targetTool));
      });
      $('#insert-tool').append(row);
   } else if (targetTool == "piSubmitApp") {
      /* review tab */
      // sendREST("GET",
      // restPath.concat("study/"+assignedID+"/comments/"+targetStage), null,
      // function(comments){
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         row.appendChild(buildSubmitComments([ "Re-Submit for Review"
         ]));
         row.appendChild(buildAppHistory());
         row.appendChild(generatePIToolTabs(targetTool));
      });
      $('#insert-tool').append(row);
   } else if (targetTool == "assignIRBAdmin") {

      getStudyMembers(assignedID, function(smcPlusIncumbents) {
         /* jshint unused:false */
         row.appendChild(buildGeneralIRBAdminSelecter());
         row.appendChild(buildCurrentAdminstrators());
         row.appendChild(generateStudyToolTabs(targetTool));
         $('#insert-tool').append(row);
         
       //check button label
         var adminToolBarButtonText = $("#navToolButton").text();
         if(adminToolBarButtonText.indexOf("General Admin Toolbar") === -1){
          if($("#navToolButton").val() === "expanded"){
             $("#navToolButton").text("Expand General Admin Toolbar");
             }else{
                $("#navToolButton").text("Collapse General Admin Toolbar");
             }
         }
         
         getCurrentAdminUsers(function(adminGroupIds, siteMemberGroupIds) {
            /* jshint unused:false */
            if (adminGroupIds.length > 0) {
               irbAdminsJson = getUsersFromSecurityUsingAjax(adminGroupIds);
               adminCollection = buildUsers(irbAdminsJson);
               // tempCollection = adminCollection.values();
            }
         });
      });

      // *var tempCollection;

      // if (spAppData.ApplicationSiteType == "Central") {
      // centralFilter = "or startswith(Title, 'Central')";
      // sendREST("GET",
      // siteUrl.concat("_api/Web/sitegroups" +
      // "?$filter=startswith(Title, '" + filter + "') eq true"), null,
      // function(centralJSON) {
      // var currentGroups = centralJSON.d.results;
      // var adminGroupId = -1;
      // var siteMembersGroupId = -1;
      // for (var i = 0; i < currentGroups.length; i++) { // find the
      // admin group and site
      // // users' group for
      // // the
      // if (currentGroups[i].Title.indexOf("Administrators") > -1) {
      // adminGroupId = currentGroups[i].Id;
      // adminGroupIds = buildGroupIds(adminGroupIds, adminGroupId);
      // }
      //
      // if ((currentGroups[i].Title.indexOf("Administrators") > -1)
      // || (currentGroups[i].Title.indexOf("Members") > -1)
      // || (currentGroups[i].Title.indexOf("Coordinators") > -1)) {
      // siteMembersGroupId = currentGroups[i].Id;
      // siteMemberGroupIds = buildGroupIds(siteMemberGroupIds,
      // siteMembersGroupId);
      // }
      // }
      // });

      // }

      // Get the list of security groups for the facility and if necessary the
      // CentralIRB security groups.
      // sendREST("GET", siteUrl.concat("_api/Web/sitegroups" +
      // "?$filter=startswith(Title, '" + facilityName + "') "
      // + centralFilter + " eq true&?orderby=Title asc"), null,
      // function(groupJSON) {
      // var currentGroups = groupJSON.d.results;
      // * getCurrentAdminUsers(function(adminGroupIds, siteMemberGroupIds) {

      // for (var i = 0; i < currentGroups.length; i++) { // find the
      // // admin
      // // group and site
      // // users' group for
      // // the
      // if (currentGroups[i].Title.indexOf("Administrators") > -1) {
      // // add the administrators to both the admin list and the
      // // reviewer list.
      // adminGroupIds.push(currentGroups[i].Id);
      // siteMemberGroupIds.push(currentGroups[i].Id);
      // } else if ((currentGroups[i].Title.indexOf("Members") > -1)
      // || (currentGroups[i].Title.indexOf("Coordinators") > -1)) {
      // // add the members and coordinators to the reviewer list.
      // siteMemberGroupIds.push(currentGroups[i].Id);
      // }
      // }
      // assuming we only need the admins from one site, but getting them
      // from
      // all Admin groups
      // * if (adminGroupIds.length > 0) {
      // *
      // * irbAdminsJson = getUsersFromSecurityUsingAjax(adminGroupIds);

      // sendREST("GET", siteUrl.concat("_api/Web/SiteGroups(" +
      // adminGroupId +
      // ")/Users"), null,
      // function(irbAdmins) {

      // check for size of IRB Admin group
      // * adminCollection = buildUsers(irbAdminsJson);
      // parse the JSON data for administrators, using global variable
      // * tempCollection = adminCollection.values();

      // * if (tempCollection.length == 1) {
      // if (tempCollection[0].Id == currentUser.Id) { // if
      // // current
      // // user admin,
      // // give
      // // them assigned toolbar
      // targetTool = "assignApp";
      // }
      // var intoReview = (targetTool === "assignIRBAdmin"); // boolean
      // // value
      // var data = {
      // "__metadata" : spAppData.__metadata,
      // "AssignedIRBAdministratorId" : tempCollection[0].Id
      // };
      // spAppData.AssignedIRBAdministratorId = tempCollection[0].Id;
      // if (intoReview) {
      // data.ApplicationStatus = spAppData.ApplicationStatus = "Under Review";
      // }
      //
      // insertStudyMember(tempCollection[0].Id,
      // "Study Administrator", false, assignedID, null);
      // sendREST(
      // "POST",
      // spAppData.__metadata.uri,
      // data,
      // function() {
      // if (!intoReview) {
      // swapToolGenerator(targetTool);
      // $('#navToolButton')
      // .attr("onclick",
      // "toggleNavTool('Assigned R&D Admin Toolbar')");
      // // select CurrentAdmin in the dropdown
      // if ((smcPlusIncumbents.currentAdministrator)
      // && (smcPlusIncumbents.currentAdministrator.user.SPUser)) {
      // $('#addAdmin')
      // .val(
      // smcPlusIncumbents.currentAdministrator.user.SPUser)
      // .change();
      // }
      //
      // } else {
      // sendREST(
      // "POST",
      // restPath.concat("study/"
      // + assignedID
      // + "/stage/Review"),
      // null,
      // function() {
      // swapToolGenerator(targetTool);
      // $('#navToolButton')
      // .attr("onclick",
      // "toggleNavTool('Assigned R&D Admin Toolbar')");
      // // select CurrentAdmin in the
      // // dropdown
      // if ((smcPlusIncumbents.currentAdministrator)
      // && (smcPlusIncumbents.currentAdministrator.user.SPUser)) {
      // $('#addAdmin')
      // .val(
      // smcPlusIncumbents.currentAdministrator.user.SPUser)
      // .change();
      // }
      // });
      // }
      // * }, "Merge");

      // // send admin data in POST REST calls
      // sendREST("POST", spAppData.__metadata.uri, data,
      // function() {
      // var finishUp = function() {
      // // TODO TAR should be passing UserId
      // swapToolGenerator(targetTool);
      // };
      //
      // if (!intoReview) {
      // finishUp();
      // } else {
      // sendREST("POST", restPath.concat("study/" + assignedID +
      // "/stage/Review"), null, finishUp);
      // }
      // }, "Merge");
      // // force refresh of navigation tool
      // // $('#navToolButton').attr("onclick",
      // "toggleNavTool('Assigned IRB Admin
      // // Toolbar')");
      // $('#navToolButton').attr("onclick",
      // "toggleNavTool('Assigned
      // R&D Admin Toolbar')");
      // * } else {
      // make sure executed for group size of more than 1
      // * row.appendChild(buildGeneralIRBAdminSelecter(irbAdminsJson));
      // * row.appendChild(buildCurrentAdminstrators());
      // * row.appendChild(generateStudyToolTabs(targetTool));
      // // select CurrentAdmin in the dropdown
      // if ((smcPlusIncumbents.currentAdministrator)
      // && (smcPlusIncumbents.currentAdministrator.user.SPUser)) {
      // $('#addAdmin')
      // .val(
      // smcPlusIncumbents.currentAdministrator.user.SPUser)
      // .change();
      // }
      // }
      // // });
      // }
      //
      // });

      // });
   } else if (targetTool == "reviewComments") {
      // 681460: Toolbars creating infinite loop: Commented out the
      // validSection check and recursive call because it
      // seemed like an infinite loop. In reality it was an attempt to wait
      // for the buttons on the study page to load.
      // Since it was slow the toolbar comment sections and questions
      // dropdowns do not get loaded properly. The
      // population of the dropdowns was moved to toggleNavToolbar.

      // if (validSection.length == 0) {
      // window.setTimeout(function() {
      // swapToolGenerator(targetTool);
      // }, 2000);
      // }
      // sendREST("GET",
      // restPath.concat("study/"+assignedID+"/comments/"+targetStage), null,
      // function(comments){
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         // row.appendChild(buildAdminComments());
         row.appendChild(buildMultiCommitteesComments());
         row.appendChild(buildReviewersComments());
         row.appendChild(generateReviewerToolTabs(targetTool));
         $('#insert-tool').append(row);
      });
   } else if (targetTool == "reviewChecklists") {
      sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('SiteAssets/Reviewer Checklists')/Files"),
         null, function(checklists) {
            sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID
               + "/Review%20Team')/Files"), null, function(returnedFiles) {
               row.appendChild(buildChecklistTemplates(checklists.d.results));
               row.appendChild(buildUploadedChecklists(returnedFiles.d.results, "Review%20Team"));
               row.appendChild(generateReviewerToolTabs(targetTool));
               $('#insert-tool').append(row);
            });
         });
   } else if (targetTool == "reviewerSubmit") {
      // sendREST("GET",
      // restPath.concat("study/"+assignedID+"/comments/"+targetStage), null,
      // function(comments){
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         // row.appendChild(buildSubmitComments(["Re-Submit to IRB
         // Administrator"]));
         row.appendChild(buildSubmitComments([ "Re-Submit to Administrator"
         ]));
         row.appendChild(buildAppHistory());
         row.appendChild(generateReviewerToolTabs(targetTool));
      });
      $('#insert-tool').append(row);
   } else if (targetTool == "localSite") {
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         row.appendChild(buildSubmitComments());
         row.appendChild(generateLocalSiteToolTabs(targetTool));
      });
      $('#insert-tool').append(row);
   } else if (targetTool == "CIRBAdmin") {
      commentsRollup(targetStage, function(comments) {
         mapComments(comments);
         // Custom Assign Study ID field, AND assignment to cIRB reviewer?
         row.appendChild(generateCIRBToolTabs(targetTool));
      });
      $('#insert-tool').append(row);
   }

   // $('#insert-tool').append(row);
}

/**
 * retrieve the study comments by calling the study/{id}/comments/{stage}/rollup service.
 * 
 * @param {number} stage number that represents the stage the study comments belong to.
 * @param {function} callback the function to return to when complete.
 */
function commentsRollup(stage, callback) {
   sendREST("GET", restPath + "study/" + assignedID + "/comments/" + stage + "/rollup", null, callback);
}

/**
 * Divide comments by user and puts them into the {@link appReviewerComments}
 * 
 * @param {Comments[]} array of comment objects
 */
function mapComments(comments) {
   var i, name;
   appReviewComments = new Hashtable();
   for (i = 0; i < comments.length; i++) {
      name = comments[i].user.displayName;
      if (!appReviewComments.containsKey(name)) {
         appReviewComments.put(name, "");
         reviewersComments.push(name);
      }
      appReviewComments.put(name, appReviewComments.get(name) + comments[i].hyperText + "\t" + comments[i].createdAt
         + "\n\n");
   }
}

/**
 * builds and returns a list of files that include only files with the name specified in the filter.
 * 
 * @param {string[]} checklists a list of files
 * @param {string} filter the value to search for in the checklists
 * @returns {filteredChecklists[]}
 */
function filterChecklists(checklists, filter) {
   var filteredChecklists = [];

   for (var i = 0; i < checklists.length; i++) {
      if ((checklists[i].Name).indexOf(filter) > -1) {
         filteredChecklists.push(checklists[i]);
      }
   }

   return filteredChecklists;
}

/**
 * builds the admin reviewer portion of the toolbar
 * 
 * @returns {container} container dom portion that represents the toolbar
 */
function buildAdminReviewers() {
   var container, tableDiv, table, tbody, trow, row, i, crow;
   // reviewerCollection = new HashSet();
   // var tempCollection;
   var reviewerCollection = [];

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /* Header Table */
   table = tableHeaderHelper("Current Reviewers", [ "Name", "Committee", "Role", "Remove"
   ]);
   row.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   /* table body */
   tbody = document.createElement("tbody");

   // reviewerCollection = buildUsers(irbReviewers);
   // tempCollection = reviewerCollection.values();

   /* Adding the primary reviewer to the reviewers collection */
   for (i = 0; i < COMMITTEETYPE.length; i++) {
      if (smcPlusIncumbents[COMMITTEETYPE[i]].currentPrimaryReviewer) {
         trow = document.createElement("tr");
         // trow.appendChild(textCellHelper("td",
         // smcPlusIncumbents[COMMITTEETYPE[i]].currentPrimaryReviewer.user.SPUser,
         // "SPU" + smcPlusIncumbents[COMMITTEETYPE[i]].currentPrimaryReviewer.user.SPUser, "SPU"));
         trow.appendChild(textCellHelper("td",
            smcPlusIncumbents[COMMITTEETYPE[i]].currentPrimaryReviewer.user.displayName));
         trow.appendChild(textCellHelper("td", COMMITTEETYPE[i]));
         trow.appendChild(textCellHelper("td", "Primary"));
         trow.appendChild(textCellHelper("td", "Primary Reviewers can only be replaced."));
         tbody.appendChild(trow);
         // reviewerCollection.push(smcPlusIncumbents[COMMITTEETYPE[i]].currentPrimaryReviewer.user);
      }
   }
   // * if (smcPlusIncumbents.currentPrimaryReviewer) {
   // * reviewerCollection.push(smcPlusIncumbents.currentPrimaryReviewer.user);
   // * }

   // if (spAppData.PrimaryReviewersId != null) {
   // // grabbing current reviewer's names from their IDs
   // tempCollection = ramsUsers.values();
   // reviewerCollection = [];
   //
   // for (var j = 0; j < tempCollection.length; j++) {
   // if (spAppData.PrimaryReviewersId.results == tempCollection[j].Id) {
   // reviewerCollection.push(tempCollection[j]);
   // }
   // }

   /*
    * Building the Reviewers Table based on users who have been added to the
    * reviewers collection
    */
   // * for (i = 0; i < reviewerCollection.length; i++) {
   // * trow = document.createElement("tr");
   // * trow.appendChild(textCellHelper("td", reviewerCollection[i].SPUser,
   // * "SPU" + reviewerCollection[i].SPUser, "SPU"));
   // * trow
   // * .appendChild(textCellHelper("td",
   // * reviewerCollection[i].displayName));
   // * trow.appendChild(textCellHelper("td", "Primary"));
   // * trow.appendChild(textCellHelper("td",
   // * "Primary Reviewers can only be replaced."));
   // * //* trow.appendChild(buttonCellHelper("Remove",
   // * //* "removePrimaryReviewer('"+reviewerCollection[i].Id+"')"));
   // * tbody.appendChild(trow);
   // * }
   // }
   reviewerCollection = [];
   // * if (smcPlusIncumbents.currentAssignedReviewers
   // * && (smcPlusIncumbents.currentAssignedReviewers.length > 0)) {
   // * for ( var car in smcPlusIncumbents.currentAssignedReviewers) {
   // * if (smcPlusIncumbents.currentAssignedReviewers.hasOwnProperty(car)) {
   // * reviewerCollection
   // * .push(smcPlusIncumbents.currentAssignedReviewers[car].user);
   // * }
   // * }
   // * }

   for (i = 0; i < COMMITTEETYPE.length; i++) {
      if (smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers) {
         for ( var car in smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers) {
            if (smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers.hasOwnProperty(car)) {
               trow = document.createElement("tr");
               // trow.appendChild(textCellHelper("td",
               // smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers[car].user.SPUser,
               // "SPU" + smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers[car].user.SPUser, "SPU"));
               trow.appendChild(textCellHelper("td",
                  smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers[car].user.displayName));
               trow.appendChild(textCellHelper("td", COMMITTEETYPE[i]));
               trow.appendChild(textCellHelper("td", "Assigned"));
               trow.appendChild(buttonCellHelper("Remove", "removeAssignedReviewer('"
                  + smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers[car].user.SPUser + "')"));
               tbody.appendChild(trow);
               // reviewerCollection
               // .push(smcPlusIncumbents[COMMITTEETYPE[i]].currentAssignedReviewers[car].user);
            }
         }
      }
   }

   // /* Adding assigned reviewers to the the reviewers collection */
   // if (smcPlusIncumbents.currentAssignedReviewers &&
   // smcPlusIncumbents.currentAssignedReviewers.length > 0) {
   // // grabbing current reviewer's names from their IDs
   // tempCollection = ramsUsers.values();
   // reviewerCollection = [];
   // // console.log("tempCollection: " + tempCollection);
   // // console.log("spAppData.ReviewersId.length " +
   // spAppData.ReviewersId.results.length);
   // for (var i = 0; i < spAppData.ReviewersId.results.length; i++) {
   // for (var j = 0; j < tempCollection.length; j++) {
   // if (spAppData.ReviewersId.results[i] == tempCollection[j].Id) {
   // reviewerCollection.push(tempCollection[j]);
   // }
   // }
   // }

   /*
    * Building the Reviewers Table based on users who have been added to the
    * reviewers collection
    */
   // * for (i = 0; i < reviewerCollection.length; i++) {
   // * trow = document.createElement("tr");
   // * trow.appendChild(textCellHelper("td", reviewerCollection[i].SPUser,
   // * "SPU" + reviewerCollection[i].SPUser, "SPU"));
   // * trow
   // * .appendChild(textCellHelper("td",
   // * reviewerCollection[i].displayName));
   // * trow.appendChild(textCellHelper("td", "Assigned"));
   // * trow.appendChild(buttonCellHelper("Remove", "removeAssignedReviewer('"
   // * + reviewerCollection[i].SPUser + "')"));
   // * tbody.appendChild(trow);
   // * }
   // }
   // BEGIN FROM SUSTAINMENT 2.1.1
   // Add CIRB custom id
   crow = document.createElement("tr");
   crow.appendChild(textCellHelper("td", "CIRB ID"));
   var cirbText = textCellHelper('input', "CIRB ID", "cirb_id");
   cirbText.value = (spAppData.CIRB_ID === null ? "" : spAppData.CIRB_ID);
   crow.appendChild(cirbText);
   tbody.appendChild(crow);
   // END FROM SUSTAINMENT 2.1.1

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   row.appendChild(tableDiv);
   container.appendChild(row);
   var rows = document.getElementsByName("SPU");
   for (i = 0; i < rows.length; i++) {
      rows[i].style.display = "none";
   }

   return container;
}

/**
 * builds a button with the provide attributes
 * 
 * @param {string} buttonType the name of the button
 * @param {string} onclickFunc the function to call in the onclick event
 * @param {string} id the id to be assigned to the button (optional)
 * @param {boolean} disabled whether the button should be enabled or disabled (optional)
 * @returns {Object} cell the dom element representing the button
 */
function buttonCellHelper(buttonType, onclickFunc, id, disabled) {
   var cell, button, text;

   cell = document.createElement("td");
   cell.setAttribute("align", "center");

   button = document.createElement("button");
   button.setAttribute("type", "button");
   button.setAttribute("class", "btn btn-default btn-xs");
   button.setAttribute("title", buttonType);
   button.setAttribute("onclick", onclickFunc);

   if (id) {
      button.setAttribute("id", id);
   }
   if (disabled) {
      button.setAttribute("disabled", true);
   }

   text = document.createTextNode(buttonType);

   button.appendChild(text);
   cell.appendChild(button);

   return cell;
}

/**
 * builds a select dropdown with the provided attributes
 * 
 * @param {string} id the id to be assigned to the control
 * @param {string} labelText the text for the dropdown's label
 * @param {string} displayOptions array of options to add to the option element
 * @param {string} onChangeFunc the function to call in the onChangeFunc event (optional)
 * @param {string} disabled whether the control should be enabled or disabled (optional)
 * @returns {Object} cell the dom element representing the control
 */
/* exported selectCellHelper */
function selectCellHelper(id, labelText, displayOptions, onChangeFunc, disabled) {
   var cell, label, text, select, option;

   cell = document.createElement("td");
   cell.setAttribute("align", "center");

   label = document.createElement("label");
   label.setAttribute("class", "sr-only");
   label.setAttribute("for", id);

   text = document.createTextNode(labelText);
   label.appendChild(text);

   cell.appendChild(label);

   select = document.createElement("select");
   select.setAttribute("class", "form-control");
   select.setAttribute("id", id);

   if (onChangeFunc) {
      select.setAttribute("onchange", onChangeFunc);
   }

   if (disabled) {
      select.setAttribute("disabled", true);
   }

   for (var i = 0; i < displayOptions.length; i++) {
      option = document.createElement("option");
      option.setAttribute("value", displayOptions[i]);

      text = document.createTextNode(displayOptions[i]);
      option.appendChild(text);
      select.appendChild(option);
   }

   cell.appendChild(select);
   return cell;
}

/**
 * builds a checkbox with the provided attributes
 * 
 * @param {string} id the id to be assigned to the control
 * @param {string} labelText labelText the text for the dropdown's label
 * @param {string} onclickFunc onclickFunc the function to call in the onclick event
 * @returns {string} cell the dom element representing the control
 */
function checkboxCellHelper(id, labelText, onclickFunc) {
   // var cell, checkbox, label, id, text;
   var cell, checkbox, label, text;

   cell = document.createElement("td");
   // cell.setAttribute("align", "left");

   label = document.createElement("label");
   label.setAttribute("class", "sr-only");
   label.setAttribute("for", id);

   text = document.createTextNode(labelText);
   label.appendChild(text);

   cell.appendChild(label);

   checkbox = document.createElement("input");
   checkbox.setAttribute("id", id);
   checkbox.setAttribute("type", "checkbox");

   if (onclickFunc) {
      checkbox.setAttribute("onclick", onclickFunc);
   }

   cell.appendChild(checkbox);

   return cell;
}

/**
 * builds a text element with the provided attributes
 * 
 * @param {string} type the type of control to create. valid values are:
 *           <ul>
 *           <li>input
 *           <li>td
 *           <li>th
 *           <li>tr
 *           </ul>
 * @param {string} displayText the text added to the TextNode (optional)
 * @param {string} id the id to be assigned to the control (optional)
 * @param {string} name the value set in the name attribute (optional)
 * @returns {Object} cell the dom element representing the control
 */
function textCellHelper(type, displayText, id, name) {
   var cell, text;

   cell = document.createElement(type);
   cell.setAttribute("align", "center");
   cell.setAttribute("tabindex", "0");

   if (displayText) {
      text = document.createTextNode(displayText);
   } else {
      text = document.createTextNode("");
   }

   if (id) {
      cell.setAttribute("id", id);
   }
   if (name) {
      cell.setAttribute("name", name);
   }

   cell.appendChild(text);

   return cell;
}

// hoping to use this to re-write the code, but would have to add in
// tool-tip and required astrick as well.
/**
 * builds a text form element with the provided attributes
 * 
 * @param {string} id the id to be assigned to the control
 * @param {string} displayText the text added to the TextNode
 * @param {string} onChangeFunc the function to call in the onChangeFunc event (optional)
 * @returns {Object} form the dom element representing the control
 */
/* exported textFieldHelper */
function textFieldHelper(id, displayText, onChangeFunc) {
   var form, formGroup, label, input;

   form = document.createElement("form");
   form.setAttribute("role", "form");

   formGroup = document.createElement("div");
   formGroup.setAttribute("class", "form-group");

   label = document.createElement("label");
   label.setAttribute("for", id);
   formGroup.appendChild(label);

   var text = document.createTextNode(displayText);
   formGroup.appendChild(text);

   input = document.createElement("input");
   input.setAttribute("name", id);
   input.setAttribute("class", "form-control normal");
   input.setAttribute("id", id);
   input.setAttribute("type", "text");
   if (onChangeFunc) {
      input.setAttribute("onchange", onChangeFunc);
   }
   formGroup.appendChild(input);

   form.appendChild(formGroup);

   return form;
}

// see description for textFieldHelper. Would also need to add id's
// for each radio button
/**
 * builds a text form element with the provided attributes
 * 
 * @param {string} id the id to be assigned to the control
 * @param {string} displayText the text added to the TextNode
 * @param {string} onChangeFunc the function to call in the onChangeFunc event (optional)
 * @returns {Object} form the dom element representing the control
 */
/* exported radioFieldHelper */
function radioFieldHelper(id, displayText, radioQuestions) {
   var form, formGroup, fieldSet, legend, text, input, label;

   form = document.createElement("form");

   formGroup = document.createElement("div");
   formGroup.setAttribute("class", "form-group");

   fieldSet = document.createElement("fieldset");

   legend = document.createElement("legend");
   text = document.createTextNode(displayText);
   legend.appendChild(text);
   fieldSet.appendChild(legend);

   for (var i = 0; i < radioQuestions.length; i++) {
      label = document.createElement("label");
      label.setAttribute("class", "radio");
      label.setAttribute("for", id + "_" + i);

      input = document.createElement("input");
      input.setAttribute("name", id);
      input.setAttribute("id", id + "_" + i);
      input.setAttribute("type", "radio");
      // if onclick and/or onchange
      label.appendChild(input);

      text = document.createTextNode(radioQuestions[i]);
      label.appendChild(text);

      fieldSet.appendChild(label);
   }

   formGroup.appendChild(fieldSet);
   form.appendChild(formGroup);

   return form;
}

/**
 * builds a select form element with the provided attributes
 * 
 * @param {string} id the id to be assigned to the control
 * @param {string} displayText the text added to the TextNode
 * @param {options[]} selectOptions the options added to the element
 * @param {string} onChangeFunc the function to call in the onChangeFunc event (optional)
 * @returns {Object} form the dom element representing the control
 */
/* exported selectFieldHelper */
function selectFieldHelper(id, displayText, selectOptions, onChangeFunc) {
   var form, formGroup, label, text, select, option;

   form = document.createElement("form");
   form.setAttribute("role", "form");

   formGroup = document.createElement("div");
   formGroup.setAttribute("class", "form-group");

   label = document.createElement("label");
   label.setAttribute("for", id);

   text = document.createTextNode(displayText);
   label.appendChild(text);
   formGroup.appendChild(label);

   select = document.createElement("select");
   select.setAttribute("id", id);
   select.setAttribute("class", "form-control normal select");
   if (onChangeFunc) {
      select.setAttribute("onchange", onChangeFunc);
   }

   for (var i = 0; i < selectOptions.length; i++) {
      option = document.createElement("option");
      // If the target is a user selection field and assuming the first entry
      // is guidance text
      // "Please Select a User",
      // add the users to the selection with SPUserId and displayName
      if ((id == "initPerson") && (i !== 0)) {
         option.setAttribute("value", selectOptions[i].Id);
         text = document.createTextNode(selectOptions[i].Title);
      } else {
         option.setAttribute("value", selectOptions[i]);
         text = document.createTextNode(selectOptions[i]);
      }

      option.appendChild(text);
      select.appendChild(option);
   }

   formGroup.appendChild(select);
   form.appendChild(formGroup);

   return form;
}

/**
 * builds a link cell element with the provided attributes
 * 
 * @param {string} type the type of control to create. valid values are:
 * @param {string} displayText the text added to the TextNode
 * @param {string} teamFolder the name of the folder to create a link to
 * @param {string} linkName the name of the link
 * @param {string} target sets the link's target
 * @returns {Object} cell the dom element representing the control
 */
function linkCellHelper(type, displayText, teamFolder, linkName, target) {
   var cell, text, link;

   cell = document.createElement(type);
   cell.setAttribute("align", "center");

   link = document.createElement("a");
   if (teamFolder && linkName) {
      link.setAttribute("href", siteUrl.concat("Shared%20Documents/" + assignedID + "/" + teamFolder + "/" + linkName));
   } else {
      link.setAttribute("href", linkName);
   }

   if (target) {
      link.target = target; // open link based on target selected
   }
   link.style.cursor = "pointer";

   text = document.createTextNode(displayText);
   link.appendChild(text);

   cell.appendChild(link);

   return cell;
}

/**
 * builds a textarea cell element with the provided attributes
 * 
 * @param {string} type the type of control to create.
 * @param {string} id the id to be assigned to the control
 * @param {string} labelText the text added to the TextNode
 * @returns {Object} cell the dom element representing the control
 */
/* exported textareaCellHelper */
function textareaCellHelper(type, id, labelText) {
   var cell, text, label, input;

   cell = document.createElement(type);
   cell.setAttribute("align", "center");

   label = document.createElement("label");
   label.setAttribute("class", "sr-only");
   label.setAttribute("for", id);

   text = document.createTextNode(labelText);
   label.appendChild(text);

   cell.appendChild(label);

   input = document.createElement("input");
   input.setAttribute("class", "form-control");
   input.setAttribute("id", id);

   cell.appendChild(input);

   return cell;
}

/**
 * builds the administrator dropdown with the provided administrators
 * 
 * @param {irbAdmins[]} irbAdmins a collection of administrators to add to the dropdown selecter
 * @returns {Object} container the dom element representing the control
 */
function buildGeneralIRBAdminSelecter() {
   var committeeSelecter, container;
   // var tempCollection;
   var adminSelecter, row;
   // var availableAdmins = []; // array of administrator names
   adminCollection = new HashSet(); // global variable, similar to
   // associative array
   // var currentAdmin = null;
   // var smc = smcPlusIncumbents.smc;

   getCommitteeList(function(committeeList) {
      container = document.createElement("div");
      container.setAttribute("class", "col-xs-4 col-md-offset-1 selecter-div");
      row = document.createElement("div");
      row.setAttribute("class", "row selecter-text");
      committeeSelecter = buildSelecterHelper("selectCommittee", "Committee: ", [ "Select Committee"
      ].concat(committeeList), "12", "populateAdminAndReviewerDropDowns('selectCommittee', 'addAdmin')", false, "yes");

      row.appendChild(committeeSelecter); // add dropdown to row
      container.appendChild(row);
      adminSelecter = buildSelecterHelper("addAdmin", "Administrator: ", [ "Please Select an Administrator"
      ], "12", "", true, "yes");
      row.appendChild(adminSelecter); // add dropdown to row
      container.appendChild(row);

      var selectedCommittee = $("#selectCommittee").find(':selected').val();
      if (selectedCommittee) {
         $('#addAdmin').val(smcPlusIncumbents[selectedCommittee].currentAdministrator.user.SPUser).change();
      }
   });

   return container;

   // var userExistinIRBAdminGrp = false;
   // var userExistinIACUCAdminGrp = false;
   // var userExistinRDCAdminGrp = false;
   // var userExistinSRSAdminGrp = false;
   // var userExistinSC1Grp = false;
   // var userExistinSC2Grp = false;
   // var userExistinSC3Grp = false;
   // var committeeList = [];
   //
   // if ((committeeTable.containsKey(assignedID))) {
   // committeeList = committeeTable.get(assignedID);
   // } else {
   // $
   // .ajax({
   // async : false,
   // type : 'GET',
   // url : siteUrl.concat("_api/Web/CurrentUser/Groups"),
   // headers : {
   // "accept" : "application/json;odata=verbose"
   // },
   // dataType : 'json',
   // success : function(returnedUserGroups) {
   // currentUsers = returnedUserGroups.d.results;
   // var currentUserGroups = returnedUserGroups.d.results;
   // for (var i = 0; i < currentUserGroups.length; i++) {
   // if (currentUserGroups[i].Title
   // .indexOf("RDC Administrator") > -1) {
   // userExistinRDCAdminGrp = true;
   // } else if (currentUserGroups[i].Title
   // .indexOf("IRB Administrator") > -1) {
   // userExistinIRBAdminGrp = true;
   // } else if (currentUserGroups[i].Title
   // .indexOf("IACUC Administrator") > -1) {
   // userExistinIACUCAdminGrp = true;
   // } else if (currentUserGroups[i].Title
   // .indexOf("SRS Administrator") > -1) {
   // userExistinSRSAdminGrp = true;
   // } else if (currentUserGroups[i].Title
   // .indexOf("SC1 Administrator") > -1) {
   // userExistinSC1Grp = true;
   // } else if (currentUserGroups[i].Title
   // .indexOf("SC2 Administrator") > -1) {
   // userExistinSC2Grp = true;
   // } else if (currentUserGroups[i].Title
   // .indexOf("SC3 Administrator") > -1) {
   // userExistinSC3Grp = true;
   // }
   // }
   //
   // if (spAppData.IRBReviewRequired === true
   // && userExistinIRBAdminGrp === true) {
   // committeeList.push(IRB);
   // }
   // if (spAppData.RDCReviewRequired === true
   // && userExistinRDCAdminGrp === true) {
   // committeeList.push(RDC);
   // }
   // if (spAppData.IACUCReviewRequired === true
   // && userExistinIACUCAdminGrp === true) {
   // committeeList.push(IACUC);
   // }
   // if (spAppData.SRSReviewRequired === true
   // && userExistinSRSAdminGrp === true) {
   // committeeList.push(SRS);
   // }
   // if (spAppData.SC1ReviewRequired === true
   // && userExistinSC1Grp === true) {
   // committeeList.push(SC1);
   // }
   // if (spAppData.SC2ReviewRequired === true
   // && userExistinSC1Grp === true) {
   // committeeList.push(SC2);
   // }
   // if (spAppData.SC3ReviewRequired === true
   // && userExistinSC1Grp === true) {
   // committeeList.push(SC3);
   // }
   //
   // // store in committeeTable
   // committeeTable.put(assignedID, committeeList);
   //
   // },
   // error : function(XMLHttpRequest, textStatus, errorThrown) {
   // logError(siteUrl.concat(uri), errorThrown, textStatus,
   // JSON.stringify(XMLHttpRequest));
   // }
   // });
   // }
   //
   // container = document.createElement("div");
   // container.setAttribute("class", "col-xs-4 col-md-offset-3 selecter-div");
   //
   // row = document.createElement("div");
   // row.setAttribute("class", "row selecter-text");
   //
   // // adminCollection = buildUsers(irbAdmins); // parse the JSON data for
   // // // administrators
   // // tempCollection = adminCollection.values();
   // // for (var i = 0; i < tempCollection.length; i++) {
   // // if ((smcPlusIncumbents.currentAdministrator)
   // // && (tempCollection[i].Id == smcPlusIncumbents.currentAdministrator.user.SPUser)) {
   // // currentAdmin = tempCollection[i];
   // // availableAdmins.push(tempCollection[i]);
   // // } else {
   // // // if ((smcPlusIncumbents.currentPI) && (tempCollection[i].Id !=
   // // // smcPlusIncumbents.currentPI.user.SPUser)
   // // // && (!(smcPlusIncumbents.currentCoPI) ||
   // // // (smcPlusIncumbents.currentCoPI) && (tempCollection[i].Id !=
   // // // smcPlusIncumbents.currentCoPI.user.SPUser))){
   // // if ((smc[tempCollection[i].Id] === undefined)
   // // || ((smcPlusIncumbents.currentAdministrator) && (tempCollection[i].Id ==
   // smcPlusIncumbents.currentAdministrator.user.SPUser))) {
   // // availableAdmins.push(tempCollection[i]);
   // // }
   // // // }
   // // }
   // // }
   // // // sort array before adding values to dropdown
   // // availableAdmins.sort();
   //
   // // TODO TAR the following 3 lines were commented out in tst
   // // if(currentAdmin != null){
   // // adminSelecter = buildSelecterHelper("addAdmin", "Administrator: ", ["The
   // // Current
   // // Administrator is: " +
   // // currentAdmin.Title].concat(availableAdmins), "12");
   // // }else{
   // committeeSelecter = buildSelecterHelper("selectCommittee", "Committee: ",
   // [ "Select Committee" ].concat(committeeList), "12",
   // "getAdminForGivenCommittee('selectCommittee', 'addAdmin')", false,
   // "yes");
   //
   // row.appendChild(committeeSelecter); // add dropdown to row
   // container.appendChild(row);
   // // adminSelecter = buildSelecterHelper("addAdmin", "Administrator: ", [
   // // "Please Select an Administrator"
   // // ].concat(availableAdmins), "12");
   //
   // adminSelecter = buildSelecterHelper("addAdmin", "Administrator: ",
   // [ "Please Select an Administrator" ], "12", "", true, "yes");
   // // TODO TAR the following 1 lines were commented out in tst
   // // }
   //
   // row.appendChild(adminSelecter); // add dropdown to row
   // container.appendChild(row);
   // // Attempt to select and show the item in the dropdown
   // // if (currentAdmin) {
   // // for (var j = 0; j < availableAdmins.length; j++) {
   // // if (availableAdmins[j].Id == currentAdmin.Id) {
   // // $('#addAdmin').val(availableAdmins[j].Id).change();
   // // // $('#addAdmin').change();
   // // // var selectedValue = $('#addAdmin').val();
   // // break;
   // // }
   // // }
   // // }
   //
   // return container;
}

/**
 * builds the approval toolbar
 * 
 * @returns {Object} container the dom element representing the control
 */
function buildPostApproveReview() {
   var container, row, date, label, text, table, cell, trow;
   var risklevelOptions = [];
   var risklevelSelecter;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 col-md-offset-1 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   table = document.createElement("table");
   table.setAttribute("class", "table");

   trow = document.createElement("tr");
   cell = document.createElement("td");

   label = document.createElement("label");
   label.setAttribute("class", "control-label");
   label.setAttribute("for", "continueReview");

   text = document.createTextNode("Set Renewal Date:");
   label.appendChild(text);

   cell.appendChild(label);
   trow.appendChild(cell);

   cell = document.createElement("td");

   date = document.createElement("input");
   date.setAttribute("type", "text");
   date.setAttribute("id", "continueReview");
   date.setAttribute("data-provide", "datepicker");
   date.setAttribute("date-date-format", "yyyy/mm/dd");
   date.setAttribute("class", "form-control datepicker");

   if ((typeof spAppData.RenewalDate !== "undefined") && (spAppData.RenewalDate !== null)) {
      var renewalDate = spAppData.RenewalDate.split("T")[0];
      renewalDate = renewalDate.split("-");
      renewalDate = renewalDate[1] + "/" + renewalDate[2] + "/" + renewalDate[0];
      date.setAttribute("value", renewalDate);
   }

   cell.appendChild(date);
   trow.appendChild(cell);
   table.appendChild(trow);

   if (grabApplicationSiteType() == "Central") {
      trow = document.createElement("tr");
      cell = document.createElement("td");

      label = document.createElement("label");
      label.setAttribute("class", "control-label");
      label.setAttribute("for", "commentPeriod");

      text = document.createTextNode("Select Comment Period Start Date:");
      label.appendChild(text);

      cell.appendChild(label);
      trow.appendChild(cell);
      cell = document.createElement("td");

      date = document.createElement("input");
      date.setAttribute("type", "text");
      date.setAttribute("id", "commentPeriod");
      date.setAttribute("data-provide", "datepicker");
      date.setAttribute("date-date-format", "yyyy/mm/dd");
      date.setAttribute("class", "form-control");

      if ((typeof spAppData.OData__x0031_5DayCommentPeriod !== "undefined")
         && (spAppData.OData__x0031_5DayCommentPeriod !== null)) {
         var commentPeriod = spAppData.OData__x0031_5DayCommentPeriod.split("T")[0];
         commentPeriod = commentPeriod.split("-");
         commentPeriod = commentPeriod[1] + "/" + commentPeriod[2] + "/" + commentPeriod[0];
         date.setAttribute("value", commentPeriod);
      }

      // date = document.createElement("button");
      // date.setAttribute("type", "button");
      // date.setAttribute("class", "btn btn-default");
      // date.setAttribute("id", "commentPeriod");

      // text = document.createTextNode("Start");
      // date.appendChild(text);

      cell.appendChild(date);
      trow.appendChild(cell);
      table.appendChild(trow);
   }

   row.appendChild(table);
   container.appendChild(row);

   /* User can select risk level */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   risklevelOptions.push("Select Risk Level", "High", "Low", "None");

   risklevelSelecter = buildSelecterHelper("selectRiskLevel", "Select Risk Level: ", risklevelOptions, "12");
   row.appendChild(risklevelSelecter);
   container.appendChild(row); // add dropdown

   return container;
}

// function dateHelper(){
// var container, date, label;

// container = document.createElement("div")

// date = document.createElement("input");
// date.setAttribute("type", "text");
// date.setAttribute("data-provide", "datepicker");

// cell.appendChild(date);

// return cell;
// }

/**
 * builds all of the selecters in the Assignments tab
 * 
 * @param {irbAdmins[]} irbAdmins array of administrators
 * @param {availableReviewers[]} availableReviewers array of reviewers
 * @returns {object} selectersDiv the dom element representing the control
 */
function buildAdminSelecters(irbAdmins, availableReviewers) {
   /* jshint unused:false */

   var committeeSelecter, selectersDiv, adminSelecter, reviewerSelecter, reviewMethod, row, roleSelecter;
   // var tempCollection, i;
   // var availableAdmins = []; // array of administrator names
   // var smc = smcPlusIncumbents.smc;

   // adminCollection = new HashSet(); // global variable, similar to
   // associative array
   // var currentAdmin = null;
   getCommitteeList(function(committeeList) {

      selectersDiv = document.createElement("div");
      selectersDiv.setAttribute("class", "col-xs-4 col-md-offset-1 selecter-div");
      row = document.createElement("div");
      row.setAttribute("class", "row selecter-text");
      committeeSelecter = buildSelecterHelper("selectCommittee", "Committee: ", [ "Select Committee"
      ].concat(committeeList), "12", "populateAdminAndReviewerDropDowns('selectCommittee', 'addAdmin','addReviewer')",
         false, "yes");

      row.appendChild(committeeSelecter); // add dropdown to row
      selectersDiv.appendChild(row);
      /* IRB Admin drop down */
      row = document.createElement("div");
      row.setAttribute("class", "row selecter-text");

      // // collect admin names for dropdown
      // adminCollection = buildUsers(irbAdmins);
      // tempCollection = adminCollection.values();
      // for (i = 0; i < tempCollection.length; i++) {
      // // if ((typeof smcPlusIncumbents.currentAdministrator !== "undefined")
      // // && (tempCollection[i].Id == smcPlusIncumbents.currentAdministrator.user.SPUser)) {
      // // currentAdmin = tempCollection[i];
      // availableAdmins.push(tempCollection[i]);
      // // } else if (typeof smc[tempCollection[i].Id] === "undefined") {
      // // availableAdmins.push(tempCollection[i]);
      // // }
      // }

      // // sort arrays before adding values to dropdown
      // availableAdmins.sort(function(a, b) {
      // return (a.Title > b.Title) ? 1 : ((b.Title > a.Title) ? -1 : 0);
      // });
      // availableReviewers.sort(function(a, b) {
      // return (a.Title > b.Title) ? 1 : ((b.Title > a.Title) ? -1 : 0);
      // });

      adminSelecter = buildSelecterHelper("addAdmin", "Administrator: ",
      // [ "Please Select an Administrator" ].concat(availableAdmins), "12");
      [ "Please Select an Administrator"
      ], "12", "", true, "yes");
      row.appendChild(adminSelecter);
      selectersDiv.appendChild(row);
      // Attempt to select and show the item in the dropdown
      // if (typeof smcPlusIncumbents.currentAdministrator !== "undefined") {
      // $('#addAdmin').val(smcPlusIncumbents.currentAdministrator.user.SPUser)
      // .change();
      // }

      // var selectedCommittee = $("#selectCommittee").find(':selected').val();
      // if(selectedCommittee){
      // $('#addAdmin').val(smcPlusIncumbents[selectedCommittee].currentAdministrator.user.SPUser).change();
      // }

      /* Add reviewer and assign a role */
      row = document.createElement("div");
      row.setAttribute("class", "row selecter-text");

      reviewerSelecter = buildSelecterHelper("addReviewer", "Reviewer: ",
      // [ "Please Select a Reviewer" ].concat(availableReviewers), "6");
      [ "Please Select a Reviewer"
      ], "6", "", true, "yes");
      row.appendChild(reviewerSelecter);

      roleSelecter = buildSelecterHelper("assignRole", "Role: ", [ "Primary", "Assigned"
      ], "6", "", true);
      row.appendChild(roleSelecter);
      selectersDiv.appendChild(row);

      /* compliant method of review drop down */
      row = document.createElement("div");
      row.setAttribute("class", "row selecter-text");

      if ((typeof spAppData.ApplicationType !== "undefined") && (spAppData.ApplicationType !== null)) {
         var temp = spAppData.ApplicationType.split(' ');
         spAppData.Checkboxes = temp[1];
         spAppData.ApplicationType = temp[0];
         reviewMethod = buildSelecterHelper("assignReviewMethod", "Compliant Method of Review: ", [
            "The Current Compliant Method of Review is: " + spAppData.ApplicationType + "", "Exempt", "Expedited",
            "Full"
         ], "12", "selectCheckBoxes('assignReviewMethod','reviewMethodCategory')", true);
      } else {
         reviewMethod = buildSelecterHelper("assignReviewMethod", "Compliant Method of Review: ", [
            "Please Select a Compliant Method of Review", "Exempt", "Expedited", "Full"
         ], "12", "selectCheckBoxes('assignReviewMethod','reviewMethodCategory')", true);
      }

      row.appendChild(reviewMethod);
      selectersDiv.appendChild(row);

      /* Additional Category Information dropdown */
      row = document.createElement("div");
      row.setAttribute("class", "row selecter-text");
      row.setAttribute("id", "reviewMethodCategory");
      row.style.fontSize = "10px";
      row.style.lineHeight = "1.0";
      selectersDiv.appendChild(row);

      // if(committeeList.length === 1){
      // $('#selectCommittee').val(committeeList[0]);
      // populateAdminAndReviewerDropDowns('selectCommittee', 'addAdmin','addReviewer', committeeList[0]);
      // }

   });

   return selectersDiv;
}

/**
 * builds the list of checkboxes for Expedited and Exempt Compliant Method of Review
 * 
 * @param {string} selectID the id of the compliant method of review
 * @param {string} insertID the id of the CMOD checkboxes
 */
/* exported selectCheckBoxes */
function selectCheckBoxes(selectID, insertID) {
   var selectOptions;
   var tempChoice;
   var chkResponses = spAppData.Checkboxes;

   if (selectID == '0') {
      tempChoice = spAppData.ApplicationType;
   } else {
      tempChoice = $('#' + selectID).val();
   }

   if (tempChoice.indexOf("Expedited") != -1) {
      selectOptions = RAMS.Environment.SharePoint.List.Application.ExpeditedCategories;
   } else if (tempChoice.indexOf("Exempt") != -1) {
      selectOptions = RAMS.Environment.SharePoint.List.Application.ExemptCategories;
   } else {
      selectOptions = [];
   }

   $("#" + insertID).empty();
   selectOptions.forEach(function(o, s) {
      var opt = checkboxCellHelper("chkOpt" + s, o);
      $("#" + insertID).append(opt);
      $("#" + insertID).append(textCellHelper("td", o));
      if ((s + 1) % 2 === 0) {
         $("#" + insertID).append(document.createElement('br'));
      }
   });

   if ((typeof chkResponses !== "undefined") && (chkResponses !== null)
      && (chkResponses.length == selectOptions.length)) {
      for (var i = 0; i < chkResponses.length; i++) {
         if (chkResponses.charAt(i) == "*") {
            $("#chkOpt" + i).prop('checked', true);
         }
      }
   }
}

/**
 * helps to build the selecters in the assignments tab
 * 
 * @param {string} id the id to be assigned to the control
 * @param {string} displayText the text added to the TextNode
 * @param {string} displayOptions the text to the select dropdowns
 * @param {string} length the size of the bootstrap number of cells
 * @param {string} onChangeFunc function to call onChange (optional)
 * @param {string} disabled the text added to the TextNode (optional)
 * @param {string} required the text added to the TextNode (optional)
 * @returns {Object} selecterDiv the dom element representing the control
 */
function buildSelecterHelper(id, displayText, displayOptions, length, onChangeFunc, disabled, required) {
   var selecterDiv, label, selecter, option, text;
   var locDisabled = false;
   var locRequired = false;

   if (required === undefined) {
      if (typeof disabled === "boolean") {
         locDisabled = disabled;
      } else if ((typeof disabled === "string") && (disabled.toLowerCase() == "yes")) {
         locRequired = true;
      }
   } else if (required.toLowerCase() == "yes") {
      locRequired = true;
   }

   if (disabled !== undefined) {
      if (typeof disabled === "boolean") {
         locDisabled = disabled;
      } else if ((typeof disabled === "string") && (disabled.toLowerCase() == "yes")) {
         locRequired = true;
      }
   }

   selecterDiv = document.createElement("div");
   selecterDiv.setAttribute("class", "col-xs-" + length);

   if (locRequired) {
      var reqSpan = document.createElement("span");
      reqSpan.setAttribute("class", "required");
      reqSpan.setAttribute("aria-hidden", "true");
      reqSpan.textContent = "*";
      selecterDiv.appendChild(reqSpan);

      reqSpan = document.createElement("span");
      reqSpan.setAttribute("class", "required sr-only");
      reqSpan.textContent = "A response is required.";
      selecterDiv.appendChild(reqSpan);
   }

   label = document.createElement("label");
   label.setAttribute("for", id);
   label.setAttribute("class", "control-label");

   text = document.createTextNode(displayText);
   label.appendChild(text);
   selecterDiv.appendChild(label);

   selecter = document.createElement("select");
   selecter.setAttribute("class", "form-control");
   selecter.setAttribute("id", id);

   if (onChangeFunc) {
      selecter.setAttribute("onchange", onChangeFunc);
   }

   if (locDisabled) { // if disabled argument is set to true
      selecter.setAttribute("disabled", locDisabled); // disable dropdown
   }

   for (var i = 0; i < displayOptions.length; i++) {
      option = document.createElement("option");
      // If the target is a user selection field and assuming the first entry
      // is guidance text
      // "Please Select a User",
      // add the users to the selection with SPUserId and displayName
      if (((id == "addAdmin") || (id == "addReviewer")) && (i !== 0)) {
         option.setAttribute("value", displayOptions[i].Id);
         text = document.createTextNode(displayOptions[i].Title);
      } else {
         option.setAttribute("value", displayOptions[i]);
         text = document.createTextNode(displayOptions[i]);
      }
      option.appendChild(text);
      selecter.appendChild(option);
   }

   selecterDiv.appendChild(selecter);
   return selecterDiv;
}

/**
 * builds the comments textarea
 * 
 * @param {string} id the id to be assigned to the control
 * @param {string} spacing the value that will be set as the class attribute
 * @param {string} displayText the text added to the TextNode
 * @param {publishedComments[]} publishedComments the comments that will be added as text
 * @returns {Object} container the dom element representing the control
 */
/* exported generateViewgenComments */
function generateViewgenComments(id, spacing, displayText, publishedComments) {
   var container, label, textarea, text, row;

   container = document.createElement("div");
   container.setAttribute("class", spacing);

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   label = document.createElement("label");
   label.setAttribute("for", id);

   text = document.createTextNode(displayText);
   label.appendChild(text);
   row.appendChild(label);

   textarea = document.createElement("textarea");
   textarea.setAttribute("id", id);
   textarea.setAttribute("class", "form-control");
   textarea.setAttribute("rows", "10");

   if ((typeof publishedComments !== "undefined") && (publishedComments !== null)) {
      for (var i = 0; i < publishedComments.length; i++) {
         text = document.createTextNode(publishedComments[i].hyperText);
         textarea.appendChild(text);
      }
   }

   row.appendChild(textarea);
   container.appendChild(row);

   return container;
}

/**
 * toolbar structure includes the navbutton and the general floating nav at the bottom of the page.
 * 
 * @param {string} toolbarText the navbutton text.
 */
function generateToolStructure(toolbarText) {
   var navButton, navTool, containerDiv, collapseDiv;

   navButton = generateNavButton(toolbarText);
   $('body').append(navButton);

   navTool = document.createElement("nav");
   navTool.setAttribute("class", "navbar navbar-fixed-bottom navbar-test collapse");
   navTool.setAttribute("id", "responsive-nav");

   containerDiv = document.createElement("div");
   containerDiv.setAttribute("class", "container-fluid");

   collapseDiv = document.createElement("div");
   collapseDiv.setAttribute("class", "collapse navbar-collapse");
   collapseDiv.setAttribute("id", "insert-tool");

   containerDiv.appendChild(collapseDiv);
   navTool.appendChild(containerDiv);

   $('body').append(navTool);
}

/**
 * builds a navigation button with he provided text
 * 
 * @param {string} toolbarText the text to add to the nav tool
 * @returns {Object} nav the dom element representing the control
 */
function generateNavButton(toolbarText) {
   var nav, button, icon, text;

   nav = document.createElement("nav");
   nav.setAttribute("class", "navbar navbar-fixed-top");
   nav.setAttribute("style", "width: 160px");

   button = document.createElement("button");
   button.setAttribute("class", "btn btn-default");
   button.setAttribute("type", "button");
   button.setAttribute("data-toggle", "collapse");
   button.setAttribute("data-target", "#responsive-nav");
   button.setAttribute("aria-expanded", "false");
   button.setAttribute("aria-controls", "responsive-nav");
   button.setAttribute("id", "navToolButton");
   button.setAttribute("onclick", "toggleNavTool('" + toolbarText + "')");
   button.setAttribute("value", "expanded");

   text = document.createTextNode("Expand " + toolbarText);

   button.appendChild(text);

   icon = document.createElement("span");
   icon.setAttribute("class", "glyphicon glyphicon-chevron-up");
   icon.setAttribute("aria-hidden", "true");
   icon.setAttribute("alt", "Expand " + toolbarText);

   button.appendChild(icon);

   if ($('#navToolButton').length === 0) {
      nav.appendChild(button);
   }

   return nav;
}

/**
 * builds the toggler for the navigation button with he provided text
 * 
 * @param {string} toolbarText the text to add to the nav tool
 */
/* exported toggleNavTool */
function toggleNavTool(toolbarText) {
   var icon;
   var navToolButton = $('#navToolButton');
   var navBarClass = $('.navbar-test').attr("class");
   var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

   if (navBarClass.indexOf("collapse") > -1) {
      getStudyMembers(assignedID,function(smcPlusIncumbents) {
      /* jshint unused:false */
      // 681460: Toolbars creating infinite loop: This code check if the
      // section and question dropdowns are loaded
      // properly and if they are not it reloads them.
      if ((toolbarText == "Reviewer Toolbar") || (toolbarText == "PI Toolbar")
         || (toolbarText == "Study Member Toolbar") && ($("#addCommentSection"))
         && (($("#addCommentSection option").length == 1) || ($("#addCommentQuestion option").length == 1))) {
         if ((validSection.length > 0) && ($("#addCommentSection option").length === 1)) {
            /* Specify Section to add comment to */
            for (var i = 0; i < validSection.length; i++) {
               var sectionValue = "Section " + sectionTitle.charAt(validSection[i]);
               $("#addCommentSection").append($('<option value="' + sectionValue + '">' + sectionValue + '</option>'));
            }
         }
         var formLength = $("form").length;
         if ((formLength > 0) && ($("#addCommentQuestion")) && ($("#addCommentQuestion option").length == 1)) {
            /* Specify Section to add comment to */
            for (var j = 0; j < formLength; j++) {
               var questionValue = "Question " + (j + 1);
               $("#addCommentQuestion").append(
                  $('<option value="' + questionValue + '">' + questionValue + '</option>'));
            }
         }
      }
      
     

      // toolbar currently expanded/up
      navToolButton.val("collapsed");

      // removes children and replaces text
      navToolButton.text("Collapse " + toolbarText);

      // adding back the icon
      icon = document.createElement("span");
      icon.setAttribute("class", "glyphicon glyphicon-chevron-down");
      icon.setAttribute("aria-hidden", "true");
      icon.setAttribute("alt", "Collapse " + toolbarText);
      navToolButton.append(icon);
      //reset it to defaut values.
      if(toolbarText == "General Admin Toolbar"){
         if (($("#selectCommittee")) && ($("#selectCommittee option").length != 1)) {
            $("#selectCommittee").val($("#selectCommittee").find('option').first().val());
            $("#addAdmin").val($("#addAdmin").find('option').first().val());
         }
      }
      });

   } else {
      // toolbar currently collapsed/down
      navToolButton.val("expanded");

      // removes children and replaces text
      navToolButton.text("Expand " + toolbarText);

      // adding back the icon
      icon = document.createElement("span");
      icon.setAttribute("class", "glyphicon glyphicon-chevron-up");
      icon.setAttribute("aria-hidden", "true");
      icon.setAttribute("alt", "Expand " + toolbarText);
      navToolButton.append(icon);
   }
}

/**
 * builds the assignments, review, checklists, Submit, Cancel, Print, and Save buttons on the toolbar
 * 
 * @param {string} targetTool the name of the function to call on save
 * @returns {Object} container the dom element representing the control
 */
function generateToolTabs(targetTool) {
   var save, cancel, container, column, row, print;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-3");

   row = document.createElement("div");
   row.setAttribute("class", "row");

   column = document.createElement("div");
   column.setAttribute("class", "col-xs-6 btn-group-vertical");
   column.setAttribute("role", "group");

   /* Assignments button */
   column.appendChild(tabLabelMaker("assignApp", "Assignments"));
   column.appendChild(tabButtonMaker("assignApp", "1.Assignments", "user"));

   /* Review button */
   column.appendChild(tabLabelMaker("appReview", "Review"));
   column.appendChild(tabButtonMaker("appReview", "2.Review", "pencil"));

   /* Checklists button */
   column.appendChild(tabLabelMaker("appChecklists", "Checklists"));
   column.appendChild(tabButtonMaker("appChecklists", "3.Checklists", "list-alt"));

   /* Approve button */
   column.appendChild(tabLabelMaker("submitApp", "Submit Application"));
   column.appendChild(tabButtonMaker("submitApp", "4.Update Status", "send"));

   /* Events button */
   if ((spAppData.ApplicationStatus.indexOf(RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDCHOICE) > -1)
      && (currentFormType == "Parent")) {
      column.appendChild(tabLabelMaker("appEvent", "Events"));
      column.appendChild(tabButtonMaker("appEvent", "Events", "bullhorn"));
   }

   row.appendChild(column);
   column = document.createElement("div");
   column.setAttribute("class", "col-xs-6 btn-group-vertical");
   column.setAttribute("role", "group");

   /* Cancel Button */
   column.appendChild(tabLabelMaker("cancelToolbar", "Cancel Changes"));
   cancel = tabButtonMaker("cancelToolbar", "Exit", "trash");
   cancel.removeAttribute("onclick");
   cancel.setAttribute("onclick", "Function(window.history.back())");
   column.appendChild(cancel);

   /* Print button */
   column.appendChild(tabLabelMaker("piPrint", "Print Comments"));
   print = tabButtonMaker("piPrint", "Print", "print");
   print.removeAttribute("onclick");
   print.setAttribute("onclick", "printComments()");
   column.appendChild(print);

   /* Save button */
   column.appendChild(tabLabelMaker("saveAdminTool", "Save Admin Comments"));
   save = tabButtonMaker("saveAdminTool", "Save", "floppy-disk");
   save.removeAttribute("onclick");
   save.setAttribute("onclick", "saveProgress('" + targetTool + "')");
   column.appendChild(save);

   row.appendChild(column);
   container.appendChild(row);

   return container;
}

/**
 * builds the Cancel, Print, and Save buttons on the toolbar
 * 
 * @param {string} targetTool the name of the function to call on save
 * @returns {Object} container the dom element representing the control
 */
function generateStudyToolTabs(targetTool) {
   var container, save, cancel, print;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-2 btn-group-vertical");
   container.setAttribute("role", "group");

   /* Cancel Button */
   container.appendChild(tabLabelMaker("cancelToolbar", "Cancel Changes"));
   cancel = tabButtonMaker("cancelToolbar", "Exit", "trash");
   cancel.removeAttribute("onclick");
   cancel.setAttribute("onclick", "Function(window.history.back())");
   container.appendChild(cancel);

   /* Print button */
   container.appendChild(tabLabelMaker("piPrint", "Print Comments"));
   print = tabButtonMaker("piPrint", "Print", "print");
   print.removeAttribute("onclick");
   print.setAttribute("onclick", "printComments()");
   container.appendChild(print);

   /* Save button */
   container.appendChild(tabLabelMaker("saveAdminTool", "Save Admin Comments"));
   save = tabButtonMaker("saveAdminTool", "Save", "floppy-disk");
   save.removeAttribute("onclick");
   save.setAttribute("onclick", "saveProgress('" + targetTool + "')");
   container.appendChild(save);

   return container;
}

/**
 * builds the review, checklists, Submit, Cancel, Print, and Save buttons on the toolbar
 * 
 * @param {string} targetTool the name of the function to call on save
 * @returns {Object} container the dom element representing the control
 */
function generateReviewerToolTabs(targetTool) {
   var container, save, print, cancel;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-2 btn-group-vertical");
   container.setAttribute("role", "group");

   /* Review/comments */
   container.appendChild(tabLabelMaker("reviewComments", "Review"));
   container.appendChild(tabButtonMaker("reviewComments", "1.Review", "pencil"));

   /* Checklists */
   container.appendChild(tabLabelMaker("reviewChecklists", "Checklists"));
   container.appendChild(tabButtonMaker("reviewChecklists", "2.Checklists", "list-alt"));

   /* Submit (back to IRB Admin) */
   container.appendChild(tabLabelMaker("reviewerSubmit", "Submit Application"));
   container.appendChild(tabButtonMaker("reviewerSubmit", "3.Update Status", "send"));

   /* Cancel Button */
   container.appendChild(tabLabelMaker("cancelToolbar", "Cancel Changes"));
   cancel = tabButtonMaker("cancelToolbar", "Exit", "trash");
   cancel.removeAttribute("onclick");
   cancel.setAttribute("onclick", "Function(window.history.back())");
   container.appendChild(cancel);

   /* Print button */
   container.appendChild(tabLabelMaker("piPrint", "Print Comments"));
   print = tabButtonMaker("piPrint", "Print", "print");
   print.removeAttribute("onclick");
   print.setAttribute("onclick", "printComments()");
   container.appendChild(print);

   /* Save button */
   container.appendChild(tabLabelMaker("saveAdminTool", "Save Admin Comments"));
   save = tabButtonMaker("saveAdminTool", "Save", "floppy-disk");
   save.removeAttribute("onclick");
   save.setAttribute("onclick", "saveProgress('" + targetTool + "')");
   container.appendChild(save);

   return container;

}

/**
 * builds the review, Submit, Cancel, Print, and Save buttons on the toolbar
 * 
 * @param {string} targetTool the name of the function to call on save
 * @returns {Object} container the dom element representing the control
 */
function generateLocalSiteToolTabs(targetTool) {
   var container, save, print, cancel;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-2 btn-group-vertical");
   container.setAttribute("role", "group");

   /* Review/comments */
   container.appendChild(tabLabelMaker("reviewComments", "Review"));
   container.appendChild(tabButtonMaker("reviewComments", "1.Review", "pencil"));

   /* Submit (back to IRB Admin? */
   container.appendChild(tabLabelMaker("reviewerSubmit", "Submit Application"));
   container.appendChild(tabButtonMaker("reviewerSubmit", "2.Update Status", "send"));

   /* Cancel Button */
   container.appendChild(tabLabelMaker("cancelToolbar", "Cancel Changes"));
   cancel = tabButtonMaker("cancelToolbar", "Exit", "trash");
   cancel.removeAttribute("onclick");
   cancel.setAttribute("onclick", "Function(window.history.back())");
   container.appendChild(cancel);

   /* Print button */
   container.appendChild(tabLabelMaker("piPrint", "Print Comments"));
   print = tabButtonMaker("piPrint", "Print", "print");
   print.removeAttribute("onclick");
   print.setAttribute("onclick", "printComments()");
   container.appendChild(print);

   /* Save button */
   container.appendChild(tabLabelMaker("saveAdminTool", "Save Admin Comments"));
   save = tabButtonMaker("saveAdminTool", "Save", "floppy-disk");
   save.removeAttribute("onclick");
   save.setAttribute("onclick", "saveProgress('" + targetTool + "')");
   container.appendChild(save);

   return container;
}

/**
 * builds the Cancel, Print, and Save buttons on the toolbar
 * 
 * @param {string} targetTool the name of the function to call on save
 * @returns {Object} container the dom element representing the control
 */
function generateCIRBToolTabs(targetTool) {
   var container, save, cancel;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-2 btn-group-vertical");
   container.setAttribute("role", "group");

   /* Cancel Button */
   container.appendChild(tabLabelMaker("cancelToolbar", "Cancel Changes"));
   cancel = tabButtonMaker("cancelToolbar", "Exit", "trash");
   cancel.removeAttribute("onclick");
   cancel.setAttribute("onclick", "Function(window.history.back())");
   container.appendChild(cancel);

   /* Save button */
   container.appendChild(tabLabelMaker("saveAdminTool", "Save Admin Comments"));
   save = tabButtonMaker("saveAdminTool", "Save", "floppy-disk");
   save.removeAttribute("onclick");
   save.setAttribute("onclick", "saveProgress('" + targetTool + "')");
   container.appendChild(save);

   /* Submit button */
   container.appendChild(tabLabelMaker("piSubmitApp", "Submit Application"));
   container.appendChild(tabButtonMaker("piSubmitApp", "Update Status", "send"));

   return container;
}

/**
 * builds the review, checklists, Submit, Cancel, Print, and Save buttons on the toolbar
 * 
 * @param {string} targetTool the name of the function to call on save
 * @returns {Object} container the dom element representing the control
 */
function generatePIToolTabs(targetTool) {
   var save, print, container, cancel;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-2 btn-group-vertical");
   container.setAttribute("role", "group");

   /* comments button */
   container.appendChild(tabLabelMaker("piComments", "Review"));
   container.appendChild(tabButtonMaker("piComments", "1.Review", "pencil"));

   /* adverse events */// moving to approved toolbar
   // container.appendChild(tabLabelMaker("adverseEvent", "Adverse Event"));
   // container.appendChild(tabButtonMaker("postApproveEvents", "Adverse
   // Event", "bullhorn"));
   /* checklists */
   container.appendChild(tabLabelMaker("piChecklists", "Checklists"));
   container.appendChild(tabButtonMaker("piChecklists", "2.Checklists", "list-alt"));

   /* Approve button */
   container.appendChild(tabLabelMaker("piSubmitApp", "Submit Application"));
   container.appendChild(tabButtonMaker("piSubmitApp", "3.Update Status", "send"));

   /* Cancel Button */
   container.appendChild(tabLabelMaker("cancelToolbar", "Cancel Changes"));
   cancel = tabButtonMaker("cancelToolbar", "Exit", "trash");
   cancel.removeAttribute("onclick");
   cancel.setAttribute("onclick", "Function(window.history.back())");
   container.appendChild(cancel);

   /* Print button */
   container.appendChild(tabLabelMaker("piPrint", "Print Comments"));
   print = tabButtonMaker("piPrint", "Print", "print");
   print.removeAttribute("onclick");
   print.setAttribute("onclick", "printComments()");
   container.appendChild(print);

   /* Save button */
   container.appendChild(tabLabelMaker("saveAdminTool", "Save Admin Comments"));
   save = tabButtonMaker("saveAdminTool", "Save", "floppy-disk");
   save.removeAttribute("onclick");
   save.setAttribute("onclick", "saveProgress('" + targetTool + "')");
   container.appendChild(save);

   return container;
}

/**
 * opens the comments in another tab or window so they can be printed
 */
/* exported printComments */
function printComments() {
   var winPrint = window.open('about:blank', '_blank');
   var text = "<title>Print Report</title><br /><br /><span style='white-space: pre-line'>";
   $("#reviewerComments").find('option').each(function() {
      if ($(this).val().indexOf(",") != -1) {
         text += "<br />-------" + $(this).val() + "----------<br />";
         text += appReviewComments.get($(this).val());
      }
   });
   text += "</span>";
   winPrint.document.write(text);
   winPrint.document.close();
   winPrint.focus();
   winPrint.print();
}

/**
 * create a label
 * 
 * @param {string} id the id of the contol to add the label to
 * @param {string} title the text to put on the textNode
 * @returns {label} label the dom element representing the control
 */
function tabLabelMaker(id, title) {
   var label, text;

   /* only the screen reader will see this */
   label = document.createElement("label");
   label.setAttribute("class", "sr-only");
   label.setAttribute("for", id);

   text = document.createTextNode(title);
   label.appendChild(text);

   return label;
}

/**
 * create a button
 * 
 * @param {string} id the id of the contol to add the label to
 * @param {string} title the text to put on the textNode
 * @returns {Object} button the dom element representing the control
 */
function tabButtonMaker(id, displayText, iconType) {
   var button, icon, text;

   button = document.createElement("button");
   button.setAttribute("type", "button");
   button.setAttribute("class", "btn btn-default");
   button.setAttribute("id", id);

   text = document.createTextNode(displayText);
   button.appendChild(text);

   icon = document.createElement("span");
   icon.setAttribute("class", "glyphicon glyphicon-" + iconType);
   icon.setAttribute("aria-hidden", "true");
   button.appendChild(icon);
   button.setAttribute("onclick", "swapToolGenerator('" + id + "')");

   return button;
}

/**
 * builds the comments section of the toolbar
 * 
 * @returns {Object} container the dom element representing the control
 */
function buildMultiCommitteesComments() {
   var container, row, commentTypeSelecter, sectionSelecter, questionSelecter, commentArea;
   var commentOptions = [];
   var question = [];
   var section = [];
   var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
   var committeeOptions = [];
   var committeeSelecter;
   var displayCommitteeName = getCommitteeName();
   var i;

   if (committeeUrl === "") {
      committeeOptions.push("Select Committee");
      for (i = 0; i < displayCommitteeName.length; i++) {
         if (displayCommitteeName[i] !== "") {
            committeeOptions.push(displayCommitteeName[i]);
         }
      }
   }
   if (committeeUrl == "RDC") {
      committeeOptions.push("Select Committee", "R&D");
   }
   if (committeeUrl == "IRB") {
      committeeOptions.push("Select Committee", "IRB");
   }
   if (committeeUrl == "IACUC") {
      committeeOptions.push("Select Committee", "IACUC");
   }
   if (committeeUrl == "SRS") {
      committeeOptions.push("Select Committee", "SRS");
   }
   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 col-md-offset-1 selecter-div");

   /* User must write general or specific comment */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   commentOptions.push("Select Comment Type: ", "General Comment", "Specific Comment");
   commentTypeSelecter = buildSelecterHelper("addCommentType", "Select Comment Type:", commentOptions, "6",
      "enableDropdowns(['addCommentSection', 'addCommentQuestion'])", "yes");
   row.appendChild(commentTypeSelecter);

   /* User can select belonged committee */
   getCommitteeList(function(committeeList) {
      committeeSelecter = buildSelecterHelper("selectCommittee", "Select Committee: ", [ "Select Committee"].concat(committeeList), "6", null, "yes");
      row.appendChild(committeeSelecter);
   });
   //committeeSelecter = buildSelecterHelper("selectCommittee", "Select Committee: ", committeeOptions, "6", null, "yes");
   //row.appendChild(committeeSelecter);

   container.appendChild(row); // add 1st dropdown to toolbar

   /* Add comment */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   // Default text added to section dropdown
   section.push("Select a Section: ");

   /* Specify Section to add comment to */
   for (i = 0; i < validSection.length; i++) {
      section.push("Section " + sectionTitle.charAt(validSection[i]));
   }

   sectionSelecter = buildSelecterHelper("addCommentSection", "Add Comment, Section: ", section, "6",
      "switchSection('addCommentSection', 'addCommentQuestion')", true);
   row.appendChild(sectionSelecter);

   // Default text added to question dropdown
   question.push("Select a Question: ");

   /* Specify Question to add comment to */
   for (i = 0; i < $("form").length; i++) {
      question.push("Question " + (i + 1));
   }

   questionSelecter = buildSelecterHelper("addCommentQuestion", "Question: ", question, "6", null, true);
   row.appendChild(questionSelecter);

   container.appendChild(row);

   /* Area to type comments into */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   commentArea = buildTextareaHelper("addComment", null, "12", "Add Comment:", "9");
   row.appendChild(commentArea);
   container.appendChild(row);

   return container;
}

/**
 * builds the admin comments section of the toolbar
 * 
 * @returns {Object} container the dom element representing the control
 */
function buildAdminComments() {
   var container, row, commentTypeSelecter, sectionSelecter, questionSelecter, commentArea, i;
   var commentOptions = [];
   var question = [];
   var section = [];
   var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 col-md-offset-1 selecter-div");

   /* User must write general or specific comment */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   commentOptions.push("Select Comment Type: ", "General Comment", "Specific Comment");
   commentTypeSelecter = buildSelecterHelper("addCommentType", "Select General or Specific Comment", commentOptions,
      "12", "enableDropdowns(['addCommentSection', 'addCommentQuestion'])", "yes");
   row.appendChild(commentTypeSelecter);
   container.appendChild(row); // add 1st dropdown to toolbar

   /* Add comment */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   // Default text added to section dropdown
   section.push("Select a Section: ");

   /* Specify Section to add comment to */
   for (i = 0; i < validSection.length; i++) {
      section.push("Section " + sectionTitle.charAt(validSection[i]));
   }

   sectionSelecter = buildSelecterHelper("addCommentSection", "Add Comment, Section: ", section, "6",
      "switchSection('addCommentSection', 'addCommentQuestion')", true);
   row.appendChild(sectionSelecter);

   // Default text added to question dropdown
   question.push("Select a Question: ");

   /* Specify Question to add comment to */
   for (i = 0; i < $("form").length; i++) {
      question.push("Question " + (i + 1));
   }

   questionSelecter = buildSelecterHelper("addCommentQuestion", "Question: ", question, "6", null, true);
   row.appendChild(questionSelecter);

   container.appendChild(row);

   /* Area to type comments into */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   commentArea = buildTextareaHelper("addComment", null, "12", "Add Comment:", "9");
   row.appendChild(commentArea);
   container.appendChild(row);

   return container;
}

/**
 * builds the reviewer comments section of the toolbar
 * 
 * @returns {Object} container the dom element representing the control
 */
function buildReviewersComments() {
   var container, row, reviewerSelecter, reviewerComments;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 selecter-div");

   /* Pick a reviewer's comments */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /*
    * pass buildselecter an onclick function this function will be insert
    * review comments pass the person determine if person is reviewer or admin
    * insert comments accordingly
    */

   /*
    * build list of people to send, if there are no people, remove onclick and
    * display "no reviews"
    */

   // currently assuming everyone is a reviewer
   reviewerSelecter = buildSelecterHelper("reviewerComments", "View Comments: ", [ "Please Select a User:"
   ].concat(appReviewComments.keys(), "View All"), "12", "selectToText('reviewerComments', 'viewComments')");
   reviewerSelecter.setAttribute("style", "padding-bottom: 4px;");
   row.appendChild(reviewerSelecter);

   container.appendChild(row);

   /* View a reviewer's comments */
   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   reviewerComments = buildTextareaHelper("viewComments", "true", "12", "View Comments:", "13");
   row.appendChild(reviewerComments);

   container.appendChild(row);

   return container;
}

/**
 * Correlates a user in a drop down box to their comments
 * 
 * @param {string} selectID the Id of the control to get the text from
 * @param {string} insertID the Id of the control to add the person to.
 */
/* exported selectToText */
function selectToText(selectID, insertID) {
   var person = $("#" + selectID).find(':selected').val();
   var allComments = "";

   /*
    * assuming that if a person in the drop-down, then they'll have a comma in
    * their name
    */
   if (person.indexOf(":") != -1) {
      person = currentUser.Title;
   }

   if (person.indexOf(",") == -1) {
      $("#" + selectID).find('option').each(function() {
         if ($(this).val().indexOf(",") != -1) {
            allComments += "\n-------" + $(this).val() + "----------\n";
            allComments += appReviewComments.get($(this).val());
         }
      });
      $("#" + insertID).val(allComments);
   } else if ((appReviewComments) && (appReviewComments.containsKey(person))) {
      $("#" + insertID).val(appReviewComments.get(person));
   } else {
      $("#" + insertID).val("No comments available.");
   }
}

/**
 * the review toolbar uses this as the onChange event to enable or disable other controls
 * 
 * @param {string[]} dropdownList the Ids of the controls to be enabled or disabled
 */
/* exported enableDropdowns */
function enableDropdowns(dropdownList) { // enable (or disable) section and
   // question dropdown for
   var i;

   // specific comment
   var commentType = $("#" + 'addCommentType').find(':selected').val();
   if (commentType == "Specific Comment") {
      // enable dropdowns again
      for (i = 0; i < dropdownList.length; i++) {
         $("#" + (dropdownList[i])).removeAttr("disabled");
      }
   } else { // make sure other dropdowns disabled
      for (i = 0; i < dropdownList.length; i++) {
         if (dropdownList[i] == "addCommentSection") {
            // set back to default value before disabling because they get
            // set to blank value for some reason
            $("#addCommentSection").val("Select a Section: ");
            $("#addCommentQuestion").val("Select a Question: ");
         }
         if (($("#" + (dropdownList[i])).attr("disabled") !== typeof undefined)
            && ($("#" + (dropdownList[i])).attr("disabled") !== false)) {
            // check if dropdown isn't disabled
            $("#" + (dropdownList[i])).attr("disabled", true);
         }
      }
   }
}

/**
 * moves question dropdown to specific section & generates the options in the question dropdown
 * 
 * @param {string} sectionID the Id of the control to find
 * @param {string} questionID the Id of the question to remove and re-add
 */
/* exported switchSection */
function switchSection(sectionID, questionID) {
   var targetSection = $("#" + sectionID).find(':selected').val();
   var option, text, i;

   if (targetSection != "Select a Section: ") {
      targetSection = targetSection.split(" ")[1]; // get letter of section

      var letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
      targetSection = letters.indexOf(targetSection); // find index of letter
      // in alphabet

      continueSectionBuilder(targetSection);

      $("#" + questionID).children().remove(); // remove old options
      for (i = 0; i < ($("form").length); i++) { // add new options
         option = document.createElement("option");
         option.setAttribute("value", "Question " + (i + 1));

         text = document.createTextNode("Question " + (i + 1));
         option.appendChild(text);

         $('#' + questionID).append(option);
      }
   }
}

/**
 * build a textarea with the provided parameters
 * 
 * @param {string} id the Id of the control
 * @param {string} disabler not used
 * @param {number} length the bootstrap number of cells to use
 * @param {string} title the title of the TextNode
 * @param {number} rows the number of rows
 * @returns {Object} container the dom element representing the control
 */
function buildTextareaHelper(id, disabler, length, title, rows) {
   /* jshint unused: false */
   var container, label, textarea, text;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-" + length);

   label = document.createElement("label");
   label.setAttribute("class", "sr-only");
   label.setAttribute("for", id);

   text = document.createTextNode(title);
   label.appendChild(text);
   container.appendChild(label);

   textarea = document.createElement("textarea");
   textarea.setAttribute("id", id);
   textarea.setAttribute("class", "form-control");

   if (rows) {
      textarea.setAttribute("rows", rows);
   } else { // default number of rows for textbox is 7
      textarea.setAttribute("rows", "7");
   }
   container.appendChild(textarea);

   return container;
}

/**
 * builds the table of checklists
 * 
 * @param {checklists[]} checklists an array of files to add to the checklist
 * @returns {Object} container the dom element representing the control
 */
function buildChecklistTemplates(checklists) {
   var container, row, table, tbody, trow, tableDiv, i;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 col-md-offset-1 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /* Header Table */
   table = tableHeaderHelper("Available Checklist Templates", [ "Checklist Type", "Download"
   ]);
   row.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content");
   tableDiv.setAttribute("style", "height: 225px");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   tbody = document.createElement("tbody");
   for (i = 0; i < checklists.length; i++) {
      trow = document.createElement("tr");
      trow.appendChild(textCellHelper("td", checklists[i].Name));
      trow.appendChild(buttonCellHelper("Download", "downloadChecklist('" + checklists[i].ServerRelativeUrl + "')"));
      tbody.appendChild(trow);
   }

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   row.appendChild(tableDiv);
   container.appendChild(row);

   container.appendChild(fileSelectorBuilder("uploadChecklist", "Upload Checklist:"));

   return container;
}

/**
 * build the file selector using the provided attributes
 * 
 * @param {string} id the id of the control being created
 * @param {string} displayText the text for the textNode
 * @returns {Object} container the dom element representing the control
 */
function fileSelectorBuilder(id, displayText) {
   var container, label, input, labelDiv, inputDiv, text;

   container = document.createElement("div");
   container.setAttribute("class", "row selecter-text");

   labelDiv = document.createElement("div");
   labelDiv.setAttribute("class", "col-xs-4");

   label = document.createElement("label");
   label.setAttribute("for", id);

   text = document.createTextNode(displayText);
   label.appendChild(text);

   labelDiv.appendChild(label);
   container.appendChild(labelDiv);

   inputDiv = document.createElement("div");
   inputDiv.setAttribute("class", "col-xs-8");

   input = document.createElement("input");
   input.setAttribute("type", "file");
   input.setAttribute("id", id);

   inputDiv.appendChild(input);
   container.appendChild(inputDiv);

   return container;
}

/**
 * build a container of files uploaded for the study
 * 
 * @param {attachments[]} attachments list of uploaded attachments to display
 * @param {string} teamFolder the folder to pull the list of files from
 * @returns {Object} container the dom element representing the control
 */
function buildUploadedChecklists(attachments, teamFolder) {
   var container, row, tableDiv, table, tbody, trow, text, fileName, i;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /* Header Table */
   table = tableHeaderHelper("Current Uploaded Checklists", [ "Checklist Type", "Date"
   ]);
   row.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   tbody = document.createElement("tbody");

   for (i = 0; i < attachments.length; i++) {
      trow = document.createElement("tr");

      if (teamFolder == "Study%20Team") {
         fileName = (attachments[i].Name).replace("-CHECKLIST-", "");
      } else {
         fileName = attachments[i].Name;
      }

      trow.appendChild(linkCellHelper("td", fileName, teamFolder, attachments[i].Name));
      text = attachments[i].TimeCreated.split("T")[1].replace("Z", "") + " on "
         + attachments[i].TimeCreated.split("T")[0];
      trow.appendChild(textCellHelper("td", text));
      tbody.appendChild(trow);
   }

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   row.appendChild(tableDiv);
   container.appendChild(row);

   return container;
}

/**
 * Builds the table structure as well as the thead
 * 
 * @param {string} captionText text to be added to the table as a caption
 * @param {string[]} headers a list of headers for the table
 * @returns {Object} table the dom element representing the control
 */
function tableHeaderHelper(captionText, headers) {
   var table, thead, trow, text, caption, i;

   // if there aren't enough elements for the table to scroll, then you don't
   // need to
   // include "tool-table" in the table's class
   table = document.createElement("table");
   table.setAttribute("class", "table table-striped selecter-text");

   if (captionText) {
      caption = document.createElement("caption");
      text = document.createTextNode(captionText);
      caption.appendChild(text);
      table.appendChild(caption);
   }

   thead = document.createElement("thead");
   trow = document.createElement("tr");

   for (i = 0; i < headers.length; i++) {
      trow.appendChild(textCellHelper("th", headers[i]));
   }

   thead.appendChild(trow);
   table.appendChild(thead);

   return table;
}

// TODO TAR I don't think this function is used.
/**
 * NOT USED. get the comments for the requested user.
 * 
 * @deprecated
 */
/* exported grabComments */
function grabComments(comments, user) {
   var userComments = [];

   for (var i = 0; i < comments.length; i++) {
      if (user == comments[i].user.SPUser) {
         userComments.push(comments[i] + "\n");
      }
   }

   // publishedComments is not defined probably should be userComments
   // return publishedComments;
   return userComments;
}

/**
 * function used by the save button to save the toolbar
 * 
 * @param {string} targetTool the tool name to save. valid values are:
 *           <ul>
 *           <li>appReview
 *           <li>assignApp
 *           <li>submitApp
 *           <li>studyComments
 *           <li>piSubmitApp
 *           <li>assignIRBAdmin
 *           <li>reviewerSubmit
 *           <li>reviewComments
 *           <li>appEvent
 *           <li>appChecklists
 *           <li>reviewChecklists
 *           <li>piChecklists
 *           <li>piComments
 *           </ul>
 */
/* exported saveProgress */
function saveProgress(targetTool) {
   // var saveButton = $('#saveAdminTool');

   // saveButton.fadeOut(400, function(){
   // saveButton.attr("class", "btn btn-default label-success");
   // saveButton.children().attr("class", "glyphicon glyphicon-floppy-saved");
   // }).fadeIn(600);

   // console.log("the target tool is: " + targetTool);
   getRAMSUsers(function() {

      if (targetTool == "appReview") {
         sendAppReview(targetTool);
      } else if (targetTool == "assignApp") {
         sendAssignApp(targetTool);
      } else if (targetTool == "submitApp") {
         var approvalType = $('#selectApproval').val();
         if (approvalType == "Re-Submit to Study Team") {
            sendSubmitApp(RAMS.Environment.SharePoint.List.CommitteeStatus.AWAITINGREVISIONSCHOICE, "Revision");
         } else if (approvalType == "Approve Application") {
            sendSubmitApp(RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDCHOICE,
               RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDCHOICE);
            notifyLocalSitesOnParentStatus(assignedID, RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDCHOICE); // this
            // will update the children studies with status of parent Approved status
         } else if (approvalType == "Approve with Minor Modifications") {
            sendSubmitApp(RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDWITHMINORMODIFICATIONSCHOICE,
               "Minor Mods");
         } else if (approvalType == "Approve with Major Modifications") {
            sendSubmitApp(RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDWITHMAJORMODIFICATIONSCHOICE,
               "Major Mods");
         } else if (approvalType == "Suspend") {
            sendSubmitApp(RAMS.Environment.SharePoint.List.CommitteeStatus.SUSPENDEDCHOICE,
               RAMS.Environment.SharePoint.List.CommitteeStatus.SUSPENDEDCHOICE);
         } else if (approvalType == "Rejected") {
            sendSubmitApp(RAMS.Environment.SharePoint.List.CommitteeStatus.DISAPPROVEDCHOICE, "Rejected");
         }
      } else if (targetTool == "studyComments") {
         sendAppReview(targetTool);
      } else if (targetTool == "piSubmitApp") {
         sendSubmitApp(RAMS.Environment.SharePoint.List.CommitteeStatus.UNDERREVIEWCHOICE, "Review");
      } else if (targetTool == "assignIRBAdmin") {
         sendAssignApp(targetTool);
      } else if (targetTool == "reviewerSubmit") {
         publishComments(targetTool);
         window.location.href("/sites/irbapplications/sitepages/irb%20applications.aspx"); // added
         // line
      } else if (targetTool == "reviewComments") {
         sendAppReview(targetTool);
      } else if (targetTool == "appEvent") {
         sendRenewalDate(targetTool);
      } else if ((targetTool == "appChecklists") || (targetTool == "reviewChecklists")) {
         sendReviewChecklist(targetTool, "Review%20Team");
      } else if (targetTool == "piChecklists") {
         sendReviewChecklist(targetTool, "Study%20Team");
      } else if (targetTool == "piComments") {
         sendAppReview(targetTool);
      }
   });
}

/**
 * build the controls to submit the study and collect submission comments
 * 
 * @param {string[]} submissionTypes the types of submissions. see caller for list of valid values.
 * @returns {Object} table the dom element representing the control
 */
function buildSubmitComments(submissionTypes, committee) {
   var container, row, selecter, textarea;
   var committeeList = [];

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 col-md-offset-1 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   if (committee) {
      committeeList = committeeTable.get(assignedID);
      selecter = buildSelecterHelper("selectCommittee", "Committee: ", [ "Select Committee"
      ].concat(committeeList), "12",
         "hideAndShowTriennialExpirationDate('triennialExpirationLabel', 'triennialExpirationId')", false, "yes");
      row.appendChild(selecter);
   }

   selecter = buildSelecterHelper("selectApproval", "Submission Type: ", [ "Please Select a Submission"
   ].concat(submissionTypes), "12", "selectToText('selectApproval', 'submissionComment')", false, "yes");
   row.appendChild(selecter);

   // selecter = buildSelecterHelper("renewalDate", "Renewal Date: ", ["Date
   // 1", "Date 2", "Date
   // 3"], "6", null);
   // row.appendChild(selecter);

   container.appendChild(row);

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   textarea = buildTextareaHelper("submissionComment", null, "12", "Submission Comments: ");
   row.appendChild(textarea);

   container.appendChild(row);

   return container;
}

/**
 * builds the application history table
 * 
 * @returns {Object} container the dom element representing the control
 */
function buildAppHistory() {
   var container, row, table, tbody, trow, tableDiv, erow;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /* Header Table */
   table = tableHeaderHelper("Application History", [ "Event Type", "Date", "Actor"
   ]);
   row.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   // console.log("we have " + currentStudyStatus.length + " events");

   tbody = document.createElement("tbody");
   for (var i = 0; i < currentStudyStatus.length; i++) {
      trow = document.createElement("tr");
      trow.appendChild(textCellHelper("td", currentStudyStatus[i].studyState.name));
      trow.appendChild(textCellHelper("td", currentStudyStatus[i].createdAt.split("T")[0]));
      trow.appendChild(textCellHelper("td", currentStudyStatus[i].createdBy.displayName));
      tbody.appendChild(trow);
   }

   // BEGIN FROM SUSTAINMENT 2.1.1
   // Only show if Submission Type is of type approval:
   erow = document.createElement("tr");
   erow.appendChild(textCellHelper("td", "Expiration Date"));
   //erow.appendChild(textCellHelper("td", "Expiration Period"));
   var expText = textCellHelper('input', "Exp", "expirationId");
   var dobj = new Date();

   // TODO TAR which committee expiredate?
    var expD = spAppData.ExpiringDate;
   expText.value = (expD === null ? (dobj.getUTCMonth() + 1) + "/" + dobj.getUTCDate() + "/"
    + (dobj.getUTCFullYear() + 1) : expD.substring(5, 10).replace("-", "/") + "/" + expD.substring(0, 4));
   //expText.value = 12;
   erow.appendChild(expText);
   tbody.appendChild(erow);
   // END FROM SUSTAINMENT 2.1.1

   erow = document.createElement("tr");
   erow.appendChild(textCellHelper("td", "Triennial Expiration Date", "triennialExpirationLabel"));
   //erow.appendChild(textCellHelper("td", "Triennial Expiration Period", "triennialExpirationLabel"));
   var triExpText = textCellHelper('input', "TriExp", "triennialExpirationId");

   // expD = spAppData.ExpiringDate;
   triExpText.value = (expD === null ? (dobj.getUTCMonth() + 1) + "/" + dobj.getUTCDate() + "/"
   + (dobj.getUTCFullYear() + 3) : expD.substring(5, 10).replace("-", "/") + "/" + expD.substring(0, 4));
   //triExpText.value = 36;
   erow.appendChild(triExpText);
   tbody.appendChild(erow);

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   row.appendChild(tableDiv);
   container.appendChild(row);

   return container;
}

/**
 * build the add event controls to create additional event studies
 * 
 * @returns {Object} table the dom element representing the control
 */
function buildAddEvent() {
   var container, row, table, tbody, trow, tableDiv;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 col-md-offset-2 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /* Header Table */
   table = tableHeaderHelper("Create New Event", [ "Event Type", "Create"
   ]);
   row.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   tbody = document.createElement("tbody");

   /* Local Event */
   // BEGIN SUSTAINMENT 2.1.1 Commented out
   // trow = document.createElement("tr");
   // trow.appendChild(textCellHelper("td", "Local Site"));
   // trow.appendChild(buttonCellHelper("Create Event", "createLocalEvent()"));
   // tbody.appendChild(trow);
   // END SUSTAINMENT 2.1.1 Commented out
   /* Adverse Event */
   trow = document.createElement("tr");
   trow.appendChild(textCellHelper("td", "Adverse/Reportable"));
   trow.appendChild(buttonCellHelper("Create Event", "createAdverseEvent()"));
   tbody.appendChild(trow);

   /* Continuing Review */
   trow = document.createElement("tr");
   trow.appendChild(textCellHelper("td", "Continuing Review"));
   trow.appendChild(buttonCellHelper("Create Event", "createContReview()"));
   tbody.appendChild(trow);

   /* Triennial Review */
   trow = document.createElement("tr");
   trow.appendChild(textCellHelper("td", "Triennial Review"));
   trow.appendChild(buttonCellHelper("Create Event", "createTriennialReview()"));
   tbody.appendChild(trow);

   /* Closeout */
   trow = document.createElement("tr");
   trow.appendChild(textCellHelper("td", "Closeout"));
   trow.appendChild(buttonCellHelper("Create Event", "createCloseoutReview()"));
   tbody.appendChild(trow);

   /* Admin Update */
   trow = document.createElement("tr");
   trow.appendChild(textCellHelper("td", "Administrative Update"));
   trow.appendChild(buttonCellHelper("Create Event", "createAdminUpdate()"));
   tbody.appendChild(trow);

   /* Amendment */
   trow = document.createElement("tr");
   trow.appendChild(textCellHelper("td", "Amendment"));
   trow.appendChild(buttonCellHelper("Create Event", "createAmendment()"));
   tbody.appendChild(trow);

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   row.appendChild(tableDiv);
   container.appendChild(row);

   return container;
}

/**
 * build the controls to display event history
 * 
 * @returns {Object} table the dom element representing the control
 */
function buildEventHistory() {
   var container, row, table, tbody, trow, tableDiv;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /* Header Table */
   table = tableHeaderHelper("Event History", [ "Event Type", "Date", "View"
   ]);
   row.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   tbody = document.createElement("tbody");

   var tempStudies = childrenStudies.values();
   for (var i = 0; i < tempStudies.length; i++) {
      trow = document.createElement("tr");
      trow.appendChild(textCellHelper("td", tempStudies[i].ApplicationType));
      trow.appendChild(textCellHelper("td", tempStudies[i].Created));
      trow.appendChild(buttonCellHelper("View", "Function(window.location.href('irb-applications-general.html?"
         + tempStudies[i].Study_ID + "'))"));
      tbody.appendChild(trow);
   }

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   row.appendChild(tableDiv);
   container.appendChild(row);

   return container;
}

/**
 * build the controls to submit the study and collect submission comments
 * 
 * @returns {Object} table the dom element representing the control
 */
function buildRenewalHistory() {
   var container, row, table, tbody, trow, tableDiv;

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /* Header Table */
   table = tableHeaderHelper("Renewal Reminder", [ "Renewal Date", "Risk Level"
   ]);
   row.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   tbody = document.createElement("tbody");

   trow = document.createElement("tr");
   if ((typeof spAppData.RenewalDate !== "undefined") && (spAppData.RenewalDate !== null)) {
      trow.appendChild(textCellHelper("td", spAppData.RenewalDate.split("T")[0]));
   } else {
      trow.appendChild(textCellHelper("td", spAppData.RenewalDate));
   }
   trow.appendChild(textCellHelper("td", spAppData.RiskLevel));
   tbody.appendChild(trow);

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   row.appendChild(tableDiv);
   container.appendChild(row);

   return container;
}

/**
 * update the study with reviewer comments
 * 
 * @param {string} targetTool the tool to switch to when done
 */
function sendAppReview(targetTool) {
   // var displayName = GetFacilityandCommitteeName();
   // var saveButton = $('#saveAdminTool');
   var selectCommittee = $('#selectCommittee').find(':selected').val();
   var commentType = $('#addCommentType').find(':selected').val();
   var section = $('#addCommentSection').find(':selected').val();
   var question = $('#addCommentQuestion').find(':selected').val();

   var comment = $('#addComment').val();
   var generalSave = false;
   var specificSave = false;

   var textToDisplay;
   var data = [];

   // customize how comment displayed based on type of comment
   if (commentType == "General Comment") { // comment type is general
      textToDisplay = "(" + selectCommittee + ": " + commentType + ")\n" + comment + "\n";
   } else { // comment type is specific - pertaining to a section and
      // question number
      textToDisplay = "(" + selectCommittee + ": " + commentType + " -- " + section + " -- " + question + ")\n"
         + comment + "\n";
   }

   // Enable Save if the 1st 4 questions are selected for a Specific Comment
   // and the last 2 for a General Comment
   // selected.
   if ((commentType == "Specific Comment") && (selectCommittee != "Select Committee")
      && (section != "Select a Section: ") && (question != "Select a Question: ") && (comment != "undefined")
      && (comment.trim() !== "")) {
      specificSave = true;
   } else if ((commentType == "General Comment") && (selectCommittee != "Select Committee") && (comment != "undefined")
      && (comment.trim() !== "")) {
      generalSave = true;
   }

   if (generalSave || specificSave) {

      var studyPackage = {
         // "formsID": section,
         "formsID" : "250",
         "hyperText" : textToDisplay,
         "published" : "false",
         "questionsID" : "8518",
         "redFlagged" : "false"
      };

      data.push(studyPackage);

      sendREST("POST", restPath.concat("study/" + assignedID + "/comments/" + targetStage), data, function() {
         swapToolGenerator(targetTool);
         // saveButton.fadeOut(400, function(){
         // saveButton.attr("class", "btn btn-default label-success");
         // saveButton.children().attr("class", "glyphicon
         // glyphicon-floppy-saved");
         // }).fadeIn(600);
      });

   } else {
      alert("Some of the fields do not contain valid values. Check that a selection is made for each enabled drop down and that the comments contain text.");
   }

}

/**
 * update the study with renewal date
 * 
 * @param {string} targetTool the tool to switch to when done, but not used
 */
function sendRenewalDate(targetTool) {
   /* jshint unused:false */
   var data = {};
   var saveButton = $('#saveAdminTool');
   var renewalDate = $('#continueReview').val();
   var commentDate = $('#commentPeriod').val();
   var riskLevel = $('#selectRiskLevel').val();

   if (((typeof renewalDate !== "undefined") && (renewalDate !== null) && (renewalDate !== ""))
      || ((typeof commentDate !== "undefined") && (commentDate !== null) && (commentDate !== ""))) {

      if ((renewalDate !== "undefined") && (renewalDate !== null) && (renewalDate !== "")) {
         renewalDate = renewalDate.split("/");
         renewalDate = renewalDate[2] + "-" + renewalDate[0] + "-" + renewalDate[1] + "T05:00:00Z";
      }

      if ((typeof commentDate !== "undefined") && (commentDate !== null) && (commentDate !== "")) {
         commentDate = commentDate.split("/");
         commentDate = commentDate[2] + "-" + commentDate[0] + "-" + commentDate[1] + "T05:00:00Z";
      }
   }

   if ((renewalDate != spAppData.RenewalDate) && (commentDate != spAppData.OData__x0031_5DayCommentPeriod)) {
      // both renewal date and comment date are added - need to update
      // everything.
      data = {
         "__metadata" : spAppData.__metadata,
         "RenewalDate" : renewalDate,
         "OData__x0031_5DayCommentPeriod" : commentDate,
         "RiskLevel" : riskLevel
      };
      spAppData.RenewalDate = renewalDate;
      spAppData.OData__x0031_5DayCommentPeriod = commentDate;
      spAppData.RiskLevel = riskLevel;
   } else if (renewalDate != spAppData.RenewalDate) {
      // Only renewal date is added - need to update renewal date and risk
      // level.
      data = {
         "__metadata" : spAppData.__metadata,
         "RenewalDate" : renewalDate,
         "RiskLevel" : riskLevel
      };

      spAppData.RenewalDate = renewalDate;
      spAppData.RiskLevel = riskLevel;
   } else if ((renewalDate == spAppData.RenewalDate) && (riskLevel != spAppData.RiskLevel)) {
      // renewal date did not change , but risk level did - have need to
      // update renewal date and
      // risk level.
      data = {
         "__metadata" : spAppData.__metadata,
         "RenewalDate" : renewalDate,
         "RiskLevel" : riskLevel
      };

      spAppData.RenewalDate = renewalDate;
      spAppData.RiskLevel = riskLevel;
   } else if (commentDate != spAppData.OData__x0031_5DayCommentPeriod) {
      // comment date did change - have need to update comment date and risk
      // level.
      data = {
         "__metadata" : spAppData.__metadata,
         "OData__x0031_5DayCommentPeriod" : commentDate,
         "RiskLevel" : riskLevel
      };

      spAppData.OData__x0031_5DayCommentPeriod = commentDate;
      spAppData.RiskLevel = riskLevel;
   }

   sendREST("POST", spAppData.__metadata.uri, data, function() {
      // swapToolGenerator(targetTool);
      saveButton.fadeOut(400, function() {
         saveButton.attr("class", "btn btn-default label-success");
         saveButton.children().attr("class", "glyphicon glyphicon-floppy-saved");
      }).fadeIn(600);
      swapToolGenerator("appEvent");
   }, "Merge");

   // else if((renewalDate != "") && (renewalDate != null)){
   // renewalDate = renewalDate.split("/");
   // renewalDate = renewalDate[2] + "-" + renewalDate[0] + "-" +
   // renewalDate[1] + "T05:00:00Z";

   // if(renewalDate != spAppData.RenewalDate){
   // renewalData = {
   // "__metadata": spAppData.__metadata,
   // "RenewalDate": renewalDate
   // }

   // spAppData.RenewalDate = renewalDate;

   // sendREST("POST", spAppData.__metadata.uri, renewalData, function(){
   // sendREST("POST", spsubsiteAppData.__metadata.uri, renewalData,
   // function(){
   // //swapToolGenerator(targetTool);
   // saveButton.fadeOut(400, function(){
   // saveButton.attr("class", "btn btn-default label-success");
   // saveButton.children().attr("class", "glyphicon glyphicon-floppy-saved");
   // }).fadeIn(600);
   // }, "Merge");
   // }, "Merge");
   // }
   // }else if((commentDate != "") && (commentDate != null)){
   // //grab local site liaison
   // commentDate = commentDate.split("/");
   // commentDate = commentDate[2] + "-" + commentDate[0] + "-" +
   // commentDate[1] + "T05:00:00Z";

   // if(commentDate != spAppData.OData__x0031_5DayCommentPeriod){
   // commentData = {
   // "__metadata": spAppData.__metadata,
   // "OData__x0031_5DayCommentPeriod": commentDate
   // }

   // spAppData.OData__x0031_5DayCommentPeriod = commentDate;

   // sendREST("POST", spAppData.__metadata.uri, commentData, function(){
   // sendREST("POST", spsubsiteAppData.__metadata.uri, commentData,
   // function(){
   // //swapToolGenerator(targetTool);
   // saveButton.fadeOut(400, function(){
   // saveButton.attr("class", "btn btn-default label-success");
   // saveButton.children().attr("class", "glyphicon glyphicon-floppy-saved");
   // }).fadeIn(600);
   // }, "Merge");
   // }, "Merge");
   // }
   // }
}

// function sendAssignApp20171227(targetTool) {
// // var saveButton = $('#saveAdminTool');
// var admin = $('#addAdmin').find(':selected').val(); // find value from
// dropdown --"Please..."
// // or "The Current"
// var reviewer = $('#addReviewer').find(':selected').val();
// var reviewerRole = $('#assignRole').val();
// var CMOD = $('#assignReviewMethod').find(':selected').val();
// var chkBoxesInt = "";
// var tempCollection, data, currentReviewers;
// var localTargetTool = targetTool;
//
// // need to grab these people from the adminCollection and then add them to
// the SP lists.
// if ((admin != "Please Select an Administrator") && (admin.indexOf("The
// Current") == -1)) { //
// display
// // name
// // selected
// tempCollection = adminCollection.values();
// if (admin == currentUser.Id) {
// localTargetTool = "assignApp";
// }
// for (var i = 0; i < tempCollection.length; i++) {
// if (tempCollection[i].Id == admin) { // found display name in dropdown
// var intoReview = localTargetTool === "assignIRBAdmin";
// data = {
// "__metadata" : spAppData.__metadata,
// "AssignedIRBAdministratorId" : tempCollection[i].Id
// };
// spAppData.AssignedIRBAdministratorId = tempCollection[i].Id;
// if (intoReview)
// data.ApplicationStatus = spAppData.ApplicationStatus = "Under Review";
//
// // console.log("We want to add: " + admin);
// // insertStudyMember(admin, "RAMS Reviewer", false, assignedID, null);
//
// sendREST("POST", spAppData.__metadata.uri, data, function() {
// var finishUp = function() {
// insertStudyMember(admin, "Study Administrator", false, assignedID, null);
// swapToolGenerator(localTargetTool);
// // saveButton.fadeOut(400, function(){
// // saveButton.attr("class", "btn btn-default label-success");
// // saveButton.children().attr("class", "glyphicon glyphicon-floppy-saved");
// // }).fadeIn(600);
// };
// if (!intoReview) {
// finishUp();
// } else {
// sendREST("POST", restPath.concat("study/" + assignedID + "/stage/Review"),
// null,
// finishUp);
// }
// }, "Merge");
// }
// }
// }
//
// if ((reviewer != null) && (reviewer != "Please Select a Reviewer")) {
// if ((spAppData.ReviewersId == null) || (reviewerRole == "Primary")) {
// currentReviewers = [];
// } else {
// currentReviewers = spAppData.ReviewersId.results;
// }
//
// tempCollection = ramsUsers.values();
// for (var i = 0; i < tempCollection.length; i++) {
// if (tempCollection[i].Id == reviewer) {
//
// currentReviewers.push(tempCollection[i].Id);
//
// if (reviewerRole == "Assigned") {
// data = {
// "__metadata" : spAppData.__metadata,
// "ReviewersId" : {
// "__metadata" : {
// "type" : "Collection(Edm.Int32)"
// },
// "results" : currentReviewers
// }
// };
//
// spAppData.ReviewersId = {
// "results" : currentReviewers
// };
// } else if (reviewerRole == "Primary") {
// // console.log("trying to send a primary reviewer: " + currentReviewers);
//
// data = {
// "__metadata" : spAppData.__metadata,
// "PrimaryReviewersId" : {
// "__metadata" : {
// "type" : "Collection(Edm.Int32)"
// },
// "results" : currentReviewers
// }
// };
//
// spAppData.PrimaryReviewersId = {
// "results" : currentReviewers[0]
// };
// }
// // JSHINT NOTE: data contains the reviewers to be saved to the RAMS DB. There
// are
// // only 2 types of reviewer roles,
// // Primary or Assigned so the JSHINT warning can be ignored
// // since data may not contain a value at this point, if it does then update,
// // otherwise log an error
// if (data) {
// sendREST("POST", spAppData.__metadata.uri, data, function() {
// insertStudyMember(reviewer, reviewerRole + " Reviewer", false, assignedID,
// null);
// swapToolGenerator("assignApp");
// }, "Merge");
// } else {
// logError(
// spAppData.__metadata.uri,
// "404",
// "undefined data when attempting to assign the application. Reviewer role is
// probably not set.",
// "sendAssignApp");
// }
// }
// }
// }
//
// if ((CMOD != null) && (CMOD != "Please Select a Compliant Method of Review")
// && (CMOD.indexOf("The Current") == -1)) {
// for (var i = 0; i < 7; i++) {
// if ($("#chkOpt" + i).length) {
// if ($("#chkOpt" + i).is(':checked'))
// chkBoxesInt += "*";
// else
// chkBoxesInt += "-";
// }
// }
//
// CMOD += " " + chkBoxesInt;
// data = {
// "__metadata" : spAppData.__metadata,
// "ApplicationType" : CMOD
// };
//
// spAppData.ApplicationType = CMOD;
// spAppData.Checkboxes = chkBoxesInt;
//
// sendREST("POST", spAppData.__metadata.uri, data, function() {
// swapToolGenerator("assignApp");
// }, "Merge");
// }
// }

/**
 * buld the smcPlusIncumbents object of all the study users
 * 
 * @param {string} targetAssignedID the assignedID of the study to retrieve the study member collection
 * @param {function} callback function to return to when complete
 */
function getStudyMembers(targetAssignedID, callback) {
   var smc = [];
   //var assignedReviewers = [];
   var irbAssignedReviewers = [];
   var iacucAssignedReviewers = [];
   var rdcAssignedReviewers = [];
   var srsAssignedReviewers = [];
   var sc1AssignedReviewers = [];
   var sc2AssignedReviewers = [];
   var sc3AssignedReviewers = [];
   smcPlusIncumbents = {
      otherNonReviwerRoles : [],
      IRB : {},
      IACUC : {},
      RDC : {},
      SRS : {},
      SC1 : {},
      SC2 : {},
      SC3 : {}
   };
   if (typeof callback !== "function") {
      // This identifies a programming error - the users should never see this
      // alert.
      alert("ERROR: the function provided as a callback to getStudyMembers was either not provided or is not a function.");
   }

   console.log("getStudyMembers: 1. at start of function. About to call REST study/{assignedId}/members.");
   // initialize the PlusIncumbents to undefined
   // smcPlusIncumbents.currentPI = undefined;
   // smcPlusIncumbents.currentCoPI = undefined;
   // smcPlusIncumbents.currentAdministrator = undefined;
   // smcPlusIncumbents.currentPrimaryReviewer = undefined;
   // smcPlusIncumbents.currentAssignedReviewers = undefined;

   sendREST("GET", restPath.concat("study/" + targetAssignedID + "/members"), null, function(returnedMembers) {
      console.log("getStudyMembers: 2. after call REST study/{assignedId}/members. # returned: "
         + returnedMembers.length);
      smc.assignedID = targetAssignedID;
      var returnedMemberRole;
      for (var i = 0; i < returnedMembers.length; i++) {
         var com = "";
         returnedMemberRole = returnedMembers[i].memberRole;

         // Add the studyTeamMember to the associative array
         // A User may have multiple Administrator or Reviewer Roles, but only one per committee. The presence of
         // a user in the SMC is only saying that they are a member one or more times in the study.
         smc[returnedMembers[i].user.SPUser] = returnedMembers[i];

         // find the roles that can have only one member in the role
         if (returnedMemberRole === PRINCIPALINVESTIGATOR) {
            smcPlusIncumbents.currentPI = returnedMembers[i];
         } else if (returnedMemberRole === COPRINCIPALINVESTIGATOR) {
            smcPlusIncumbents.currentCoPI = returnedMembers[i];
         } else if (returnedMemberRole.indexOf("Study Administrator") > -1) {
            com = returnedMemberRole.substring(0, returnedMemberRole.indexOf(' '));
            smcPlusIncumbents[com].currentAdministrator = returnedMembers[i];
         } else if (returnedMemberRole.indexOf(LIAISONREVIEWER) > -1 || returnedMemberRole.indexOf(LOCALSITEINVESTIGATOR) > -1 || Roles.indexOf(returnedMemberRole) > -1) {
            //assignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
            smcPlusIncumbents.otherNonReviwerRoles[returnedMembers[i].user.SPUser] = returnedMembers[i];
         } else if (returnedMemberRole.indexOf(PRIMARYREVIEWER) > -1) {
            com = returnedMemberRole.substring(0, returnedMemberRole.indexOf(' '));
            smcPlusIncumbents[com].currentPrimaryReviewer = returnedMembers[i];
         } else if (returnedMemberRole.indexOf(ASSIGNEDREVIEWER) > -1) {
            com = returnedMemberRole.substring(0, returnedMemberRole.indexOf(' '));
            if (com === IRB) {
               irbAssignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
            } else if (com === IACUC) {
               iacucAssignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
            } else if (com === RDC) {
               rdcAssignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
            } else if (com === SRS) {
               srsAssignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
            } else if (com === SC1) {
               sc1AssignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
            } else if (com === SC2) {
               sc2AssignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
            } else if (com === SC2) {
               sc3AssignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
            }

         }

         // } else if (returnedMembers[i].memberRole === PRIMARYREVIEWER) {
         // smcPlusIncumbents.currentPrimaryReviewer = returnedMembers[i];
         // } else if (returnedMembers[i].memberRole === ASSIGNEDREVIEWER) {
         // assignedReviewers[returnedMembers[i].user.SPUser] = returnedMembers[i];
         // }
      }
      // if (assignedReviewers) {
      // smcPlusIncumbents.currentAssignedReviewers = assignedReviewers;
      // }

      if (irbAssignedReviewers) {
         smcPlusIncumbents[IRB].currentAssignedReviewers = irbAssignedReviewers;
      }
      if (iacucAssignedReviewers) {
         smcPlusIncumbents[IACUC].currentAssignedReviewers = iacucAssignedReviewers;
      }
      if (rdcAssignedReviewers) {
         smcPlusIncumbents[RDC].currentAssignedReviewers = rdcAssignedReviewers;
      }
      if (srsAssignedReviewers) {
         smcPlusIncumbents[SRS].currentAssignedReviewers = srsAssignedReviewers;
      }
      if (sc1AssignedReviewers) {
         smcPlusIncumbents[SC1].currentAssignedReviewers = sc1AssignedReviewers;
      }
      if (sc2AssignedReviewers) {
         smcPlusIncumbents[SC2].currentAssignedReviewers = sc2AssignedReviewers;
      }
      if (sc3AssignedReviewers) {
         smcPlusIncumbents[SC3].currentAssignedReviewers = sc3AssignedReviewers;
      }
      if (smc) {
         smcPlusIncumbents.smc = smc;
         console.log("getStudyMembers: 3. resolving with smcPlusIncumbents object");
      } else {
         console.log("getStudyMembers: 3. rejecting  with error message");
      }
      console.log("getStudyMembers: 4. returning to callback");
      callback(smcPlusIncumbents);
   });

}

/**
 * update the study with assignment data
 * 
 * @param {string} targetTool the tool to switch to when done, but not used
 */

function sendAssignApp(targetTool) {
   var selectedAdmin = $('#addAdmin').find(':selected').val(); // find value
   // from dropdown
   var selectedCommittee = $('#selectCommittee').find(':selected').val();
   // --"Please..."
   // or "The Current"
   var selectedReviewer = $('#addReviewer').find(':selected').val();
   
   if(selectedAdmin === selectedReviewer){
      alert("Administrator and Reviewer cannot be the same person for the commmittee.");
      return;
   }
   var reviewerRole = $('#assignRole').val();
   var CMOD = $('#assignReviewMethod').find(':selected').val();
   var chkBoxesInt = "";
   var tempCollection;
   var data = {
      "__metadata" : spAppData.__metadata
   };
   var currentReviewers = [];
   var localTargetTool = targetTool;
   var intoReview = false;
   // var primaryReviewers = {
   // "PrimaryReviewersId" : {
   // "__metadata" : {
   // "type" : "Collection(Edm.Int32)"
   // },
   // "results" : currentReviewers[0]
   // }
   // };
   var assignedReviewers = {
      "ReviewersId" : {
         "__metadata" : {
            "type" : "Collection(Edm.Int32)"
         },
         "results" : currentReviewers
      }
   };
   var i;

   getStudyMembers(
      assignedID,
      function(smcPlusIncumbents) {
         if (smcPlusIncumbents) {
            // if an administrator is chosen from the dropdown and
            // there is no Currrent Admin in the SMC or there is an admin in the SMC and the SMC current admin is
            // different from the selected admin (i.e. the admin is being changed)
            if ((selectedAdmin != "Please Select an Administrator")
               && (!(smcPlusIncumbents[selectedCommittee].currentAdministrator) || ((smcPlusIncumbents[selectedCommittee].currentAdministrator) && (selectedAdmin != smcPlusIncumbents[selectedCommittee].currentAdministrator.user.SPUser)))) {

//               if (selectedAdmin == currentUser.Id) {
//                  // if the selected admin is the current user, navigate to the assign reviewers toolbar
//                  localTargetTool = "assignApp";
//               } else {
//                  if (((smcPlusIncumbents[IACUC].currentAdministrator) && (smcPlusIncumbents[IACUC].currentAdministrator.user.SPUser === currentUser.Id))
//                     || ((smcPlusIncumbents[IRB].currentAdministrator) && (smcPlusIncumbents[IRB].currentAdministrator.user.SPUser === currentUser.Id))
//                     || ((smcPlusIncumbents[RDC].currentAdministrator) && (smcPlusIncumbents[RDC].currentAdministrator.user.SPUser === currentUser.Id))
//                     || ((smcPlusIncumbents[SRS].currentAdministrator) && (smcPlusIncumbents[SRS].currentAdministrator.user.SPUser === currentUser.Id))
//                     || ((smcPlusIncumbents[SC1].currentAdministrator) && (smcPlusIncumbents[SC1].currentAdministrator.user.SPUser === currentUser.Id))
//                     || ((smcPlusIncumbents[SC2].currentAdministrator) && (smcPlusIncumbents[SC2].currentAdministrator.user.SPUser === currentUser.Id))
//                     || ((smcPlusIncumbents[SC3].currentAdministrator) && (smcPlusIncumbents[SC3].currentAdministrator.user.SPUser === currentUser.Id))) {
//                     localTargetTool = "assignApp";
//                  } else {
//                     // if the selected admin NOT is the current user, navigate to the assign admin toolbar
//                     localTargetTool = "assignIRBAdmin";
//                  }
//               }

               // If there is a current administrator and the administrator is not the selected administrator (i.e. it
               // is being changed), or if there is no current administrator and there is a selected administrator, mark
               // the study as into review to create a new review stage.
               if (((smcPlusIncumbents[selectedCommittee].currentAdministrator) && (smcPlusIncumbents[selectedCommittee].currentAdministrator.user.SPUser !== selectedAdmin))
                  || (!(smcPlusIncumbents[selectedCommittee].currentAdministrator) && (selectedAdmin != "Please Select an Administrator"))
                  || ((smcPlusIncumbents[selectedCommittee].currentAdministrator) && (smcPlusIncumbents[selectedCommittee].currentAdministrator.user.SPUser === selectedAdmin) 
                     && (spAppData.ApplicationStatus === RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTEDCHOICE))) {
                  // found display name in dropdown
                  // if the localTargetTool is assignIRBAdmin - mark the study as intoReview to create the review stage
                  // intoReview = localTargetTool === "assignIRBAdmin";
                  intoReview = true;
               }
               // data to be updated
               if (selectedCommittee === IRB) {
                  data.IRBAdministratorId = selectedAdmin;
                  spAppData.IRBAdministratorId = selectedAdmin;
                  insertStudyMember(selectedAdmin, IRBSTUDYADMINISTRATOR, false, assignedID, null);
                  startRAMSWorkflow(managePersonnelNotificationWorkflowSubscriptionId, null, users[selectedAdmin].displayName,"Assigned IRB Admin","You have been assigned as IRB committee Administrator");
               } else if (selectedCommittee === IACUC) {
                  data.IACUCAdministratorId = selectedAdmin;
                  spAppData.IACUCAdministratorId = selectedAdmin;
                  insertStudyMember(selectedAdmin, IACUCSTUDYADMINISTRATOR, false, assignedID, null);
                  startRAMSWorkflow(managePersonnelNotificationWorkflowSubscriptionId, null, users[selectedAdmin].displayName,"Assigned IACUC Admin", "You have been assigned as IACUC committee Administrator");
               } else if (selectedCommittee === RDC) {
                  data.RDCAdministratorId = selectedAdmin;
                  spAppData.RDCAdministratorId = selectedAdmin;
                  insertStudyMember(selectedAdmin, RDCSTUDYADMINISTRATOR, false, assignedID, null);
                  startRAMSWorkflow(managePersonnelNotificationWorkflowSubscriptionId, null, users[selectedAdmin].displayName,"Assigned RDC Admin", "You have been assigned as RDC committee Administrator");
               } else if (selectedCommittee === SRS) {
                  data.SRSAdministratorId = selectedAdmin;
                  spAppData.SRSAdministratorId = selectedAdmin;
                  insertStudyMember(selectedAdmin, SRSSTUDYADMINISTRATOR, false, assignedID, null);
                  startRAMSWorkflow(managePersonnelNotificationWorkflowSubscriptionId, null, users[selectedAdmin].displayName,"Assigned SRS Admin", "You have been assigned as SRS committee Administrator");
               } else if (selectedCommittee === SC1) {
                  data.SC1AdministratorId = selectedAdmin;
                  spAppData.SC1AdministratorId = selectedAdmin;
                  insertStudyMember(selectedAdmin, SC1STUDYADMINISTRATOR, false, assignedID, null);
                  startRAMSWorkflow(managePersonnelNotificationWorkflowSubscriptionId, null, users[selectedAdmin].displayName,"Assigned SC1 Admin", "You have been assigned as SC1 committee Administrator");
               } else if (selectedCommittee === SC2) {
                  data.SC2AdministratorId = selectedAdmin;
                  spAppData.SC2AdministratorId = selectedAdmin;
                  insertStudyMember(selectedAdmin, SC2STUDYADMINISTRATOR, false, assignedID, null);
                 startRAMSWorkflow(managePersonnelNotificationWorkflowSubscriptionId, null, users[selectedAdmin].displayName,"Assigned SC2 Admin", "You have been assigned as SC2 committee Administrator");
               } else if (selectedCommittee === SC3) {
                  data.SC3AdministratorId = selectedAdmin;
                  spAppData.SC3AdministratorId = selectedAdmin;
                  insertStudyMember(selectedAdmin, SC3STUDYADMINISTRATOR, false, assignedID, null);
                 startRAMSWorkflow(managePersonnelNotificationWorkflowSubscriptionId, null, users[selectedAdmin].displayName,"Assigned SC3 Admin", "You have been assigned as SC3 committee Administrator");
               }
               // data.AssignedIRBAdministratorId =
               // tempCollection[i].Id;
               // spAppData.AssignedIRBAdministratorId =
               // tempCollection[i].Id;
               if (intoReview) {
                  data.ApplicationStatus = spAppData.ApplicationStatus = RAMS.Environment.SharePoint.List.CommitteeStatus.UNDERREVIEWCHOICE;
               }
               // insertStudyMember(selectedAdmin,
               // "Study Administrator", false,
               // assignedID, null);
               // break;
               // }
            } else if ((selectedAdmin != "Please Select an Administrator")
               && (smcPlusIncumbents[selectedCommittee].currentAdministrator) && (smcPlusIncumbents[selectedCommittee].currentAdministrator.user.SPUser == selectedAdmin) 
               && (spAppData.ApplicationStatus === RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTEDCHOICE)) {
               intoReview = true;
               data.ApplicationStatus = spAppData.ApplicationStatus = RAMS.Environment.SharePoint.List.CommitteeStatus.UNDERREVIEWCHOICE;
            }
         }

         // need to grab the reviewer and then add them to the smc
         // and SP application list.
         // JSHINT NOTE: data contains the reviewers to be saved to
         // the RAMS DB. There are
         // only 2 types of reviewer roles,
         // Primary or Assigned so the JSHINT warning can be ignored
         // since data may not contain a value at this point, if it
         // does then update,
         // otherwise log an error
         if ((typeof selectedReviewer !== "undefined") && (selectedReviewer !== null)
            && (selectedReviewer != "Please Select a Reviewer")) {
            // var car = smcPlusIncumbents.currentAssignedReviewers;
            var car = smcPlusIncumbents[selectedCommittee].currentAssignedReviewers;
            if (!(car) || (car.length === 0) || (reviewerRole == "Primary")) {
               currentReviewers = [];
            } else {
               for (i in car) {
                  if (car.hasOwnProperty(i)) {
                     currentReviewers.push(car[i].user.SPUser);
                  }
               }
            }

            if (!(smcPlusIncumbents[selectedCommittee].currentPrimaryReviewer)
               || ((smcPlusIncumbents[selectedCommittee].currentPrimaryReviewer) && (selectedReviewer != smcPlusIncumbents[selectedCommittee].currentPrimaryReviewer.user.SPUser))) {
               tempCollection = ramsUsers.values();
               for (i = 0; i < tempCollection.length; i++) {
                  if (tempCollection[i].Id == selectedReviewer) {

                     currentReviewers.push(tempCollection[i].Id);

                     if (reviewerRole == "Assigned") {
                        var comReviewersId = selectedCommittee + "ReviewersId";
                        data[comReviewersId] = assignedReviewers;
                        data[comReviewersId].results = currentReviewers;
                        spAppData[comReviewersId] = {
                           "results" : currentReviewers
                        };
                     } else if (reviewerRole == "Primary") {
                        var comPrimaryReviewerId = selectedCommittee + "PrimaryReviewerId";
                        data[comPrimaryReviewerId] = selectedReviewer;
                        // data[comPrimaryReviewerId].results = currentReviewers;
                        spAppData[comPrimaryReviewerId] = selectedReviewer;
                     }
                     if ((reviewerRole == "Primary") || (reviewerRole == "Assigned")) {
                        insertStudyMember(selectedReviewer, selectedCommittee + " " + reviewerRole + " Reviewer",
                           false, assignedID, null);
                        // localTargetTool = "assignApp";
                        startRAMSWorkflow(managePersonnelNotificationWorkflowSubscriptionId, null, users[selectedReviewer].displayName,"Assigned "+selectedCommittee+" Reviewer", "You have been assigned as an "+selectedCommittee+" Committee "+reviewerRole+" Reviewer");
                     }
                     break;
                  }
               }

            }
         }

         // Setup the CMOD fields for storing in the Application List
         if ((typeof CMOD !== "undefined") && (CMOD !== null) && (CMOD != "Please Select a Compliant Method of Review")
            && (CMOD.indexOf("The Current") == -1)) {
            for (i = 0; i < 7; i++) {
               if ($("#chkOpt" + i).length) {
                  if ($("#chkOpt" + i).is(':checked')) {
                     chkBoxesInt += "*";
                  } else {
                     chkBoxesInt += "-";
                  }
               }
            }

            CMOD += " " + chkBoxesInt;
            data.ApplicationType = CMOD;
            spAppData.ApplicationType = CMOD;
            spAppData.Checkboxes = chkBoxesInt;
            // localTargetTool = "assignApp";

         }

         var cirbId = document.getElementById('cirb_id');
         if ((typeof cirbId !== "undefined") && (cirbId !== null) && (typeof cirbId.value !== "undefined")
            && (cirbId.value !== null)) {
            data.CIRB_ID = cirbId.value;
            spAppData.CIRB_ID = cirbId.value;
         }

         var refreshPersonnel = false;
         if ($('table:contains("Personnel")').length) {
            refreshPersonnel = true;
         }
         
         //set this value not to trigger committe broker for Assign Admin, reviewer and CMOD
         
         data.CommitteeBrokerDirective = "IgnoreCommitteeBroker";

         if (intoReview) {
            sendREST("POST", restPath.concat("study/" + assignedID + "/stage/Review"), null, function() {
               
               sendREST("GET", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq '" + assignedID
                     + "' or ParentStudyID eq '" + assignedID + "'", null, function(json) {
                     /* jshint unused:false */
                     sendREST("GET", restPath + "study/" + assignedID, null, function(studyResponse) {
     
                        /*jshint -W020 */
                        // ignore the jshint read-only warning here because we are intentionally updating fileds defined 
                        // in dynamicGeneration
                        targetStage = studyResponse.studyStatusCollection.length;
                        targetState = studyResponse.studyStatusCollection[targetStage - 1].studyState.studyStatesID;
                        currentFormVersion = studyResponse.formVersionsID;
                        currentStudyStatus = studyResponse.studyStatusCollection;
                        studyTitle = studyResponse.title;
                        
                        sendREST("POST", spAppData.__metadata.uri, data, function() {
                           if (refreshPersonnel) {
                              refreshDisplay('Personnel');
                           }
                           getStudyMembers(assignedID, function(smcPlusIncumbents) {
                              if (((smcPlusIncumbents[IACUC].currentAdministrator) && (smcPlusIncumbents[IACUC].currentAdministrator.user.SPUser === currentUser.Id))
                                 || ((smcPlusIncumbents[IRB].currentAdministrator) && (smcPlusIncumbents[IRB].currentAdministrator.user.SPUser === currentUser.Id))
                                 || ((smcPlusIncumbents[RDC].currentAdministrator) && (smcPlusIncumbents[RDC].currentAdministrator.user.SPUser === currentUser.Id))
                                 || ((smcPlusIncumbents[SRS].currentAdministrator) && (smcPlusIncumbents[SRS].currentAdministrator.user.SPUser === currentUser.Id))
                                 || ((smcPlusIncumbents[SC1].currentAdministrator) && (smcPlusIncumbents[SC1].currentAdministrator.user.SPUser === currentUser.Id))
                                 || ((smcPlusIncumbents[SC2].currentAdministrator) && (smcPlusIncumbents[SC2].currentAdministrator.user.SPUser === currentUser.Id))
                                 || ((smcPlusIncumbents[SC3].currentAdministrator) && (smcPlusIncumbents[SC3].currentAdministrator.user.SPUser === currentUser.Id))) {
                                 localTargetTool = "assignApp";
                              } else {
                                 // if the selected admin NOT is the current user, navigate to the assign admin toolbar
                                 localTargetTool = "assignIRBAdmin";
                              }
                              swapToolGenerator(localTargetTool);
                           });
                        }, "Merge");
                 });
               });
            });
         } else {
            sendREST("POST", spAppData.__metadata.uri, data, function() {
               if (refreshPersonnel) {
                  refreshDisplay('Personnel');
               }
               getStudyMembers(assignedID, function(smcPlusIncumbents) {
                  if (((smcPlusIncumbents[IACUC].currentAdministrator) && (smcPlusIncumbents[IACUC].currentAdministrator.user.SPUser === currentUser.Id))
                     || ((smcPlusIncumbents[IRB].currentAdministrator) && (smcPlusIncumbents[IRB].currentAdministrator.user.SPUser === currentUser.Id))
                     || ((smcPlusIncumbents[RDC].currentAdministrator) && (smcPlusIncumbents[RDC].currentAdministrator.user.SPUser === currentUser.Id))
                     || ((smcPlusIncumbents[SRS].currentAdministrator) && (smcPlusIncumbents[SRS].currentAdministrator.user.SPUser === currentUser.Id))
                     || ((smcPlusIncumbents[SC1].currentAdministrator) && (smcPlusIncumbents[SC1].currentAdministrator.user.SPUser === currentUser.Id))
                     || ((smcPlusIncumbents[SC2].currentAdministrator) && (smcPlusIncumbents[SC2].currentAdministrator.user.SPUser === currentUser.Id))
                     || ((smcPlusIncumbents[SC3].currentAdministrator) && (smcPlusIncumbents[SC3].currentAdministrator.user.SPUser === currentUser.Id))) {
                     localTargetTool = "assignApp";
                  } else {
                     // if the selected admin NOT is the current user, navigate to the assign admin toolbar
                     localTargetTool = "assignIRBAdmin";
                  }
                  swapToolGenerator(localTargetTool);
               });
            }, "Merge");
         }
         // }
      });
}

/**
 * save the PI Review toolbar
 * 
 * @depricated
 */
// NOT USED
// function sendPIReview() {
// // var saveButton = $('#saveAdminTool');
// var comment = $('#submissionComment').val();
// var data = [];
//
// var commentPackage = {
// "formsID" : "250",
// "hyperText" : comment + "\n",
// "published" : "true",
// "questionsID" : "8518",
// "redFlagged" : "false"
// };
//
// data.push(commentPackage);
//
// sendREST("POST", restPath.concat("study/" + assignedID + "/comments/" +
// targetStage), data, function() {
// swapToolGenerator("studyComments");
// });
//
// }
// NO LONGER USED
// function removePrimaryReviewer(id) {
// data = {
// "__metadata" : spAppData.__metadata,
// "PrimaryReviewersId" : null
// };
//
// sendREST("POST", spAppData.__metadata.uri, data, function() {
// swapToolGenerator("assignApp");
// }, "Merge");
// }
/**
 * remove the assignedReviewer from the study
 * 
 * @param {number} SPUserId the SPUserId of the user to be removed
 */
/* exported removeAssignedReviewer */
function removeAssignedReviewer(SPUserId) {
   // var data;
   // var reviewers = [];
   //
   // /* Building new reviewers list, without the one to be removed */
   // for (var i = 0; i < spAppData.ReviewersId.results.length; i++) {
   // if (spAppData.ReviewersId.results[i] != id) {
   // reviewers.push(spAppData.ReviewersId.results[i]);
   // }
   // }
   //
   // /* packaging new reviewers list */
   // data = {
   // "__metadata" : spAppData.__metadata,
   // "ReviewersId" : {
   // "__metadata" : {
   // "type" : "Collection(Edm.Int32)"
   // },
   // "results" : reviewers
   // }
   // };
   //
   // /* updating our local records */
   // spAppData.ReviewersId.results = reviewers;

   var currentReviewers = [];
   var data = {
      "__metadata" : spAppData.__metadata
   };
   var assignedReviewers = {
      "ReviewersId" : {
         "__metadata" : {
            "type" : "Collection(Edm.Int32)"
         },
         "results" : currentReviewers
      }
   };

   getRAMSUsers(function() {
      // remove the Assigned Reviewer from the SMC and remove folder
      // permissions
      removeStudyMember(SPUserId, "Assigned Reviewer", true, assignedID, null);
      // refresh the smc
      getStudyMembers(assignedID, function(smcPlusIncumbents) {
         /* jshint unused:false */
         // if ((smcPlusIncumbents)
         // && (smcPlusIncumbents.currentAssignedReviewers)
         // && (smcPlusIncumbents.currentAssignedReviewers.length > 0)) {
         // for ( var car in smcPlusIncumbents.currentAssignedReviewers) {
         // if (smcPlusIncumbents.currentAssignedReviewers
         // .hasOwnProperty(car)) {
         // currentReviewers
         // .push(smcPlusIncumbents.currentAssignedReviewers[car].user.SPUser);
         // }
         // }
         // }
         data.ReviewersId = assignedReviewers;
         data.ReviewersId.results = currentReviewers;
         spAppData.ReviewersId = {
            "results" : currentReviewers
         };

         // update application list in SharePoint and then regenerate
         // the toolbar
         sendREST("POST", spAppData.__metadata.uri, data, function() {
            swapToolGenerator("assignApp");
         }, "Merge");
      });
   });
}

/**
 * submit the study application
 * 
 * @param {string} appStatus the status of the application
 * @param {number} appStage the stage of the study
 */
function sendSubmitApp(appStatus, appStage) {
   var currentTime = new Date();
   var month = currentTime.getMonth() + 1;
   var day = currentTime.getDate();
   var year = currentTime.getFullYear();
   currentTime = month + '/' + day + '/' + year;

   var committeeVal = $('#selectCommittee').find(':selected').val();
   var comments = $('#submissionComment').val();
   var commentPackage;
   var data = [];

   var spData = null;
   var expirId = document.getElementById('expirationId');
   var triExpirId = document.getElementById('triennialExpirationId');

   var comStatus = committeeVal + "StatusId";
   var comApprovedDate =  committeeVal + "ApprovedDate";
   var comExpiresDate = committeeVal + "ExpiresDate";
   var comTriennialReviewDate = committeeVal + "TriennialReviewDate";

   //var comExpiresPeriod = committeeVal + "ExpiresPeriod";
   //var comTriennialReviewPeriod = committeeVal + "TriennialReviewPeriod";
   if(committeeVal == RDC){
      if ((spAppData.IRBReviewRequired !== null && spAppData.IRBReviewRequired === true && spAppData.IRBStatusId > 6 ) || (spAppData.IACUCReviewRequired !== null && spAppData.IACUCReviewRequired === true && spAppData.IACUCStatusId > 6) 
         || (spAppData.SRSReviewRequired !==null && spAppData.SRSReviewRequired === true && spAppData.SRSStatusId > 6)  || (spAppData.SC1ReviewRequired !== null && spAppData.SC1ReviewRequired === true && spAppData.SC1StatusId > 6)
         || (spAppData.SC2ReviewRequired !== null && spAppData.SC2ReviewRequired === true && spAppData.SC2StatusId > 6) || (spAppData.SC3ReviewRequired !== null && spAppData.SC3ReviewRequired === true && spAppData.SC3StatusId > 6)) {
         
         var r = confirm("**WARNING**\n\nAll the required sub-committees have NOT completed their review and approval.\n\nAre you sure you want to approve the study?\n\nClick the [Ok] button to continue with RDC approval. Click the [Cancel] button to return.");
         if (r !== true){
            return;
         }
      }
   }
   
  
  

   var appStatusId = appStatus.replace(/ /g, "").toUpperCase();
   appStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus[appStatusId];

   if ((appStatus.indexOf(RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDCHOICE) > -1)
      && ((typeof spAppData.ParentStudyID === "undefined") || (spAppData.ParentStudyID === null))) {
      spData = {
         "__metadata" : spAppData.__metadata,
         // "ApplicationStatus" : appStatus,
         //comStatus : appStatusId,
         //"InitialApprovalDate" : currentTime
      };
      if(committeeVal == RDC){
         spData.InitialApprovalDate = currentTime;
         spData.ApplicationStatus = "Approved";
      }else{ 
         spData.ApplicationStatus = RAMS.Environment.SharePoint.List.CommitteeStatus.UNDERREVIEWCHOICE; // Defect 752061
      }
      spData[comStatus] = appStatusId;
      spData[comApprovedDate] = currentTime;
      
      // TODO TAR Which expiresDate should be set?
      if ((typeof expirId !== "undefined") && (expirId !== null) && (typeof expirId.value !== "undefined")
         && (expirId.value !== null)) {
         // spData.ExpiringDate = expirId.value;
         //spData[comExpiresPeriod] = expirId.value;
         spData[comExpiresDate] = expirId.value;
      }

      if ((typeof triExpirId !== "undefined") && (triExpirId !== null) && (typeof triExpirId.value !== "undefined")
         && (triExpirId.value !== null) && (committeeVal == IACUC)) {
         // spData.IACUCTriennialReviewDate = triExpirId.value;
         //spData[comTriennialReviewPeriod] = triExpirId.value;
         spData[comTriennialReviewDate] = triExpirId.value;
      }
   } else if ((appStatus.indexOf(RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDCHOICE) > -1)
      && (typeof spAppData.ParentStudyID !== "undefined") && (spAppData.ParentStudyID !== null)) {
      spData = {
         "__metadata" : spAppData.__metadata,
         // "ApplicationStatus" : appStatus,
         //comStatus : appStatusId,
         //"LastApprovalDate" : currentTime
      };
      if(committeeVal == RDC){
         spData.LastApprovalDate = currentTime;
         spData.ApplicationStatus = "Approved";
      }else{
         spData.ApplicationStatus = RAMS.Environment.SharePoint.List.CommitteeStatus.UNDERREVIEWCHOICE; // Defect 752061
      }
      spData[comStatus] = appStatusId;
      spData[comApprovedDate] = currentTime;
      
      // TODO TAR Which expiresDate should be set?
      if ((typeof expirId !== "undefined") && (expirId !== null) && (typeof expirId.value !== "undefined")
         && (expirId.value !== null)) {
         // spData.ExpiringDate = expirId.value;
         //spData[comExpiresPeriod] = expirId.value;
         spData[comExpiresDate] = expirId.value;
      }

      if ((typeof triExpirId !== "undefined") && (triExpirId !== null) && (typeof triExpirId.value !== "undefined")
         && (triExpirId.value !== null)) {
         // spData.IACUCTriennialReviewDate = triExpirId.value;
         //spData[comTriennialReviewPeriod] = triExpirId.value;
         spData[comTriennialReviewDate] = triExpirId.value;
      }

      if ((spAppData.StudyTitle.indexOf("Closeout") != -1) && (committeeVal == RDC)) {
         var data2 = {
            "__metadata" : {
               "type" : "SP.Data.ApplicationListItem"
            },
            "Study_ID" : spAppData.ParentStudyID,
            "ApplicationStatus" : RAMS.Environment.SharePoint.List.CommitteeStatus.CLOSEDCHOICE
         };
         sendREST("GET", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq '"
            + spAppData.ParentStudyID + "'", null, function(json) {
            var temp = json.d.results;
            sendREST("POST", temp[0].__metadata.uri, data2, function() {
            }, "Merge");
         });
      }
   } else {
      spData = {
         "__metadata" : spAppData.__metadata,
         "ApplicationStatus" : appStatus
      };
      
      if(committeeVal){
         spData[comStatus] = appStatusId;
      }
   }

   // TODO What is this hard-coding horror?
   if (comments !== "") {
      commentPackage = {
         // "formsID": section,
         "formsID" : "250",
         "hyperText" : comments + "\n",
         "published" : "true",
         "questionsID" : "8518",
         "redFlagged" : "false"
      };
   }

   // JSHINT NOTE: commentPackage may not contain a value at this point, if it
   // does then add to
   // data for the update
   if (commentPackage) {
      data.push(commentPackage);
   }

   // BEGIN SUSTAINMENT 2.1.1
   // TAR merged it above
   // Only update Exiring Date if appStatus contains the word Approved
   // var expirId = document.getElementById('expirationId');
   // if (appStatus.indexOf("Approved") > -1) {
   // if (expirId != null && expirId.value != null) {
   // var data2 = {
   // "__metadata" : {
   // "type" : "SP.Data.ApplicationListItem"
   // },
   // "Study_ID" : assignedID,
   // "ExpiringDate" : expirId.value,
   // "ParentStudyID" : parentID
   // };
   // sendREST("GET", siteUrl +
   // "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq
   // '"
   // + assignedID + "'", null, function(json) {
   // var temp = json.d.results;
   // sendREST("POST", temp[0].__metadata.uri, data2, function() {
   // }, "Merge");
   // });
   // }
   // if (spAppData.StudyTitle.indexOf("Closeout") != -1) {
   // var data2 = {
   // "__metadata" : {
   // "type" : "SP.Data.ApplicationListItem"
   // },
   // "Study_ID" : spAppData.ParentStudyID,
   // "ApplicationStatus" : "Closed"
   // };
   // sendREST("GET", siteUrl +
   // "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq
   // '"
   // + spAppData.ParentStudyID + "'", null, function(json) {
   // var temp = json.d.results;
   // sendREST("POST", temp[0].__metadata.uri, data2, function() {
   // }, "Merge");
   // });
   // }
   // }
   // END SUSTAINMENT 2.1.1

   if (appStatus.indexOf(RAMS.Environment.SharePoint.List.CommitteeStatus.APPROVEDCHOICE) > -1) {
      var stage="";
      sendREST("POST", spAppData.__metadata.uri, spData, function() {
         if(committeeVal == RDC){
            stage = "Approved";
         }else{
            stage = "Review";
         }
         sendREST("POST", restPath.concat("study/" + assignedID + "/stage/" + stage), null, function() {
            if (comments !== "") {
               sendREST("POST", restPath.concat("study/" + assignedID + "/comments/" + (targetStage + 1)), data,
                  function() {
                     window.location.href("/sites/irbapplications/sitepages/irb%20applications.aspx");
                  });
            } else {
               window.location.href("/sites/irbapplications/sitepages/irb%20applications.aspx");
            }
         });
      }, "Merge");
      

   } else {

      sendREST("POST", spAppData.__metadata.uri, spData, function() {
         sendREST("POST", restPath.concat("study/" + assignedID + "/stage/" + appStage), null, function() {
            if (comments !== "") {
               sendREST("POST", restPath.concat("study/" + assignedID + "/comments/" + (targetStage + 1)), data,
                  function() {
                     window.location.href("/sites/irbapplications/sitepages/irb%20applications.aspx");
                  });
            } else {
               window.location.href("/sites/irbapplications/sitepages/irb%20applications.aspx");
            }
         });
      }, "Merge");
   }

}

/**
 * save the comments
 * 
 * @param targetTool
 */
function publishComments(targetTool) {
   var comments = $('#submissionComment').val();
   var data = [];

   var commentPackage = {
      // "formsID": section,
      "formsID" : "250",
      "hyperText" : comments + "\n",
      "published" : "true",
      "questionsID" : "8518",
      "redFlagged" : "false"
   };

   data.push(commentPackage);

   sendREST("POST", restPath.concat("study/" + assignedID + "/comments/" + targetStage), data, function() {
      swapToolGenerator(targetTool);
   });
}

/**
 * download the selected checklist
 * 
 * @param {string} relativeUrl Url of the document do download
 */
/* exported downloadChecklist */
function downloadChecklist(relativeUrl) {
   var splitAt;
   var destUrl = relativeUrl;

   // console.log("the url is " + relativeUrl);

   splitAt = siteUrl.split("sites")[1];
   destUrl = relativeUrl.split(splitAt)[1];

   // console.log("the url is " + relativeUrl);

   window.open(siteUrl.concat(destUrl));
}

// BEGIN SUSTAINMENT 2.1.1 - commented out
// function createLocalEvent() {
// generateAssignedId("Local");
// }
// END SUSTAINMENT 2.1.1 - commented out

/**
 * onclick function to create the adverse event child application
 */
/* exported createAdverseEvent */
function createAdverseEvent() {
   generateAssignedId("Adverse");
}

/**
 * onclick function to create the ContReview event child application
 */
/* exported createContReview */
function createContReview() {
   generateAssignedId("ContReview");
}

/**
 * onclick function to create the Closeout event child application
 */
/* exported createCloseoutReview */
function createCloseoutReview() {
   generateAssignedId("Closeout");
}

/**
 * onclick function to create the AdministrativeUpdate event child application
 */
/* exported createAdminUpdate */
function createAdminUpdate() {
   generateAssignedId("AdministrativeUpdate");
}

/**
 * onclick function to create the Amendment event child application
 */
/* exported createAmendment */
function createAmendment() {
   generateAssignedId("Amendment");
}

/**
 * onclick function to create the Triennial Review event child application
 */
/* exported createTriennialReview */
function createTriennialReview() {
   generateAssignedId("TriennialReview");
}

/**
 * upload the user completed checklist to the designated folder
 * 
 * @param {string} targetTool the name of the tool to go to when complete
 * @param {string} teamFolder the folder name to upload the file to
 */
function sendReviewChecklist(targetTool, teamFolder) {
   var fileList, fileName;
   var upload = $('#uploadChecklist');

   if ((typeof window.FileReader !== "undefined") && (window.FileReader !== null)) {
      fileList = upload[0].files;
      // console.log(fileList);
      for (var i = 0; i < fileList.length; i++) {
         if (teamFolder == "Study%20Team") {
            fileName = "-CHECKLIST-" + fileList[i].name;
         } else {
            fileName = fileList[i].name;
         }

         // console.log("we really want to upload: " + fileName);
         sendFile(fileList[i], fileName, fileList[i].size, teamFolder, function() {
            swapToolGenerator(targetTool);
         });
      }
   } else {
      alert('Please upgrade your browser to upload files.');
   }
}

// this should take in the current facility > then find the committee lookup >
// then see if this
// users groups
// match the committee member and irb admins of that site.
/**
 * determine if the user is in the committee Member security group or the Admin groups.
 * 
 * @param {function} callback the function to return to when complete
 */
function groupCheck(callback) {
   
   var isAdmin = false;
   sendREST("GET", siteUrl.concat("_api/Web/CurrentUser/Groups"), null, function(returnedGroups) {
      var currentGroups = returnedGroups.d.results;
      var mySecurityGroups=[];
      for (var i = 0; i < currentGroups.length; i++) {
         mySecurityGroups[currentGroups[i].Id] = currentGroups[i];
      }
    
      ficObj.get(spAppData.FacilitiesLookupId, function(locfiObj) {
         var reviewRequired;
         if (spAppData.ReviewRequired) {
            reviewRequired= spAppData.ReviewRequired.split(",");
            for ( var com in reviewRequired) {
               var ci = locfiObj.fi.getCommitteeGroup(reviewRequired[com]);
               if((ci) && (mySecurityGroups[ci.getAdministrators().id])){
                  isAdmin = true;
                  break;
               }
            }
            if (!(isAdmin) && (spAppData.ApplicationSiteType == "Central")) {
               ficObj.get(RAMS.Environment.SharePoint.CentralIRBSite, function(centralfiObj) {
                  if (spAppData.ReviewRequired) {
                     reviewRequired= spAppData.ReviewRequired.split(",");
                     for ( var com in reviewRequired) {
                        var ci = locfiObj.fi.getCommitteeGroup(reviewRequired[com]);
                        if(mySecurityGroups[ci.getAdministrators().id]){
                           isAdmin = true;
                           break;
                        }
                     }
                  }
                  callback(isAdmin);
            });
            } else {
               callback(isAdmin);
            }
         } else {
            callback(isAdmin);
         }
      });
   });
   
   
  /** var committeeMember = false;
   var irbAdmin = false;
   sendREST("GET", siteUrl.concat("_api/Web/CurrentUser/Groups"), null, function(returnedGroups) {
      var currentGroups = returnedGroups.d.results;

      for (var i = 0; i < currentGroups.length; i++) {
         // console.log("you are a part of: " +
         // currentGroups[i].Title);
         if (currentGroups[i].Title.indexOf("Committee Members") > -1) {
            committeeMember = true;
         } else if ((currentGroups[i].Title.indexOf("RDC Administrator") > -1)
            || (currentGroups[i].Title.indexOf("IRB Administrator") > -1)
            || (currentGroups[i].Title.indexOf("IACUC Administrator") > -1)
            || (currentGroups[i].Title.indexOf("SRS Administrator") > -1)) {
            // console.log("you are an irb admin");
            irbAdmin = true;
         }
      }
      // console.log("sending: committeeMember " + committeeMember +
      // ", irbAdmin " + irbAdmin);
      callback(committeeMember, irbAdmin);
   }); **/
}

/**
 * Grab all local site applications of a parent and update status for each list item
 * 
 * @param {string} parentId the assignedID of the parent study
 * @param {} parentStatus the parent study's status
 */
function notifyLocalSitesOnParentStatus(parentId, parentStatus) {
   var applicationUrl = siteUrl.concat("_api/web/lists/getbytitle('Application')/items?$filter=ParentStudyID eq '"
      + parentId + "'");
   return $.ajax({
      async : false,
      type : 'GET',
      url : applicationUrl,
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         updateLocalSiteParentStatus(json, parentStatus);
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((applicationUrl), errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
      }
   });
}

/**
 * update the child study witht he parent's status
 * 
 * @param {ApplicationListItems[]} localSites the application list items with the same parent id
 * @param {string} status the status to set on the study
 */
function updateLocalSiteParentStatus(localSites, status) {

   var temp = localSites.d.results;
   for (var i = 0; i < temp.length; i++) {
      // alert('temp index is '+ i);

      // alert (temp[i].ParticipatingFacilitiesLookupId + ' eq' +
      // currentFacility);
      var spData = {
         "__metadata" : spAppData.__metadata,
         "ParentStudyStatus" : status
      };
      sendREST("POST", temp[i].__metadata.uri, spData, null, "Merge");
   }
}

/**
 * determine the facility and committee name from the url
 * 
 * @returns {string} displayName the "Facility Committee:"
 * @depricated
 */
// NOT USED
// function GetFacilityandCommitteeName() {
// var user;
// var displayName = "";
// $.ajax({
// async : false,
// type : 'GET',
// url : siteUrl.concat("_api/Web/CurrentUser/Groups"),
// headers : {
// "accept" : "application/json;odata=verbose"
// },
// dataType : 'json',
// success : function(json) {
// user = json.d.results;
// for (var i = 0; i < user.length; i++) {
// if ((user[i].Title.indexOf("Administrators") > -1) && (facilityUrl !== "") &&
// (committeeUrl !== "")) {
// displayName = facilityUrl + " " + committeeUrl + ": ";
// }
// }
// }
// });
// return displayName;
// }
/**
 * determine what committees the user belongs to based on the security groups the user belongs to.
 * 
 * @returns {string[]} displayCommittees the list of committees that the user belongs to.
 */
function getCommitteeName() {
   var user;
   var displayIACUC = "";
   var displayIRB = "";
   var displayRDC = "";
   var displaySRS = "";
   var displaySC1 = "";
   var displaySC2 = "";
   var displaySC3 = "";
   $.ajax({
      async : false,
      type : 'GET',
      url : siteUrl.concat("_api/Web/CurrentUser/Groups"),
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         user = json.d.results;
         for (var i = 0; i < user.length; i++) {
            if ((user[i].Title.indexOf(IACUC) > -1)) {
               displayIACUC = IACUC;
            } else if ((user[i].Title.indexOf(IRB) > -1)) {
               displayIRB = IRB;
            } else if ((user[i].Title.indexOf(RDC) > -1)) {
               displayRDC = RDC;
            } else if ((user[i].Title.indexOf(SRS) > -1)) {
               displaySRS = SRS;
            } else if ((user[i].Title.indexOf(SC1) > -1)) {
               displaySC1 = SC1;
            } else if ((user[i].Title.indexOf(SC2) > -1)) {
               displaySC2 = SC2;
            } else if ((user[i].Title.indexOf(SC3) > -1)) {
               displaySC3 = SC3;
            }

         }
      }
   });
   return [ displayRDC, displayIRB, displaySRS, displayIACUC
   ];
}

// /*
// * function buildAdminIds (array, id) { var adminGroupIds = []; adminGroupIds =
// * array; adminGroupIds.push(id); return adminGroupIds ; } function
// * buildSiteGroupIds (array, id) { var siteGroupIds = []; siteGroupIds = array;
// * siteGroupIds.push(id); return siteGroupIds; }
// */
//
// /**
// * @param {number[]} array groupIDs that the user belongs to
// * @param {number} id number to add to array
// * @returns {number[]} groupIds array with id added
// */
// function buildGroupIds(array, id) {
// var groupIds = [];
// groupIds = array;
// groupIds.push(id);
// return groupIds;
// }

/**
 * read the facility list by its Id and return attributes
 * 
 * @param {number} id the id if the facility entry
 * @returns {string} FacilityName the name of the facility
 */
function getFacilityNameById(id) {
   var facilityJson;
   var facilityName;
   var uri = "_api/Web/lists/getbytitle('Facilities')/items" + "?$filter=Id eq '" + id + "'";
   $.ajax({
      async : false,
      type : 'GET',
      url : siteUrl.concat(uri),
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         facilityJson = json.d.results;
         facilityName = facilityJson[0].Title;

      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError(siteUrl.concat(uri), errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
         // This could be caused by 1+ REST Calls timing out.
         alert("Unable to retrieve facility name for id " + id);
         facilityName = null;
      }
   });
   return facilityName;
}

// function getUsersFromSecurityGroups(securityGroupIds) {
// var json = "";
// var users = [];
// var uniqueUsers = [];
//
// for (var i = 0; i < securityGroupIds.length; i++) {
// sendREST("GET", siteUrl.concat("_api/Web/SiteGroups(" + securityGroupIds[i] +
// ")/Users"), null,
// function(userJSON) {
// var currentUsers = userJSON.d.results;
// for (var j = 0; j < currentUsers.length; j++) {
// if (!(users[currentUsers[j].Id])) {
// users[currentUsers.Id] = currentUsers[j];
// uniqueUsers.push(currentUsers[j]);
// }
// }
// });
// }
//
// return JSON.stringify(uniqueUsers);
//
// }
//
// function getUsersFromSecurityGroupsPlural(securityGroupIds) {
// var json = "";
// var users = [];
// var userIDList = [];
//
// for (var i = 0; i < securityGroupIds.length; i++) {
// json = getUsersFromSecurityGroupSingle(securityGroupIds[i]);
// var currentUsers = json.d.results;
// var uniqueUsers = [];
// for (var j = 0; j < currentUsers.length; j++) {
// if (!(users[currentUsers[j].Id])) {
// users[currentUsers.Id] = currentUsers[j];
// uniqueUsers.push(currentUsers[j]);
// }
// }
// }
// return JSON.stringify(uniqueUsers);
// }
// function getUsersFromSecurityGroupSingle(groupId) {
// sendREST("GET", siteUrl.concat("_api/Web/SiteGroups(" + groupId + ")/Users"),
// null,
// function(userJSON) {
// return userJSON;
// });
// }

/**
 * given a list of security groups, read all the names from each group and build a list of unique users
 * 
 * @param {number[]} securityGroupIds list of security groups to pull the users from
 * @returns {json} json collection of unique users in json.d.results
 */
function getUsersFromSecurityUsingAjax(securityGroupIds, committee, listType) {
   var json = {
      "d" : {
         "results" : []
      }
   };

   var users = [];
   var currentUsers = [];
   var uniqueUsers = [];
   var uri;

   for (var i = 0; i < securityGroupIds.length; i++) {
      uri = "_api/Web/SiteGroups(" + securityGroupIds[i] + ")/Users";

      $
         .ajax({
            async : false,
            type : 'GET',
            url : siteUrl.concat(uri),
            headers : {
               "accept" : "application/json;odata=verbose"
            },
            dataType : 'json',
            success : function(jsonusers) {
               currentUsers = jsonusers.d.results;
               for (var j = 0; j < currentUsers.length; j++) {
                  // if the user is already in the list of users, skip
                  // them, otherwise see if they should be added.
                  if (typeof users[currentUsers[j].Id] === "undefined") {
                     // Service Accunt Should Never Show and PI
                     // Should not show
                     // PI and CoPI should not be added to the list
                     // of admins or reviewers.
                     if ((currentUsers[j].Title.indexOf("ServiceAccount") > -1)
                           || ((typeof smcPlusIncumbents.currentPI !== "undefined") && (currentUsers[j].Id == smcPlusIncumbents.currentPI.user.SPUser))
                           || ((typeof smcPlusIncumbents.currentCoPI !== "undefined") && (currentUsers[j].Id == smcPlusIncumbents.currentCoPI.user.SPUser))
                           || ((listType === 'reviewer')
                              && (typeof smcPlusIncumbents[committee].currentAdministrator !== "undefined") && (currentUsers[j].Id == smcPlusIncumbents[committee].currentAdministrator.user.SPUser))
                           || ((listType === 'admin' || listType === 'reviewer')
                              && (typeof smcPlusIncumbents.otherNonReviwerRoles !== "undefined") && (typeof smcPlusIncumbents.otherNonReviwerRoles[currentUsers[j].Id] !== "undefined"))
                           || ((listType === 'admin')
                              && (typeof smcPlusIncumbents[committee].currentPrimaryReviewer !== "undefined") && (currentUsers[j].Id == smcPlusIncumbents[committee].currentPrimaryReviewer.user.SPUser))
                           || ((listType === 'admin')
                              && (typeof smcPlusIncumbents[committee].currentAssignedReviewer !== "undefined") && (typeof smcPlusIncumbents[committee].currentAssignedReviewer[currentUsers[j].Id] !== "undefined"))) {
                           // JSHINT Empty block - The empty blocks are
                        // intentional for debugging purposes.
                        // jshint -W035
                        // skip user
                     } else {
                        users[currentUsers[j].Id] = currentUsers[j];
                        uniqueUsers.push(currentUsers[j]);
                     }
                  }
               }
            },
            error : function(XMLHttpRequest, textStatus, errorThrown) {
               logError(siteUrl.concat(uri), errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
               // This could be caused by 1+ REST Calls timing out.
               alert("Unable to retrieve users from security group.");
            }
         });
   }
   json.d.results = uniqueUsers.sort(function(a, b) {
      return (a.Title > b.Title) ? 1 : ((b.Title > a.Title) ? -1 : 0);
   });
   return json;
}

/**
 * This function populates adminstrators and reviewer dropdowns when user selects committee in admin tool bar.
 * 
 * @param {committeeId} committee dom element id
 * @param {adminDropDownId} administrator dom element id
 * @param {reviewerDropDownId} reviewer dom element id
 * @returns {facilityAdminIDs, reviewerIDs} the list of facilityAdminIDs and reviewerIDs
 */
function populateAdminAndReviewerDropDowns(committeeId, adminDropDownId, reviewerDropDownId) {
   var committeeVal = $("#" + committeeId).find(':selected').val();
   // if dom was not build then use the committee value
   if ((typeof committeeVal === "undefined") || (committeeVal === null)) {
      return;
   }
   var option, text, i;
   var committeeAdminUsers = [];
   var committeeReviewers = [];
   var targetFirstOptVal;
   // NOT USED var facilityIDs = [];

   if (committeeVal != "Select Committee") {

      // get list admin and reviewer security groupd id's for given committee
      getAdminAndReviewersForGivenCommittee(committeeVal, function(committeeAdminIds, committeeReviewerIDs) {

         if (adminDropDownId) {

            // get the group members from all admin security groups
            var adminJson = getUsersFromSecurityUsingAjax(committeeAdminIds, committeeVal, 'admin');
            committeeAdminUsers = adminJson.d.results;

            // Admin dropdown replace
            $("#" + adminDropDownId).removeAttr('disabled');
            targetFirstOptVal = $("#" + adminDropDownId).find('option').first().val();

            $("#" + adminDropDownId).children().remove(); // remove old options

            option = document.createElement("option");
            option.setAttribute("value", targetFirstOptVal);
            text = document.createTextNode(targetFirstOptVal);
            option.appendChild(text);
            $('#' + adminDropDownId).append(option);

            $('#' + adminDropDownId).append(option);
            for (i = 0; i < (committeeAdminUsers.length); i++) { // add new options
               var currentAdminUser = committeeAdminUsers[i];
               option = document.createElement("option");
               option.setAttribute("value", currentAdminUser.Id);
               text = document.createTextNode(currentAdminUser.Title);
               option.appendChild(text);
               $('#' + adminDropDownId).append(option);
            }

            if (smcPlusIncumbents[committeeVal].currentAdministrator) {
               $('#' + adminDropDownId).val(smcPlusIncumbents[committeeVal].currentAdministrator.user.SPUser).change();
            }
         }

         if (reviewerDropDownId) {
            // get the group members from all reviewer security groups
            var reviewerJson = getUsersFromSecurityUsingAjax(committeeReviewerIDs, committeeVal, 'reviewer');
            committeeReviewers = reviewerJson.d.results;

            // Reviewer dropdown replace
            $("#" + reviewerDropDownId).removeAttr('disabled');
            targetFirstOptVal = $("#" + reviewerDropDownId).find('option').first().val();

            $("#" + reviewerDropDownId).children().remove(); // remove old options

            option = document.createElement("option");
            option.setAttribute("value", targetFirstOptVal);
            text = document.createTextNode(targetFirstOptVal);
            option.appendChild(text);
            $('#' + reviewerDropDownId).append(option);

            $('#' + reviewerDropDownId).append(option);
            for (i = 0; i < (committeeReviewers.length); i++) { // add new options
               var currentReviewer = committeeReviewers[i];
               option = document.createElement("option");
               option.setAttribute("value", currentReviewer.Id);
               text = document.createTextNode(currentReviewer.Title);
               option.appendChild(text);
               $('#' + reviewerDropDownId).append(option);
            }

            $("#assignReviewMethod").removeAttr('disabled');
            $("#assignRole").removeAttr('disabled');
         }

      });
   } else {
      $("#" + adminDropDownId).prop("selectedIndex", 0);
      $("#" + adminDropDownId).prop("disabled", "disabled");
      $("#" + reviewerDropDownId).prop("selectedIndex", 0);
      $("#" + reviewerDropDownId).prop("disabled", "disabled");
      $("#assignReviewMethod").prop("disabled", "disabled");
      $("#assignRole").prop("disabled", "disabled");
   }
}

/**
 * Returns the adminId's and reviewerId's for all committees.
 * 
 * @param {function} callback the place to return to when finished
 * @returns {facilityAdminIDs, reviewerIDs} the list of facilityAdminIDs and reviewerIDs
 */

function getCurrentAdminUsers(callback) {

   // NOT USED var currentAdminUsers = [];
   var facilityAdminIDs = [];
   var reviewerIDs = [];
   var userExistinIRBAdminGrp = false;
   var userExistinIACUCAdminGrp = false;
   var userExistinRDCAdminGrp = false;
   var userExistinSRSAdminGrp = false;
   var userExistinSC1Grp = false;
   var userExistinSC2Grp = false;
   var userExistinSC3Grp = false;
   var committeeList = [];
   var uri = "_api/Web/CurrentUser/Groups";

   if ((committeeTable.containsKey(assignedID))) {
      committeeList = committeeTable.get(assignedID);
   } else {
      $.ajax({
         async : false,
         type : 'GET',
         url : siteUrl.concat(uri),
         headers : {
            "accept" : "application/json;odata=verbose"
         },
         dataType : 'json',
         success : function(returnedUserGroups) {
            /*jshint -W020 */
            //ignoring jsHint read-only warning - intentionally overwriting currentUsers
            currentUsers = returnedUserGroups.d.results;
            var currentUserGroups = returnedUserGroups.d.results;
            for (var i = 0; i < currentUserGroups.length; i++) {
               if (currentUserGroups[i].Title.indexOf("RDC Administrator") > -1) {
                  userExistinRDCAdminGrp = true;
               } else if (currentUserGroups[i].Title.indexOf("IRB Administrator") > -1) {
                  userExistinIRBAdminGrp = true;
               } else if (currentUserGroups[i].Title.indexOf("IACUC Administrator") > -1) {
                  userExistinIACUCAdminGrp = true;
               } else if (currentUserGroups[i].Title.indexOf("SRS Administrator") > -1) {
                  userExistinSRSAdminGrp = true;
               } else if (currentUserGroups[i].Title.indexOf("SC1 Administrator") > -1) {
                  userExistinSC1Grp = true;
               } else if (currentUserGroups[i].Title.indexOf("SC2 Administrator") > -1) {
                  userExistinSC2Grp = true;
               } else if (currentUserGroups[i].Title.indexOf("SC3 Administrator") > -1) {
                  userExistinSC3Grp = true;
               }
            }

            if (spAppData.IRBReviewRequired === true && userExistinIRBAdminGrp === true) {
               committeeList.push(IRB);
            }
            if (spAppData.RDCReviewRequired === true && userExistinRDCAdminGrp === true) {
               committeeList.push(RDC);
            }
            if (spAppData.IACUCReviewRequired === true && userExistinIACUCAdminGrp === true) {
               committeeList.push(IACUC);
            }
            if (spAppData.SRSReviewRequired === true && userExistinSRSAdminGrp === true) {
               committeeList.push(SRS);
            }
            if (spAppData.SC1ReviewRequired === true && userExistinSC1Grp === true) {
               committeeList.push(SC1);
            }
            if (spAppData.SC2ReviewRequired === true && userExistinSC1Grp === true) {
               committeeList.push(SC2);
            }
            if (spAppData.SC3ReviewRequired === true && userExistinSC1Grp === true) {
               committeeList.push(SC3);
            }

            // store in committeeTable
            committeeTable.put(assignedID, committeeList);

         },
         error : function(XMLHttpRequest, textStatus, errorThrown) {
            logError(siteUrl.concat(uri), errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
         }
      });
   }

   var facilityName = getFacilityNameById(spAppData.FacilitiesLookupId);
   if ((typeof facilityName === "undefined") || (facilityName === null)) {
      return;
   }

   ficObj.get(facilityName, function(locfiObj) {
      for (var i = 0; i < committeeList.length; i++) {
         var ci = locfiObj.fi.getCommitteeGroup(committeeList[i]);
         // get security group id's
         facilityAdminIDs.push(ci.getAdministrators().id);
         reviewerIDs.push(ci.getAdministrators().id);
         reviewerIDs.push(ci.getCoordinators().id);
         reviewerIDs.push(ci.getMembers().id);
      }
      // var locJson = getUsersFromSecurityUsingAjax(facilityIDs);
      // currentAdminUsers = locJson.d.results;
      if (spAppData.ApplicationSiteType == "Central") {
         ficObj.get(RAMS.Environment.SharePoint.CentralIRBSite, function(centralfiObj) {
            for (var i = 0; i < committeeList.length; i++) {
               facilityAdminIDs.push(centralfiObj.fi.getCommitteeGroup(committeeList[i]).getAdministrators().id);
               reviewerIDs.push(centralfiObj.fi.getCommitteeGroup(committeeList[i]).getAdministrators().id);
               reviewerIDs.push(centralfiObj.fi.getCommitteeGroup(committeeList[i]).getCoordinators().id);
               reviewerIDs.push(centralfiObj.fi.getCommitteeGroup(committeeList[i]).getMembers().id);
            }
            callback(facilityAdminIDs, reviewerIDs);

         });
      } else {
         callback(facilityAdminIDs, reviewerIDs);
      }

   });

}

/**
 * This function returns the committee list.
 * 
 * @param {function} callback the place to return to when finished
 * @returns {committeeList} the committee list.
 */
function getCommitteeList(callback) {

   var userExistinIRBAdminGrp = false;
   var userExistinIACUCAdminGrp = false;
   var userExistinRDCAdminGrp = false;
   var userExistinSRSAdminGrp = false;
   var userExistinSC1Grp = false;
   var userExistinSC2Grp = false;
   var userExistinSC3Grp = false;
   var committeeList = [];
   var currentUsers = [];
   var uri = "_api/Web/CurrentUser/Groups";
   if ((committeeTable.containsKey(assignedID))) {
      committeeList = committeeTable.get(assignedID);
      callback(committeeList);
   } else {
      $.ajax({
         async : false,
         type : 'GET',
         url : siteUrl.concat(uri),
         headers : {
            "accept" : "application/json;odata=verbose"
         },
         dataType : 'json',
         success : function(returnedUserGroups) {
            currentUsers = returnedUserGroups.d.results;
            var currentUserGroups = returnedUserGroups.d.results;
            for (var i = 0; i < currentUserGroups.length; i++) {
               if (currentUserGroups[i].Title.indexOf("RDC Administrator") > -1) {
                  userExistinRDCAdminGrp = true;
               } else if (currentUserGroups[i].Title.indexOf("IRB Administrator") > -1) {
                  userExistinIRBAdminGrp = true;
               } else if (currentUserGroups[i].Title.indexOf("IACUC Administrator") > -1) {
                  userExistinIACUCAdminGrp = true;
               } else if (currentUserGroups[i].Title.indexOf("SRS Administrator") > -1) {
                  userExistinSRSAdminGrp = true;
               } else if (currentUserGroups[i].Title.indexOf("SC1 Administrator") > -1) {
                  userExistinSC1Grp = true;
               } else if (currentUserGroups[i].Title.indexOf("SC2 Administrator") > -1) {
                  userExistinSC2Grp = true;
               } else if (currentUserGroups[i].Title.indexOf("SC3 Administrator") > -1) {
                  userExistinSC3Grp = true;
               }
            }

            if (spAppData.IRBReviewRequired === true && userExistinIRBAdminGrp === true && spAppData.IRBStatusId > 3) {
               committeeList.push(IRB);
            }
            if (spAppData.RDCReviewRequired === true && userExistinRDCAdminGrp === true && spAppData.RDCStatusId > 3) {
               committeeList.push(RDC);
            }
            if (spAppData.IACUCReviewRequired === true && userExistinIACUCAdminGrp === true && spAppData.IACUCStatusId > 3) {
               committeeList.push(IACUC);
            }
            if (spAppData.SRSReviewRequired === true && userExistinSRSAdminGrp === true && spAppData.SRSStatusId > 3) {
               committeeList.push(SRS);
            }
            if (spAppData.SC1ReviewRequired === true && userExistinSC1Grp === true && spAppData.SC1StatusId > 3) {
               committeeList.push(SC1);
            }
            if (spAppData.SC2ReviewRequired === true && userExistinSC1Grp === true && spAppData.SC2StatusId > 3) {
               committeeList.push(SC2);
            }
            if (spAppData.SC3ReviewRequired === true && userExistinSC1Grp === true && spAppData.SC3StatusId > 3) {
               committeeList.push(SC3);
            }

            // store in committeeTable
            committeeTable.put(assignedID, committeeList);
            callback(committeeList);

         },
         error : function(XMLHttpRequest, textStatus, errorThrown) {
            logError(siteUrl.concat(uri), errorThrown, textStatus, JSON.stringify(XMLHttpRequest));
         }

      });

      // callback(committeeList);
   }

}

/**
 * builds the admin reviewer portion of the toolbar
 * 
 * @returns {container} container dom portion that represents the toolbar
 */
function buildCurrentAdminstrators() {
   var container, tableDiv, table, tbody, trow, row
   //var i, crow;

   //var reviewerCollection = [];

   container = document.createElement("div");
   container.setAttribute("class", "col-xs-4 selecter-div");

   row = document.createElement("div");
   row.setAttribute("class", "row selecter-text");

   /* Header Table */
   // table = tableHeaderHelper("Current Administrators", [ "Name", "Committee", "Role", "Remove" ]);
   table = tableHeaderHelper("Current Administrators", [ "Name", "Role", "Remove"
   ]);
   row.appendChild(table);

   /* Scrollable Body Table */
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content");

   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");

   /* table body */
   tbody = document.createElement("tbody");

   // getRAMSUsers(function() {

   var rr = spAppData.ReviewRequired.split(",");
   for ( var com in rr) {
      if ((typeof smcPlusIncumbents[rr[com]].currentAdministrator !== "undefined")
         && (smcPlusIncumbents[rr[com]].currentAdministrator !== null)) {
         var ca = smcPlusIncumbents[rr[com]].currentAdministrator;
         if (rr[com] === IACUC) {
            trow = document.createElement("tr");
            trow.appendChild(textCellHelper("td", ca.user.displayName));
            // trow.appendChild(textCellHelper("td", IACUC));
            trow.appendChild(textCellHelper("td", IACUCSTUDYADMINISTRATOR));
            // trow.appendChild(buttonCellHelper("Remove",
            // "removeAssignedAdministrator(spAppData.IACUCAdministratorId,'IACUCAdministratorId',IACUCSTUDYADMINISTRATOR)"));
            trow.appendChild(textCellHelper("td", "Administrators can only be replaced."));
            tbody.appendChild(trow);
         } else if (rr[com] === IRB) {
            if (spAppData.IRBAdministratorId) {
               trow = document.createElement("tr");
               trow.appendChild(textCellHelper("td", users[spAppData.IRBAdministratorId].displayName));
               // trow.appendChild(textCellHelper("td", IRB));
               trow.appendChild(textCellHelper("td", IRBSTUDYADMINISTRATOR));
               // trow.appendChild(buttonCellHelper("Remove",
               // "removeAssignedAdministrator(spAppData.IRBAdministratorId,'IRBAdministratorId',IRBSTUDYADMINISTRATOR)"));
               trow.appendChild(textCellHelper("td", "Administrators can only be replaced."));
               tbody.appendChild(trow);
            }
         } else if (rr[com] === RDC) {
            if (spAppData.RDCAdministratorId) {
               trow = document.createElement("tr");
               trow.appendChild(textCellHelper("td", users[spAppData.RDCAdministratorId].displayName));
               // trow.appendChild(textCellHelper("td", RDC));
               trow.appendChild(textCellHelper("td", RDCSTUDYADMINISTRATOR));
               // trow.appendChild(buttonCellHelper("Remove",
               // "removeAssignedAdministrator(spAppData.RDCAdministratorId,'RDCAdministratorId',RDCSTUDYADMINISTRATOR)"));
               trow.appendChild(textCellHelper("td", "Administrators can only be replaced."));
               tbody.appendChild(trow);
            }
         } else if (rr[com] === SRS) {
            if (spAppData.SRSAdministratorId) {
               trow = document.createElement("tr");
               trow.appendChild(textCellHelper("td", users[spAppData.SRSAdministratorId].displayName));
               // trow.appendChild(textCellHelper("td", SRS));
               trow.appendChild(textCellHelper("td", SRSSTUDYADMINISTRATOR));
               // trow.appendChild(buttonCellHelper("Remove",
               // "removeAssignedAdministrator(spAppData.SRSAdministratorId,'SRSAdministratorId',SRSSTUDYADMINISTRATOR)"));
               trow.appendChild(textCellHelper("td", "Administrators can only be replaced."));
               tbody.appendChild(trow);
            }
         } else if (rr[com] === SC1) {
            if (spAppData.SC1AdministratorId) {
               trow = document.createElement("tr");
               trow.appendChild(textCellHelper("td", users[spAppData.SC1AdministratorId].displayName));
               // trow.appendChild(textCellHelper("td", SC1));
               trow.appendChild(textCellHelper("td", SC1STUDYADMINISTRATOR));
               // trow.appendChild(buttonCellHelper("Remove",
               // "removeAssignedAdministrator(spAppData.SC1AdministratorId,'SC1AdministratorId',SC1STUDYADMINISTRATOR)"));
               trow.appendChild(textCellHelper("td", "Administrators can only be replaced."));
               tbody.appendChild(trow);
            }
         } else if (rr[com] === SC2) {
            if (spAppData.SC2AdministratorId) {
               trow = document.createElement("tr");
               trow.appendChild(textCellHelper("td", users[spAppData.SC2AdministratorId].displayName));
               // trow.appendChild(textCellHelper("td", SC2));
               trow.appendChild(textCellHelper("td", SC2STUDYADMINISTRATOR));
               // trow.appendChild(buttonCellHelper("Remove",
               // "removeAssignedAdministrator(spAppData.SC2AdministratorId,'SC2AdministratorId',SC2STUDYADMINISTRATOR)"));
               trow.appendChild(textCellHelper("td", "Administrators can only be replaced."));
               tbody.appendChild(trow);
            }
         } else if (rr[com] === SC3) {
            if (spAppData.SC3AdministratorId) {
               trow = document.createElement("tr");
               trow.appendChild(textCellHelper("td", users[spAppData.SC3AdministratorId].displayName));
               // trow.appendChild(textCellHelper("td", SC3));
               trow.appendChild(textCellHelper("td", SC3STUDYADMINISTRATOR));
               // trow.appendChild(buttonCellHelper("Remove",
               // "removeAssignedAdministrator(spAppData.SC3AdministratorId,'SC3AdministratorId',SC3STUDYADMINISTRATOR)"));
               trow.appendChild(textCellHelper("td", "Administrators can only be replaced."));
               tbody.appendChild(trow);
            }
         }
      }
   }

   table.appendChild(tbody);
   tableDiv.appendChild(table);
   row.appendChild(tableDiv);
   container.appendChild(row);
   // });
   return container;
}

/**
 * Returns the adminId's and reviewerId's for given committee.
 * 
 * @returns {facilityAdminIDs, reviewerIDs}
 */
function getAdminAndReviewersForGivenCommittee(committee, callback) {

   var facilityAdminIDs = [];
   var reviewerIDs = [];

   var facilityName = getFacilityNameById(spAppData.FacilitiesLookupId);
   if ((typeof facilityName === "undefined") || (facilityName === null)) {
      return;
   }

   ficObj.get(facilityName, function(locfiObj) {
      var ci = locfiObj.fi.getCommitteeGroup(committee);
      // get security group id's
      facilityAdminIDs.push(ci.getAdministrators().id);
      reviewerIDs.push(ci.getAdministrators().id);
      reviewerIDs.push(ci.getCoordinators().id);
      reviewerIDs.push(ci.getMembers().id);

      if (spAppData.ApplicationSiteType == "Central") {
         ficObj.get(RAMS.Environment.SharePoint.CentralIRBSite, function(centralfiObj) {
            facilityAdminIDs.push(centralfiObj.fi.getCommitteeGroup(committee).getAdministrators().id);
            reviewerIDs.push(centralfiObj.fi.getCommitteeGroup(committee).getAdministrators().id);
            reviewerIDs.push(centralfiObj.fi.getCommitteeGroup(committee).getCoordinators().id);
            reviewerIDs.push(centralfiObj.fi.getCommitteeGroup(committee).getMembers().id);
            callback(facilityAdminIDs, reviewerIDs);

         });
      } else {
         callback(facilityAdminIDs, reviewerIDs);
      }

   });
}

function hideAndShowTriennialExpirationDate(triennialExpirationLabel, triennialExpirationId) {
   /* jshint unused:false */
   var committeeVal = $("#selectCommittee").find(':selected').val();

   if (committeeVal) {
      if (committeeVal === IACUC) {
         $("#triennialExpirationLabel").show();
         $("#triennialExpirationId").show();
      } else {
         $("#triennialExpirationLabel").hide();
         $("#triennialExpirationId").hide();
      }
   } else {
      getCommitteeList(function(committeeList) {
         if (committeeList.length == 1) {
            if (committeeList[0] === IACUC) {
               $("#triennialExpirationLabel").show();
               $("#triennialExpirationId").show();
            } else {
               $("#triennialExpirationLabel").hide();
               $("#triennialExpirationId").hide();
            }
         } else {
            $("#triennialExpirationLabel").hide();
            $("#triennialExpirationId").hide();
         }
      });
   }

}
