/**
 * @fileOverview Responsible for the generation, editing, and saving of the study application. This file depends on the
 *               inclusion of the following files in the html or aspx page where it is used.
 * 
 * <pre>
 * &lt;script type=&quot;text/javascript&quot; src=&quot;jshashtable-3.0.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;jshashset-3.0.js&quot;&gt;&lt;/script&gt;
 * &lt;script type=&quot;text/javascript&quot; src=&quot;rams-environment.js&quot;&gt;&lt;/script&gt;
 * </pre>
 * 
 * In some cases the jshint errors are suppressed with jshint -W### codes, jshint globals, or jshint
 *               exported comments
 * @version 2.3.0
 */
// XXX REINTRODUCE ASAP "use strict";
// JSHINT globals declaration so references to these global variables do not get flagged as warning.
/* globals RAMS, XMLHttpRequest, ActiveXObject, consoleLoggingEnabled, FileReader, sessionStorage, confirm */
/* globals FacilityInfoCollection, SecurityGroup, determineCommittees, getStudyMembers, smcPlusIncumbents, Roles */
/**
 * template to hold overall study with all display info
 * 
 * @type {StudyObject}
 */
var study;

/**
 * template to keep track of which questions are answered in a section
 * 
 * @type {Hashset}
 */
var changedQuestions;

/**
 * template to keep track of all the current users
 * 
 * @type {Hashset}
 */
var ramsUsers = new HashSet();

// template to keep track of members of the study team
// Not Used
// var studyMembers = new HashSet();

/**
 * array of users from the RAMS DB
 * 
 * @type {User[]}
 */
var sqlUsers = [];

/**
 * associative array of users from the RAMS DB, indexed by SPUserId. Value is the user object.
 * 
 * @type {User[]}
 */
var users = [];

/**
 * template to hold all responses from a user from the /study/{id}/responses/{stage}/rollup api call.
 * 
 * @type {rollup[]}
 */
var rollup;

/**
 * location to temporarily hold the HIPAA form and restore it during validation
 * 
 * @type {formStatusCollection[].formsID}
 */
var deletedHIPAA;

/**
 * holds an array of attachments connected to the study application
 * 
 * @type {Attachments}
 */
var attachments;

/**
 * template to keep track of how many stages the study has been through
 * 
 * @type {number}
 */
var targetStage;

/**
 * template to keep track of if the study is in draft or in review
 * 
 * @type {{string|number}}
 */
var targetState;

// /**
// * NOT USED?? template to hold responses from previous application stages
// */
// var previousStageResponses = [];

/**
 * template for the study title
 * 
 * @type {string}
 */
var studyTitle = "";

/**
 * holds the ContextInfo json.d.GetContextWebInformation.FormDigestValue for use during REST calls
 * 
 * @type {Object}
 */
var requestDigest = null;

/**
 * study title's question id
 */
var studyTitleID = "0";

/**
 * template for the study ID. When the value is 1, it is the initial state of creating a study. Once it is saved it will
 * have a value that is 14 numbers followed by a H or A.
 * 
 * @type {string}
 */
var assignedID = "1";

/**
 * template for the parent study's study ID. It will have a value that is 14 numbers followed by a H or A.
 * 
 * @type {string}
 */
var parentID;

/**
 * template for the parent study's PI ID. It will have the SPUserId. It will only be set when adding the child study so
 * the parent PI can be added to the child Study.
 * 
 * @type {string}
 */
var parentPIId;

/**
 * template for the parent study's CoPI ID. It will have the SPUserId. It will only be set when adding the child study
 * so the parent CoPI can be added to the child Study.
 * 
 * @type {string}
 */
var parentCoPIId;

/**
 * template for map between parent/child questions used to require additional answers based on the answer to other
 * questions.
 * 
 * @type {Hashtable}
 */
var dependency = new Hashtable();

/**
 * Set of all answers IDs that are in the rollup
 * 
 * @type {Hashset}
 */
var rollupAnswers = new HashSet();

/**
 * Set of all Questions IDs that are in the rollup
 * 
 * @type {Hashset}
 */
var rollupQuestions = new HashSet();

/**
 * Set of all answers and answer IDs that are in a previous application stage's rollup
 * 
 * @type {Hashtable}
 */
var previousStageAnswers = new Hashtable();

/**
 * Set of all answers and questions IDs that are in a previous application stage's rollup
 * 
 * @type {Hashtable}
 */
var previousStageQuestions = new Hashtable();

/**
 * maps SPUserIds to RAMS User ID
 * 
 * @type {Hashtable}
 */
var userIDMap = new Hashtable();

/**
 * holds a collection of child studies indexed by facility id
 * 
 * @type {Hashtable}
 */
var childrenStudies = new Hashtable();

// /**
// * NOT USED?
// *
// * @type {number[]}
// */
// var previousStage = [];

/**
 * an array of numbers representing the sections that are complete and valid.
 * 
 * @type {number[]}
 */
var validSection = [];

/**
 * holds the study status collection
 * 
 * @type {StudyStatusCollection[]}
 */
var currentStudyStatus;

/**
 * the RAMS web service URL from the rams-environment.js
 * 
 * @type {string}
 */
var restPath = RAMS.Environment.WebService.URL;
/**
 * the SharePoint site collection URL from the rams-environment.js
 * 
 * @type {string}
 */
var siteUrl = RAMS.Environment.SharePoint.URL;

// /**
// * NOT USED? it used to be the URL for the facility subsite.
// *
// * @type {string}
// */
// var subCommittee;

/**
 * the currently logged in user's user object from _api/Web/CurrentUser
 * 
 * @type {string}
 */
var currentUser;

// /**
// * NOT USED? hidden by local variable
// */
// var currentFacility = null;

/**
 * contains a collection of FacilityInfo objects. When created it will have the Central IRB pre-populated.
 * 
 * @type {FacilityInfoCollection}
 */
var ficObj;
/**
 * FacilityInfo Object contains the Facility Information for the retrieved facility.
 * 
 * @type {FacilityInfo}
 */
var fiObj;

/**
 * holds array of facilities from _api/web/lists/getbytitle('Facilities')/items
 * 
 * @type {facility[]}
 */
var facilities = [];

/**
 * holds the Application List data for the study application (from
 * "_api/Web/Lists/GetByTitle('Application')/Items?$filter=Study_ID eq assignedID", for example)
 * 
 * @type {ApplicationListItem}
 */
var spAppData = null;

// TAR Not used anymore - No need to update subsite application list data as it is not used anymore
// var spsubsiteAppData = null;

/**
 * Holds the currentFromVersion number as identified in rams-environment.js. The code assumes a relationship between the
 * currentFormVersion and the childFormVersion in that they are a set distance apart. Local is 1 away from the Parent,
 * Averse is 2 away, etc. Parent: 59, 59: "Parent", Local: 60, 60: "Local", Adverse: 61, 61: "Adverse", ContReview: 62,
 * 62: "ContReview", Closeout: 63, 63: "Closeout", AdministrativeUpdate: 64, 64: "AdministrativeUpdate"
 * 
 * @type {number}
 */
var currentFormVersion;

/**
 * Holds the currentFromType name as identified in rams-environment.js. The code assumes a relationship between the
 * currentFormVersion and the childFormVersion in that they are a set distance apart. Local is 1 away from the Parent,
 * Averse is 2 away, etc. Parent: 59, 59: "Parent", Local: 60, 60: "Local", Adverse: 61, 61: "Adverse", ContReview: 62,
 * 62: "ContReview", Closeout: 63, 63: "Closeout", AdministrativeUpdate: 64, 64: "AdministrativeUpdate"
 * 
 * @type {string}
 */
var currentFormType;

/**
 * identifies if the study is readonly. Initially set to readonly and then turned off if the specific conditions are
 * met.
 * 
 * @type {boolean}
 */
var readOnlyState = true;

/**
 * Save the number of required questions left to answer on the page so that their color can be changed to more easily
 * identify the missing answers.
 * 
 * @type {section[question[]]}
 */
var remainingQuestions = [ []
];

/**
 * object to hold data (like the application list data) being updated during REST POST calls
 * 
 * @type {Object}
 */
var data = {};

/**
 * holds the facility name from the question on the study or from the cookie. May be reset in other methods.
 * 
 * @type {string}
 */
// var facilityName = getFacilityAnswer();
// CONSTANTS - this section holds the constants used in this and other related javascript code. It should be kept in
// sync with the StudyMemberRoles.java in the java code.
/* exported ADMINISTRATOR, ASSIGNEDREVIEWER, PRIMARYREVIEWER */

/**
 * @constant {string} to identify the Administrator. This role may have the type of administrator prepended to the role
 *           (e.g. Study Administrator). In future versions of javascript that support const, this will be changed to a
 *           const.
 */
var ADMINISTRATOR = "Administrator";

/**
 * @constant {string} to identify the Assigned Reviewer. In future versions of javascript that support const, this will
 *           be changed to a const.
 */
var ASSIGNEDREVIEWER = "Assigned Reviewer";

/**
 * @constant {string} to identify the Co-Principal Investigator or CoPI. In future versions of javascript that support
 *           const, this will be changed to a const.
 */
var COPRINCIPALINVESTIGATOR = "Co-Principal Investigator";

/**
 * @constant {string} to identify the Liaison Reviewer. This role will have the Facility Name prepended to the role. In
 *           future versions of javascript that support const, this will be changed to a const.
 */
var LIAISONREVIEWER = "Liaison Reviewer";

/**
 * @constant {string} to identify the Local Site Investigator. This role will have the Facility Name prepended to the
 *           role. In future versions of javascript that support const, this will be changed to a const
 */
var LOCALSITEINVESTIGATOR = "Local Site Investigator";

/**
 * @constant {string} to identify the Parent Study PI. In future versions of javascript that support const, this will be
 *           changed to a const.
 */
var PARENTSTUDYPI = "Parent Study PI";

/**
 * @constant {string} to identify the Parent Study CoPI. In future versions of javascript that support const, this will
 *           be changed to a const.
 */
var PARENTSTUDYCOPI = "Parent Study CoPI";

/**
 * @constant {string} to identify the Primary Reviewer. In future versions of javascript that support const, this will
 *           be changed to a const.
 */
var PRIMARYREVIEWER = "Primary Reviewer";

/**
 * @constant {string} to identify the Principal Investigator or PI. In future versions of javascript that support const,
 *           this will be changed to a const.
 */
var PRINCIPALINVESTIGATOR = "Principal Investigator";

/**
 * @constant {string} to identify the Study Member. This is typically the person that creates the study that picks
 *           someone else as the PI or COPI. This role is needed so that the author retains edit rights to the
 *           application. In future versions of javascript that support const, this will be changed to a const.
 */
var STUDYMEMBER = "Study Member";

/* exported IACUC, IRB, RDC, SRS, SC1, SC2, SC3 */
/**
 * @constant {string} for use with IACUC committee type.
 */
var IACUC = "IACUC";

/**
 * @constant {string} for use with IRB committee type.
 */
var IRB = "IRB";

/**
 * @constant {string} for use with RDC committee type.
 */
var RDC = "RDC";

/**
 * @constant {string} for use with SRS committee type.
 */
var SRS = "SRS";

/**
 * @constant {string} for use with Supplemental Committee 1 (SC1) committee type.
 */
var SC1 = "SC1";

/**
 * @constant {string} for use with Supplemental Committee 2 (SC2) committee type.
 */
var SC2 = "SC2";

/**
 * @constant {string} for use with Supplemental Committee 3 (SC3) committee type.
 */
var SC3 = "SC3";

/* exported COMMITTEETYPE */
var COMMITTEETYPE = [ IRB, IACUC, RDC, SRS, SC1, SC2, SC3
];
/**
 * @constant {string} standard string for IACUC Study Administrator
 */
var IACUCSTUDYADMINISTRATOR = "IACUC Study Administrator";
/**
 * @constant {string} standard string for IRB Study Administrator
 */
var IRBSTUDYADMINISTRATOR = "IRB Study Administrator";
/**
 * @constant {string} standard string for RDC Study Administrator
 */
var RDCSTUDYADMINISTRATOR = "RDC Study Administrator";
/**
 * @constant {string} standard string for SRS Study Administrator
 */
var SRSSTUDYADMINISTRATOR = "SRS Study Administrator";
/**
 * @constant {string} standard string for SC1 Study Administrator
 */
var SC1STUDYADMINISTRATOR = "SC1 Study Administrator";
/**
 * @constant {string} standard string for SC2 Study Administrator
 */
var SC2STUDYADMINISTRATOR = "SC2 Study Administrator";
/**
 * @constant {string} standard string for SC3 Study Administrator
 */
var SC3STUDYADMINISTRATOR = "SC3 Study Administrator";
/* exported IACUCPRIMARYREVIEWER */
/**
 * @constant {string} standard string for IACUC Primary Reviewer
 */
var IACUCPRIMARYREVIEWER = "IACUC Primary Reviewer";
/* exported IRBSPRIMARYREVIEWER */
/**
 * @constant {string} standard string for IRB Primary Reviewer
 */
var IRBSPRIMARYREVIEWER = "IRB Primary Reviewer";
/* exported RDCPRIMARYREVIEWER */
/**
 * @constant {string} standard string for RDC Primary Reviewer
 */
var RDCPRIMARYREVIEWER = "RDC Primary Reviewer";
/* exported SRSPRIMARYREVIEWER */
/**
 * @constant {string} standard string for SRS Primary Reviewer
 */
var SRSPRIMARYREVIEWER = "SRS Primary Reviewer";
/* exported SC1PRIMARYREVIEWER */
/**
 * @constant {string} standard string for SC1 Primary Reviewer
 */
var SC1PRIMARYREVIEWER = "SC1 Primary Reviewer";
/* exported SC2PRIMARYREVIEWER */
/**
 * @constant {string} standard string for SC2 Primary Reviewer
 */
var SC2PRIMARYREVIEWER = "SC2 Primary Reviewer";
/* exported SC3PRIMARYREVIEWER */
/**
 * @constant {string} standard string for SC3 Primary Reviewer
 */
var SC3PRIMARYREVIEWER = "SC3 Primary Reviewer";
/* exported IACUCASSIGNEDREVIEWER */
/**
 * @constant {string} standard string for IACUC Assigned Reviewer
 */
var IACUCASSIGNEDREVIEWER = "IACUC Assigned Reviewer";
/* exported IRBSASSIGNEDREVIEWER */
/**
 * @constant {string} standard string for IRB Assigned Reviewer
 */
var IRBSASSIGNEDREVIEWER = "IRB Assigned Reviewer";
/* exported RDCASSIGNEDREVIEWER */
/**
 * @constant {string} standard string for RDC Assigned Reviewer
 */
var RDCASSIGNEDREVIEWER = "RDC Assigned Reviewer";
/* exported SRSASSIGNEDREVIEWER */
/**
 * @constant {string} standard string for SRS Assigned Reviewer
 */
var SRSASSIGNEDREVIEWER = "SRS Assigned Reviewer";
/* exported SC1ASSIGNEDREVIEWER */
/**
 * @constant {string} standard string for SC1 Assigned Reviewer
 */
var SC1ASSIGNEDREVIEWER = "SC1 Assigned Reviewer";
/* exported SC2ASSIGNEDREVIEWER */
/**
 * @constant {string} standard string for SC2 Assigned Reviewer
 */
var SC2ASSIGNEDREVIEWER = "SC2 Assigned Reviewer";

/* exported SC3ASSIGNEDREVIEWER */
/**
 * @constant {string} standard string for SC3 Assigned Reviewer
 */
var SC3ASSIGNEDREVIEWER = "SC3 Assigned Reviewer";

// CONSTANTS end

/* exported process */
/**
 * Initial function called by HTML. Sets up baseline user context and context information needed then continues
 * processing.
 * 
 * @param {string} behavior the desired action (edit or print)
 */
function process(behavior) {
   // Grabs username and study ID
   getContext(function() {
      sendREST("POST", siteUrl + "_api/ContextInfo", null, function(json) {
         requestDigest = json.d.GetContextWebInformation.FormDigestValue;
         getRAMSUsers(function() {
            if (typeof ficObj === "undefined") {
               ficObj = new FacilityInfoCollection();
            }
            continueProcess(behavior);
         });
      });
   });
}

/**
 * Following the retrieval of the context do the edit or print behavior
 * 
 * @param {string} behavior behavior the desired action (edit or print)
 */
function continueProcess(behavior) {
   if (assignedID === "1") { // Not yet created.
      targetStage = 1;
      targetState = 1;
      readOnlyState = false;
      currentFormType = "Parent";
      currentFormVersion = RAMS.Environment.WebService.Form[currentFormType]; // XXX gross
      sendREST("GET", restPath + "forms/" + currentFormVersion, null, function(json) { // most recent forms
         study = json;
         // This section will intercept the study object and delete HIPAA from the form collection
         for (var x = 0; x < study.containedFormCollection.length; x++) {
            var formTitle = study.containedFormCollection[x].containedForm.title;
            if (formTitle.indexOf("HIPAA") !== -1) {
               deletedHIPAA = study.containedFormCollection[x].containedForm.formsID;
               study.containedFormCollection.splice(x, 1);
            }
         }
         sectionBuilder(0);
      });
      return;
   }

   // Getting the most recent stage and state to use later
   sendREST("GET", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq '" + assignedID
      + "' or ParentStudyID eq '" + assignedID + "'", null, function(json) {
      sendREST("GET", restPath + "study/" + assignedID, null, function(studyResponse) {
         if (targetStage === undefined) {
            targetStage = studyResponse.studyStatusCollection.length;
         }
         targetState = studyResponse.studyStatusCollection[targetStage - 1].studyState.studyStatesID;
         currentFormVersion = studyResponse.formVersionsID;
         currentStudyStatus = studyResponse.studyStatusCollection;
         studyTitle = studyResponse.title;

         sendREST("GET", restPath + "forms/formType/" + currentFormVersion, null, function(formVersion) {
            currentFormType = formVersion.formType;
            // JSHINT getStudyMembers is not defined - it is defined in RAMS_Tools
            // jshint -W117
            getStudyMembers(assignedID, function() {
               // JSHINT {var} is defined but never used.
               // jshint -W098
               // JSHINT smcPlusIncumbents is not defined - it is defined in RAMS_Tools
               // jshint -W117
               var smc = smcPlusIncumbents.smc;
               // moving below study members for leverage
               sendREST("GET", restPath + "forms/" + currentFormVersion, null, function(rootForm) {
                  study = rootForm;
                  grabAppData(json, function() {
                     // alert('Done with grabAppData...checking for behavior');
                     switch (behavior) {
                     case "edit":
                        // alert('grabbing Child Studies');
                        grabChildStudies(json);
                        if (targetState == 1) {
                           readOnlyState = false;
                        } else {
                           // JSHINT selectToolbar is not defined - it is defined in RAMS_Tools
                           selectToolbar(targetState);
                        }
                        if (targetStage > 1) {
                           // alert('Getting previous stage responses');
                           getPreviousStageResponses();
                        }
                        // alert('Invoking Section Builder');
                        sectionBuilder(0);
                        break;
                     case "print":
                        if (targetStage > 1) {
                           // alert('Getting previous stage responses');
                           getPreviousStageResponses();
                        }
                        // The printable view of all sections/responses keeps the readOnlyState set to true.
                        printableSectionsBuilder();
                        break;
                     default:
                        alert("Unknown processing behavior: " + behavior);
                     }
                  });
               });
            });
         });
      });
   });
}

/**
 * This function will grab the children studies of the main study and build the childrenStudies hashtable indexed by the
 * facility.
 * 
 * @param {json} json collection of child studies
 */
function grabChildStudies(json) {
   var i, studies = json.d.results;
   for (i = 0; i < studies.length; i++) {
      // this indicates that this is a the child study
      if ((studies[i].ParentStudyID == assignedID) && (typeof studies[i].Study_ID !== "undefined")
         && (studies[i].Study_ID !== null)) {
         //childrenStudies.put(studies[i][RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId], studies[i]);
         childrenStudies.put(studies[i].Study_ID, studies[i]);
      }
   }
   if (json.d.__next) {
      sendREST("GET", json.d.__next, null, function(returnedJson) {
         grabChildStudies(returnedJson);
      });
   }
}

// 552083: REFACTOR: Update Study Application submit process to populate the new CommiteeTopics /Application CT at the
// Site Collection level
// made check to make point to application list

// Looking only for Study_ID match, not ParentStudyID, here.
/**
 * process the passed in application list data in json collection from a call similar to sendREST("GET", siteUrl +
 * "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq '" + assignedID + "' or ParentStudyID eq '" +
 * assignedID + "'", null, function(json) and put the study Application data into the spAppData global.
 * 
 * @param {json} json holds the incoming data to be processed
 * @param {function} callback location to return when finished processing the studies
 */
function grabAppData(json, callback) {
   // console.log("looking for appdata");
   for (var i = 0; i < json.d.results.length; i++) {
      if (json.d.results[i].Study_ID == assignedID) {
         // var andThen = function(returnedUrl) {
         // subCommittee = returnedUrl;
         // // JSHINT subsiteJson is defined but never used - it is returned by default from sendREST
         // // jshint -W098
         // sendREST("GET", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=Study_ID eq '"
         // + assignedID + "'", null, function(subsiteJson) {
         // callback();
         // });
         // };
         spAppData = json.d.results[i];

         /* ONLY entering this area if we need to search for the subsite. */
         // need to look for the sub-sites url the hard way
         if ((targetStage == 1)
            && ((typeof spAppData.ParentStudyID === "undefined") || (spAppData.ParentStudyID === null))) {
            if ((typeof spAppData.ApplicationStatus !== "undefined") && (spAppData.ApplicationStatus !== null)
               && (spAppData.ApplicationSiteType == "Central")) {
               // console.log("looking for a central substring?");
               // getCentralUrl(andThen);
               callback();
            } else if ((typeof spAppData.ApplicationStatus !== "undefined") && (spAppData.ApplicationStatus !== null)
               && (spAppData.ApplicationStatus != RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFTCHOICE)) {
               // console.log("looking for a regular substring?");
               // getSites(andThen);
               callback();
            } else {
               // we don't need to find the subsite url, but we do
               // need to signify we're done.
               callback();
            }
         } else {
            /* the application has already been submitted. */
            // getCommitteeSubString(spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId],
            // andThen);
            callback();
         }
      }
   }
   // JSHINT Empty block - The empty blocks are intentional for debugging purposes.
   // jshint -W035
   if ((typeof spAppData === "undefined") || (spAppData === null)) {
      if (json.d.__next) {
         sendREST("GET", json.d.__next, null, function(returnedJson) {
            grabAppData(returnedJson, callback);
         });
      } else {
         // console.log("there is no next");
      }
   } else {
      /* SHOULD NEVER GET HERE, but just in case. */
      // console.log("got to the end, spappdata is: " + JSON.stringify(spAppData));
      // callback();
   }
}
// function grabspsubsiteAppData(subsiteJson, callback) {
// // //console.log("looking for subsite app data");
// for (var i = 0; i < subsiteJson.d.results.length; i++) {
// if (subsiteJson.d.results[i].Study_ID == assignedID) {
// spsubsiteAppData = subsiteJson.d.results[i];
// callback();
// break;
// }
// }
// if (spsubsiteAppData == null) {
// if (subsiteJson.d.__next) {
// sendREST("GET", subsiteJson.d.__next, null, function(returnedJson) {
// grabspsubsiteAppData(returnedJson, callback);
// });
// } else {
// alert("Application not found in Committee Topics.");
// }
// }
// }

/**
 * Getting user's responses from the database and updating the local version of the user's responses, called
 * rollupAnswers
 * 
 * @param {number} targetSection a number that represents the section of the study application
 */
function sectionBuilder(targetSection) {
   rollupAnswers = new HashSet();
   rollupQuestions = new HashSet();

   // all user's responses that are in the database
   if (assignedID != "1") {
      // When the assignedID is set to 1, we know that there is no data in the database for a study
      sendREST("GET", restPath.concat("study/" + assignedID + "/responses/" + targetStage + "/rollup"), null, function(
         json) {
         rollup = json;

         for (var j = 0; j < rollup.responses.length; j++) {
            // adding selected radios and checkboxes
            if (typeof rollup.responses[j].answersID !== "undefined") {
               rollupAnswers.add(rollup.responses[j].answersID);
            }
            // adding questions regardless
            rollupQuestions.add(rollup.responses[j].questionsID);
         }

         continueSectionBuilder(targetSection);
      });
   } else {
      // setting an empty rollup
      rollup = {};
      continueSectionBuilder(targetSection);
   }
}

// function determineFormType(currentForms) {
/**
 * lookup the form version in rams-environment.js and set currentFormType to the name of the form
 * 
 * @param {number} formVersion the number that represents the form
 */
function determineFormType(formVersion) {
   /*
    * XXX this is awful and only "works" for the most recent uploaded forms (that we hardcoded into rams-environment.js)
    */
   currentFormType = RAMS.Environment.WebService.Form[formVersion];
}

/**
 * Retrieve the user context and see if the user is in the RAMS database, if they are not in the RAMS User table, add
 * them. If they are in the RAMS User table, see if they need to be updated and update them if necessary.
 * 
 * @param {function} andThen return to the async process that originated the call
 */
function getContext(andThen) {
   var splitURL;

   // finding the user information in the database
   sendREST(
      "GET",
      siteUrl.concat("_api/web/SiteUsers"),
      null,
      function(siteUsersJson) {
         sendREST(
            "GET",
            siteUrl.concat("_api/Web/CurrentUser"),
            null,
            function(currentUserJson) {
               sendREST(
                  "POST",
                  restPath.concat("user"),
                  null,
                  function(sqlUserJson) {
                     splitURL = window.location.href.split("?");

                     // study id that was passed
                     assignedID = splitURL[1];

                     // stage number that was passed
                     targetStage = splitURL[2];

                     ramsUsers = buildUsers(siteUsersJson, null);
                     var tempUsers = ramsUsers.values();
                     // TAR SQLUSERJSON will always be a single user, the current user as read form
                     // the RAMS DB or as added to the RAMS DB
                     sqlUsers = sqlUserJson;

                     currentUser = currentUserJson.d;

                     tempUsers = siteUsersJson.d.results;
                     for (var i = 0; i < tempUsers.length; i++) {
                        // TAR for(var j=0; j<sqlUserJson.length; j++){
                        // if(tempUsers[i].Title == sqlUserJson[j].displayName){
                        // tempUsers.loginName contains i:0#.w|domain\\userid and
// sqlUserJson.loginName contains domain\\userid
// var loginName = tempUsers[i].LoginName;
                        // var startIndex = tempUsers[i].LoginName.indexOf("|") + 1;
                        // var strippedLoginName = loginName.substring(startIndex);
                        if ((sqlUserJson)
                           && (tempUsers[i].LoginName.substring(tempUsers[i].LoginName.indexOf("|") + 1) == sqlUserJson.loginName)) {
                           // console.log("for " + tempUsers[i].Title + ", mapping " +
                           // sqlUserJson.SPUser + " to " + tempUsers[i].Id);
                           userIDMap.put(sqlUserJson.SPUser, tempUsers[i].Id);
                           break;
                        }
                        // }
                     }
                     andThen();
                  });
            });
      });
}

/**
 * get the RAMS users from the RAMS Web Services and build an associative array of users indexed by SPUser so users can
 * be looked up instead of iterated over to find them.
 * 
 * @param {function} callback return to the caller
 */
function getRAMSUsers(callback) {
   sendREST("GET", restPath.concat("user"), null, function(returnedUsers) {
      if (consoleLoggingEnabled) {
         console.log("getUsers: 2. after call REST user. # returned: " + returnedUsers.length);
      }
      for (var i = 0; i < returnedUsers.length; i++) {
         // add each user the the users associative array
         if (typeof returnedUsers[i].SPUser !== "undefined") {
            users[returnedUsers[i].SPUser] = returnedUsers[i];
         }
      }
      if (callback) {
         callback();
      }
   });
}

/**
 * Takes a json collection of users and builds a hashset of user objects
 * 
 * @param {json} json incoming list of users
 * @param {Hashset} aUsers a holding object for recursive calls to build the full list of users.
 * @returns tempUsers {HashSet} the full list of users.
 */
function buildUsers(json, aUsers) {
   var tempUsers = aUsers;
   if (!(tempUsers)) {
      tempUsers = new HashSet();
   }

   for (var i = 0; i < json.d.results.length; i++) {
      tempUsers.add({
         "__metadata" : json.d.results[i].__metadata,
         "Id" : json.d.results[i].Id,
         "IsHiddenInUI" : json.d.results[i].IsHiddenInUI,
         "LoginName" : json.d.results[i].LoginName,
         "Title" : json.d.results[i].Title,
         "PrincipalType" : json.d.results[i].PrincipalType,
         "Email" : json.d.results[i].Email,
         "IsSiteAdmin" : "false",
         "UserID" : json.d.results[i].UserID
      });
   }
   if (json.d.__next) {
      sendREST("GET", json.d.__next, null, function(returnedJson) {
         buildUsers(returnedJson, tempUsers);
      });
   } else {
      return tempUsers;
   }
}

/**
 * creates a cookie that contains the expire time and the facilityAnswer
 * 
 * @param {string} cname variable to set in the cookie (e.g. facilityAnswer)
 * @param {string} cvalue value to set for the cname
 * @param {number} exdays number of days until expire
 */
function setCookie(cname, cvalue, exdays) {
   var d = new Date();
   d.setTime(d.getTime() + (exdays * 24 * 60 * 60 * 1000));
   var expires = "expires=" + d.toUTCString();
   document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/";
}

/**
 * retrieve the cookie and return the value of cname
 * 
 * @param {string} cname variable to retrieve from the cookie (e.g. facilityAnswer)
 * @returns value {string} value read from cookie
 */
function getCookie(cname) {
   var name = cname + "=";
   var decodedCookie = decodeURIComponent(document.cookie);
   var ca = decodedCookie.split(';');
   for (var i = 0; i < ca.length; i++) {
      var c = ca[i];
      while (c.charAt(0) == ' ') {
         c = c.substring(1);
      }
      if (c.indexOf(name) === 0) {
         return c.substring(name.length, c.length);
      }
   }
   return "";
}

/**
 * reads the facility dropdown answer puts the answer into a cookie for use later when the study application may not be
 * available.
 * 
 * @returns facilityAnswer {string} the name of the facility as selected on the study application
 */
function getFacilityAnswer(callback) {
   var question = document.getElementById("facilitydropdown");
   // read studyId from url query string
   // var c = location.queryString[0];
   var cVarName = "facilityAnswer";

   var facilityAnswer = "";
   if ((typeof question !== "undefined") && (question !== null)) {
      facilityAnswer = question.options[question.selectedIndex].value;
      setCookie(cVarName, facilityAnswer, 1);
      callback(facilityAnswer);
   } else if ((typeof spAppData !== "undefined") && (spAppData.FacilitiesLookupId !== null)
      && (typeof spAppData.FacilitiesLookupId !== "undefined") && (spAppData.FacilitiesLookupId !== null)) {
      ficObj.get(spAppData.FacilitiesLookupId, function(fi) {
         fiObj = fi;
         facilityAnswer = fiObj.getFacilityName();
         setCookie(cVarName, facilityAnswer, 1);
         callback(facilityAnswer);
      });
   } else {
      facilityAnswer = getCookie(cVarName);
      if (facilityAnswer !== "") {
         // validate the cookie value
         var regexp = /^[a-zA-Z0-9.-_\s*]+$/;
         if (facilityAnswer.search(regexp) == -1) {
            alert('Warning: invalid Facility name, please update the facility name and try submit again');
            callback("");
         } else {
            callback(facilityAnswer);
         }
      } else {
         callback("");
      }
   }
}

// /**
// * gets the first subsite where the user is a member of the facility Users security group in order to return the
// * subsiteUrl This probably no longer works as originally coded nor is really needed.
// *
// * @param {function} callback the function to return to when done
// */
// function getSites_old(callback) {
// var currentGroups;
// var groupId;
// var currentFacilities = [];
// var subUrl = "";
// // read studyId from url query string
// // var c = location.queryString[0];
// var facilityAnswer;
// getFacilityAnswer(function(facAnsw) {
// facilityAnswer = facAnsw;
//
// currentFacilities = getFacilityItems();
// for (var f = 0; f < currentFacilities.length; f++) {
// // Adding two records to the associative
// facilities[currentFacilities[f].Id] = currentFacilities[f];
// facilities[currentFacilities[f].Title] = currentFacilities[f];
// }
//
// if ((typeof facilities !== "undefined")
// && (typeof spAppData !== "undefined")
// && (spAppData.FacilitiesLookupId)
// && ((facilityAnswer == "none") || (facilityAnswer === "") || ((typeof facilityAnswer === "undefined") ||
// (facilityAnswer === null)))
// && (currentFormType != "Parent")) {
// facilityAnswer = facilities[spAppData.FacilitiesLookupId].Title;
// }
//
// // TODO TAR update the query to read only groups that have Users in the name to reduce the number of items that
// // need
// // to be iterated over.
// // TODO TAR need to account for the fact that the user may belong to multiple facilities
// sendREST("GET", siteUrl.concat("_api/Web/CurrentUser/Groups" + "?$filter=substringof(' Users', Title)"), null,
// function(groupJson) {
// currentGroups = groupJson.d.results;
// var siteFound = false;
// for (var i = 0; i < currentGroups.length; i++) {
// var done = false;
// // console.log("A group is: " + currentGroups[i].Title);
// if (currentGroups[i].Title.indexOf(facilityAnswer + " Users") > -1) {
// siteFound = true;
// // groupId is the Id of the FacilityUsers group that the user belongs to, assuming that the user only
// // belongs to one
// groupId = currentGroups[i].Id;
// // Loop throught the currentFacilities until the groupId is found and return its attributes in the
// // callback
// for (var j = 0; j < currentFacilities.length; j++) {
// if (groupId == currentFacilities[j].UsersId) {
// subUrl = currentFacilities[j].FacilitySubsite.Url;
// subUrl = subUrl.replace(siteUrl, "");
// subUrl = subUrl + "/";
// callback(subUrl, currentFacilities[j].Id, currentFacilities[j].FacilityLiaisonId);
// done = true;
// if (done) {
// break;
// }
// }
// }
// }
//
// if (done) {
// break;
// }
// }
// if (siteFound === false) {
// alert('Submit Error: Please contact your Local Site Liaison to align you to an active IRB Site.');
// }
// });
// });
//
// }

/**
 * gets the first subsite where the user is a member of the facility Users security group in order to return the
 * subsiteUrl This probably no longer works as originally coded nor is really needed.
 * 
 * @param {function} callback the function to return to when done
 */
function getSites(callback) {
   var currentGroups;
   var sg;
   var siteFound = false;
   // var currentFacilities = [];
   var subUrl = "";
   var facilityAnswer;
   getFacilityAnswer(function(facAnsw) {
      facilityAnswer = facAnsw;
      ficObj.get(facilityAnswer, function(fiObj) {

         // TODO TAR update the query to read only groups that have Users in the name to reduce the number of items
         // that need to be iterated over.
         // TODO TAR need to account for the fact that the user may belong to multiple facilities
         sendREST("GET", siteUrl.concat("_api/Web/CurrentUser/Groups" + "?$filter=substringof(' Users', Title)"), null,
            function(groupJson) {
               currentGroups = groupJson.d.results;
               // This loop is going over all the FacilityUsers groups the user belongs to to see if the study is
               // submitted to one of these facilities. It the user is a member of the facility, return
               // to the caller the necessaary information, otherwise issue the error message.
               for (var i = 0; i < currentGroups.length; i++) {
                  sg = new SecurityGroup(currentGroups[i].Id, currentGroups[i].Title);
                  if (sg.getFacilityName() == fiObj.getFacilityName()) {
                     siteFound = true;
                     subUrl = fiObj.getFacilitySubsiteUrl();
                     subUrl = subUrl.replace(siteUrl, "");
                     subUrl = subUrl + "/";
                     callback(subUrl, fiObj.getFacilityId(), fiObj.getFacilityGroup("liaison").id);
                     break;
                  }
               }

               if (siteFound === false) {
                  alert("Submit Error: Please contact the " + facilityAnswer
                     + " Local Site Administrators to add you to the " + facilityAnswer + " users group.");
               }
            });
      });
   });
}

// /**
// * retrieve the list of facilities and return the Central IRB site as identified in rams-environment.js
// *
// * @param {function} callback return to caller with the url, facility id, and facility liaison id
// */
// function getCentralUrl(callback) {
// var subUrl = "";
// // console.log("looking for central IRB URL");
// var cirbFI = ficObj.get(RAMS.Environment.SharePoint.CentralIRBSite);
// if (typeof cirbFI !== "undefined") {
// subUrl = cirbFI.getFacilitySubsiteUrl();
// subUrl = subUrl.replace(siteUrl, "");
// subUrl = subUrl + "/";
// callback(subUrl, cirbFI.getFacilityId(), cirbFI.getFacilityGroup("liaison").id);
// } else {
// alert("Submit Error: Unable to retrieve Central IRB site information. "
// + "Please contact your Local Site Administrators.");
// }
//
// // var siteFound = false;
// // var currentFacilities = getFacilityItems();
// // for (var j = 0; j < currentFacilities.length; j++) {
// //
// // if (currentFacilities[j].Title == RAMS.Environment.SharePoint.CentralIRBSite) {
// // siteFound = true;
// // subUrl = currentFacilities[j].FacilitySubsite.Url;
// // subUrl = subUrl.replace(siteUrl, "");
// // subUrl = subUrl + "/";
// // callback(subUrl, currentFacilities[j].Id, currentFacilities[j].FacilityLiaisonId);
// // }
// //
// // }
// // if (siteFound === false) {
// // alert('Submit Error: Please contact your Local Site Liaison to align you to an active IRB Site.');
// // }
// }

/**
 * retrieve the list of facilities ordered by the facility name
 * 
 * @returns facilityJson {json} collection of facilities
 */
function getFacilityItems() {
   var facilityJson;
   $.ajax({
      async : false,
      type : 'GET',
      url : siteUrl.concat("_api/Web/lists/getbytitle('Facilities')/items?" + "$orderby=Title"),
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         facilityJson = json.d.results;
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError(siteUrl.concat("_api/Web/lists/getbytitle('Facilities')/items?" + "$orderby=Title"), errorThrown,
            textStatus, JSON.stringify(XMLHttpRequest));
      }
   });
   // JSHINT NOTE: facilityJson may be uninitialized, but it is used in For loops that go through the collection and
   // the
   // loop
   // should already be protected for undefined variables (the length should be 0).
   return facilityJson;
}

// FacilityLookupId
// /**
// * read the facility list and committee list to retrieve the subsite URL.
// *
// * @param {number} facilityLookup the id of the facility
// * @param {function} callback return to the caller with the subsiteUrl
// */
// function getCommitteeSubString(facilityLookup, callback) {
// var subUrl;
// // // var sliceAt;
// // // grab all facilities, find yours, find the committee lookup
// // // TODO TAR the facility list also has the subsite URL why not just use it instead of also reading the committee
// // // list?
// // sendREST("GET", siteUrl.concat("_api/web/lists/getbytitle('Facilities')/items(" + facilityLookup + ")"), null,
// // function(returnedFacility) {
// // // console.log("our facility is " + returnedFacility.d.Title + " with committee lookup " +
// // // returnedFacility.d.CommitteeLookupId);
// // // sendREST("GET", siteUrl.concat("_api/web/lists/getbytitle('Committees')/items("
// // // + returnedFacility.d.CommitteeLookupId + ")"), null, function(returnedCommittee) {
// // // console.log("our committee is: " + returnedCommittee.d.Title + " with substring " +
// // // returnedCommittee.d.CommitteeSub_x002d_Site.Url);
// // // subUrl = (returnedCommittee.d.CommitteeSub_x002d_Site.Url).replace(siteUrl, "");
// // subUrl = returnedFacility.d.FacilitySubsite.Url;
// // subUrl = subUrl + "/";
// // // sliceAt = subUrl.indexOf("/") + 1;
// // // subUrl = subUrl.slice(0, sliceAt);
// // // console.log("the subsite is: " + subUrl);
// // callback(subUrl);
// // // });
// // });
//
// fiObj = ficObj.get(facilityLookup);
// subUrl = fiObj.getFacilitySubsite() + "/";
// callback(subUrl);
// }

/**
 * call RAMS Web Services to return the study responses and build the previousStageAnswers and previousStageQuestions.
 */
function getPreviousStageResponses() {
   for (var i = 1; i < targetStage; i++) {
      sendREST("GET", restPath.concat("study/" + assignedID + "/responses/" + i), null,
         function(stageResponseValidator) {
            for (var j = 0; j < stageResponseValidator.length; j++) {
               if (typeof stageResponseValidator[j].answersID !== "undefined") {
                  previousStageAnswers.put(stageResponseValidator[j].answersID, stageResponseValidator[j]);
               }
               previousStageQuestions.put(stageResponseValidator[j].questionsID, stageResponseValidator[j]);
            }
         });
   }
}

/**
 * sets up the structure of the question/sidebar section and attaches to the proper area in webpage.
 * 
 * @param {number} targetSection number that represents the section of the study application
 */
function continueSectionBuilder(targetSection) {
   window.scrollTo(0, 0); // before reloading page with new section, scroll to the top of the page
   // rollupAnswers = new HashSet();

   // removing any trace of the last section that was built
   $('#refresh-section').remove();
   var row, column, sideColumn;

   // setting up the initial structure of page
   row = document.createElement("div");
   row.setAttribute("class", "row");
   row.setAttribute("id", "refresh-section");
   $('div#insert').append(row);

   column = document.createElement("div");
   column.setAttribute("class", "col-xs-9");
   row.appendChild(column);

   columnBuilder(targetSection);

   sideColumn = document.createElement("div");
   sideColumn.setAttribute("class", "col-xs-3 toolbar");
   row.appendChild(sideColumn);

   sideColumnBuilder(targetSection);

   // TODO TAR instead of hard coding the 54, the value should be looked up from rams-environment.js
   // if (currentFormVersion == 54) { // Local Site Study
   if (currentFormType == "Local") {
      // Grab the title of the principal
      var lookupSPUserId;
      if (spAppData.ParticipatingSiteInvestigatorId === null) {
         if (spAppData.PrincipalInvestigatorId === null) {
            alert("Invalid user id");
            throw "Invalid user id when trying to create child application";
         } else {
            lookupSPUserId = spAppData.PrincipalInvestigatorId;
         }
      } else {
         lookupSPUserId = spAppData.ParticipatingSiteInvestigatorId;
      }
      sendREST("GET", siteUrl + "_api/Web/GetUserById(" + lookupSPUserId + ")/", null, function(userPrincipal) {
         // TODO LOOK - do we also need the userPrincipal.Id? or instead of the title, use the Id?
         var title = userPrincipal.d.Title;
         var id = userPrincipal.d.Id;
         var pid = study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID;
         // JSHINT consoleLoggingEnabled is not defined - it is defined in rams-environment
         // jshint -W117
         if (consoleLoggingEnabled) {
            console.log("continueSectionBuilder: User Id: " + id + "   pid: " + pid + "   Title: " + title);
         }
         // console.log("User Title:" + title + " pid:" + pid);
         // Set the PI to the LocalSite Investigator
         $("#QID_" + pid).val(id).change();
         // Disable text area
         $("#QID_" + pid).attr('disabled', true);
         changedQuestions.add(pid);
      });
   }
}

/**
 * retrieve the responses rollup and call the printColumnBuilder for each section
 */
function printableSectionsBuilder() {
   sendREST("GET", restPath.concat("study/" + assignedID + "/responses/" + targetStage + "/rollup"), null, function(
      responseRollup) {
      rollup = responseRollup;
      var targetSection, validityBySection = rollup.formValidationReport.formStatusCollection;
      rollupAnswers = new HashSet();
      rollupQuestions = new HashSet();
      for (var response = 0; response < rollup.responses.length; response++) {
         // adding selected radios and checkboxes
         if (typeof rollup.responses[response].answersID !== "undefined") {
            rollupAnswers.add(rollup.responses[response].answersID);
         }
         // adding questions regardless
         rollupQuestions.add(rollup.responses[response].questionsID);
      }

      for (targetSection = 0; targetSection < validityBySection.length; targetSection++) {
         switch (validityBySection[targetSection].status) {
         case "EMPTY":
         case "NOT_APPLICABLE":
            break;
         default:
            var section = $(
               $.parseHTML("<div class=container id=section-" + targetSection + ">" + "<div class=row>"
                  + "<div class=col-xs-9/>" + "</div>" + "</div>")[0]).appendTo("body");
            printColumnBuilder(targetSection, section);
         }
      }
   });
}

/**
 * sets up the structure of the question area; includes header, questions, and button dock (progress buttons)
 * 
 * @param {number} targetSection number representing the section of the study application
 */
function columnBuilder(targetSection) {
   var section, pageHeader, formContent, progressContainer, i;

   section = study.containedFormCollection[targetSection];

   // page header
   pageHeader = document.createElement("div");
   pageHeader.setAttribute("class", "page-header");
   pageHeader.setAttribute("tabindex", "0");
   pageHeader = $(pageHeader).appendTo("div.col-xs-9");

   headerBuilder(targetSection, pageHeader);

   // questions
   formContent = document.createElement("div");
   formContent.setAttribute("class", "formContent");
   formContent = $(formContent).appendTo("div.col-xs-9");

   if (currentFormType == "Parent") {
      if ((targetSection == 1) || (targetSection == 2) || (targetSection == 5) || (targetSection == 6)
         || (targetSection == 7) || (targetSection == 11)) {
         disclaimerBuilder(targetSection);
      }
   }

   questionBuilder(section, formContent);

   // Save the number of required questions left to answer on the page
   if (remainingQuestions[0].length === 0) {
      for (i = 0; i < study.containedFormCollection.length; i++) {
         remainingQuestions[0].push(0);
      }
   }

   if ((targetSection !== 0) && (remainingQuestions[0][targetSection] !== 1)) {
      var temp = [];
      var allFormQuestions = study.containedFormCollection[targetSection].containedForm.formQuestionCollection;
      for (i = 0; i < allFormQuestions.length; i++) {
         temp.push(allFormQuestions[i].question.questionsID);
      }
      remainingQuestions[targetSection] = temp;
      remainingQuestions[0][targetSection] = 1;
   }

   // progress
   progressContainer = document.createElement("div");
   progressContainer.setAttribute("class", "progress-container");
   progressContainer.setAttribute("aria-live", "assertive");
   $('div.col-xs-9').append(progressContainer);

   // buttons
   $('div.col-xs-9').append("<div class=button-container/>");

   progressBuilder();
}

/**
 * Sets up the structure of the sidebar; includes section buttons, print and upload buttons, and review and submit
 * buttons.
 * 
 * @param {number} targetSection number representing the section of the study application
 */
function sideColumnBuilder(targetSection) {
   var actionBar, wsBar;

   // action bar
   actionBar = document.createElement("div");
   actionBar.setAttribute("class", "actionbar");
   $('.toolbar').append(actionBar);

   actionBarBuilder(targetSection);

   // well sidebar
   wsBar = document.createElement("div");
   wsBar.setAttribute("class", "well sidebar");
   $('.toolbar').append(wsBar);

   if (assignedID != "1") {
      sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID
         + "/Study%20Team')/Files"), null, function(returnedFiles) {
         // attachments = filterChecklists(returnedFiles.d.results, "-RAMS-");
         attachments = returnedFiles.d.results;
         wellSidebarBuilder(targetSection, attachments.length);
      });
   } else {
      wellSidebarBuilder(targetSection, "0");
   }
}

/**
 * creating the disclaimer that is seen at the top of certain page
 * 
 * @param {number} targetSection number representing the section of the study application
 */
function disclaimerBuilder(targetSection) {
   var cont, close, hide, closer, paragraph, text;
   cont = document.createElement("div");
   cont.setAttribute("class", "well notificationArea alert-dismissible fade in");
   cont.setAttribute("role", "alertdialog");
   cont.setAttribute("aria-describedby", "disclaimerText");

   close = document.createElement("button");
   close.setAttribute("data-dismiss", "alert");
   close.setAttribute("class", "close");
   close.setAttribute("type", "button");

   hide = document.createElement("span");
   hide.setAttribute("aria-hidden", "true");
   text = document.createTextNode("x");
   hide.appendChild(text);

   closer = document.createElement("span");
   closer.setAttribute("class", "sr-only");
   text = document.createTextNode("Close");
   closer.appendChild(text);

   close.appendChild(hide);
   close.appendChild(closer);

   paragraph = document.createElement("p");
   paragraph.setAttribute("id", "disclaimerText");

   text = "";
   if (targetSection == 1) {
      text = "Disclaimer: Answers to some questions in this section will require your answer to additional questions "
         + "in subsequent sections of the Research Study Application.";
   } else if (targetSection == 2) {
      text = "Disclaimer: Answers in this section may be hidden based on your answers in Section A";
   } else if (targetSection == 5) {
      text = "Disclaimer: Based on your answers in Section B, you are required to answer these additional questions.";
   } else if (targetSection == 6) {
      text = "Disclaimer: Based on your answers in Section B, you are required to answer these additional questions.";
   } else if (targetSection == 7) {
      text = "Disclaimer: Based on your answers in Section B, you are required to answer these additional questions.";
   } else if (targetSection == 11) {
      text = "Disclaimer: Based on your answers in Section B, you are required to answer these additional questions.";
   }
   // text may be uninitialized or undefined.
   if (text) {
      paragraph.textContent = text;
   }

   cont.appendChild(close);
   cont.appendChild(paragraph);
   $('.formContent').append(cont);
}

/**
 * builds the header that displays information about a study and section.
 * 
 * @param {number} targetSection number representing the section of the study application
 * @param {object} headerArea jquery control where the header is located
 */
function headerBuilder(targetSection, headerArea) {
   var appTitle, id, section, sectionContainer, genInfo, text;

   // Study title
   appTitle = document.createElement("h2");
   appTitle.setAttribute("class", "applicationTitle");

   // Sets up the default study title in the header
   if (studyTitle === "") {
      text = document.createTextNode("New Research Study Application");
   } else {
      text = document.createTextNode(studyTitle);
   }
   appTitle.appendChild(text);

   // Study ID
   id = document.createElement("small");

   // Sets up the default study ID in the header
   if (assignedID == "1") {
      text = document.createTextNode(" ID: TBD");
   } else {
      text = document.createTextNode(" ID: " + assignedID);
   }
   id.appendChild(text);
   appTitle.appendChild(id);

   headerArea.append(appTitle);

   // Current Section Display
   section = document.createElement("h3");
   section.setAttribute("class", "sectionNumber");

   if ((targetSection != "Sites") && (targetSection != "Personnel") && (targetSection != "Attachments")) {
      var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
      text = document.createTextNode("Section " + sectionTitle.charAt(targetSection));
   } else {
      text = document.createTextNode(targetSection);
   }
   section.appendChild(text);

   headerArea.append(section);

   // General Information Display
   sectionContainer = document.createElement("div");
   sectionContainer.setAttribute("class", "sectionContainer");

   genInfo = document.createElement("h5");
   genInfo.setAttribute("class", "sectionTitle");

   if ((targetSection != "Sites") && (targetSection != "Personnel") && (targetSection != "Attachments")) {
      text = document.createTextNode(study.containedFormCollection[targetSection].containedForm.title);
      genInfo.appendChild(text);
   }

   sectionContainer.appendChild(genInfo);

   if (readOnlyState) {
      $(sectionContainer).append(
         "<h5 class=sectionLocation>Application stage " + targetStage + ": "
            + currentStudyStatus[targetStage - 1].studyState.name + "</h5>");
   } else {
      $(sectionContainer).append("<h5 class=sectionLocation>Completed: 0/11</h5>");
   }

   headerArea.append(sectionContainer);
}

/**
 * sets up the structure of the question areas and headers, questions and responses, but no buttons
 * 
 * @param {number} targetSection number representing the section of the study application
 * @param {object} sectionArea jquery control where the section is located
 */
function printColumnBuilder(targetSection, sectionArea) {
   var pageHeader;
   var section = study.containedFormCollection[targetSection];
   var colXs9 = sectionArea.find("div.col-xs-9");

   // page header
   pageHeader = $($.parseHTML("<div class=page-header tabindex=0/>")[0]).appendTo(colXs9);
   headerBuilder(targetSection, pageHeader);

   // questions
   var formContent = $($.parseHTML("<div class=formContent/>")[0]).appendTo(colXs9);
   questionBuilder(section, formContent);
}

/**
 * Creates a general warning to display. Takes in a message and a warning type, which will move the user to the progress
 * bar when called.
 * 
 * @param {string} message the message to display to the user.
 * @param {string} type type of message where alert-warning will show as "Warning!" and alert-success will show as
 *           "Congratulations!"
 */
function insertWarning(message, type) {
   var warnCont, bolded, warnClose, warnSpan, text, breaker;

   // Removing any traces of previous warnings.
   $('.alert').remove();

   warnCont = document.createElement("div");
   warnCont.setAttribute("class", "alert " + type);
   warnCont.setAttribute("role", "alert");
   warnCont.setAttribute("aria-role", "assertive");

   // close warning
   warnClose = document.createElement("button");
   warnClose.setAttribute("type", "button");
   warnClose.setAttribute("class", "close");
   warnClose.setAttribute("data-dismiss", "alert");
   warnClose.setAttribute("aria-label", "Close");

   warnSpan = document.createElement("span");
   warnSpan.setAttribute("aria-hidden", "true");

   text = document.createTextNode("x");
   warnSpan.appendChild(text);
   warnClose.appendChild(warnSpan);
   warnCont.appendChild(warnClose);

   bolded = document.createElement("strong");

   // Creating different content based on what is requested This isn't warnCont.innerHTML
   // because it wipes out everything that's in the div instead of appending to it.
   if (type == "alert-warning") {
      text = document.createTextNode("Warning!");
   } else if (type == "alert-success") {
      text = document.createTextNode("Congratulations!");
   }

   // bolding the warning header
   bolded.appendChild(text);
   warnCont.appendChild(bolded);

   // break
   breaker = document.createElement("br");
   warnCont.appendChild(breaker);

   // passed warning message
   text = document.createTextNode(message);
   warnCont.appendChild(text);

   $('.progress-container').prepend(warnCont);
   $('.progress-container').show();

   // JQUERY moving page to the generated warning
   if (type == "alert-warning") {
      $('html, body').animate({
         scrollTop : $(document).height()
      }, 1000);
   }

   $(warnCont).parent().delay("3000").fadeOut("3000");
}

// "GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID
// + "/Study%20Team')/Files"), null, function(returnedFiles)

/**
 * asynchronously connects to a rest endpoint and either sends or receives data. Will catch exceptions from the
 * operation and generically display them to the user and log them via the logerror function
 * 
 * @param {string} httpMethod valid values are GET, PUT, DELETE, POST
 * @param {string} endpoint the web service uri
 * @param {object} dataPack the data to update when POSTing
 * @param {function} callback the place to return to when finished
 * @param {string} spHttpMethod "Merge" to update existing records or null to insert
 * @return xmlResponse {json} the data retrieved from the operation. May be null or false in some situations.
 */
function sendREST(httpMethod, endpoint, dataPack, callback, spHttpMethod) {
   var xmlResponse;

   // accounting for internet explorer's needs
   if (window.XMLHttpRequest) {
      xmlResponse = new XMLHttpRequest();
   } else {
      xmlResponse = new ActiveXObject("Microsoft.XMLHTTP");
   }

   try {
      // setting up general headers
      xmlResponse.open(httpMethod, endpoint);
      xmlResponse.withCredentials = true;
      // xmlResponse.setRequestHeader("Content-Type", "application/json");
      xmlResponse.setRequestHeader("Accept", "application/json; odata=verbose");
      xmlResponse.setRequestHeader("X-RequestDigest", requestDigest);

      if ((httpMethod == "PUT") || (httpMethod == "DELETE")) {
         xmlResponse.setRequestHeader("sm_user", dataPack);
         xmlResponse.send(null);

         // 204 is the response status indicating success, while
         // 403 indicates failure
         if (xmlResponse.status === 204) {
            return true;
         } else if (xmlResponse.status === 403) {
            logError(endpoint, 403, "PUT or DELETE Failure", xmlResponse.responseText);
            return false;
         }
      } else if (httpMethod == "POST") {
         try {
            xmlResponse.setRequestHeader("Content-Type", "application/json; odata=verbose");

            if (spHttpMethod == "Merge") {
               xmlResponse.setRequestHeader("X-HTTP-Method", "MERGE");
               xmlResponse.setRequestHeader("If-Match", "*");
            } else if (spHttpMethod == "Delete") {
               xmlResponse.setRequestHeader("X-HTTP-Method", "DELETE");
               xmlResponse.setRequestHeader("If-Match", "*");
            }

            // sending dataPack and alerting the user
            if (dataPack) {
               // logMessageToULS("POST with Data - " + endpoint, "dynamicGeneration.js"); //show what data is
               // being sent
               xmlResponse.send(JSON.stringify(dataPack));

            } else {
               xmlResponse.send(null);
            }

            if (callback) {
               xmlResponse.onreadystatechange = function() {
                  if (xmlResponse.readyState === 4) {

                     if (readOnlyState === false) {
                        insertWarning('You have successfully saved your responses', 'alert-success');
                        $("#saveAppProgress").prop("disabled", true);
                        setTimeout(function() {
                           $('html, body').animate({
                              scrollTop : $("#insert").offset().top
                           }, 2000);
                        }, 4000); // scroll back to the top after displaying success message
                     }

                     if ((xmlResponse.status === 200) && (xmlResponse.responseText)) {
                        // response has content
                        xmlResponse = JSON.parse(xmlResponse.responseText);
                        callback(xmlResponse);
                     } else {
                        // logMessageToULS(httpMethod + "-" + endpoint + "-" + xmlResponse.status,
                        // "dynamicGeneration.js");
                        if (xmlResponse.status !== 204) {
                           logError(endpoint, xmlResponse.status, "POST error statusText=" + xmlResponse.statusText
                           + " result=" + xmlResponse.result + " responseText=" + xmlResponse.responseText, httpMethod);
                        }
                        callback();
                     }
                  }
               };
            }
         } catch (e) {
            logError(endpoint, xmlResponse.status, "POST error statusText=" + xmlResponse.statusText + " result="
               + xmlResponse.result, httpMethod);
            // alert(e.toString());
            // logMessageToULS(httpMethod + "-" + endpoint + "-" + e.toString(), "dynamicGeneration.js");
         }
      } else if (httpMethod == "GET") {
         // retrieving the needed data
         xmlResponse.onreadystatechange = function() {
            if (xmlResponse.readyState === 4) {
               if (xmlResponse.status === 200) {
                  try {
                     xmlResponse = JSON.parse(xmlResponse.responseText);
                     if (callback) {
                        callback(xmlResponse);
                     }
                  } catch (e) {
                     logError(endpoint, xmlResponse.status, "GET error", xmlResponse.responseText);
                     alert(e.toString());
                  }
               } else {
                  logError(endpoint, xmlResponse.status, "GET error", xmlResponse.responseText);
                  if ((xmlResponse.status !== 204) && (xmlResponse.status !== 1223)) {
                     var msg = xmlResponse.status + " " + xmlResponse.statusText + " from " + endpoint;
                     if (xmlResponse.status === 401) {
                        msg = "You are not authorized for the requested information. \n\n" + msg;
                        alert(msg);
                        window.history.back();
                     } else {
                        // alert(xmlResponse.status);
                        alert(msg);
                     }
                  }
               }
            }
         };
         xmlResponse.send(null);
         // passing the needed data to the function that is requesting it
         return xmlResponse;
      }

   } catch (e) {
      logError(endpoint, e, "Failure on sendREST function call", "n/a");
      // alert(e.toString());
      alert(e.toString() + " from " + endpoint);
   }
}

/**
 * Takes the response from the database and builds the questions displayed in the application.
 * 
 * @param {object} section collection of questions
 * @param {object} divFormContent jquery area of the dom to put the questions
 */
function questionBuilder(section, divFormContent) {
   // setting up a fresh set to keep track of questions that have been
   // changed while the user is in a particular section.
   changedQuestions = new HashSet();
   var panelGroup, users;
   var i, j, x; // iterators
   var form, formGroup, textLabel, text, input, toolSpan, field, reqSpan, breaker, italic, inputOption;
   var responseFound = false;

   // filtering through each of the questions in a section to build them individually
   for (i = 0; i < section.containedForm.formQuestionCollection.length; i++) {
      // variables that are used in commented out code only
      // var legend;
      var formQuestion = section.containedForm.formQuestionCollection[i];
      var currentValues = new Hashtable();
      var answeredValues = [];
      // var currentAnswers = new HashSet();
      // var currentAnswers = [];
      var checkedAnswers = 0;

      form = document.createElement("form");

      // div containing complete questions
      formGroup = document.createElement("div");
      formGroup.setAttribute("class", "form-group");
      formGroup.setAttribute("aria-live", "assertive");

      // holds the required astrik, question text, and tooltip
      textLabel = document.createElement("label");
      textLabel.setAttribute("tabindex", "0");

      // required asterisk
      // XXX We hide this from a screen reader. Should the question not be in tabindex=0 if it is disabled?
      if (formQuestion.requiredPredecessorAnswersID !== null) {
         form.setAttribute("rams-required", "true");
         reqSpan = document.createElement("span");
         reqSpan.setAttribute("class", "required");
         reqSpan.setAttribute("aria-hidden", "true");
         reqSpan.textContent = "*";
         textLabel.appendChild(reqSpan);

         reqSpan = document.createElement("span");
         reqSpan.setAttribute("class", "required sr-only");
         reqSpan.textContent = "A response is required.";
         textLabel.appendChild(reqSpan);
      } else {
         form.setAttribute("rams-required", "false");
         reqSpan = document.createElement("span");
         reqSpan.setAttribute("class", "sr-only");
         reqSpan.textContent = "A response is not required.";
         textLabel.appendChild(reqSpan);
      }

      // question text. We see the introdocution of QID_ (Question IDs) and later
      // we will see AID_ (Answer IDs). These have been put in to comply 508 standards that
      // input id attributes must start with a letter.
      textLabel.setAttribute("for", "QID_" + formQuestion.question.questionsID);
      if (formQuestion.question.responseFormat.name == "LinkTextWithButton") {
         var questionText = formQuestion.question.questionHyperText;
         var linkStartPosition = questionText.lastIndexOf(";");
         var questionPart = questionText.slice(0, linkStartPosition);
         var linkName = questionText.slice(linkStartPosition + 1, questionText.length);
         var functionParam = linkName.replace(/\s/g, "");
         var params = [ functionParam, formQuestion.question.questionsID
         ];
         var functionName = 'doCreateTemplate';
         var link = createHyperlink(linkName, functionName, params);

         text = document.createTextNode(formQuestion.formSequencePosition + ". " + questionPart + " ");
         textLabel.appendChild(text);
         textLabel.appendChild(link);
      } else {
         text = document.createTextNode(formQuestion.formSequencePosition + ". "
            + formQuestion.question.questionHyperText);
         textLabel.appendChild(text);
      }

      // 543178: Section 508 Issue 2
      // Created hidden style to add text that is contained in the tooltip logic and hide to sighted user.
      reqSpan = document.createElement("span");
      reqSpan.setAttribute("class", "hidden");
      reqSpan.setAttribute("aria-hidden", "false");
      reqSpan.textContent = formQuestion.question.helpHyperText;
      textLabel.appendChild(reqSpan);

      // tooltip logic
      if ((formQuestion.question.helpHyperText) && (readOnlyState === false)) {
         toolSpan = document.createElement("span");
         toolSpan.setAttribute("class", "glyphicon glyphicon-question-sign");
         toolSpan.setAttribute("tabindex", "0");
         toolSpan.style.cursor = "help";
         toolSpan.setAttribute("data-toggle", "popover");
         toolSpan.setAttribute("data-placement", "top");
         toolSpan.setAttribute("data-content", formQuestion.question.helpHyperText);
         $(toolSpan).popover({
            trigger : "hover focus manual click",
            delay : {
               show : 250
            }
         });

         textLabel.appendChild(toolSpan);
      }

      formGroup.appendChild(textLabel);

      // Building the questions based on their type: radio, checkbox, select, textarea, file button, etc
      if ((formQuestion.question.responseFormat.name == "radio")
         || (formQuestion.question.responseFormat.name == "checkbox")) {

         // question input for radios and checkboxes
         field = document.createElement("fieldset");
         // legend = textLabel;
         field.setAttribute("id", "QID_" + formQuestion.question.questionsID);
         field.setAttribute("name", "QID_" + formQuestion.question.questionsID);
         // legend.setAttribute("for", "QID_" + formQuestion.question.questionsID);

         // building the individual radio/checkbox buttons one by one.
         for (j = 0; j < formQuestion.question.answerCollection.length; j++) {

            // used to determine pixels needed for answer
            var answerValueLength = (formQuestion.question.answerCollection[j].value).length;
            var tinyAnswer = (answerValueLength < 5);
            var smallAnswer = (answerValueLength >= 5) && (answerValueLength < 18);
            var mediumAnswer = (answerValueLength >= 18) && (answerValueLength < 100);
            var neededLength = 8 * answerValueLength;

            var answerLabel = document.createElement("label");
            if (formQuestion.question.responseFormat.name == "radio") {
               answerLabel.setAttribute("class", "radio");
            } else if (formQuestion.question.responseFormat.name == "checkbox") {
               answerLabel.setAttribute("class", "checkbox");
            }

            // set label width for radio or checkbox based on answer length
            if (tinyAnswer) {
               answerLabel.style.width = "60px";
            } else if (smallAnswer) {
               answerLabel.style.width = "150px";
            } else if (mediumAnswer) {
               answerLabel.style.width = neededLength + "px";
            } else { // answer is 100 or more characters long
               answerLabel.style.width = "700px"; // keep long answer within form margin
            }

            answerLabel.setAttribute("for", "AID_" + formQuestion.question.answerCollection[j].answersID);

            // putting together general structure
            input = document.createElement("input");
            input.setAttribute("type", formQuestion.question.responseFormat.name);
            input.setAttribute("name", "QID_" + formQuestion.question.questionsID + "_multi");
            input.setAttribute("id", "AID_" + formQuestion.question.answerCollection[j].answersID);

            // disabling questions if the study is in review
            if (readOnlyState) {
               input.setAttribute("disabled", true);
            }

            // Specifying the method to notify which questions are changed. This method helps group radio buttons
            // together, while allowing checkboxes to be saved individually.
            if (formQuestion.question.responseFormat.name == "checkbox") {
               input.setAttribute("title", formQuestion.question.answerCollection[j].sequencePosition);
            }
            input.setAttribute("onchange", "checkEnableSaveButton(" + formQuestion.question.questionsID + ")");

            if ((rollupAnswers.contains(formQuestion.question.answerCollection[j].answersID))
               && (rollupQuestions.contains(formQuestion.question.questionsID))) {
               input.checked = true;
               checkedAnswers++;
               answeredValues.push(formQuestion.question.answerCollection[j].value);
            }

            if ((targetStage != 1)
               && (previousStageAnswers.containsKey(formQuestion.question.answerCollection[j].answersID))
               && (rollupAnswers.contains(formQuestion.question.answerCollection[j].answersID) === false)) {
               // console.log("inserting into currentValues:" +
               // JSON.stringify(formQuestion.question.answerCollection[j]));
               currentValues.put(formQuestion.question.answerCollection[j].answersID,
                  formQuestion.question.answerCollection[j].value);
            }

            var value = document.createTextNode(formQuestion.question.answerCollection[j].value);

            answerLabel.appendChild(input);
            answerLabel.appendChild(value);

            field.appendChild(answerLabel);
            // XXX NVDA Reads this out the question twice if we do this.
            // if(formQuestion.question.responseFormat.name == "radio")
            // field.setAttribute("role", "radiogroup");
            formGroup.appendChild(field);
            form.appendChild(formGroup);

            divFormContent.append(form);
         }

         if (readOnlyState) {
            var insertHere = $('[name="QID_' + formQuestion.question.questionsID + '"]').parent();
            $('[name="QID_' + formQuestion.question.questionsID + '"]').remove();
            input = document.createElement("blockquote");
            if (answeredValues.length >= 1) {
               for (x = 0; x < answeredValues.length; x++) {
                  if (x >= 1) {
                     breaker = document.createElement("br");
                     input.appendChild(breaker);
                     text = document.createTextNode(answeredValues[x]);
                  } else {
                     text = document.createTextNode(answeredValues[x]);
                  }
                  input.appendChild(text);
               }
            } else {
               italic = document.createElement("em");
               text = document.createTextNode("No answer given.");
               italic.appendChild(text);
               input.appendChild(italic);
            }
            insertHere.append(input);
         }

         if (currentValues.keys().length >= 1) {
            panelGroup = generatePreviousResponses(null, currentValues);
            formGroup.appendChild(panelGroup);
         }

      } else if (formQuestion.question.responseFormat.name == "select") {
         form.setAttribute("role", "form");
         form.appendChild(formGroup);

         // setting up the general structure of a drop down
         input = document.createElement("select");
         inputOption = document.createElement("option");
         inputOption.setAttribute("value", "none");
         text = document.createTextNode("none");
         inputOption.appendChild(text);
         input.appendChild(inputOption);

         // populating drop down based on users grabbed from the database
         users = ramsUsers.values();
         for (j = 0; j < users.length; j++) {
            inputOption = document.createElement("option");
            inputOption.setAttribute("value", users[j].Id);
            text = document.createTextNode(users[j].Title);
            // 600489: Section 508 Issue 12 Release 2.1
            inputOption.setAttribute("role", "option");
            inputOption.setAttribute("aria-label", users[j].Title);
            inputOption.setAttribute("aria-selected", "true");
            inputOption.appendChild(text);
            input.appendChild(inputOption);
         }

         input.setAttribute("name", "QID_" + formQuestion.question.questionsID);
         input.setAttribute("id", "QID_" + formQuestion.question.questionsID);
         input.setAttribute("class", "form-control normal select");
         input.setAttribute("onchange", "checkEnableSaveButton(" + formQuestion.question.questionsID + ")");
         // input.setAttribute("onkeypress", "changedQuestions.add("+formQuestion.question.questionsID+")");

         // disabling question if the study is in review
         if (readOnlyState) {
            input.setAttribute("disabled", true);
         }

         if (rollup.responses) {
            responseFound = false;
            // changing which option in the dropdown is initially selected
            for (x = 0; x < rollup.responses.length; x++) {
               if (rollup.responses[x].questionsID == formQuestion.question.questionsID) {
                  if ((targetStage != 1)
                     && (previousStageQuestions.get(formQuestion.question.questionsID) && ((previousStageQuestions
                        .get(formQuestion.question.questionsID)).response != rollup.responses[x].response))) {
                     currentValues.put(formQuestion.question.questionsID, (previousStageQuestions
                        .get(formQuestion.question.questionsID)).response);
                  }
                  answeredValues.push(rollup.responses[x].response);
                  responseFound = true;
                  if (readOnlyState) {
                     input = document.createElement("blockquote");
                     text = document.createTextNode(sharepointIDtoTitle(rollup.responses[x].response));
                     input.appendChild(text);
                  } else {
                     input.value = rollup.responses[x].response;
                  }
               }
            }
            if ((readOnlyState === true) && (responseFound === false)) {
               input = document.createElement("blockquote");
               italic = document.createElement("em");
               text = document.createTextNode("No answer given.");
               italic.appendChild(text);
               input.appendChild(italic);
            }
         }

         formGroup.appendChild(input);

         if (currentValues.keys().length >= 1) {
            panelGroup = generatePreviousResponses(currentValues, null);
            formGroup.appendChild(panelGroup);
         }

         divFormContent.append(form);

      } else if (formQuestion.question.responseFormat.name == "Facility") {
         var facilityName;
         var currentFacilities = getFacilityItems();

         form.setAttribute("role", "form");
         form.appendChild(formGroup);

         // setting up the general structure of a drop down
         input = document.createElement("select");
         // populating drop down based on users grabbed from the database
         if (assignedID == "1") {
            inputOption = document.createElement("option");
            inputOption.setAttribute("value", "none");
            text = document.createTextNode("");
            inputOption.appendChild(text);
            input.appendChild(inputOption);
         }
         for (j = 0; j < currentFacilities.length; j++) {
            facilityName = currentFacilities[j].Title;
            if (facilityName != "Central IRB") {
               inputOption = document.createElement("option");
               inputOption.setAttribute("value", currentFacilities[j].Title);
               text = document.createTextNode(currentFacilities[j].Title);
               // 600489: Section 508 Issue 12 Release 2.1
               inputOption.setAttribute("role", "option");
               inputOption.setAttribute("aria-label", facilityName);
               inputOption.setAttribute("aria-selected", "true");
               // if (facilityName == "Central IRB") {
               // inputOption.setAttribute("disabled", "true");
               // }
               inputOption.appendChild(text);
               input.appendChild(inputOption);
            }

         }

         input.setAttribute("id", "facilitydropdown");
         input.setAttribute("name", "QID_" + formQuestion.question.questionsID);
         input.setAttribute("class", "form-control normal select");
         input.setAttribute("aria-label", "Facility");
         input.setAttribute("onchange", "checkEnableSaveButton(" + formQuestion.question.questionsID + ")");
         // input.setAttribute("onkeypress", "changedQuestions.add("+formQuestion.question.questionsID+")");

         // disabling question if the study is in review
         if (readOnlyState) {
            input.setAttribute("disabled", true);
         }

         if (rollup.responses) {
            responseFound = false;
            // changing which option in the dropdown is initially selected
            for (j = 0; j < rollup.responses.length; j++) {
               if (rollup.responses[j].questionsID == formQuestion.question.questionsID) {
                  if ((targetStage != 1)
                     && (previousStageQuestions.get(formQuestion.question.questionsID) && ((previousStageQuestions
                        .get(formQuestion.question.questionsID)).response != rollup.responses[j].response))) {
                     currentValues.put(formQuestion.question.questionsID, (previousStageQuestions
                        .get(formQuestion.question.questionsID)).response);
                  }
                  answeredValues.push(rollup.responses[j].response);
                  responseFound = true;
                  if (readOnlyState) {
                     input = document.createElement("blockquote");
                     text = document.createTextNode(rollup.responses[j].response);
                     input.appendChild(text);
                  } else {
                     input.value = rollup.responses[j].response;
                  }
               }
            }
            if ((readOnlyState === true) && (responseFound === false)) {
               input = document.createElement("blockquote");
               italic = document.createElement("em");
               text = document.createTextNode("No answer given.");
               italic.appendChild(text);
               input.appendChild(italic);
            }
         }

         formGroup.appendChild(input);

         if (currentValues.keys().length >= 1) {
            panelGroup = generatePreviousResponses(currentValues, null);
            formGroup.appendChild(panelGroup);
         }

         divFormContent.append(form);

      } else if ((formQuestion.question.responseFormat.name == "button")
         || (formQuestion.question.responseFormat.name == "LinkTextWithButton")) {
         var uploadInput;

         // setting up the general structure for a file upload button
         form.setAttribute("role", "form");

         // setting up the upload button (Section C, Question 5)
         uploadInput = document.createElement("input");
         // 573974: 508 FireEyes Defect_Quantity=16
         uploadInput.setAttribute("id", "QID_" + formQuestion.question.questionsID);
         uploadInput.setAttribute("name", "QID_" + formQuestion.question.questionsID);
         uploadInput.setAttribute("type", "file");

         if (rollup.responses) {
            responseFound = false;
            // looking for responses to populate the textarea with
            for (j = 0; j < rollup.responses.length; j++) {
               if (rollup.responses[j].questionsID == formQuestion.question.questionsID) {
                  responseFound = true;
                  if (readOnlyState) {
                     uploadInput = document.createElement("blockquote");
                     text = document.createTextNode(rollup.responses[j].response);
                     uploadInput.appendChild(text);
                  } else {
                     text = document.createTextNode("Uploaded File: " + rollup.responses[j].response);
                     formGroup.appendChild(text);
                     uploadInput.setAttribute("disabled", "disabled");
                  }
               }
            }
            if ((readOnlyState === true) && (responseFound === false)) {
               uploadInput = document.createElement("blockquote");
               italic = document.createElement("em");
               text = document.createTextNode("No file uploaded.");
               italic.appendChild(text);
               uploadInput.appendChild(italic);
            }
         }

         if (readOnlyState) {
            uploadInput.setAttribute("disabled", "disabled");
         }

         // attaching the upload button
         formGroup.appendChild(uploadInput);

         form.appendChild(formGroup);
         divFormContent.append(form);

      } else {
         // question input for everything else, which should cover text areas.
         form.setAttribute("role", "form");
         form.appendChild(formGroup);

         // setting up the general structure for a text area
         if (formQuestion.question.responseFormat.name == "textarea") {
            input = document.createElement("textarea");
            input.setAttribute("rows", "10");
         } else {
            input = document.createElement("input");
            input.setAttribute("type", formQuestion.question.responseFormat.name);
         }

         // setting up the general structure for a textarea
         input.setAttribute("name", "QID_" + formQuestion.question.questionsID);
         input.setAttribute("id", "QID_" + formQuestion.question.questionsID); // Still used?
         input.setAttribute("class", "form-control normal");
         input.setAttribute("onchange", "checkEnableSaveButton(" + formQuestion.question.questionsID + ")");
         // making sure the textarea is disabled if the application is in review
         if (readOnlyState) {
            input.setAttribute("disabled", true);
         }

         if (rollup.responses) {
            responseFound = false;
            // looking for responses to populate the textarea with
            for (j = 0; j < rollup.responses.length; j++) {
               if (rollup.responses[j].questionsID == formQuestion.question.questionsID) {
                  if ((targetStage != 1)
                     && (previousStageQuestions.get(formQuestion.question.questionsID) && ((previousStageQuestions
                        .get(formQuestion.question.questionsID)).response != rollup.responses[j].response))) {
                     currentValues.put(formQuestion.question.questionsID, (previousStageQuestions
                        .get(formQuestion.question.questionsID)).response);
                  }
                  answeredValues.push(rollup.responses[j].response);
                  responseFound = true;
                  if (readOnlyState) {
                     input = document.createElement("blockquote");
                     text = document.createTextNode(rollup.responses[j].response);
                     input.appendChild(text);
                  } else {
                     input.value = rollup.responses[j].response;
                  }
               }
            }
            if ((readOnlyState === true) && (responseFound === false)) {
               input = document.createElement("blockquote");
               italic = document.createElement("em");
               text = document.createTextNode("No answer given.");
               italic.appendChild(text);
               input.appendChild(italic);
            }
         }

         formGroup.appendChild(input);

         if (currentValues.keys().length >= 1) {
            // generatePreviousResponses(currentValues, null);
            panelGroup = generatePreviousResponses(currentValues, null);
            formGroup.appendChild(panelGroup);
         }

         divFormContent.append(form);
      }

      // checking to see if the question has a dependency on another answer
      if (formQuestion.requiredPredecessorAnswersID) {// JSHINT NOTE: input may not be initialized at this point. It is
         // up to the called method to behave
         // appropriately.
         questionRequirement(formQuestion, input);
      }

      // making sure the question is completely disabled if the application is in review
      if (readOnlyState) {
         // JSHINT NOTE: input may not be initialized at this point. It is up to the called method to behave
         // appropriately.
         input.setAttribute("disabled", true);
      }

   }
}

/**
 * builds either the questions or answers for comparison and display
 * 
 * @param {Hashtable} buildQuestions
 * @param {Hashtable} buildAnswers
 * @returns panelGroup {object} jquery object to add question and ansswer elements to the dom
 */
function generatePreviousResponses(buildQuestion, buildAnswers) {
   // console.log("getting into previous responses. questions: " + buildQuestion + ", answers: " + buildAnswers);
   var panelGroup, panel, panelHeading, panelTitle, panelAnchor, panelDiv, panelBody, text, breaker, bold, question, find;
   var key = "key";

   if ((typeof buildQuestion !== "undefined") && (buildQuestion !== null)) {
      find = buildQuestion.keys();
      question = previousStageQuestions.get(find[0]);
   } else {
      find = buildAnswers.keys();
      question = previousStageAnswers.get(find[0]);
   }

   panelGroup = document.createElement("div");
   panelGroup.setAttribute("class", "panel-group");
   panelGroup.setAttribute("id", "accordion" + question.questionsID);
   panelGroup.setAttribute("role", "tablist");
   panelGroup.setAttribute("aria-multiselectable", "true");

   panel = document.createElement("div");
   panel.setAttribute("class", "panel panel-default");

   panelHeading = document.createElement("div");
   panelHeading.setAttribute("class", "panel-heading");
   panelHeading.setAttribute("role", "tab");
   panelHeading.setAttribute("id", "heading" + question.questionsID + key);

   panelTitle = document.createElement("h4");
   panelTitle.setAttribute("class", "panel-title");

   panelAnchor = document.createElement("a");
   panelAnchor.setAttribute("data-toggle", "collapse");
   panelAnchor.setAttribute("data-parent", "#accordion" + question.questionsID);
   panelAnchor.setAttribute("href", "#collapse" + question.questionsID + key);
   panelAnchor.setAttribute("aria-expanded", "false");
   panelAnchor.setAttribute("aria-controls", "collapse" + question.questionsID + key);

   text = document.createTextNode("Answer from " + question.updatedAt.split("T")[0]);
   panelAnchor.appendChild(text);
   panelTitle.appendChild(panelAnchor);
   panelHeading.appendChild(panelTitle);
   panel.appendChild(panelHeading);

   panelDiv = document.createElement("div");
   panelDiv.setAttribute("id", "collapse" + question.questionsID + key);
   panelDiv.setAttribute("class", "panel-collapse collapse");
   panelDiv.setAttribute("role", "tabpanel");
   panelDiv.setAttribute("aria-labelledby", "heading" + question.questionsID + key);

   panelBody = document.createElement("div");
   panelBody.setAttribute("class", "panel-body");

   bold = document.createElement("b");
   text = document.createTextNode("You previously answered: ");
   bold.appendChild(text);
   panelBody.appendChild(bold);

   breaker = document.createElement("br");
   panelBody.appendChild(breaker);

   if ((typeof buildQuestion === "undefined") || (buildQuestion === null)) {
      // JSHINT {var} is defined but never used.
      // jshint -W098
      buildAnswers.each(function(displayID, displayText) {
         text = document.createTextNode(displayText);
         breaker = document.createElement("br");
         panelBody.appendChild(text);
         panelBody.appendChild(breaker);
      });
   } else {
      text = document.createTextNode(question.response);
      panelBody.appendChild(text);
   }

   panelDiv.appendChild(panelBody);
   panel.appendChild(panelDiv);

   panelGroup.appendChild(panel);

   return panelGroup;
}

/* exported checkSave */
/**
 * checks to see if there are any unsaved questions within a section and notifies the user when they try to change
 * sections
 * 
 * @param targetSection {number} The section number used to determine if any answers have been modified
 */
function checkSave(targetSection) {
   if (changedQuestions.isEmpty()) {
      sectionBuilder(targetSection);
   } else {
      if ($('.alert-warning').length !== 0) {
         sectionBuilder(targetSection);
      } else {
         insertWarning("You have unsaved responses. Please click the Save Progress button at the bottom of the page "
            + "or your changes will be lost.", "alert-warning");
      }
   }
}

/**
 * some questions are optional, this goes through the optional questions and sets up the proper dependencies on the
 * front end to allow for instant activation and deactivation of questions.
 * 
 * @param formQuestion {object} the question to check for predecessors.
 * @param input {object} jQuery dom element for the input control
 */
function questionRequirement(formQuestion, input) {

   // general queries to grab parent question
   var parentAnswer = $('#AID_' + formQuestion.requiredPredecessorAnswersID);
   var radioPair, span, j, x;

   // checkboxes have a listener on the one checkbox, but radios need a
   // listener to turn on the question and a listener to turn off the question.
   // this requires us to query the yes/no radio buttons and attach a listener to each.
   if (parentAnswer.attr("type") == "checkbox") {
      if ((typeof (parentAnswer.attr('onclick')) === "undefined") || ((parentAnswer.attr('onclick')) === null)) {
         parentAnswer.attr("onclick", "activateQuestion(" + formQuestion.requiredPredecessorAnswersID + ")");
         // parentAnswer.attr("onkeypress", "activateQuestion("+formQuestion.requiredPredecessorAnswersID+")");
      }
   } else if (parentAnswer.attr("type") == "radio") {
      // radioPair = $('[name="'+($('#AID_'+formQuestion.requiredPredecessorAnswersID).attr("name"))+'"]');
      radioPair = $('[for="AID_' + formQuestion.requiredPredecessorAnswersID + '"]').parent();
      if ((typeof (radioPair.attr('onclick')) === "undefined") || ((radioPair.attr('onclick')) === null)) {
         for (j = 0; j < radioPair.length; j++) {
            radioPair[j].setAttribute("onclick", "activateQuestion(" + formQuestion.requiredPredecessorAnswersID + ")");
            // radioPair[j].setAttribute("onkeypress",
            // "activateQuestion("+formQuestion.requiredPredecessorAnswersID+")");
         }
      } else {
         // var multipleDependency = true;

         for (j = 0; j < radioPair.length; j++) {
            var lookupID = parseInt(radioPair[j].getAttribute("id"));
            // checking to see if radio buttons in the same question
            // have different dependencies.
            if ((dependency.containsKey(lookupID))) {
               if (lookupID != formQuestion.requiredPredecessorAnswersID) {

                  // building the consolodated string of IDs that need to be updated
                  var activateString;
                  activateString = "Function(";
                  for (x = 0; x < radioPair.length; x++) {
                     var secondLookup = parseInt(radioPair[x].getAttribute("id"));
                     activateString += "activateQuestion(" + secondLookup + ")";
                  }
                  activateString += ")";
                  activateString = activateString.replace(")a", "),a");

                  // updating the activation attribute
                  for (x = 0; x < radioPair.length; x++) {
                     radioPair[x].setAttribute("onclick", activateString);
                     // radioPair[x].setAttribute("onkeypress", activateString);
                  }
               }
               // JSHINT Empty block - The empty blocks are intentional for debugging purposes.
               // jshint -W035
            } else {
               // multipleDependency = false;
            }
         }
      }
   }

   // We want to make sure that all children are disabled when their parents are disabled.
   if (parentAnswer.attr("disabled") == "disabled") {
      $('[name="QID_' + formQuestion.question.questionsID + '"]').attr("disabled", true);
      $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().parent().attr("rams-required", false);
      $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().find("span.required").remove();
   } else {
      $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().parent().attr("rams-required", true);
      $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().find("span.required").remove();
      span = document.createElement("span");
      span.setAttribute("class", "required sr-only");
      span.textContent = "A response is required.";
      // $('[name="QID_'+formQuestion.question.questionsID+'"]').before(span);
      $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().children().first().prepend(span);

      span = document.createElement("span");
      span.setAttribute("class", "required");
      span.setAttribute("aria-hidden", "true");
      span.textContent = "*";
      // $('[name="QID_'+formQuestion.question.questionsID+'"]').before(span);
      $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().children().first().prepend(span);

   }

   // Below we set up a data structure to hold parent/child relationships. This includes setting up
   // a hash map where they key is a question ID and the value is a set of question IDs that the key effects

   // checking to see if there is already an established key for this answer
   if (dependency.containsKey(formQuestion.requiredPredecessorAnswersID)) {

      // checking to see if the parent answer is checked or not, which directly affects whether the child is
      // active/deactive
      if (((parentAnswer.is(':checked')) === false)
         && ((rollupAnswers.contains(formQuestion.requiredPredecessorAnswersID)) === false)) {
         if ((input.getAttribute("type") == "radio") || (input.getAttribute("type") == "checkbox")) {
            $('[name="QID_' + formQuestion.question.questionsID + '"]').attr("disabled", true);
            $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().find("span.required").remove();
         } else {
            input.setAttribute("disabled", true);
            $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().find("span.required").remove();
         }

         // points to an off page dependency. In this case we remove the question completely.
         if (((rollupAnswers.contains(formQuestion.requiredPredecessorAnswersID)) === false)
            && ($('#AID_' + formQuestion.requiredPredecessorAnswersID).length === 0)) {
            $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().hide();
         }

      }

      // adding id to the set of questions affected by an answer.
      if (((dependency.get(formQuestion.requiredPredecessorAnswersID)).contains(formQuestion.question.questionsID)) === false) {
         (dependency.get(formQuestion.requiredPredecessorAnswersID)).add(formQuestion.question.questionsID);
      }

   } else {
      // instantiating the answer in the dependency map and giving it a set of questions it affects.
      // Going through the same logic as above.
      var temp = new HashSet();
      temp.add(formQuestion.question.questionsID);
      dependency.put(formQuestion.requiredPredecessorAnswersID, temp);

      if (((parentAnswer.is(':checked')) === false)
         && ((rollupAnswers.contains(formQuestion.requiredPredecessorAnswersID)) === false)) {
         if ((input.getAttribute("type") == "radio") || (input.getAttribute("type") == "checkbox")) {
            $('[name="QID_' + formQuestion.question.questionsID + '"]').attr("disabled", true);
            $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().find("span.required").remove();
         } else {
            input.setAttribute("disabled", true);
            $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().find("span.required").remove();
         }

         if (((rollupAnswers.contains(formQuestion.requiredPredecessorAnswersID)) === false)
            && ($('#AID_' + formQuestion.requiredPredecessorAnswersID).length === 0)) {
            $('[name="QID_' + formQuestion.question.questionsID + '"]').parent().hide();
         }
      }
   }
}

/* exported activateQuestion */
/**
 * This function toggles if a question is activated or deactivated base on the target state
 * 
 * @param answersId {number} the answer to enable or disable
 */
function activateQuestion(answersId) {
   var set = (dependency.get(answersId)).values();
   var span;

   if ((targetState == "1") || (targetState == "3")) {
      for (var i = 0; i < set.length; i++) {
         if ($('#AID_' + answersId).attr("disabled")) {
            $('[name="QID_' + set[i] + '"]').attr("disabled", true);
            $('[name="QID_' + set[i] + '"]').parent().parent().attr("rams-required", false);
            $('[name="QID_' + set[i] + '"]').parent().find("span.required").remove();
         } else {
            if (($('#AID_' + answersId).is(':checked')) === false) {
               $('[name="QID_' + set[i] + '"]').attr("disabled", true);
               $('[name="QID_' + set[i] + '"]').parent().parent().attr("rams-required", false);
               $('[name="QID_' + set[i] + '"]').parent().find("span.required").remove();

               var questionRequired = $('[name="QID_' + set[i] + '"]').parent().parent().attr("rams-required");

               // 5543162: This allows Response Is Required and Response is not required to be only read at the
               // correct time by the screen reader.
               if (questionRequired == "false") {
                  // If block is not required add not required message.
                  span = document.createElement("span");
                  span.setAttribute("class", "sr-only");
                  span.textContent = "A response is not required.";
                  $('[name="QID_' + set[i] + '"]').parent().children().first().prepend(span);
               }

            } else {
               // 5543162 Remove not required message to prevent duplicates required & not required from occuring
               $('[name="QID_' + set[i] + '"]').parent().find(".sr-only").remove();

               $('[name="QID_' + set[i] + '"]').attr("disabled", false);
               $('[name="QID_' + set[i] + '"]').parent().parent().attr("rams-required", true);
               $('[name="QID_' + set[i] + '"]').parent().find("span.required").remove();
               span = document.createElement("span");
               span.setAttribute("class", "required");
               span.setAttribute("aria-hidden", "true");
               span.textContent = "*";
               // $('[name="QID_'+set[i]+'"]').before(span);
               $('[name="QID_' + set[i] + '"]').parent().children().first().prepend(span);

               span = document.createElement("span");
               span.setAttribute("class", "required sr-only");
               span.textContent = "A response is required.";
               // $('[name="QID_'+set[i]+'"]').before(span);
               $('[name="QID_' + set[i] + '"]').parent().children().first().prepend(span);
            }
         }
         // recursively going through the quetions to activate/deactivate children
         var setQuestions = $('[name="QID_' + set[i] + '"]');
         for (var j = 0; j < setQuestions.length; j++) {
            var setID = setQuestions[j].getAttribute("id");
            if (dependency.containsKey(parseInt(setID))) {
               activateQuestion(parseInt(setID));
            }
         }
      }
   }
}

// Builds the buttons that appear at the bottom of the page
function progressBuilder(allowSave) {
   var spButton, csButton;

   // Cancel Button. Takes the user back to the home page.
   csButton = document.createElement("button");
   csButton.setAttribute("value", "Cancel");
   csButton.setAttribute("type", "button");
   csButton.setAttribute("role", "button");
   csButton.setAttribute("class", "btn btn-default pull-left");
   csButton.setAttribute("title", "Cancel");
   // csButton.setAttribute("onclick", "location.href='../SiteAssets/irb-applications.html'");
   csButton.setAttribute("onclick", "location.href='../SitePages/irb%20applications.aspx'");
   // csButton.setAttribute("onkeypress", "location.href='../SitePages/irb%20applications.aspx'");

   var text = document.createTextNode("Cancel");
   csButton.appendChild(text);
   $('div.button-container').append(csButton);

   if ((readOnlyState === false) || (allowSave === true)) {
      // save progress button. Runs the function to package the user's responses.
      spButton = document.createElement("button");
      spButton.setAttribute("value", "Save Progress");
      spButton.setAttribute("type", "button");
      spButton.setAttribute("role", "button");
      spButton.setAttribute("class", "btn btn-default pull-right");
      spButton.setAttribute("title", "Save Progress");
      spButton.setAttribute("id", "saveAppProgress");
      spButton.setAttribute("onclick", "createResponses()");
      // spButton.setAttribute("onkeypress", "createResponses()");

      text = document.createTextNode("Save Progress");

      spButton.appendChild(text);
      $('div.button-container').append(spButton);
   }
}

// JSHINT targetSection is defined but never used.
// jshint -W098
// Builds the buttons that appear on top of the sidebar which
// allow users to upload documents and print a section
function actionBarBuilder(targetSection) {
   /* jshint unused: false */
   var actionTools;
   var uploadList, uploadLink, uploadIcon, uploadInput, trainingLink, trainingIcon;

   // general structure for print button
   actionTools = document.createElement("ul");
   actionTools.setAttribute("class", "actionTools");
   if (assignedID != 1) {
      actionTools.appendChild(printMenuBuilder());
   }

   uploadList = document.createElement("li");
   // general structure for upload button
   // creating the normal button that stays hidden
   uploadInput = document.createElement("input");
   uploadInput.setAttribute("id", "upload-general");
   uploadInput.setAttribute("type", "file");
   uploadInput.setAttribute("name", "0000");
   uploadInput.style.display = "none";
   uploadList.appendChild(uploadInput);

   // setting up the stylized button that triggers the usual
   // file upload button which is hidden.
   uploadLink = document.createElement("a");
   // uploadLink.setAttribute("href", "#");
   // uploadLink.setAttribute("onclick", "checkEnableSaveButton('0000'); $('#upload-general').trigger('click');
   // uploadFiles();"); //previous code (working)
   uploadLink.setAttribute("onclick", "triggerUploadModal()");
   // uploadLink.setAttribute("onkeypress", "Function($('#upload-general').trigger('click'),
   // changedQuestions.add('0000'))");

   uploadLink.style.cursor = "pointer";

   uploadIcon = document.createElement("span");
   uploadIcon.setAttribute("class", "glyphicon glyphicon-upload");
   uploadLink.appendChild(uploadIcon);

   var text = document.createTextNode("Upload Document...");
   uploadLink.appendChild(text);

   uploadList.appendChild(uploadLink);
   actionTools.appendChild(uploadList);

   if (assignedID != 1) {
      var trainingList = document.createElement("li");

      trainingLink = document.createElement("a");
      trainingLink.setAttribute("id", "trainingLink");
      trainingLink.setAttribute("onclick", "getTrainingReport()");
      trainingLink.style.cursor = "pointer";

      trainingIcon = document.createElement("span");
      trainingIcon.setAttribute("class", "glyphicon glyphicon-book");
      trainingLink.appendChild(trainingIcon);

      var trainingText = document.createTextNode("Verify Training");
      trainingLink.appendChild(trainingText);
      trainingList.appendChild(trainingLink);
      actionTools.appendChild(trainingList);
   }

   $('div.actionbar').append(actionTools);
}

/* exported getTrainingReport */
function getTrainingReport() {
   // var trainingReportLink = protocol + reportsServerName + "/ReportServer/Pages/" +
   // "ReportViewer.aspx?%2fRAMS+Reports%2fTrainingReport&rs:Command=Render&listOfIds=";
   // JSHINT dynamicTrainingReportURL is not defined - it is defined in RAMS-environment
   // jshint -W117
   var trainingReportLink = dynamicTrainingReportURL;
   var users = [];

   sendREST("GET", restPath.concat("study/" + assignedID + "/members"), null, function(studyMembers) {
      if ((typeof studyMembers !== "undefined") && (studyMembers !== null)) {
         for (i = 0; i < studyMembers.length; i++) {
            if ((typeof studyMembers[i].user.SPUser !== "undefined") && (studyMembers[i].user.SPUser !== null)) {
               var id = studyMembers[i].user.SPUser;
               users.push(id);
            }

         }
      }
      trainingReportLink += users.join(",");
      trainingReportLink += "&ReportFor=Study ";
      trainingReportLink += assignedID;
      window.open(trainingReportLink, "_blank");
   });
}

function printMenuBuilder() {
   var template = "<li><div class=dropdown>"
      + "    <a id=printLabel href='#' data-toggle=dropdown role=button aria-haspopup=true aria-expanded=false>"
      + "         <span class='glyphicon glyphicon-print'></span>" + "         Print All Sections"
      + "         <span class=caret></span>" + "    </a>" + "    <ul class=dropdown-menu aria-labelledby=printLabel>";

   var stage;
   for (stage = 1; stage <= currentStudyStatus.length; stage++) {
      template += "<li>";
      template += "<a target=_blank href='irb-applications-print.html?" + assignedID + "?" + stage + "'>Stage " + stage
         + ": " + currentStudyStatus[targetStage - 1].studyState.name;
      if (stage == targetStage) {
         template += " (currently displayed)";
      }
      template += "</a></li>";
   }
   template += "</ul></div></li>";
   return $.parseHTML(template)[0];
}

// creates the rest of the sidebar including the sections
// and the buttons that allow you to review and submit
function wellSidebarBuilder(targetSection, fileCount) {
   var listGroup, lGroup;
   var link, text, icon, title;
   var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

   $('.list-group').remove();

   listGroup = document.createElement("div");
   listGroup.setAttribute("class", "list-group");

   // creating the sidebar menu button for each section
   for (var i = 0; i < study.containedFormCollection.length; i++) {
      link = document.createElement("a");
      link.style.cursor = "pointer";
      link.setAttribute("tabindex", "0");

      // setting up the activated section styling
      if (i == targetSection) {
         link.setAttribute("class", "list-group-item active sectionLink");
      } else {
         link.setAttribute("class", "list-group-item sectionLink");
      }

      // var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
      title = study.containedFormCollection[i].containedForm.title;
      text = document.createTextNode(sectionTitle.charAt(i) + ": " + title);
      link.appendChild(text);

      listGroup.appendChild(link);
   }

   $('div.well.sidebar').append(listGroup);

   lGroup = document.createElement("div");
   lGroup.setAttribute("class", "list-group");

   // setting up the Attachments button
   link = document.createElement("a");
   link.style.cursor = "pointer";
   link.setAttribute("class", "list-group-item");
   // link.setAttribute("onkeypress", "refreshDisplay('Files')");
   link.setAttribute("onclick", "refreshDisplay('Files')");
   link.setAttribute("tabindex", "0");

   text = document.createTextNode("Attachments");
   link.appendChild(text);

   icon = document.createElement("span");
   icon.setAttribute("class", "badge");

   text = document.createTextNode(fileCount);
   icon.appendChild(text);

   link.appendChild(icon);

   lGroup.appendChild(link);

   // setting up the Add Personnel button
   link = document.createElement("a");
   link.style.cursor = "pointer";
   link.setAttribute("class", "list-group-item");
   link.setAttribute("onclick", "refreshDisplay('Personnel')");
   // link.setAttribute("onkeypress", "refreshDisplay('Personnel')");
   link.setAttribute("tabindex", "0");

   text = document.createTextNode("Manage Personnel");
   link.appendChild(text);

   icon = document.createElement("span");
   icon.setAttribute("class", "glyphicon glyphicon-chevron-right");

   link.appendChild(icon);

   lGroup.appendChild(link);

   /* Should only be set if this is a central applicaiton */
   if ((currentFormType == "Parent")
      && ((grabApplicationSiteType() == "Central") || (grabApplicationSiteType() == "Multi"))) {
      // setting up the Local Site button
      link = document.createElement("a");
      link.style.cursor = "pointer";
      link.setAttribute("class", "list-group-item");
      link.setAttribute("onclick", "refreshDisplay('Facility')");
      // link.setAttribute("onkeypress", "refreshDisplay('Facility')");
      link.setAttribute("tabindex", "0");

      text = document.createTextNode("Local Site Application");
      link.appendChild(text);

      icon = document.createElement("span");
      icon.setAttribute("class", "glyphicon glyphicon-chevron-right");

      link.appendChild(icon);

      lGroup.appendChild(link);
   }

   // setting up the Review & Submit button
   if (targetState == 1) {
      link = document.createElement("a");
      link.style.cursor = "pointer";
      link.setAttribute("class", "list-group-item");
      link.setAttribute("onclick", "reviewApp()");
      // link.setAttribute("onkeypress", "reviewApp()");
      link.setAttribute("tabindex", "0");

      // if(targetState == 1){
      text = document.createTextNode("Review & Submit");
      link.appendChild(text);
      // }

      icon = document.createElement("span");
      icon.setAttribute("class", "glyphicon glyphicon-chevron-right");

      link.appendChild(icon);

      lGroup.appendChild(link);
   }

   $('div.well.sidebar').append(lGroup);

   // updating sidebar icons
   updateSidebarIcons(targetSection);
}

// organizes questions in sections to be either answered or unanswered and then
// reports to the user which questions they still need to fill out.
function validateQuestions() {
   var formsResponse = study;
   var rollupResponse = rollup;

   var form_id_questions_id_map = {};
   var questions_predecessor_map = {};
   var formValidCont;
   // var sectionCont;
   var formTable, row, questionTable;
   var x, xx;
   var text, breaker, paragraph, submission;

   formTable = document.createElement("ul");
   questionTable = document.createElement("ul");

   // putting together a map of parent/child questions
   for (x = 0; x < formsResponse.containedFormCollection.length; x++) {
      for (xx = 0; xx < formsResponse.containedFormCollection[x].containedForm.formQuestionCollection.length; xx++) {
         if (formsResponse.containedFormCollection[x].containedForm.formsID in form_id_questions_id_map) {
            form_id_questions_id_map[formsResponse.containedFormCollection[x].containedForm.formsID]
               .push(formsResponse.containedFormCollection[x].containedForm.formQuestionCollection[xx].question.questionsID);
         } else {
            form_id_questions_id_map[formsResponse.containedFormCollection[x].containedForm.formsID] = [ formsResponse.containedFormCollection[x].containedForm.formQuestionCollection[xx].question.questionsID
            ];
         }

         if (formsResponse.containedFormCollection[x].containedForm.formQuestionCollection[xx].requiredPredecessorAnswersID) {
            questions_predecessor_map[formsResponse.containedFormCollection[x].containedForm.formQuestionCollection[xx].question.questionsID] = formsResponse.containedFormCollection[x].containedForm.formQuestionCollection[xx].requiredPredecessorAnswersID;
         }
      }
   }

   var form_id_empty = [];
   var form_id_incomplete = [];

   var sectionTitle = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
   formValidCont = document.createElement("div");
   // sectionCont = document.createElement("div");
   if (deletedHIPAA !== "") {
      for (x = 0; x < rollupResponse.formValidationReport.formStatusCollection.length; x++) {

         if (rollupResponse.formValidationReport.formStatusCollection[x].formsID == deletedHIPAA) {

            rollupResponse.formValidationReport.formStatusCollection.splice(x, 1);
         }
      }
   }
   // grabbing questions that still need to be answered
   for (x = 0; x < rollupResponse.formValidationReport.formStatusCollection.length; x++) {

      if (rollupResponse.formValidationReport.formStatusCollection[x].status === "INCOMPLETE") {
         form_id_incomplete.push(rollupResponse.formValidationReport.formStatusCollection[x].formsID);

      } else if (rollupResponse.formValidationReport.formStatusCollection[x].status === "EMPTY") {
         form_id_empty.push(rollupResponse.formValidationReport.formStatusCollection[x].formsID);
         // notifying users of sections that are completely empty
         row = document.createElement("li");
         row.textContent = "Section " + sectionTitle.charAt(x) + ": "
            + study.containedFormCollection[x].containedForm.title + " is empty";
         formTable.appendChild(row);
      }
   }

   formValidCont.appendChild(formTable);

   // below we grab questions that have been answered
   var filled_questions = [];

   for (x = 0; x < rollupResponse.responses.length; x++) {
      filled_questions.push(rollupResponse.responses[x].questionsID);
   }

   var filled_answers = [];

   for (x = 0; x < rollupResponse.responses.length; x++) {
      if (rollupResponse.responses[x].answersID) {
         filled_answers.push(rollupResponse.responses[x].answersID);
      }
   }

   function hasPredecessorAnswered(question) {
      if (filled_answers.indexOf(questions_predecessor_map[question]) == -1) {
         return false;
      }
      return true;
   }

   var incomplete_forms_questions_map = {};

   // putting together a map of incomplete sections as the key and unanswered questions and the values
   for (x = 0; x < form_id_incomplete.length; x++) {
      for (xx = 0; xx < form_id_questions_id_map[form_id_incomplete[x]].length; xx++) {
         if (filled_questions.indexOf(form_id_questions_id_map[form_id_incomplete[x]][xx]) == -1) {
            if (!(form_id_questions_id_map[form_id_incomplete[x]][xx] in questions_predecessor_map)) {
               if (form_id_incomplete[x] in incomplete_forms_questions_map) {
                  incomplete_forms_questions_map[form_id_incomplete[x]]
                     .push(form_id_questions_id_map[form_id_incomplete[x]][xx]);
               } else {
                  incomplete_forms_questions_map[form_id_incomplete[x]] = [ form_id_questions_id_map[form_id_incomplete[x]][xx]
                  ];
               }
            } else {
               if (hasPredecessorAnswered(form_id_questions_id_map[form_id_incomplete[x]][xx])) {
                  if (form_id_incomplete[x] in incomplete_forms_questions_map) {
                     incomplete_forms_questions_map[form_id_incomplete[x]]
                        .push(form_id_questions_id_map[form_id_incomplete[x]][xx]);
                  } else {
                     incomplete_forms_questions_map[form_id_incomplete[x]] = [ form_id_questions_id_map[form_id_incomplete[x]][xx]
                     ];
                  }
               }
            }
         }
      }
   }

   // going through the map that was just created to notify the users of what specific questions still need to be
   // answered.
   for (x = 0; x < study.containedFormCollection.length; x++) {
      for (var i = 0; i < form_id_incomplete.length; i++) {
         if (form_id_incomplete[i] == study.containedFormCollection[x].containedForm.formsID) {
            for (var y = 0; y < study.containedFormCollection[x].containedForm.formQuestionCollection.length; y++) {
               for ( var key in incomplete_forms_questions_map) {
                  if (incomplete_forms_questions_map.hasOwnProperty(key)) {
                     for (var j = 0; j < incomplete_forms_questions_map[key].length; j++) {
                        if (incomplete_forms_questions_map[key][j] == study.containedFormCollection[x].containedForm.formQuestionCollection[y].question.questionsID) {
                           row = document.createElement("li");
                           row.textContent = "In Section "
                              + sectionTitle.charAt(x)
                              + ", you still need to answer: "
                              + study.containedFormCollection[x].containedForm.formQuestionCollection[y].question.questionHyperText;
                           questionTable.appendChild(row);
                        }
                     }
                  }
               }
            }
         }
      }
   }

   formValidCont.appendChild(questionTable);

   // checking the completeness of the application. All of the sections have to be labled as complete
   var complete = true;
   var expedited_complete = true;
   for (x = 0; x < rollup.formValidationReport.formStatusCollection.length; x++) {
      if ((rollup.formValidationReport.formStatusCollection[x].status != "COMPLETE")
         && (rollup.formValidationReport.formStatusCollection[x].status != "NOT_APPLICABLE")) {
         complete = false;
      }
   }

   // if a user is submitting an application as an expedited study, then they need to meet the three requirements,
   // which the below logic looks for
   if ((currentFormType == "Parent")
      && (rollupAnswers
         .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[5].question.answerCollection[1].answersID))) {
      if (!(rollupAnswers
         .contains(study.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[0].answersID))) {
         expedited_complete = false;
      }

      if (!(rollupAnswers
         .contains(study.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[1].answersID))) {
         expedited_complete = false;
      }

      if (!(rollupAnswers
         .contains(study.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[2].answersID))) {
         expedited_complete = false;
      }
   }

   // finally, for a user to submit an application, they have to either be a PI or CO-PI
   if (targetState == "1") {
      if (complete) {
         if (expedited_complete) {
            // todo strip login name AND PUT DISPLAYNAME TO LOGINNAME
            if (((currentFormType != "Parent") && (currentFormType != "Local"))
               || ((currentUser.Id == grabPI()) || (currentUser.Id == grabCoPI()))) {
               // setting up the structure for a user to submit their application
               var submitCont = document.createElement("div");
               // submitCont.setAttribute("action", "../SitePages/irb%20applications.aspx");

               breaker = document.createElement("br");
               formValidCont.appendChild(breaker);

               // inserting an official narrative for users to accept while submitting
               paragraph = document.createElement("p");
               text = document
                  .createTextNode("I accept responsibility for assuring that this protocol will be conducted in "
                     + "compliance with ethical clinical practices and all applicable VA, IRB, FDA regulations, "
                     + "IRB and Institutional policies relative to the protection of the rights, welfare and safety "
                     + "of human subjects.  I acknowledge that I have the primary and ultimate responsibility for "
                     + "protecting the rights and welfare of research participants and that I understand the ethical "
                     + "principles of human subject protection.  I affirm that I have the competencies and adequate "
                     + "resources and facilities to carry out the proposed research and I attest that this application "
                     + "is scientifically and ethically sound.  Finally, all Conflicts of Interests have been "
                     + "disclosed.");
               paragraph.appendChild(text);
               submitCont.appendChild(paragraph);

               paragraph = document.createElement("p");
               text = document
                  .createTextNode("In submitting this application, I certify the acknowledgements previously "
                     + "described and that the research team is qualified and credentialed to conduct this research "
                     + "study.  I and my study team will conduct and oversee this project in accordance with all "
                     + "ethical standards required by the VA for the conduct of human subjects research.");
               paragraph.appendChild(text);
               submitCont.appendChild(paragraph);

               submission = document.createElement("button");
               submission.setAttribute("id", "submitApplication"); // add ID to button
               submission.setAttribute("onclick", "submitIRB()");
               // submission.setAttribute("onkeypress", "submitIRB()");
               text = document.createTextNode("Submit Application");
               submission.appendChild(text);
               submitCont.appendChild(submission);

               formValidCont.appendChild(submitCont);
            } else {
               breaker = document.createElement("br");
               formValidCont.appendChild(breaker);

               // notifying users why their application is incomplete
               paragraph = document.createElement("p");
               text = document.createTextNode("You must be either the PI or CO-PI to submit an application");
               paragraph.appendChild(text);
               formValidCont.appendChild(paragraph);
            }
         } else {
            breaker = document.createElement("br");
            formValidCont.appendChild(breaker);

            // notifying users why their application is incomplete
            paragraph = document.createElement("p");
            text = document
               .createTextNode("You have idenitfied your application as being expedited, however you haven't met "
                  + "all three expedited cagtegories outlined in Section C");
            paragraph.appendChild(text);
            formValidCont.appendChild(paragraph);
         }
      } else {
         breaker = document.createElement("br");
         formValidCont.appendChild(breaker);

         // notifying users why their application is incomplete
         paragraph = document.createElement("p");
         text = document.createTextNode("You must complete the Research Study Application before you can submit.");
         paragraph.appendChild(text);
         formValidCont.appendChild(paragraph);
      }
   }
   return formValidCont;
}
/* exported submitIRB */
// changes the status of the application from "draft" to "review"
function submitIRB() {
   var cirb, appLink, liaison;
   var iacucReviewRequired = false;
   var irbReviewRequired = false;
   var rdcReviewRequired = false;
   var srsReviewRequired = false;
   var sc1ReviewRequired = false;
   var sc2ReviewRequired = false;
   var sc3ReviewRequired = false;
   /*
    * If the review criteria for a committee is met reviewsRequired.push("committeeName"); A new row for each
    * required committee
    */

   $("#submitApplication").prop("disabled", true); // disable the "Submit Application" button for the application
   $("#closeWindow").prop("disabled", true); // disable the "Close Window" button
   // encourage user to wait for redirect, user must click OK for the process to continue need to figure out how
   // to send the current user as well as how to grab the facility that the user is connected to.
   alert("You have just submitted your Research Study Application.\n\nPlease click [OK] to be redirected "
      + "to the Research Study Applications page");

   // Populate the list of RAMS Users for use by later functions
   getRAMSUsers(function() {

      // Why would the IRBAdministrator be set when the application is submitted? Answer, some child studies carry the
      // parent administrator forward.
      if ((typeof spAppData.IACUCAdministratorId !== "undefined") && (spAppData.IACUCAdministratorId !== null)) {
         insertStudyMember(spAppData.IACUCAdministratorId, IACUCSTUDYADMINISTRATOR, false, assignedID, null);
      }
      if ((typeof spAppData.IRBAdministratorId !== "undefined") && (spAppData.IRBAdministratorId !== null)) {
         insertStudyMember(spAppData.IRBAdministratorId, IRBSTUDYADMINISTRATOR, false, assignedID, null);
      }
      if ((typeof spAppData.RDCAdministratorId !== "undefined") && (spAppData.RDCAdministratorId !== null)) {
         insertStudyMember(spAppData.RDCAdministratorId, RDCSTUDYADMINISTRATOR, false, assignedID, null);
      }
      if ((typeof spAppData.SRSAdministratorId !== "undefined") && (spAppData.SRSAdministratorId !== null)) {
         insertStudyMember(spAppData.SRSAdministratorId, SRSSTUDYADMINISTRATOR, false, assignedID, null);
      }
      if ((typeof spAppData.SC1AdministratorId !== "undefined") && (spAppData.SC1AdministratorId !== null)) {
         insertStudyMember(spAppData.SC1AdministratorId, SC1STUDYADMINISTRATOR, false, assignedID, null);
      }
      if ((typeof spAppData.SC2AdministratorId !== "undefined") && (spAppData.SC2AdministratorId !== null)) {
         insertStudyMember(spAppData.SC2AdministratorId, SC2STUDYADMINISTRATOR, false, assignedID, null);
      }
      if ((typeof spAppData.SC3AdministratorId !== "undefined") && (spAppData.SC3AdministratorId !== null)) {
         insertStudyMember(spAppData.SC3AdministratorId, SC3STUDYADMINISTRATOR, false, assignedID, null);
      }

      // *****only want to grab the pi if parent or local
      // if()
      var piTitle = sharepointIDtoTitle(grabPI());
      appLink = {
         "__metadata" : {
            "type" : "SP.FieldUrlValue"
         },
         // todo sharepointid to title for grabpi
         "Description" : piTitle + " - " + studyTitle + ": " + assignedID,
         "Url" : window.location.href
      };

      /*
       * The tissue bank answer is hardcoded to match the tissue bank in the database. If the questionID ever changes
       * this will need to be updated in the future. There may be a better way of doing this.
       */
      var tissueBankAnswer = "";
      for (var j = 0; j < rollup.responses.length; j++) {
         if (rollup.responses[j].questionsID == "13541") {
            tissueBankAnswer = rollup.responses[j].response;
            break;
         }
      }

      var adverseReportableEventAnswers = getAdverseReportableEventAnswers();

      var submitDate = new Date().toJSON();

      // var reviewsRequired;
      determineCommittees(function(reviewsRequired) {
         if (reviewsRequired === null || reviewsRequired === "") {
            throw("Unable to determine the committees required to review the study.");
         } else {
            if (reviewsRequired.indexOf(IRB) >= 0) {
               irbReviewRequired = true;
            }
            if (reviewsRequired.indexOf(RDC) >= 0) {
               rdcReviewRequired = true;
            }
            if (reviewsRequired.indexOf(IACUC) >= 0) {
               iacucReviewRequired = true;
            }
            if (reviewsRequired.indexOf(SRS) >= 0) {
               srsReviewRequired = true;
            }
            if (reviewsRequired.indexOf(SC1) >= 0) {
               sc1ReviewRequired = true;
            }
            if (reviewsRequired.indexOf(SC2) >= 0) {
               sc2ReviewRequired = true;
            }
            if (reviewsRequired.indexOf(SC3) >= 0) {
               sc3ReviewRequired = true;
            }
         }

         if ((spAppData.ApplicationSiteType == "Central")
            || ((currentFormType == "Parent") && (grabApplicationSiteType() == "Central"))) {
            cirb = "Yes";
            // getCentralUrl(function(returnedUrl, returnedFacilityId, returnedLiaisonId) {
            // JSHINT returnedUrl and returnedFacilityId is defined but never used.
            // jshint -W098
            getSites(function(returnedUrl, returnedFacilityId, returnedLiaisonId) {

               // insertStudyMember(returnedLiaisonId, "Liaison Reviewer", false, assignedID, null);
               // console.log("in central, was given: " + returnedUrl + ", " + returnedFacilityId + ", " +
               // returnedLiaisonId);
               // NOT USED subsiteUrl = returnedUrl;

               if ((typeof spAppData.CoordinatingSitesLiaisonId === "undefined")
                  || (spAppData.CoordinatingSitesLiaisonId === null)) {
                  liaison = {
                     "__metadata" : {
                        "type" : "Collection(Edm.Int32)"
                     },
                     "results" : []
                  };
               } else {
                  liaison = spAppData.CoordinatingSitesLiaisonId;
               }
               data = {
                  "__metadata" : {
                     "type" : "SP.Data.ApplicationListItem"
                  },
                  "ApplicationStatus" : RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTEDCHOICE,
                  "Study_ID" : assignedID,
                  "ParentStudyID" : spAppData.ParentStudyID,
                  "Title" : studyTitle,
                  "StudyTitle" : studyTitle,
                  "CIRB" : cirb,
                  "ApplicationSiteType" : grabApplicationSiteType(),
                  "CoordinatingSitesId" : spAppData.CoordinatingSitesId,
                  // "CoordinatingSitesLiaisonId": liaison
                  "ReviewRequired" : reviewsRequired.toString(),
                  // 460601: Create view of studies to show COI disclosure status for personnel assigned to the study
                  // 4.7
                  // JSHINT getCOIStatusAnswer is not defined - it is defined in COI.js
                  // jshint -W117
                  "COI" : getCOIStatusAnswer(),
                  // "CommitteesLookupId" : CommitteesLookupId,
                  "TissueBank" : tissueBankAnswer,
               };
               // The following condition will determine if the study is parent study or child study and will
               // populate facilityID accordingly
               if ((typeof spAppData.ParticipatingFacilitiesLookupId !== "undefined")
                  && (spAppData.ParticipatingFacilitiesLookupId !== null)
                  && (typeof spAppData.ParentStudyID !== "undefined") && (spAppData.ParentStudyID !== null)) {
                  // this is the child study
                  data[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId] = spAppData.ParticipatingFacilitiesLookupId;
                  data[RAMS.Environment.SharePoint.List.Application.SiteLookupId] = spAppData.ParticipatingFacilitiesLookupId;
               } else {
                  data[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId] = returnedFacilityId;
                  data[RAMS.Environment.SharePoint.List.Application.SiteLookupId] = returnedFacilityId;
               }
               // Sustainment Defect #662683 The metadata on the Research Study Applications table is showing the Local
               // Site/Participating Site Investigator as the Principal Investigator.
               // For the Local Site Application on a multi site or central IRB workflow, the PI should remain as the
               // Main Study PI and only the Participating Site Investigator field should be updated to the correct
               // Local Site Investigator(s)
               // data[RAMS.Environment.SharePoint.List.Application.PrincipalInvestigatorId] = currentUser.Id;
               // Sustainment Defect #662683 End
               data[RAMS.Environment.SharePoint.List.Application.LinkToApplication] = appLink;

               data[RAMS.Environment.SharePoint.List.Application.OffsiteWaiverRequests] = getOffsiteWaiverRequests();

               if (irbReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.IRBReviewRequired] = true;
                  data.IRBSubmitDate = submitDate;
                  data.IRBStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (rdcReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.RDCReviewRequired] = true;
                  data.RDCSubmitDate = submitDate;
                  data.RDCStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (iacucReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.IACUCReviewRequired] = true;
                  data.IACUCSubmitDate = submitDate;
                  data.IACUCStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (srsReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.SRSReviewRequired] = true;
                  data.SRSSubmitDate = submitDate;
                  data.SRSStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (sc1ReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.SC1ReviewRequired] = true;
                  data.SC1SubmitDate = submitDate;
                  data.SC1StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (sc2ReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.SC2ReviewRequired] = true;
                  data.SC2SubmitDate = submitDate;
                  data.SC2StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (sc3ReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.SC3ReviewRequired] = true;
                  data.SC3SubmitDate = submitDate;
                  data.SC3StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }

               if (studyTitle.indexOf("Adverse/Reportable Event") !== -1) {
                  data[RAMS.Environment.SharePoint.List.Application.AdverseReportableEvent] = adverseReportableEventAnswers.AdverseReportableEvent;
                  data[RAMS.Environment.SharePoint.List.Application.DeviationReportableEvent] = adverseReportableEventAnswers.DeviationReportableEvent;
               }

               // console.log("the url we are using is: " + subsiteUrl + ", we are sending " + JSON.stringify(data));
               sendREST("POST", spAppData.__metadata.uri, data, function() {
                  if (currentFormType == "Parent") {
                     sendREST("POST", restPath.concat("study/" + assignedID + "/stage/Review"), null, function() {
                        window.location.assign("../SitePages/irb%20applications.aspx");
                     });
                  } else {
                     sendREST("POST", restPath.concat("study/" + assignedID + "/stage/Review"), null, function() {
                        window.location.assign("../SitePages/irb%20applications.aspx");
                     });
                  }

               }, "Merge");
            });
         } else {
            cirb = "No";
            getSites(function(returnedUrl, returnedFacilityId, returnedLiaisonId) {

               // ***For 1604 Release, as business requirement this code has to be excluded since users do not want
               // default creation of RAMS Reviewer ***
               // Reversing this above decision due to Liason name is not being recorded...thus throwin 404
               // per Review and Planning meeting on 20171220 - Liaison should only be a reviewer on CIRB studies.
               // insertStudyMember(returnedLiaisonId, "Liaison Reviewer", false, assignedID, null);
               // console.log("in regular, was given: " + returnedUrl + ", " + returnedFacilityId + ", " +
               // returnedLiaisonId);
               // NOT USED subsiteUrl = returnedUrl;
               data = {
                  "__metadata" : {
                     "type" : "SP.Data.ApplicationListItem"
                  },
                  "ApplicationStatus" : RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTEDCHOICE,
                  "Study_ID" : assignedID,
                  "ParentStudyID" : spAppData.ParentStudyID,
                  "Title" : studyTitle,
                  "StudyTitle" : studyTitle,
                  "CIRB" : cirb,
                  "ApplicationSiteType" : grabApplicationSiteType(),
                  "ReviewRequired" : reviewsRequired.toString(),
                  // 460601: Create view of studies to show COI disclosure status for personnel assigned to the study
                  // 4.7
                  // JSHINT getCOIStatusAnswer is not defined - it is defined in COI.js
                  // jshint -W117
                  "COI" : getCOIStatusAnswer(),
                  // "CommitteesLookupId" : CommitteesLookupId,
                  "TissueBank" : tissueBankAnswer,
               };

               // The following condition will determine if the study is parent study or child study and will
               // populate
               // facilityID accordingly
               if ((typeof spAppData.ParticipatingFacilitiesLookupId !== "undefined")
                  && (spAppData.ParticipatingFacilitiesLookupId !== null)
                  && (typeof spAppData.ParentStudyID !== "undefined") && (spAppData.ParentStudyID !== null)) {
                  // this is the child study
                  data[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId] = spAppData.ParticipatingFacilitiesLookupId;
                  data[RAMS.Environment.SharePoint.List.Application.SiteLookupId] = spAppData.ParticipatingFacilitiesLookupId;
               } else {
                  data[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId] = returnedFacilityId;
                  data[RAMS.Environment.SharePoint.List.Application.SiteLookupId] = returnedFacilityId;
               }
               data[RAMS.Environment.SharePoint.List.Application.PrincipalInvestigatorId] = currentUser.Id;
               data[RAMS.Environment.SharePoint.List.Application.LinkToApplication] = appLink;

               data[RAMS.Environment.SharePoint.List.Application.OffsiteWaiverRequests] = getOffsiteWaiverRequests();

               if (irbReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.IRBReviewRequired] = true;
                  data.IRBSubmitDate = submitDate;
                  data.IRBStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (rdcReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.RDCReviewRequired] = true;
                  data.RDCSubmitDate = submitDate;
                  data.RDCStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (iacucReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.IACUCReviewRequired] = true;
                  data.IACUCSubmitDate = submitDate;
                  data.IACUCStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (srsReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.SRSReviewRequired] = true;
                  data.SRSSubmitDate = submitDate;
                  data.SRSStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (sc1ReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.SC1ReviewRequired] = true;
                  data.SC1SubmitDate = submitDate;
                  data.SC1StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (sc2ReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.SC2ReviewRequired] = true;
                  data.SC2SubmitDate = submitDate;
                  data.SC2StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }
               if (sc3ReviewRequired) {
                  data[RAMS.Environment.SharePoint.List.Application.SC3ReviewRequired] = true;
                  data.SC3SubmitDate = submitDate;
                  data.SC3StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.SUBMITTED;
               }

               if (studyTitle.indexOf("Adverse/Reportable Event") !== -1) {
                  data[RAMS.Environment.SharePoint.List.Application.AdverseReportableEvent] = adverseReportableEventAnswers.AdverseReportableEvent;
                  data[RAMS.Environment.SharePoint.List.Application.DeviationReportableEvent] = adverseReportableEventAnswers.DeviationReportableEvent;
               }

               sendREST("POST", spAppData.__metadata.uri, data, function() {
                  if (currentFormType == "Parent") {
                     sendREST("POST", restPath.concat("study/" + assignedID + "/stage/Review"), null, function() {
                        window.location.assign("../SitePages/irb%20applications.aspx");
                     });
                  } else {
                     sendREST("POST", restPath.concat("study/" + assignedID + "/stage/Review"), null, function() {
                        window.location.assign("../SitePages/irb%20applications.aspx");
                     });
                  }
               }, "Merge");
            });
         }
      });
   });
}

/* exported generateHipaa */
/**
 * opens a new tab and starts the hipaa or icf download.
 * 
 * @param {number} id the id of the section where hipaa = 15 and icf = 14
 */
function generateHipaa(id) {
   var sideButtons = $('.list-group-item.sectionLink');

   if (id == "15") {
      var hipaaSection = sideButtons.length - 1;
      window.open(restPath.concat("study/" + assignedID + "/documentGeneration/" + targetStage + "/" + hipaaSection));
   } else if (id == "14") {
      var icfSection = sideButtons.length;
      window.open(restPath.concat("study/" + assignedID + "/documentGeneration/" + targetStage + "/" + icfSection));
   }
}

/* exported reviewApp */
/**
 * puts together the content of the popup modal that allows users to see what files they've uploaded, delete uploaded
 * files, what questions they have left to answer, allows users to generate the hipaa and consent form, and submit the
 * application.
 */
function reviewApp() {
   var reviewCont;
   // var reviewsRequired;
   // var uploaded, fileTable, tableCont, fileHead, row, fileBody;

   // reviewsRequired = determineCommittees();

   // JSHINT setupCOIStatusAnswers is not defined - it is defined in COI.js
   // jshint -W117
   // 460601: Create view of studies to show COI disclosure status for personnel assigned to the study 4.7
   setupCOIStatusAnswers();

   // removing any trace of a previous popup modal
   $('#review').remove();

   // putting together the basic structure and style of the popup
   reviewCont = document.createElement("div");
   reviewCont.setAttribute("id", "review");
   reviewCont.setAttribute("class", "well");
   reviewCont.setAttribute("style", "background: #fff");

   // getting all of the files
   // uploaded = sendREST("GET", restPath.concat("fascade/file/querystudy/" + assignedID), null, false);

   // var text = document.createTextNode("Below are all the files that you have uploaded:");
   // reviewCont.appendChild(text);

   // tableCont = document.createElement("div");
   // tableCont.setAttribute("class", "col-xs-12");

   // fileTable = document.createElement("table");
   // fileTable.setAttribute("class", "table table-bordered table-striped table-hover");

   // fileHead = document.createElement("thead");

   // row = document.createElement("tr");
   // row.appendChild(createCell("th", "File", null));
   // row.appendChild(createCell("th", "Remove", null));

   // fileHead.appendChild(row);
   // fileTable.appendChild(fileHead);

   // fileBody = document.createElement("tbody");

   // //putting together the table of files
   // if((uploaded.files === undefined)||(uploaded.files.length == 0)){
   // row = document.createElement("tr");
   // row.appendChild((createCell("td", "No Files Uploaded", null)));
   // fileBody.appendChild(row);
   // fileTable.appendChild(fileBody);
   // }else{
   // for(var i=0; i<uploaded.files.length; i++){
   // row = document.createElement("tr");
   // row.setAttribute("name", uploaded.files[i].file);
   // row.appendChild(createCell("td", uploaded.files[i].file, null));
   // row.appendChild(createCell("td", "delete", uploaded.files[i].file, uploaded.files[i].details[0].tags));
   // fileBody.appendChild(row);
   // }
   // fileTable.appendChild(fileBody);
   // }
   // tableCont.appendChild(fileTable);
   // reviewCont.appendChild(tableCont);

   // var text = document.createTextNode("Review your application below:");
   // reviewCont.appendChild(text);

   // validate questions inserts the rest of the reviewing content
   reviewCont.appendChild(validateQuestions());

   var breaker = document.createElement("br");
   reviewCont.appendChild(breaker);

   // button to close the window
   var closer = document.createElement("button");
   closer.setAttribute("type", "button");
   closer.setAttribute("id", "closeWindow"); // add id to close window button
   closer.setAttribute("onclick", "$('#review').popup('toggle')");
   // closer.setAttribute("onkeypress", "Function($('#review').popup('toggle'))");

   var text = document.createTextNode("Close Window");
   closer.appendChild(text);

   reviewCont.appendChild(closer);

   // specific library code to show popup
   $('body').append(reviewCont);
   $('#review').popup('show');
}

// NOT USED
// // creates a cell in a table. type is what kind of cell to create,
// // content is what goes in the cell, tag is used to pass file
// // information (since the only table is used with files)
// function createCell(type, content, tag, id) {
//
// var cell = document.createElement(type);
//
// var text = document.createTextNode(content);
//
// if (content == "delete") {
// // styling delete text
// var anchor = document.createElement("a");
// anchor.style.color = "blue";
// anchor.style.textDecoration = "underline";
// anchor.style.textDecorationColor = "blue";
// anchor.style.cursor = "pointer";
//
// anchor.appendChild(text);
// cell.appendChild(anchor);
//
// // tag here is used to link the remove button to a specific file
// cell.setAttribute("onclick", "removeFile('" + tag + "', " + id + ")");
// // cell.setAttribute("onkeypress", "removeFile('"+tag+"', "+id+")");
// } else if (content == "No Files Uploaded") {
// cell.appendChild(text);
// cell.setAttribute("colspan", "2");
// } else {
// cell.appendChild(text);
// }
//
// return cell;
// }

// rest call for removing a file from the cache and from
// the table
// function removeFile(file, id){
// var status;
// var qResponse = {};
// var data = [];

// status = sendREST("DELETE", restPath.concat("fascade/file/delete/study/"+assignedID+"/filename/"+file), null, false);
// $('[name="'+file+'"]').remove();

// response = {
// "questionsID": id
// };

// data.push(response);

// sendREST("POST", restPath.concat("study/"+assignedID+"/responses/"+targetStage), data, false);

// };

// JSHINT targetSection is defined but never used.
// jshint -W098
// assigning the appropriate icons to the buttons in the sidebar
function updateSidebarIcons(targetSection) {
   /* jshint unused: false */
   // grabbing all of the section buttons
   var sideButtons;
   var incomplete = 0;
   var complete = 0;
   var empty = 0;
   var total;

   if (assignedID == "1") {
      sideButtons = $('.list-group-item');
      // removing previous icons
      $('.glyphicon-adjust').remove();
      $('.glyphicon-record').remove();
      $('.glyphicon-ok-circle').remove();
      $('.glyphicon-ban-circle').remove();

      for (var i = 0; i < sideButtons.length; i++) {
         var icon = document.createElement("span");
         if (i === 0) {
            icon.setAttribute("class", "glyphicon glyphicon-record");
         } else {
            icon.setAttribute("class", "glyphicon glyphicon-ban-circle");
            sideButtons[i].setAttribute("style", "cursor: not-allowed");
         }

         sideButtons[i].appendChild(icon);
         sideButtons[i].setAttribute("onclick",
            "insertWarning('You must complete Section A and Save your responses before "
               + "proceeding.', 'alert-warning')");
         // sideButtons[i].setAttribute("onkeypress", "insertWarning('You must complete Section A and Save your
         // responses before proceeding.', 'alert-warning')");
      }
   } else {
      validSection = [];
      sideButtons = $('.list-group-item.sectionLink');
      sendREST("GET", restPath.concat("study/" + assignedID + "/responses/" + targetStage + "/validation"), null,
         function(validationReport) {
            // removing previous icons
            $('.glyphicon-adjust').remove();
            $('.glyphicon-record').remove();
            $('.glyphicon-ok-circle').remove();
            $('.glyphicon-ban-circle').remove();

            // going through the most recent user's responses and assigning them the correct icons as
            // well as giving the relevant buttons a link.
            for (var i = 0; i < sideButtons.length; i++) {
               var icon = document.createElement("span");
               if (validationReport.formStatusCollection[i].status == "INCOMPLETE") {
                  icon.setAttribute("class", "glyphicon glyphicon-adjust");
                  icon.setAttribute("alt", " has missing responses");
                  incomplete += 1;
                  sideButtons[i].setAttribute("onclick", "checkSave(" + i + ")");
                  // sideButtons[i].setAttribute("onkeypress", "checkSave("+i+")");
                  sideButtons[i].setAttribute("style", "cursor: pointer");
               } else if (validationReport.formStatusCollection[i].status == "NOT_APPLICABLE") {
                  // when a section isn't required a user shouldn't be able to click on it
                  icon.setAttribute("class", "glyphicon glyphicon-ban-circle");
                  icon.setAttribute("alt", " is not applicable");
                  sideButtons[i].removeAttribute("onclick");
                  // sideButtons[i].removeAttribute("onkeypress");
                  sideButtons[i].setAttribute("style", "cursor: not-allowed");
               } else if (validationReport.formStatusCollection[i].status == "EMPTY") {
                  icon.setAttribute("class", "glyphicon glyphicon-record");
                  icon.setAttribute("alt", " has not been started");
                  sideButtons[i].setAttribute("onclick", "checkSave(" + i + ")");
                  // sideButtons[i].setAttribute("onkeypress", "checkSave("+i+")");
                  sideButtons[i].setAttribute("style", "cursor: pointer");
                  empty += 1;
               } else if (validationReport.formStatusCollection[i].status == "COMPLETE") {
                  icon.setAttribute("class", "glyphicon glyphicon-ok-circle");
                  icon.setAttribute("alt", " has been completed");
                  sideButtons[i].setAttribute("onclick", "checkSave(" + i + ")");
                  // sideButtons[i].setAttribute("onkeypress", "checkSave("+i+")");
                  sideButtons[i].setAttribute("style", "cursor: pointer");
                  complete += 1;
                  validSection.push(i);
               }
               sideButtons[i].appendChild(icon);
               sideButtons[i].setAttribute("rams-form-status", validationReport.formStatusCollection[i].status);
            }

            // updates the general information section that gives the user
            // a quick glance of how much they have left to do
            total = incomplete + empty + complete;
            $('.sectionLocation').html("Completed: " + complete + "/" + total);
         });
   }
}

function generateAssignedId(eventType, facilityID, liaisonID) {
   // grabs a new generated study id

   // if the user object has not yet been initialized, initialize it because it will be needed later.
   if (!(users) || (users.length === 0)) {
      getRAMSUsers();
   }

   if ((typeof eventType !== "undefined") && (eventType !== null)) {
      parentID = assignedID;
      parentPIId = spAppData.PrincipalInvestigatorId;
      parentCoPIId = spAppData.CoPIId;
   }

   sendREST("GET", restPath.concat("fascade/studyidgenerator/sitecode/1111/humorani/H"), null, function(json) {
      assignedID = json.studyAssignedID;

      // //console.log("in generate assigned id, sending: " + eventType + ", " + facilitiesID);
      createStudy(assignedID, eventType, facilityID, liaisonID);
   });

}

// NOT USED
// function grabApplicationType() {
// var applicationType = null;
// var exemptID, expeditedID, fullID;
//
// if (currentFormType == "Parent") {
// exemptID =
// study.containedFormCollection[0].containedForm.formQuestionCollection[5].question.answerCollection[0].answersID;
// expeditedID =
// study.containedFormCollection[0].containedForm.formQuestionCollection[5].question.answerCollection[1].answersID;
// fullID =
// study.containedFormCollection[0].containedForm.formQuestionCollection[5].question.answerCollection[2].answersID;
//
// if (rollupAnswers.contains(exemptID)) {
// applicationType = "Exempt";
// } else if (rollupAnswers.contains(expeditedID)) {
// applicationType = "Expedited";
// } else if (rollupAnswers.contains(fullID)) {
// applicationType = "Full";
// }
// } else {
// applicationType = spAppData.ApplicationType;
// }
//
// return applicationType;
// }

function grabApplicationSiteType() {
   var applicationSiteType = null;
   var singleSiteID, multisiteID, centralID;

   if (currentFormType == "Parent") {
      singleSiteID = study.containedFormCollection[1].containedForm.formQuestionCollection[7].question.answerCollection[0].answersID;
      multisiteID = study.containedFormCollection[1].containedForm.formQuestionCollection[8].question.answerCollection[0].answersID;
      centralID = study.containedFormCollection[1].containedForm.formQuestionCollection[9].question.answerCollection[0].answersID;

      if (rollupAnswers.contains(singleSiteID)) {
         applicationSiteType = "Local";
      } else if (rollupAnswers.contains(multisiteID)) {
         applicationSiteType = "Multi";
      } else if (rollupAnswers.contains(centralID)) {
         applicationSiteType = "Central";
      }

   } else {
      applicationSiteType = spAppData.ApplicationSiteType;
   }

   return applicationSiteType;
}

function grabPI() {
   var pid;
   var currentPI = null;

   if (currentFormType == "Parent") {
      pid = study.containedFormCollection[0].containedForm.formQuestionCollection[1].question.questionsID;
   } else if (currentFormType == "Local") {
      pid = study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID;
   }

   // JSHINT NOTE: pid may not be initialized, but the if (pid) check will check if the pid is truthy and allow the
   // assignment of the PI
   if (pid) {
      var piVal = $('[name="QID_' + pid + '"]').val();
      if ((typeof piVal !== "undefined") && (piVal !== null)) {
         currentPI = $('[name="QID_' + pid + '"]').val();
      } else {
         for (var i = 0; i < rollup.responses.length; i++) {
            if (rollup.responses && (rollup.responses[i].questionsID == pid)) {
               currentPI = rollup.responses[i].response;
            }
         }
      }
   }
   // Return the Id (aka SPUserId) of the PI
   return currentPI;
}

function grabCoPI() {
   var copid;
   var currentCoPI = null;

   if (currentFormType == "Parent") {
      copid = study.containedFormCollection[0].containedForm.formQuestionCollection[3].question.questionsID;
   } else if (currentFormType == "Local") {
      copid = study.containedFormCollection[0].containedForm.formQuestionCollection[8].question.questionsID;
   }

   // JSHINT NOTE: copid may not be initialized, but the if (copid) check will check if the copid is truthy and allow
   // the
   // assignment of the copid
   if (copid) {
      var copidVal = $('[name="QID_' + copid + '"]').val();
      if ((typeof copidVal !== "undefined") && (copidVal !== null)) {
         currentCoPI = copidVal;
      } else {
         for (var i = 0; i < rollup.responses.length; i++) {
            if (rollup.responses && (rollup.responses[i].questionsID == copid)) {
               currentCoPI = rollup.responses[i].response;
            }
         }
      }
   }
   // if (copid && copid != "none") {
   // if (rollup.responses === undefined) {
   // currentCoPI = $('[name="QID_' + copid + '"]').val();
   // } else {
   // for (var i = 0; i < rollup.responses.length; i++) {
   // if (rollup.responses[i].questionsID == copid) {
   // currentCoPI = rollup.responses[i].response;
   // }
   // }
   // }
   // }
   // once we have the login
   // get
   // _api/SP.UserProfiles.PeopleManager/GetPropertiesFor(accountName=@v)?@v='vhamaster\vhaiswjogiak'
   // Return the Id (aka SPUserId) of the CoPI

   return currentCoPI;
}
function grabPriorCoPI() {
   var copid;
   var priorCoPI = null;

   if (currentFormType == "Parent") {
      copid = study.containedFormCollection[0].containedForm.formQuestionCollection[3].question.questionsID;
   } else if (currentFormType == "Local") {
      copid = study.containedFormCollection[0].containedForm.formQuestionCollection[8].question.questionsID;
   }

   // JSHINT NOTE: copid may not be initialized, but the if (copid) check will check if the copid is truthy and allow
   // the assignment of the copid
   if (copid) {
      for (var i = 0; i < rollup.responses.length; i++) {
         if (rollup.responses && (rollup.responses[i].questionsID == copid)) {
            priorCoPI = rollup.responses[i].response;
         }
      }
   }

   return priorCoPI;
}

// NOT USED line ~5650 commented out
// function grabPriorPI() {
// var pid;
// var priorPI = null;
//
// if (currentFormType == "Parent") {
// pid = study.containedFormCollection[0].containedForm.formQuestionCollection[1].question.questionsID;
// } else if (currentFormType == "Local") {
// pid = study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID;
// }
//
// // JSHINT NOTE: copid may not be initialized, but the if (copid) check will check if the copid is truthy and allow
// // the assignment of the copid
// if (pid) {
// for (var i = 0; i < rollup.responses.length; i++) {
// if (rollup.responses && (rollup.responses[i].questionsID == pid)) {
// priorPI = rollup.responses[i].response;
// }
// }
// }
//
// return priorPI;
// }

// NOT USED since 2.2
// function sharepointIDtoSqlID(spUserID) {
// var userID = null;
//
// for (var j = 0; j < sqlUsers.length; j++) {
// // if(spUserID == userIDMap.get(sqlUsers[j].SPUser)){
// if (typeof (sqlUsers[j].SPUser !== "undefined") && (spUserID == userIDMap.get(sqlUsers[j].SPUser))) {
// // console.log("we've found the user: " + sqlUsers[j].displayName + " with ID " + sqlUsers[j].usersID);
// userID = sqlUsers[j].usersID;
// }
// }
//
// return userID;
// }
// todo do we need sharepointIDtoLoginName
function sharepointIDtoTitle(spUserID) {
   var tempUsers = ramsUsers.values();
   // todo lookup by userid
   for (var i = 0; i < tempUsers.length; i++) {
      if (tempUsers[i].Id == spUserID) {
         // //console.log("found User: " + tempUsers[i].Title);
         return tempUsers[i].Title;
      }
   }

   return null;
}

// NOT USED since 2.2
// function titleToSharepointID(spUserTitle) {
// var tempUsers = ramsUsers.values();
//
// for (var i = 0; i < tempUsers.length; i++) {
// if (tempUsers[i].Title == spUserTitle) {
// return tempUsers[i].Id;
// }
// }
//
// return null;
// }

function createStudy(studyAssignedID, eventType, facilityID, liaisonID) {
   var titleID, packaging;

   // var userTable = new Hashtable();

   // var facilityName;
   var facility;
   var studyMembersCollection = [ {
      "user" : sqlUsers,
      "coiRequired" : false,
      "writePermissions" : true,
      "memberRole" : "",
      "explaination" : "Initiated this study application.",
      "notifications" : true,
      "teamMember" : true,
      "activeMember" : true,
      "createdAt" : new Date().toJSON()
   }
   ];
   // var studyMembersCollectionEntry = {
   // "user" : sqlUsers,
   // "coiRequired" : false,
   // "writePermissions" : true,
   // "memberRole" : "",
   // "explaination" : "",
   // "notifications" : true,
   // "teamMember" : true,
   // "activeMember" : true,
   // "createdAt" : new Date().toJSON()
   // };

   function createStudyItself() {
      var currentFacility;
      var facilityName;
      // we know we are only creating the study when we are on the first page, grabbing the study title and facility
      var pid = grabPI();
      var copid = grabCoPI();

      // Compare the Current User's Id, not their name to the PI and CoPI
      if (pid == currentUser.Id) {
         studyMembersCollection[0].memberRole = PRINCIPALINVESTIGATOR;
         studyMembersCollection[0].user = users[currentUser.Id];
      } else if ((copid) && (copid != "none") && (copid == currentUser.Id)) {
         studyMembersCollection[0].memberRole = COPRINCIPALINVESTIGATOR;
         studyMembersCollection[0].user = users[currentUser.Id];
      } else {
         studyMembersCollection[0].memberRole = STUDYMEMBER;
         studyMembersCollection[0].user = users[currentUser.Id];
      }

      if (pid != currentUser.Id) {
         var smcPI = {
            "user" : sqlUsers,
            "coiRequired" : false,
            "writePermissions" : true,
            "memberRole" : "",
            "explaination" : "",
            "notifications" : true,
            "teamMember" : true,
            "activeMember" : true,
            "createdAt" : new Date().toJSON()
         };
         smcPI.memberRole = PRINCIPALINVESTIGATOR;
         smcPI.user = users[pid];
         studyMembersCollection.push(smcPI);
      }
      if ((copid) && (copid != "none") && (copid != currentUser.Id)) {
         var smcCoPI = {
            "user" : sqlUsers,
            "coiRequired" : false,
            "writePermissions" : true,
            "memberRole" : "",
            "explaination" : "",
            "notifications" : true,
            "teamMember" : true,
            "activeMember" : true,
            "createdAt" : new Date().toJSON()
         };
         smcCoPI.memberRole = COPRINCIPALINVESTIGATOR;
         smcCoPI.user = users[copid];
         studyMembersCollection.push(smcCoPI);
      }

      if (studyTitle === "") {
         titleID = study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID;
         studyTitle = $("[name = 'QID_" + titleID + "']").val();

         var facilityQID = study.containedFormCollection[0].containedForm.formQuestionCollection[6].question.questionsID;
         facilityName = $("[name = 'QID_" + facilityQID + "']").val();

         if ((typeof facilities !== "undefined") && (facilityName !== "")) {
            for (var j = 0; j < facilities.d.results.length; j++) {
               if (facilityName == facilities.d.results[j].Title) {
                  // facility = currentFacility;
                  currentFacility = facilities.d.results[j];
                  break;
               }
            }

            if (!(currentFacility)) {
               logError("createStudyItself", "404",
                  "Unable to determine which facility the study belongs to. Looking for " + facilityName
                     + " in Facility List", "N/A");
               alert("Unable to determine which facility the study belongs to.");
               // reset assignedId to 1 so a new one will be generated when all the data is present.
               assignedID = 1;
               return;
            }
         } else {
            logError("createStudyItself", "404",
               "Unable to determine which facility the study belongs to. Looking for " + facilityName
                  + " in Facility List", "N/A");
            alert("Unable to determine which facility the study belongs to.");
            // reset assignedId to 1 so a new one will be generated when all the data is present.
            assignedID = 1;
            return;
         }
      }

      // TODO should facility be initialized at this point?
      packaging = {
         "assignedID" : studyAssignedID,
         "facility" : facility,
         "title" : studyTitle,
         "studyMembersCollection" : studyMembersCollection
      };

      // TODO TAR should currentFacility be initialized at this point? It may not have a value if Study Title is set.
      // SUSTAINMENT 2.1.1 added SiteDomain to the update.
      data = {
         "__metadata" : {
            "type" : "SP.Data.ApplicationListItem"
         },
         "Study_ID" : studyAssignedID,
         "Title" : studyTitle,
         "StudyTitle" : studyTitle,
         "ApplicationStatus" : RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFTCHOICE,
         "SiteDomain" : siteUrl,
         "FacilitiesLookupId" : currentFacility.Id
      };

      data[RAMS.Environment.SharePoint.List.Application.PrincipalInvestigatorId] = pid;
      data[RAMS.Environment.SharePoint.List.Application.CoPrincipalInvestigatorId] = (copid == "none" ? null : copid);

      // putting the application in SP
      sendREST("POST", siteUrl.concat("_api/web/lists/getbytitle('Application')/items"), data, function() {
         // sendREST("POST", siteUrl.concat(subCommittee.concat("_api/web/lists/getbytitle('Committee
         // Topics')/items")), data, function(){
         // putting the newly created study in the database
         sendREST("POST", restPath.concat("study/formVersionsID/" + currentFormVersion), packaging, function(
            studyResponse) {
            targetStage = studyResponse.studyStatusCollection.length;
            targetState = studyResponse.studyStatusCollection[targetStage - 1].studyState.studyStatesID;
            // // add PI to study member list
            // insertStudyMember(pid, PRINCIPALINVESTIGATOR, true, assignedID, null);
            // if ((typeof copid !== "undefined") && (copid !== null) && (copid != "none")) {
            // // add CoPI to study member
            // insertStudyMember(copid, COPRINCIPALINVESTIGATOR, true, assignedID, null);
            // }
            // list
            var headerArea = $('div.page-header');
            headerArea.empty();
            headerBuilder(0, headerArea);
            sendResponses();
         });
         // });
      });

   }

   function createChildStudy(csEventType, csFacilityID, csLiaisonID) {
      var appendingString, childFormVersion, appType, appLink;
      var ccsCurrentFacility;
      // NOT USED var subsiteUrl;
      var mainData = null;
      var shellStudy;
      var iacucReviewRequired = false;
      var irbReviewRequired = false;
      var rdcReviewRequired = false;
      var srsReviewRequired = false;
      var sc1ReviewRequired = false;
      var sc2ReviewRequired = false;
      var sc3ReviewRequired = false;

      /*
       * The code below assumes a relationship between the currentFormVersion and the childFormVersion in that they are
       * a set distance apart. Local is 1 away from the Parent, Averse is 2 away, etc. Parent: 53, 53: "Parent", Local:
       * 54, 54: "Local", Adverse: 55, 55: "Adverse", ContReview: 56, 56: "ContReview", Closeout: 57, 57: "Closeout",
       * AdministrativeUpdate: 58, 58: "AdministrativeUpdate"
       */
      if (csEventType == "Local") {
         appendingString = " - Local Site Application";
         // childFormVersion = currentFormVersion + 1;
         childFormVersion = RAMS.Environment.WebService.Form[csEventType];
         appType = spAppData.ApplicationSiteType;
         ccsCurrentFacility = csFacilityID;
      } else if (csEventType == "Adverse") {
         appendingString = " - Adverse/Reportable Event";
         // childFormVersion = currentFormVersion + 2;
         childFormVersion = RAMS.Environment.WebService.Form[csEventType];
         appType = "Adverse/Reportable Event";
         ccsCurrentFacility = spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId];
      } else if (csEventType == "ContReview") {
         appendingString = " - Continuing Review";
         // childFormVersion = currentFormVersion + 3;
         childFormVersion = RAMS.Environment.WebService.Form[csEventType];
         appType = "Renewal";
         ccsCurrentFacility = spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId];
      } else if (csEventType == "Closeout") {
         appendingString = " - Closeout";
         // childFormVersion = currentFormVersion + 4;
         childFormVersion = RAMS.Environment.WebService.Form[csEventType];
         appType = "Closeout";
         ccsCurrentFacility = spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId];
      } else if (csEventType == "AdministrativeUpdate") {
         appendingString = " - Administrative Update";
         // childFormVersion = currentFormVersion + 5;
         childFormVersion = RAMS.Environment.WebService.Form[csEventType];
         appType = "Administrative Update";
         ccsCurrentFacility = spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId];
      } else if (csEventType == "Amendment") {
         // Amendment does not have its own form, it makes the original form editable again
         appendingString = " - Amendment";
         childFormVersion = currentFormVersion;
         appType = "Amendment";
         ccsCurrentFacility = spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId];
      } else if (csEventType == "TriennialReview") {
         appendingString = " - Triennial Review";
         childFormVersion = currentFormVersion;
         appType = "Triennial Review";
         ccsCurrentFacility = spAppData[RAMS.Environment.SharePoint.List.Application.FacilityLookupId];
      }

      // TODO this will overwrite the ccsCurrentFacility for all form types even though they were just set - Is this
      // right?
      // grab assignedIRB Admin and add them to packaging
      // if anything but local, add the current PI as well.
      if ((typeof csFacilityID !== "undefined") && (csFacilityID !== null)) {
         ccsCurrentFacility = csFacilityID;
      } else {
         ccsCurrentFacility = spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId];
      }

      // Retrieve the child application list item by the parentID and the participatingSite facility. This
      // is the shellStudy created when the localSiteApplication was added to the parent study.
      sendREST(
         "GET",
         siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=ParentStudyID eq '" + parentID
            + "' and ParticipatingFacilitiesLookup eq " + ccsCurrentFacility,
         null,
         function(json) {
            shellStudy = json.d.results;

            determineFormType(childFormVersion);
            sendREST(
               "GET",
               restPath + "forms/" + childFormVersion,
               null,
               function(formVersion) {
                  study = formVersion;

                  // Compare the Current User's Id, not their name to the PI and CoPI
                  if ((csEventType == "Local") && (shellStudy[0].PrincipalInvestigatorId == currentUser.Id)) {
                     studyMembersCollection[0].memberRole = PRINCIPALINVESTIGATOR;
                     // At this point we don't know who the Co-PI will be so no need to check it
                     // } else if (grabCoPI() == currentUser.Id) {
                     // studyMembersCollection[0].memberRole = COPRINCIPALINVESTIGATOR;
                  } else {
                     studyMembersCollection[0].memberRole = PRINCIPALINVESTIGATOR;
                  }

                  appLink = {
                     "__metadata" : {
                        "type" : "SP.FieldUrlValue"
                     },
                     "Description" : studyTitle + ": " + studyAssignedID,
                     "Url" : "/sites/IRBApplications/SiteAssets/irb-applications-general.html?" + studyAssignedID
                  };

                  // TODO should facility be initialized at this point? set to csFacilityId from facility
                  // JSHINT NOTE there is a possibility that appendingString may not be initialized at this point.
                  packaging = {
                     "assignedID" : studyAssignedID,
                     "facility" : facility,
                     "title" : studyTitle + appendingString,
                     "studyMembersCollection" : studyMembersCollection
                  };

                  // Static values
                  // JSHINT NOTE there is a possibility that appendingString may not be initialized at this point.
                  if ((typeof csLiaisonID !== "undefined") && (csLiaisonID !== null)) {
                     mainData = {
                        "__metadata" : {
                           "type" : "SP.Data.ApplicationListItem"
                        },
                        "Study_ID" : studyAssignedID,
                        "Title" : studyTitle + appendingString,
                        "StudyTitle" : studyTitle + appendingString,
                        "ParentStudyID" : parentID,
                        "AssignedIRBAdministratorId" : spAppData.AssignedIRBAdministratorId,
                        "CIRB" : spAppData.CIRB,
                        "ApplicationType" : appType,
                        "ApplicationSiteType" : spAppData.ApplicationSiteType,
                        "ApplicationStatus" : RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFTCHOICE,
                        "ParticipatingSiteApplicationStat" : RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFTCHOICE,
                        "ParticipatingFacilitiesLookupId" : ccsCurrentFacility,
                        "ReviewRequired" : spAppData.ReviewRequired,
                        "ReviewersId" : {
                           "__metadata" : {
                              "type" : "Collection(Edm.Int32)"
                           },
                           "results" : [ csLiaisonID
                           ]
                        }
                     };
                     if (spAppData.IRBReviewRequired === true) {
                        irbReviewRequired = true;
                     }
                     if (spAppData.RDCReviewRequired === true) {
                        rdcReviewRequired = true;
                     }
                     if (spAppData.IACUCReviewRequired === true) {
                        iacucReviewRequired = true;
                     }
                     if (spAppData.SRSReviewRequired === true) {
                        srsReviewRequired = true;
                     }
                     if (spAppData.SC1ReviewRequired === true) {
                        sc1ReviewRequired = true;
                     }
                     if (spAppData.SC2ReviewRequired === true) {
                        sc2ReviewRequired = true;
                     }
                     if (spAppData.SC3ReviewRequired === true) {
                        sc3ReviewRequired = true;
                     }

                     if (irbReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.IRBReviewRequired] = true;
                        mainData.IRBReviewersId = {
                           "__metadata" : {
                              "type" : "Collection(Edm.Int32)"
                           },
                           "results" : [ csLiaisonID
                           ]
                        };
                        mainData.IRBStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.IRBAdministratorId !== "undefined") && (spAppData.IRBAdministratorId)) {
                           mainData.IRBAdministratorId = spAppData.IRBAdministratorId;
                        }
                     }
                     if (rdcReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.RDCReviewRequired] = true;
                        mainData.RDCReviewersId = {
                           "__metadata" : {
                              "type" : "Collection(Edm.Int32)"
                           },
                           "results" : [ csLiaisonID
                           ]
                        };
                        mainData.RDCStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.RDCAdministratorId !== "undefined") && (spAppData.RDCAdministratorId)) {
                           mainData.RDCAdministratorId = spAppData.RDCAdministratorId;
                        }
                     }
                     if (iacucReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.IACUCReviewRequired] = true;
                        mainData.IACUCReviewersId = {
                           "__metadata" : {
                              "type" : "Collection(Edm.Int32)"
                           },
                           "results" : [ csLiaisonID
                           ]
                        };
                        mainData.IACUCStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.IACUCAdministratorId !== "undefined") && (spAppData.IACUCAdministratorId)) {
                           mainData.IACUCAdministratorId = spAppData.IACUCAdministratorId;
                        }
                     }
                     if (srsReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.SRSReviewRequired] = true;
                        mainData.SRSReviewersId = {
                           "__metadata" : {
                              "type" : "Collection(Edm.Int32)"
                           },
                           "results" : [ csLiaisonID
                           ]
                        };
                        mainData.SRSStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.SRSAdministratorId !== "undefined") && (spAppData.SRSAdministratorId)) {
                           mainData.SRSAdministratorId = spAppData.SRSAdministratorId;
                        }
                     }
                     if (sc1ReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.SC1ReviewRequired] = true;
                        mainData.SC1ReviewersId = {
                           "__metadata" : {
                              "type" : "Collection(Edm.Int32)"
                           },
                           "results" : [ csLiaisonID
                           ]
                        };
                        mainData.SC1StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.SC1AdministratorId !== "undefined") && (spAppData.SC1AdministratorId)) {
                           mainData.SC1AdministratorId = spAppData.SC1AdministratorId;
                        }
                     }
                     if (sc2ReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.SC2ReviewRequired] = true;
                        mainData.SC2ReviewersId = {
                           "__metadata" : {
                              "type" : "Collection(Edm.Int32)"
                           },
                           "results" : [ csLiaisonID
                           ]
                        };
                        mainData.SC2StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.SC2AdministratorId !== "undefined") && (spAppData.SC2AdministratorId)) {
                           mainData.SC2AdministratorId = spAppData.SC2AdministratorId;
                        }
                     }
                     if (sc3ReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.SC3ReviewRequired] = true;
                        mainData.SC3ReviewersId = {
                           "__metadata" : {
                              "type" : "Collection(Edm.Int32)"
                           },
                           "results" : [ csLiaisonID
                           ]
                        };
                        mainData.SC3StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.SC3AdministratorId !== "undefined") && (spAppData.SC3AdministratorId)) {
                           mainData.SC3AdministratorId = spAppData.SC3AdministratorId;
                        }
                     }
                  } else {
                     mainData = {
                        "__metadata" : {
                           "type" : "SP.Data.ApplicationListItem"
                        },
                        "Study_ID" : studyAssignedID,
                        "Title" : studyTitle + appendingString,
                        "StudyTitle" : studyTitle + appendingString,
                        "ParentStudyID" : parentID,
                        "AssignedIRBAdministratorId" : spAppData.AssignedIRBAdministratorId,
                        "CIRB" : spAppData.CIRB,
                        "ApplicationType" : appType,
                        "ApplicationSiteType" : spAppData.ApplicationSiteType,
                        "ReviewRequired" : spAppData.ReviewRequired,
                        "ApplicationStatus" : RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFTCHOICE,
                        "ParticipatingSiteApplicationStat" : RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFTCHOICE,
                        "ParticipatingFacilitiesLookupId" : ccsCurrentFacility
                     };
                     if (spAppData.IRBReviewRequired === true) {
                        irbReviewRequired = true;
                     }
                     if (spAppData.RDCReviewRequired === true) {
                        rdcReviewRequired = true;
                     }
                     if (spAppData.IACUCReviewRequired === true) {
                        iacucReviewRequired = true;
                     }
                     if (spAppData.SRSReviewRequired === true) {
                        srsReviewRequired = true;
                     }
                     if (spAppData.SC1ReviewRequired === true) {
                        sc1ReviewRequired = true;
                     }
                     if (spAppData.SC2ReviewRequired === true) {
                        sc2ReviewRequired = true;
                     }
                     if (spAppData.SC3ReviewRequired === true) {
                        sc3ReviewRequired = true;
                     }

                     if (irbReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.IRBReviewRequired] = true;
                        mainData.IRBStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.IRBAdministratorId !== "undefined") && (spAppData.IRBAdministratorId)) {
                           mainData.IRBAdministratorId = spAppData.IRBAdministratorId;
                        }
                     }
                     if (rdcReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.RDCReviewRequired] = true;
                        mainData.RDCStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.RDCAdministratorId !== "undefined") && (spAppData.RDCAdministratorId)) {
                           mainData.RDCAdministratorId = spAppData.RDCAdministratorId;
                        }
                     }
                     if (iacucReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.IACUCReviewRequired] = true;
                        mainData.IACUCStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.IACUCAdministratorId !== "undefined") && (spAppData.IACUCAdministratorId)) {
                           mainData.IACUCAdministratorId = spAppData.IACUCAdministratorId;
                        }
                     }
                     if (srsReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.SRSReviewRequired] = true;
                        mainData.SRSStatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.SRSAdministratorId !== "undefined") && (spAppData.SRSAdministratorId)) {
                           mainData.SRSAdministratorId = spAppData.SRSAdministratorId;
                        }
                     }
                     if (sc1ReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.SC1ReviewRequired] = true;
                        mainData.SC1StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.SC1AdministratorId !== "undefined") && (spAppData.SC1AdministratorId)) {
                           mainData.SC1AdministratorId = spAppData.SC1AdministratorId;
                        }
                     }
                     if (sc2ReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.SC2ReviewRequired] = true;
                        mainData.SC2StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.SC2AdministratorId !== "undefined") && (spAppData.SC2AdministratorId)) {
                           mainData.SC2AdministratorId = spAppData.SC2AdministratorId;
                        }
                     }
                     if (sc3ReviewRequired) {
                        mainData[RAMS.Environment.SharePoint.List.Application.SC3ReviewRequired] = true;
                        mainData.SC3StatusId = RAMS.Environment.SharePoint.List.CommitteeStatus.DRAFT;
                        if ((typeof spAppData.SC3AdministratorId !== "undefined") && (spAppData.SC3AdministratorId)) {
                           mainData.SC3AdministratorId = spAppData.SC3AdministratorId;
                        }
                     }
                  }

                  // Dynamic values (read in from config file)
                  mainData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId] = ccsCurrentFacility;
                  mainData[RAMS.Environment.SharePoint.List.Application.LinkToApplication] = appLink;

                  if (csEventType == "Local") {
                     mainData[RAMS.Environment.SharePoint.List.Application.PrincipalInvestigatorId] = shellStudy[0].PrincipalInvestigatorId;
                  } else {
                     // JSHINT smcPlusIncumbents is not defined - it is defined in RAMS_Tools
                     // jshint -W117
                     mainData[RAMS.Environment.SharePoint.List.Application.PrincipalInvestigatorId] = smcPlusIncumbents.currentPI.user.SPUser;
                  }

                  // console.log("event type " + eventType + ", " + childFormVersion);
                  // sendREST("GET", siteUrl +
                  // "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=ParentStudyID eq '" + parentID +
                  // "'" , null,
                  // function(json) {
                  // var temp = json.d.results;
                  // for(var i=0; i<temp.length; i++){
                  // alert('temp index is '+ i);
                  // if (temp[i].ParticipatingFacilitiesLookupId == ccsCurrentFacility){
                  // alert (temp[i].ParticipatingFacilitiesLookupId + ' eq' + ccsCurrentFacility);
                  // sendREST("POST", temp[i].__metadata.uri, mainData, function(){
                  // // sendREST("GET",
                  // siteUrl.concat("_api/web/lists/getbytitle('Facilities')/items("+ccsCurrentFacility+")"),
                  // null,
                  // // function(returnedFacilityjson){
                  // // var tempFacility = returnedFacilityjson;
                  // // var facilityUrl = tempFacility.d.FacilitySubsite.Url;
                  // // sendREST("POST", facilityUrl.concat("/_api/web/lists/getbytitle('Committee
                  // Topics')/items"), mainData,
                  // function(){
                  // sendREST("POST", restPath.concat("study/formVersionsID/"+childFormVersion), packaging,
                  // function(studyResponse){
                  // sendREST("POST", restPath.concat("study/"+assignedID+"/parentstudy/"+parentID), null,
                  // function(json){
                  // //window.location.assign("irb-applications-general.html?") + assignedID;
                  // //refreshDisplay('Facilities');
                  // insertStudyMember(sharepointIDtoTitle(liaisonID), "RAMS Reviewer", true, assignedID, null);
                  // window.location.assign("../SitePages/irb%20applications.aspx");
                  // });
                  // });
                  // //});
                  // //});
                  // },"Merge");
                  // }
                  // };
                  // });

                  if (csEventType == "Local") {
                     // console.log("event type " + eventType + ", " + childFormVersion);
                     // This will GET the local site study belonging to the main study with Site ID value passed
                     // in from the
                     // Local Site Application window.
                     // This will UPDATE the local site study record with appropriate values
                     // This will CREATE a new row in Committee Topics List in facility subsite for that Site ID
                     // This will CREATE a new local site study in RAMS DB

                     // read the application list to retrieve all child studies with the same parentID
                     sendREST("GET", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items"
                        + "?$filter=ParentStudyID eq '" + parentID + "'", null, function(json) {
                        var temp = json.d.results;
                        // Loop through the collection until the study for the current facility is found
                        for (var i = 0; i < temp.length; i++) {
                           // alert('temp index is '+ i);
                           if (temp[i].ParticipatingFacilitiesLookupId == ccsCurrentFacility) {
                              // alert (temp[i].ParticipatingFacilitiesLookupId + ' eq' + ccsCurrentFacility);
                              // update the application list with the mainData collected above.
                              sendREST("POST", temp[i].__metadata.uri, mainData, function() {
                                 // sendREST("GET",
                                 // siteUrl.concat("_api/web/lists/getbytitle('Facilities')/items(" +
                                 // ccsCurrentFacility + ")"), null,
                                 // function(returnedFacilityjson) {
                                 // var tempFacility = returnedFacilityjson;
                                 // var facilityUrl = tempFacility.d.FacilitySubsite.Url;
                                 // sendREST("POST",
                                 // siteUrl.concat("/_api/web/lists/getbytitle('Application')/items"),
                                 // mainData, function() {
                                 // JSHINT NOTE there is a possibility that childFormVersion may
                                 // not be initialized at this point.

                                 // JSHINT studyResponse is defined but never used - it is returned by default from
                                 // sendREST
                                 // jshint -W098
                                 // add the study in the RAMS web services with the child formVersion
                                 sendREST("POST", restPath.concat("study/formVersionsID/" + childFormVersion),
                                    packaging, function(studyResponse) {

                                       // associate the parent study to the child
                                       // JSHINT json is defined but never used - it is returned by default from
                                       // sendREST
                                       // jshint -W098
                                       sendREST("POST", restPath.concat("study/" + studyAssignedID + "/parentstudy/"
                                          + parentID), null, function(json) {

                                          // var pid = grabPI();
                                          // var copid = grabCoPI();
                                          // Add the Local Site Liaison to the child study team collection
                                          insertStudyMember(csLiaisonID, LIAISONREVIEWER, false, assignedID, null);
                                          // add PI to study member
                                          insertStudyMember(shellStudy[0].PrincipalInvestigatorId,
                                             PRINCIPALINVESTIGATOR, true, assignedID, null);

                                          if (parentPIId) {
                                             insertStudyMember(parentPIId, PARENTSTUDYPI, true, assignedID, null);
                                          }
                                          if (parentCoPIId) {
                                             insertStudyMember(parentCoPIId, PARENTSTUDYCOPI, true, assignedID, null);
                                          }
                                          // if (copid) {
                                          // // add CoPI to study member
                                          // insertStudyMember(copid, COPRINCIPALINVESTIGATOR,
                                          // true, assignedID, null);
                                          // }
                                          window.location.assign("../SitePages/irb%20applications.aspx");
                                       });
                                    });
                                 // });
                                 // });
                              }, "Merge");
                           }
                        }
                     });
                  } else {// For the rest of the event type

                     sendREST("POST", siteUrl.concat("_api/web/lists/getbytitle('Application')/items"), mainData,
                        function() {
                           // sendREST("POST",
                           // siteUrl.concat("_api/web/lists/getbytitle('Application')/items"),
                           // mainData, function() {
                           // JSHINT NOTE there is a possibility that childFormVersion may not be
                           // initialized at this point.
                           // JSHINT studyResponse is defined but never used - it is returned by default from
                           // sendREST
                           // jshint -W098
                           sendREST("POST", restPath.concat("study/formVersionsID/" + childFormVersion), packaging,
                              function(studyResponse) {
                                 // JSHINT json is defined but never used - it is returned by default from
                                 // sendREST
                                 // jshint -W098
                                 sendREST("POST", restPath.concat("study/" + studyAssignedID + "/parentstudy/"
                                    + parentID), null, function(json) {
                                    // If amendment event, copy parent data to new event
                                    if (csEventType == "Amendment" || csEventType == "TriennialReview") {
                                       sendREST("GET", restPath.concat("study/" + parentID + "/responses/"
                                          + targetStage + "/rollup"), null, function(rJson) {
                                          var rData = rJson.responses;
                                          rData.forEach(function(v) {
                                             delete v.updatedAt;
                                          });
                                          sendREST("POST",
                                             restPath.concat("study/" + studyAssignedID + "/responses/1"), rData,
                                             function() {
                                                window.location.assign("../SitePages/irb%20applications.aspx");
                                             });
                                       });
                                    } else {
                                       window.location.assign("../SitePages/irb%20applications.aspx");
                                       // window.location.assign("irb-applications-general.html?") +
                                       // assignedID;
                                    }
                                 });
                              });
                        });
                     // });

                  } // END OF LOCAL v. Other Event Types
               });
         });
   } // end of createChildStudy

   sendREST("GET", siteUrl.concat("_api/web/lists/getbytitle('Facilities')/items"), null, function(returnedFacilities) {

      facilities = returnedFacilities;
      facility = {
         // "administratorSPGroup": 0,
         // "description": ccsCurrentFacility.Title,
         // "facilitiesID": ccsCurrentFacility.ID,
         // "name": ccsCurrentFacility.Facility_Name,
         // "reviewerSPCalendar": "00000000-0000-0000-0000-000000000000",
         // "reviewerSPGroup": 0,
         // "safetySPGroup": 0

         "administratorSPGroup" : 0,
         "description" : "initial facility",
         "facilitiesID" : 0,
         "name" : "General Site",
         "reviewerSPCalendar" : "00000000-0000-0000-0000-000000000000",
         "reviewerSPGroup" : 0,
         "safetySPGroup" : 0
      };

      sendREST("GET", restPath.concat("user"), null, function(returnedUsers) {
         console.log("getUsers: 2. after call REST user. # returned: " + returnedUsers.length);
         for (var i = 0; i < returnedUsers.length; i++) {
            // add each user the the users associative array
            users[returnedUsers[i].SPUser] = returnedUsers[i];
         }
         if ((typeof eventType !== "undefined") && (eventType !== null)) {
            console.log("in create study, sending " + eventType + ", " + facilityID);
            createChildStudy(eventType, facilityID, liaisonID);
         } else {
            console.log("in create study, calling create study itself " + facilityID);
            createStudyItself();
         }
      });
   });
} // end of CreateStudy

/**
 * Manages the insertion of the designated user into the studyMemberCollection.
 * 
 * @param userSPId the SharePoint UserId of the user to be added
 * @param role valid values are
 *           <ul>
 *           <li>Principal Investigator - only one allowed per studyMemberCollection
 *           <li>Co-Principal Investigator - only one allowed per studyMemberCollection
 *           <li>Study Administrator - only one allowed per studyMemberCollection
 *           <li>Liaison Reviewer
 *           <li>Primary Reviewer - only one allowed per studyMemberCollection
 *           <li>Assigned Reviewer
 *           <li>All the roles from the managePersonnel role drop down - includes "Investigator", "Co-Investigator",
 *           "Sub-Investigator", "Biostatistician", "Clinical Coordinator", "Collaborator", "CSP Program Manager", "Data
 *           Manager", "Lab Manager", "Lab Technician", "National Study Coordinator", "Nurse Assessor", "Pharmacist",
 *           "Pharmacy Technician", "Regulatory Coordinator", "Research Assistant", "Research Nurse", "Statistician",
 *           "Study Coordinator".
 *           </ul>
 * @param teamMember true or false to identify if the person in the role is considered part of the study team.
 * @param afterward Not Used
 */
// function insertStudyMember20171223(userSPId, role, teamMember, afterward) {
// const
// PRINCIPALINVESTIGATOR = "Principal Investigator";
// const
// COPRINCIPALINVESTIGATOR = "Co-Principal Investigator";
// const
// PRIMARYREVIEWER = "Primary Reviewer";
// const
// ADMINISTRATOR = "Administrator";
// var foundUser = false; // initializing this to false, but if the user is userSPId is found this will become the user
// object of the found user.
// var smRole = role;
// var smData;
// var roleChanged = false;
// var teamMemberChanged = false;
//    
// /**
// * removes study member from the studyMemberCollection
// *
// * @param user the user object to remove from the studyMemberCollection
// * @return uses jQuery deferred functionality to return success
// */
// var removeStudyMember = function removeStudyMember(user) {
// var deferred = $.Deferred();
// sendREST("POST", restPath.concat("study/" + assignedID + "/removemembers"), user, function() {
// deferred.resolve();
// return deferred.promise();
// });
// }
//
// /**
// * builds a studyMemberCollection
// *
// * @return Deferred.promise() Uses jQuery deferred functionality to return success or failure.
// * When successful the function provides an associative array of the
// * studyMemberCollection indexed by SPUserId. When failure the function returns a error
// * message to alert the user and log in the console.
// */
// function getStudyMembers(targetAssignedID, deferred) {
// //var deferred = gsmDfd;
// var smc = [];
// var smcPlusIncumbents = {};
// console.log("getStudyMembers: 1. at start of function. About to call REST study/{assignedId}/members.");
// //initialize the PlusIncumbents to undefined
// // smcPlusIncumbents.currentPI = undefined;
// // smcPlusIncumbents.currentCoPI = undefined;
// // smcPlusIncumbents.currentAdministrator = undefined;
// // smcPlusIncumbents.currentPrimaryReviewer = undefined;
//        
// sendREST("GET", restPath.concat("study/" + targetAssignedID + "/members"), null, function(returnedMembers) {
// console.log("getStudyMembers: 2. after call REST study/{assignedId}/members. # returned: "
// + returnedMembers.length);
// for (var i = 0; i < returnedMembers.length; i++) {
// // Add the studyTeamMember to the associative array
// // This assumes tha the studyMember will only have one role on the studyTeam or in the studyMemberCollection
// smc[returnedMembers[i].user.SPUser] = returnedMembers[i];
//                
// // find the roles that can have only one member in the role
// if (returnedMembers[i].memberRole === PRINCIPALINVESTIGATOR) {
// smcPlusIncumbents.currentPI = returnedMembers[i];
// } else if (returnedMembers[i].memberRole === COPRINCIPALINVESTIGATOR) {
// smcPlusIncumbents.currentCoPI = returnedMembers[i];
// } else if (returnedMembers[i].memberRole.indexOf(ADMINISTRATOR) > -1) {
// smcPlusIncumbents.currentAdministrator = returnedMembers[i];
// } else if (returnedMembers[i].memberRole === PRIMARYREVIEWER) {
// smcPlusIncumbents.currentPrimaryReviewer = returnedMembers[i];
// }
// }
// if (smc) {
// smcPlusIncumbents.smc = smc;
// console.log("getStudyMembers: 3. resolving deferred with smcPlusIncumbents object");
// deferred.resolve(smcPlusIncumbents);
// } else {
// console.log("getStudyMembers: 3. rejecting deferred with error message");
// deferred.reject("Unable to retrieve studyMembers. Please contact the Helpdesk.", "getStudyMembers("
// + targetAssignedID + ")");
// }
// console.log("getStudyMembers: 4. returning promise");
// return deferred.promise();
// });
//        
// }
//    
// /**
// * builds a usersCollection
// *
// * @return uses jQuery deferred functionality to return success or failure. When successful the
// * function provides an associative array of the users indexed by SPUserId. When failure
// * the function returns a error message to alert the user and log in the console.
// */
// function getUsers(deferred) {
// //var deferred = guDfd;
// var users = [];
// console.log("getUsers: 1. at start of function. About to call REST user.");
//        
// sendREST("GET", restPath.concat("user"), null, function(returnedUsers) {
// console.log("getUsers: 2. after call REST user. # returned: " + returnedUsers.length);
// for (var i = 0; i < returnedUsers.length; i++) {
// // add each user the the users associative array
// users[returnedUsers[i].SPUser] = returnedUsers[i];
// }
// if (users) {
// console.log("getUsers: 3. resolving deferred with users associative array");
// deferred.resolve(users);
// } else {
// console.log("getUsers: 3. rejecting deferred with error message");
// deferred.reject("Unable to retrieve users. Please contact the helpdesk.", "getUsers");
// }
// console.log("getUsers: 4. returning promise");
// return deferred.promise();
// });
//        
// }
//    
// // sendREST(
// // "GET",
// // restPath.concat("study/" + assignedID + "/members"),r
// // null,
// // function(returnedMembers) {
// //
// // for (var i = 0; i < returnedMembers.length; i++) {
// // if (returnedMembers[i].user.SPUser == userSPId) {
// // foundUser = returnedMembers[i];
// // if (returnedMembers[i].memberRole != role) {
// // roleChanged = true;
// // }
// // if (returnedMembers[i].teamMember != teamMember) {
// // teamMemberChanged = true;
// // }
// // }
// console
// .log("insertStudyMember: 1. at start of function. defining the combinedPromise with
// $.when(getStudyMembers(assignedID), getUsers())");
// //var combinedPromise = $.when(getStudyMembers(assignedID), getUsers());
// console.log("insertStudyMember: 2. executing the combinedPromise and registering a .done and .fail callback");
// //combinedPromise
// var gsmDfd = $.Deferred();
// var guDfd = $.Deferred();
// $
// .when(getStudyMembers(assignedID, gsmDfd), getUsers(guDfd))
// .done(
// function(smcPlusIncumbents, users) {
// console.log("insertStudyMember: 3. All $.when functions should be complete at this point.");
// if ( typeof smcPlusIncumbents === "undefined") {
// console
// .log("smcPlusIncumbents is undefined. It probably returned before the async call, meaining the when failed.");
// }
// if ( typeof users === "undefined") {
// console
// .log("users is undefined. It probably returned before the async call, meaining the when failed.");
// }
// if (smcPlusIncumbents.smc[userSPId]) {
// // the incoming user is in the StudyMemberCollection
// if (users[usersSPId]) {
// // the incoming user is in the RAMS DB User table.
// // if the incoming user is going into one of the restricted roles remove the person in the restricted role and
// // remove the incoming user from any prior roles. This assumes that the user can only be in one role in the StudyTeam
// or
// // StudyMemberCollection.
// if (smRole === PRINCIPALINVESTIGATOR) {
// $.when(removeStudyMember(smcPlusIncumbents.currentPI)).then(
// $.when(removeStudyMember(smcPlusIncumbents.smc[usersSPId])).then(
// function() {
// // move on
// }));
// } else if (smRole === COPRINCIPALINVESTIGATOR) {
// $.when(removeStudyMember(smcPlusIncumbents.currentCoPI)).then(
// $.when(removeStudyMember(smcPlusIncumbents.smc[usersSPId])).then(
// function() {
// // move on
// }));
// } else if (smRole.indexOf(ADMINISTRATOR) > -1) {
// $.when(removeStudyMember(smcPlusIncumbents.currentAdministrator)).then(
// $.when(removeStudyMember(smcPlusIncumbents.smc[usersSPId])).then(
// function() {
// // move on
// }));
// } else if (smRole.indexOf(PRIMARYREVIEWER) > -1) {
// $.when(removeStudyMember(smcPlusIncumbents.currentPrimaryReviewer)).then(
// $.when(removeStudyMember(smcPlusIncumbents.smc[usersSPId])).then(
// function() {
// // move on
// }));
// }
//                                
// // add the incoming person in the desired role
// // determine if the incoming user is the PI or CoPI and set their role to override whatever was provided.
// if (grabPI() == userSPId) {
// smRole = PRINCIPALINVESTIGATOR;
// } else if (grabCoPI() == userSPId) {
// smRole = COPRINCIPALINVESTIGATOR;
// }
//                                
// smData = {
// "user" : users[usersSPId],
// "activeMember" : true,
// "vaUser" : true,
// "teamMember" : teamMember,
// "memberRole" : smRole,
// "coiRequired" : false,
// "writePermissions" : true,
// "explaination" : "n/a",
// "notifications" : true
// };
//                                
// sendREST("POST", restPath.concat("study/" + assignedID + "/members"), smData,
// function() {
// console
// .log("Adding incumbent smc member and valid user with role "
// + smRole + " to the smc. user: "
// + JSON.stringify(users[usersSPId]));
// });
//                                
// } else {
// //the incoming user is NOT in the RAMS DB User table. [The execution path may never get here due to the promise.fail
// feature.]
// // TODO TAR Need to get the user's name for the alert.
// console
// .log("The "
// + role
// + " with userSPId "
// + userSPId
// + " was not found in the RAMS database. The user needs to register with RAMS by logging in and starting a
// application");
// }
// } else {
// // the incoming user is NOT in the StudyMemberCollection, so add them if they are a valid user
// // TODO TAR if they are not a valid user in the RAMS DB, should they be added automatically?
// if (users[usersSPId]) {
// // add the valid user to the studyMemberCollection if the criteria is met.
// // TODO TAR May need to see if a different PI or CoPI is already set and needs to be removed.
// // if the incoming user is going into one of the restricted roles remove the person in the restricted role and
// // remove the incoming user from any prior roles. This assumes that the user can only be in one role in the StudyTeam
// or
// // StudyMemberCollection.
// if (smRole === PRINCIPALINVESTIGATOR) {
// $.when(removeStudyMember(smcPlusIncumbents.currentPI)).then(function() {
// // move on
// })
// } else if (smRole === COPRINCIPALINVESTIGATOR) {
// $.when(removeStudyMember(smcPlusIncumbents.currentCoPI)).then(function() {
// // move on
// })
// } else if (smRole.indexOf(ADMINISTRATOR) > -1) {
// $.when(removeStudyMember(smcPlusIncumbents.currentAdministrator)).then(function() {
// // move on
// })
// } else if (smRole.indexOf(PRIMARYREVIEWER) > -1) {
// $.when(removeStudyMember(smcPlusIncumbents.currentPrimaryReviewer)).then(
// function() {
// // move on
// })
// }
//                                
// // add the incoming person in the desired role
// // determine if the incoming user is the PI or CoPI and set their role to override whatever was provided.
// if (grabPI() == userSPId) {
// smRole = PRINCIPALINVESTIGATOR;
// } else if (grabCoPI() == userSPId) {
// smRole = COPRINCIPALINVESTIGATOR;
// }
//                                
// smData = {
// "user" : users[usersSPId],
// "activeMember" : true,
// "vaUser" : true,
// "teamMember" : teamMember,
// "memberRole" : smRole,
// "coiRequired" : false,
// "writePermissions" : true,
// "explaination" : "n/a",
// "notifications" : true
// };
//                                
// sendREST("POST", restPath.concat("study/" + assignedID + "/members"), smData,
// function() {
// console.log("Adding new smc member and valid user with role " + smRole
// + " to the smc. user: " + JSON.stringify(users[usersSPId]));
// });
// } else {
// //the incoming user is NOT in the RAMS DB User table.
// // TODO TAR Need to get the user's name for the alert.
// console
// .log("The "
// + role
// + " with userSPId "
// + userSPId
// + " was not found in the RAMS database. The user needs to register with RAMS by logging in and starting a
// application");
// }
// }
// }).fail(function(errMsg, location) {
// console.log(errMsg + " " + location);
// alert(errMsg);
// });
//    
// // }
//    
// // if the incoming person is found in the SMC
// // is their incoming role is changing or their team member status is changing
// // delete the SMC entry
// // add the incoming person and role
// // else
// // no change needed???
// // else (incoming person not found)
// // if incoming role is PI or COPI
// // verify that incoming persion is valid user
// // remove the current PI or COPI
// // add the incoming person as the PI or COPI
// // else
// // add the incoming person as the incoming role
//    
// if (foundUser) {
// if (roleChanged || teamMemberChanged) {
// // The incoming person was found in the studyMemberCollectioin and their role or teamMembership status is changing
// // therefore delete the prior entry and add the new one.
// removeStudyMember(foundUser);
// addStudyMember();
// }
// } else {
//        
// }
//    
// if ( !foundUser) {
// sendREST(
// "GET",
// restPath.concat("user"),
// null,
// function(returnedUsers) {
// for (var j = 0; j < returnedUsers.length; j++) {
// if (returnedUsers[j].SPUser == userSPId) {
// foundUser = true;
// // console.log("found user: " + userTitle + ", " + returnedUsers[j].usersID);
//                            
// if (grabPI() == userSPId) {
// smRole = "Principal Investigator";
//                                
// } else if (grabCoPI() == userSPId) {
// smRole = "Co-Principal Investigator";
// }
//                            
// data = {
// "user" : returnedUsers[j],
// "activeMember" : true,
// "vaUser" : true,
// "teamMember" : teamMember,
// "memberRole" : smRole,
// "coiRequired" : false,
// "writePermissions" : true,
// "explaination" : "n/a",
// "notifications" : true
// };
//                            
// sendREST("POST", restPath.concat("study/" + assignedID + "/members"), data, function() {
//                                
// console
// .log("to this study, we want to add member: "
// + JSON.stringify(returnedUsers[j]));
// });
//                            
// }
// }
// if ( !foundUser) {
// if (userSPId != "none" && userSPId != undefined) {
// console
// .log("The "
// + role
// + " with userSPId "
// + userSPId
// + " was not found in the RAMS database. The user needs to register with RAMS by logging in and starting a
// application");
// alert('The '
// + role
// + ' with user '
// + userSPId
// + ' was not found in the RAMS database. Please notify your Local Site Administrator. The user needs to register with
// RAMS by logging in and starting a application');
// }
// }
// });
// }
// // });
// }
/**
 * insert a person into the study with the given role.
 * 
 * @param {number} userSPId - the SPUserId of the person to be added
 * @param {string} role - the role of the user. These can be any of the following roles as specified in
 *           StudyMemberRoles.java:
 *           <ul>
 *           <li>system designated roles
 *           <ul>
 *           <li><strong>Role - CONSTANT - Description and Notes</strong>
 *           <li>Assigned Reviewer - {@link ASSIGNEDREVIEWER} -
 *           <li>Co-Principal Investigator - {@link COPRINCIPALINVESTIGATOR} -
 *           <li>Liaison Reviewer - {@link LIAISONREVIEWER} -
 *           <li>Local Site Investigator - {@link LOCALSITEINVESTIGATOR} -
 *           <li>Primary Reviewer - {@link PRIMARYREVIEWER} -
 *           <li>Principal Investigator - {@link PRINCIPALINVESTIGATOR} -
 *           <li>Study Administrator - {@link ADMINISTRATOR} -
 *           </ul>
 *           <li>user designated roles - on the manage personnel page of the study
 *           <ul>
 *           <li>Biostatistician
 *           <li>Clinical Coordinator
 *           <li>Collaborator
 *           <li>Co-Investigator
 *           <li>CSP Program Manager
 *           <li>Data Manager
 *           <li>Lab Manager
 *           <li>Lab Technician
 *           <li>National Study Coordinator
 *           <li>Nurse Assessor
 *           <li>Pharmacist
 *           <li>Pharmacy Technician
 *           <li>Regulatory Coordinator
 *           <li>Research Assistant
 *           <li>Research Nurse
 *           <li>Statistician
 *           <li>Study Coordinator
 *           <li>Sub-Investigator
 *           </ul>
 *           </ul>
 * @param {boolean} teamMember - indicates if the person is considered part of the study team.
 * @param {string} studyId - the study's assignedID where the person will be added.
 * @param {function} afterward - callback function to return to when done.
 */
function insertStudyMember(userSPId, role, teamMember, studyId, afterward) {
   // This function depends on the constants for roles defined as globals in dynamicGeneration.js
   // initializing foundUser to false, but if the user is userSPId is found this will become the user
   // var foundUser = false;
   // object of the found user.
   var smRole = role;
   var smData;
   // var roleChanged = false;
   // var teamMemberChanged = false;
   // var users = [];

   if (consoleLoggingEnabled) {
      console.log("insertStudyMember: 1. attempting to add userSPId = " + userSPId + " role = '" + role
         + "' team Member = '" + teamMember + "' to study = '" + studyId + "'.");
   }

   if (!(userSPId) || (userSPId == "none") || !(studyId)) {
      alert("Error trying to insert Study Member.");
      return;
   }

   // if the user object has not yet been initialized, initialize it and then rerun insertStudyMembers
   if (!(users) || (users.length === 0)) {
      getRAMSUsers(function() {
         // give the getRAMSUsers function time to finish so that getRAMSUsers and insertStudyMembers doesn't get called
         // multiple times.
         setTimeout(function() {
            insertStudyMember(userSPId, role, teamMember, studyId, afterward);
         }, 0);
      });
   } else {

      // add the incoming person in the desired role
      // determine if the incoming user is the PI or CoPI and set their role to override whatever was provided.
      if (grabPI() == userSPId) {
         smRole = PRINCIPALINVESTIGATOR;
      } else if (grabCoPI() == userSPId) {
         smRole = COPRINCIPALINVESTIGATOR;
      }

      // if ((typeof smcPlusIncumbents.smc !== "undefined")
      // && (typeof smcPlusIncumbents.smc[userSPId] !== "undefined")) {
      // if (consoleLoggingEnabled) {
      // console.log("The user will not be added as they are already a member of the study team" + "\nSPUserID: "
      // + userSPId + "\ndispayName: " + smcPlusIncumbents.smc[userSPId].user.displayName);
      // }
      // setTimeout(function() {
      // alert("The user " + smcPlusIncumbents.smc[userSPId].user.displayName
      // + " will not be added as they are already a member of the study team.");
      // }, 0);
      // } else {

      smData = {
         "user" : users[userSPId],
         "activeMember" : true,
         "vaUser" : true,
         "teamMember" : teamMember,
         "memberRole" : smRole,
         "coiRequired" : false,
         "writePermissions" : true,
         "explaination" : "n/a",
         "notifications" : true
      };

      // Currently the insertMember call is synchronous so that we know it completed successfully before continuing
      // with
      // other updates.
      var ajaxUrl = restPath.concat("study/" + assignedID + "/insertmember");
      $.ajax({
         async : false,
         type : 'POST',
         url : ajaxUrl,
         data : JSON.stringify(smData),
         xhrFields : {
            "withCredentials" : true
         },
         headers : {
            "accept" : "application/json;odata=verbose",
            "X-RequestDigest" : requestDigest,
            "content-type" : "application/json;odata=verbose"
         },
         dataType : 'json',
         // JSHINT json is defined but never used - it is returned by default from sendREST
         // jshint -W098
         success : function(json) {
            if (consoleLoggingEnabled) {
               logError(ajaxUrl, "", "insertMember SUCCESS for user "
                  + (smData.user ? smData.user.displayName : "unknown name SPUser=" + userSPId) + " in the "
                  + smData.memberRole + " role.", null);
            }
         },
         error : function(XMLHttpRequest, textStatus, errorThrown) {
            var errorMessage;
            if ((XMLHttpRequest.responseJSON) && (XMLHttpRequest.responseJSON.error)
               && (XMLHttpRequest.responseJSON.error.message)) {
               errorMessage = JSON.stringify(XMLHttpRequest.responseJSON.error.message);
            } else if ((XMLHttpRequest.status) && (XMLHttpRequest.responseText) && (XMLHttpRequest.statusText)) {
               errorMessage = XMLHttpRequest.statusText + " (" + XMLHttpRequest.status + ") "
                  + XMLHttpRequest.responseText;
            }
            logError(ajaxUrl, errorThrown, textStatus, "insertMember FAILED for user "
               + (smData.user ? smData.user.displayName : "unknown name SPUser=" + userSPId) + " in the "
               + smData.memberRole + " role. " + errorMessage);
            alert("Unable to insert Study Member.");
         }
      });

      // sendREST("POST", restPath.concat("study/" + assignedID + "/insertmember"), smData, function() {
      // console.log("Adding new smc member and valid user with role " + smRole + " to the smc. user: "
      // + JSON.stringify(users[userSPId]));
      // });

      // });

      // }
   }
}

// JSHINT afterward is defined but never used
// jshint -W098
/**
 * remove a person from the study with the given role. Similar to {@link insertStudyMember}, but for removal of the
 * person.
 * 
 * @param {number} userSPId - the SPUserId of the person to be added
 * @param {string} role - the role of the user.
 * @param {boolean} teamMember - indicates if the person is considered part of the study team.
 * @param {string} studyId - the study's assignedID where the person will be added.
 * @param {function} afterward - callback function to return to when done.
 */
function removeStudyMember(userSPId, role, teamMember, studyId, afterward) {
   /* jshint unused: false */
   // This function depends on the constants for roles defined as globals in dynamicGeneration.js
   // initializing foundUser to false, but if the user is userSPId is found this will become the user
   // var foundUser = false;
   // object of the found user.
   var smRole = role;
   var smData;
   // var roleChanged = false;
   // var teamMemberChanged = false;
   // var users = [];

   // Nothing to remove - just return to the caller.
   if (!(userSPId) || (userSPId == "none")) {
      return;
   }

   if (consoleLoggingEnabled) {
      console.log("removeStudyMember: 1. attempting to remove userSPId = " + userSPId + " role = '" + role
         + "' team Member = '" + teamMember + "' to study = '" + studyId + "'.");
   }
   // If the user's see the following error it is a programming error as the caller is not passing the assignedId to
   // update.
   if (!(studyId)) {
      alert("Error removing a Study Member");
      return;
   }

   smData = {
      "user" : users[userSPId],
      "activeMember" : true,
      "vaUser" : true,
      "teamMember" : teamMember,
      "memberRole" : smRole,
      "coiRequired" : false,
      "writePermissions" : true,
      "explaination" : "n/a",
      "notifications" : true
   };

   // Currently the removemembers call is synchronous so that we know it completed successfully before continuing with
   // other updates.
   var ajaxUrl = restPath.concat("study/" + assignedID + "/removemembers");
   $.ajax({
      async : false,
      type : 'POST',
      url : ajaxUrl,
      data : JSON.stringify(smData),
      xhrFields : {
         "withCredentials" : true
      },
      headers : {
         "accept" : "application/json;odata=verbose",
         "X-RequestDigest" : requestDigest,
         "content-type" : "application/json;odata=verbose"
      },
      dataType : 'json',
      // JSHINT json is defined but never used - it is returned by default from sendREST
      // jshint -W098
      success : function(json) {
         if (consoleLoggingEnabled) {
            logError(ajaxUrl, "", "removeMembers SUCCESS for user "
               + (smData.user ? smData.user.displayName : "unknown name SPUser=" + userSPId) + " in the "
               + smData.memberRole + "role.", null);
         }
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError(ajaxUrl, errorThrown, textStatus, "removeMembers FAILED for user "
            + (smData.user ? smData.user.displayName : "unknown name SPUser=" + userSPId) + " in the "
            + smData.memberRole + "role. " + JSON.stringify(XMLHttpRequest.responseJSON.error.message));
         alert("Unable to remove Study Member.");
      }
   });
   // }

}

function initialSectionCheck() {
   var section, required, question;

   section = study.containedFormCollection[0].containedForm.formQuestionCollection;

   for (var i = 0; i < section.length; i++) {
      required = true;

      // grabbing a specific question
      question = section[i].question.questionsID;

      if (section[i].requiredPredecessorAnswersID) {
         // here we are looking into first level dependent questions. With the current
         // section A, there isn't a question dependency where question C depends on question
         // B which depends on question A (second level dependency);
         var parent = section[i].requiredPredecessorAnswersID;

         if (($("#" + parent).is(':checked')) === false) {
            required = false;
         }
      }

      if (required) {
         if ((section[i].question.responseFormat.name == "radio")
            || (section[i].question.responseFormat.name == "checkbox")) {
            var complete = false;
            var answers = $('[name="QID_' + question + '_multi"]');

            for (var j = 0; j < answers.length; j++) {
               var id = answers[j].getAttribute("id");

               if ($("#" + id).is(':checked')) {
                  complete = true;
               }
            }

            if (complete !== true) {
               return false;
            }

         } else if ((section[i].question.responseFormat.name == "select")
            || (section[i].question.responseFormat.name == "Facility")) {
            if ($('[name="QID_' + question + '"]').find(':selected').val() == "none") {
               return false;
            }

         } else {
            if ($('[name="QID_' + question + '"]').val() === "") {
               return false;
            }
         }
      }
   }
   return true;
}

/* exported createResponses */
// grabbing all user input and packaging it up to send to the database
function createResponses() {
   if (assignedID == "1") {
      if (initialSectionCheck()) {
         generateAssignedId(null);
      } else {
         insertWarning('You must complete Section A and save your responses before proceeding.', 'alert-warning');
         return;
      }
   } else {
      sendResponses();
   }
}

/* exported checkEnableSaveButton */
/**
 * Reenables the save progress button if a question response has been changed on the current application section
 */
function checkEnableSaveButton(questionsID) {
   changedQuestions.add(questionsID);
   // FacilityAnswer is not used in this function, but getFacilityAnswer saves the cookie with the chosen facility
   // JSHINT {var} is defined but never used.
   // jshint -W098
   var facilityAnswer = getFacilityAnswer(function(facAnsw) {
      if (!changedQuestions.isEmpty()) {
         $("#saveAppProgress").prop("disabled", false); // reenable save progress button
      }
   });
}

/* exported triggerUploadModal */
function triggerUploadModal() {

   var modal, modalDialog, modalContent, modalHeader;
   var header, heading1, lineBreak, saveButton, closeButton, text, modalFooter;
   var container, table, tableDiv, tbody, trow, trow2;
   // var tableHeader, dateAdded, cell, modalTitle, modalBody;
   // var statusTypes = [ "Approved", "Changes Requested", "Submitted", "Under Review" ];
   var documentTypes = [ "ACORP Main Body", "ACORP Appendix1", "ACORP Appendix2", "ACORP Appendix3", "ACORP Appendix4",
      "ACORP Appendix5", "ACORP Appendix6", "ACORP Appendix7", "ACORP Appendix8", "ACORP Appendix9", "Audit",
      "Case Report Forms", "COIs", "CVs", "Device Information", "HIPAA", "ICF", "Investigator Brochure/Package Insert",
      "Letters/Correspondence", "Protocol", "Questionnaires/Surveys", "Recruitment Material", "VA Form 10-9012",
      "Waiver Requests", "Other"
   ];

   modal = document.createElement("div");
   modal.setAttribute("class", "modal");
   modal.setAttribute("id", "uploadModal");
   modal.setAttribute("tabindex", "-1");
   modal.setAttribute("role", "dialog");
   modal.setAttribute("aria-labelledby", "uploadModal");

   modalDialog = document.createElement("div");
   modalDialog.setAttribute("role", "document");

   modalContent = document.createElement("div");
   modalContent.setAttribute("class", "modal-content");
   modalHeader = document.createElement("div");
   modalHeader.setAttribute("class", "modal-header");
   modalHeader.setAttribute("tabindex", "0"); // start tabbing on modal from header

   header = document.createElement("header");
   header.setAttribute("class", "modal-title");
   header.setAttribute("id", "modalTitle");

   heading1 = document.createElement("H2");
   heading1.setAttribute("class", "modal-title");
   text = document.createTextNode("Upload Document");
   heading1.appendChild(text);
   header.appendChild(heading1);
   modalHeader.appendChild(header);

   modalContent.appendChild(modalHeader);
   lineBreak = document.createElement("br");
   text = document
      .createTextNode("To successfully upload your document, select your document type, click Browse to select "
         + "your document and click the Save Document button. Then close the window to return to the " + "application.");
   // text.setAttribute("tabindex", "1");

   modalContent.appendChild(modalHeader);
   modalContent.appendChild(lineBreak);
   modalContent.appendChild(text);
   modalContent.appendChild(lineBreak);

   /* Build Table for Uploading Document */
   container = document.createElement("div");
   container.setAttribute("class", "form-content");

   /* Table Header/Division */
   // tableHeader = tableHeaderHelper("Complete the sections below: ", ["Status", "Date Approved", "Document Type",
   // "Upload Document"]);
   tableDiv = document.createElement("div");
   tableDiv.setAttribute("class", "div-table-content-big"); // div-table-content-big should be small?

   /* Table Style */
   table = document.createElement("table");
   table.setAttribute("class", "table table-striped table-hover");
   table.setAttribute("tabindex", "0");

   /* Table Body & Rows */
   tbody = document.createElement("tbody");
   trow = document.createElement("tr");
   trow.setAttribute("tabindex", "0");
   trow2 = document.createElement("tr");

   // JSHINT buildSelecterHelper is not defined - it is defined in RAMS_Tools
   // jshint -W117
   trow.appendChild(buildSelecterHelper("docType", "Document Type", documentTypes, 12, "displayOtherDocType()"));
   trow2.appendChild(fileSelectorBuilder("uploadDoc", "Upload Document:"));
   // trow2.appendChild(buildSelecterHelper("docStatus", "Status", statusTypes, 12));

   /* Add Table Elements Together */
   tbody.appendChild(trow);
   tbody.appendChild(trow2);
   table.appendChild(tbody);
   tableDiv.appendChild(table);
   // container.appendChild(tableHeader); //took out
   container.appendChild(tableDiv);
   container.appendChild(lineBreak);

   modalFooter = document.createElement("div");
   modalFooter.setAttribute("class", "modal-footer");

   /* Add Save Document Button */
   saveButton = document.createElement("button");
   saveButton.setAttribute("class", "btn btn-sm modal-button");
   saveButton.setAttribute("aria-label", "Save");

   saveButton.setAttribute("title", "Save your document"); // hover tip
   saveButton.setAttribute("type", "button");
   saveButton.setAttribute("role", "button");
   saveButton.setAttribute("id", "saveModal");

   saveButton.setAttribute("onclick", "saveAttachment()");
   text = document.createTextNode("Save Document");
   saveButton.appendChild(text);

   /* Add Close Window Button */
   closeButton = document.createElement("button");
   closeButton.setAttribute("class", "btn btn-sm modal-button modal-button-right");
   closeButton.setAttribute("data-dismiss", "modal");
   closeButton.setAttribute("aria-label", "Close");

   closeButton.setAttribute("title", "Close this window and return to the application"); // hover tip
   closeButton.setAttribute("type", "button");
   closeButton.setAttribute("role", "button");
   closeButton.setAttribute("id", "closeModal");

   // closeButton.setAttribute("onclick", "$('#uploadModal').popup('toggle')");
   closeButton.setAttribute("onclick", "closeUploadDocumentModel()");
   text = document.createTextNode("Close this window");
   closeButton.appendChild(text);

   /* Combine Modal Elements */
   modalContent.appendChild(container);
   modalFooter.appendChild(saveButton);
   modalFooter.appendChild(closeButton);
   modalContent.appendChild(modalFooter);
   modalDialog.appendChild(modalContent);
   modal.appendChild(modalDialog);
   $('body').append(modal);
   $('#uploadModal').popup('show');

}

/* exported saveAttachment */
function saveAttachment() {
   var documentType = $('#docType').find(':selected').val();
   if ("Other" == documentType) {
      if ($('#docTypeName').val() !== "") {
         documentType = $('#docTypeName').val();
      }
   }
   uploadFiles(documentType);
}

function uploadFiles(documentType) {
   var fileList, targetSection;
   var responseData = [];
   var response = {};
   var uploaders;
   var allowedExtensions = /(\.pdf|\.doc|\.docx|\.xls|\.xlsx|\.ppt|\.pptx|\.vdx|\.vsd|\.vsdx)$/i;

   // looking for uploaded files to send to the cache
   // Sustainment Defect #608335 looks for file input on upload attachment modal window
   // uploaders = $('[type="file"]');
   uploaders = $("#uploadDoc");
   // Sustainment Defect #608335 End
   for (var i = 0; i < uploaders.length; i++) {
      if (uploaders[i].value) {
         if(!allowedExtensions.exec(uploaders[i].value)){
            alert('Please upload file having extensions .pdf/.doc/.docx/.xls/.xlsx/.ppt/.pptx/.vdx/.vsd/.vsdx only.');
         }else{
         if ((typeof window.FileReader !== "undefined") && (window.FileReader !== null)) {
            fileList = uploaders[i].files;

            for (var j = 0; j < fileList.length; j++) {
               // console.log("uploading: " + uploaders[i].name.replace('QID_', '') + "-RAMS-" +fileList[j].name);

               // sendFile(fileList[j], uploaders[i].name.replace('QID_', '') + "-RAMS-" +fileList[j].name,
               // fileList[j].size, "Study%20Team", documentType, function(){
               sendFile(fileList[j], fileList[j].name, fileList[j].size, "Study%20Team", documentType, function() {
                  var sidebar = $('.list-group-item.sectionLink');
                  var current = $('.list-group-item.active');
                  for (var i = 0; i < sidebar.length; i++) {
                     if (sidebar.get(i) == current.get(0)) {
                        targetSection = i;
                        // updating sidebar icons now that we have the section that
                        // should show up as activated.
                        sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/"
                           + assignedID + "/Study%20Team')/Files"), null, function(returnedFiles) {
                           attachments = returnedFiles.d.results;
                           wellSidebarBuilder(targetSection, attachments.length);
                        });
                     }
                  }
               });
            }

            response = {
               "questionsID" : (uploaders[i].name).replace('QID_', ''),
               "response" : fileList[0].name
            };
            responseData.push(response);
            // record data associated with upload
            sendREST("POST", restPath.concat("study/" + assignedID + "/responses/" + targetStage), responseData);
         } else {
            alert('Please upgrade your browser to upload files.');
            i = uploaders.length;
         }
      }
      }
   }

}

// NOT USED the function uploadHipaa and sendFile is never used
/*
 * function uploadHipaa() {
 *    var uploaders, fileList;
 *    // looking for uploaded files to send to the cache
 *    uploaders = $('[type="file"]');
 *    for (var i = 0; i < uploaders.length; i++) {
 *       if (uploaders[i].value) {
 *          if (window.FileReader !== null) {
 *             fileList = uploaders[i].files;
 *
 *             for (var j = 0; j < fileList.length; j++) {
 *                // console.log("uploading: " + uploaders[i].name.replace('QID_', '') + "-RAMS-" +fileList[j].name);
 *
 *                sendHipaaFile(fileList[j], fileList[j].name, fileList[j].size, "Study%20Team");
 *             }
 *            
 *             * response = { "questionsID": (uploaders[i].name).replace('QID_', ''), "response": fileList[0].name }
 *             * data.push(response); sendREST("POST", restPath.concat("study/"+assignedID+"/responses/"+targetStage),
 *             * data); //record data associated with upload
 *             
 *         } else {
 *            alert('Please upgrade your browser to upload files.');
 *            i = uploaders.length;
 *         }
 *      }
 *   }
 *}
 */
/*
 * function sendFile(file, fileName, fileSize, teamFolder, callback){ 
 *    var reader = new FileReader(); 
 *    reader.onloadend = function(evt){ 
 *       if(evt.target.readyState == FileReader.DONE){ 
 *          jQuery.ajax({ 
 *             url: siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/"
 *                + assignedID + "/" + teamFolder + "')/Files/add(url='" + fileName + "',overwrite=true)"),
 *             type: "POST", 
 *             data: evt.target.result, 
 *             processData: false, 
 *             headers: { 
 *                "accept": "application/json; odata=verbose",
 *                "X-RequestDigest": requestDigest, 
 *                "content-length": fileSize 
 *             } 
 *          }).done(function(){ 
 *             insertWarning('You have successfully uploaded your document(s)', 'alert-success'); 
 *             refreshDisplay('Files'); //go to 'Attachments' page
 *             if(callback){ 
 *                callback(); 
 *             } 
 *          }); 
 *       } 
 *    } 
 *    reader.readAsArrayBuffer(file); 
 * }
 */

// This function will assign the study to multiple sites
// function assignMultiSiteStudy(localSite, localSiteInvestigator){
// var siteArray; //placeholder
// alert("Sites involved - " + siteArray.length);
// }
function sendFile(file, fileName, fileSize, teamFolder, documentType, callback) {
   var reader = new FileReader();

   // JSHINT {var} is defined but never used.
   // jshint -W098
   reader.onloadend = function(evt) {
      if (evt.target.readyState == FileReader.DONE) {
         jQuery.ajax(
            {
               url : siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID + "/"
                  + teamFolder + "')/Files/add(url='" + fileName + "',overwrite=true)"),
               type : "POST",
               data : evt.target.result,
               processData : false,
               headers : {
                  "accept" : "application/json; odata=verbose",
                  "X-RequestDigest" : requestDigest,
                  "content-length" : fileSize
               },
               error : function(XMLHttpRequest, textStatus, errorThrown) {
                  logError(siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID
                     + "/" + teamFolder + "')/Files/add(url='" + fileName + "',overwrite=true)"), errorThrown,
                     textStatus, XMLHttpRequest);
               }
            }).done(function(file, status, xhr) {

            // Get the list item that corresponds to the uploaded file.
            var getItem = getListItem(file.d.ListItemAllFields.__deferred.uri);
            // JSHINT NOTE status and xhr may be hiding a previous use of the variable
            getItem.done(function(listItem, status, xhr) {

               // Change the display name and title of the list item.
               var changeItem = updateListItem(listItem.d.__metadata, documentType);
               // JSHINT NOTE data, status and xhr may be hiding a previous use of the variable
               changeItem.done(function(data, status, xhr) {
                  alert('file uploaded and updated');
               });
               changeItem.fail(onError);
            });
            getItem.fail(onError);

            insertWarning('You have successfully uploaded your document(s)', 'alert-success');
            // JSHINT refreshDisplay is not defined - it is defined in dynamicMembers.js
            // jshint -W117
            refreshDisplay('Files'); // go to 'Attachments' page
            if (callback) {
               callback();
            }

         });
      }
   };

   reader.readAsArrayBuffer(file);
}

// Display error messages.
function onError(error) {
   alert(error.responseText);
}

function updateListItem(itemMetadata, documentType) {

   // Define the list item changes. Use the FileLeafRef property to change the display name.
   // For simplicity, also use the name as the title.
   // The example gets the list item type from the item's metadata, but you can also get it from the
   // ListItemEntityTypeFullName property of the list.

   var item = {
      "__metadata" : {
         "type" : itemMetadata.type
      },
      "DocumentType" : documentType
   };

   // Send the request and return the promise.
   // This call does not return response content from the server.
   return jQuery.ajax({
      url : itemMetadata.uri,
      type : "POST",
      data : JSON.stringify(item),
      headers : {
         "X-RequestDigest" : requestDigest,
         "content-type" : "application/json;odata=verbose",
         "IF-MATCH" : itemMetadata.etag,
         "X-HTTP-Method" : "MERGE"
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError(itemMetadata.uri, errorThrown, textStatus, XMLHttpRequest);
      }
   });
}
// Get the list item that corresponds to the file by calling the file's ListItemAllFields property.
function getListItem(fileListItemUri) {

   // Send the request and return the response.
   return jQuery.ajax({
      url : fileListItemUri,
      type : "GET",
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError(fileListItemUri, errorThrown, textStatus, XMLHttpRequest);
      }
   });
}

// Add Error into table with parameters: Timestamp, Error code, description, header
function logError(restCall, errCode, errDescription, errHeader) {
   var errLogFile = {};
   if (sessionStorage.errLogFile) {
      errLogFile = JSON.parse(sessionStorage.errLogFile);
   }

   var dt = new Date();
   var utcDate = dt.toUTCString(); // Index errors by timestamp
   errLogFile[utcDate] = restCall + " | " + errCode + " | " + errDescription + " | " + errHeader;
   sessionStorage.errLogFile = JSON.stringify(errLogFile);
}

/* exported  printErrorReport */
function printErrorReport() {
   var errLogFile = {};
   if (sessionStorage.errLogFile) {
      errLogFile = JSON.parse(sessionStorage.errLogFile);
   }

   var winPrint = window.open('about:blank', '_blank');
   var text = "<title>Error Report</title><br /><br /><span style='white-space: pre-line'>";
   for ( var e in errLogFile) {
      if (errLogFile.hasOwnProperty(e)) {
         text += "<br />-----------------<br />";
         text += e;
         text += " | ";
         text += errLogFile[e];
      }
   }
   text += "</span>";
   winPrint.document.write(text);
   winPrint.document.close();
   winPrint.focus();
}

// TAR 20180416 - commented out because nothing calls this function
// /* Add property with username to data */
// function addUsernameToProp(json, property) {
// if (property == "Reviewers") {
// data.Reviewers.push(json.d.Title);
// } else {
// data.property = json.d.Title;
// }
// }

// TAR 20180416 - commented out because nothing calls this function
/**
 * Get the username by using the SharePoint ID
 * 
 * @param userId
 * @param property
 * @returns
 */
/*function getUserTitle(userId, property) {
   return $.ajax({
      async : false,
      type : 'GET',
      url : siteUrl.concat("_api/Web/GetUserById(" + userId + ")/"),
      headers : {
         "accept" : "application/json;odata=verbose"
      },
      dataType : 'json',
      success : function(json) {
         addUsernameToProp(json, property);
      },
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError((siteUrl.concat("_api/Web/GetUserById(" + userId + ")/")), errorThrown, textStatus, JSON
            .stringify(XMLHttpRequest));
      }
   });
}*/

// TAR 20180415 - commented out because nothing calls this function
// /*
// * Get the SharePoint data from Application List and use to create Tracking & Reporting interface Query Application
// List
// * using the CIRB_tag (friendly name for users) Send the data to the function generateForm()
// */
// function getApplicationByTag(tag) {
// // get main (parent) study application
// sendREST("GET", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items" + "?$filter=CIRB_ID eq '" + tag + "'",
// null, function(jsonCIRB) { // tag
// // filter
// var app = jsonCIRB.d.results;
// var studyId = app[0].Study_ID;
// var title = app[0].StudyTitle;
//
// sendREST("GET", restPath + "study/" + studyId, null, function(studyResponse) {
// var currentStudyStat = studyResponse.studyStatusCollection;
// // var pi = app[0].PrincipalInvestigatorId;
// // var piTitle = getUserTitle(pi, "PI");
// var coPi = app[0].CoPrincipalInvestigatorId;
// // var coPiTitle;
// // if (coPi != null) { // may not have Co-PI
// // coPiTitle = getUserTitle(coPi, "Co-PI");
// // }
//
// var admin = app[0].AssignedIRBAdministratorId;
// if (admin != null) {
// adminTitle = getUserTitle(admin, "IRB Manager");
// } else {
// adminTitle = "No Admin";
// }
//
// var appType;
// if (app[0].ApplicationType != null) {
// appType = (app[0].ApplicationType).split(" ")[0]; // take off *'s and -'s
// } else {
// appType = "Not Applicable";
// }
//
// // var appStatus = app[0].ApplicationStatus;
//
// if (app[0].ReviewersId != null) {
// for (var i = 0; i < app[0].ReviewersId.results.length; i++) {
// getUserTitle(app[0].ReviewersId.results[i], "Reviewers");
// }
// }
//
// data["CIRB_ID"] = tag;
// data["Study_ID"] = studyId;
// data["Study Title"] = title;
// data["Review Type"] = appType;
//
// data["Important Dates"] = "---";
// data["Submission Started"] = currentStudyStat[0].createdAt.split("T")[0];
// data["Approval Date"] = "---";
// data["CR Expiration Date"] = "---";
//
// data["Project Closed"] = "---";
// data["Project Withdrawn"] = "---";
// data["Total Calendar Days"] = "---";
// data["Memo"] = "---";
//
// var siteId = app[0].FacilityLookupId;
// sendREST("GET", siteUrl.concat("_api/web/lists/getbytitle('Facilities')/items"), null, function(
// jsonFacility) { // find
// // name
// // of
// // site
// var returnedFacilities = jsonFacility.d.results;
// var siteName;
// // var siteLocation;
//
// for (var i = 0; i < returnedFacilities.length; i++) {
// if (returnedFacilities[i].Id == siteId) {
// siteName = returnedFacilities[i].Title;
// // siteLocation = returnedFacilities[i].FacilityLocation;
// }
// }
// // JSHINT NOTE siteName may bit be initialized at this point it is up to the generateForm call
// // to handle the
// // situation
// data["PI Site"] = siteName;
// if (coPi != null) {
// data["Co-PI Site"] = siteName;
// } else {
// data["Co-PI Site"] = "Not Applicable";
// }
// data["Coord_Center"] = siteName;
//
// // look for local (child) site applications
// sendREST("GET", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items"
// + "?$filter=ParentStudyID eq '" + studyId + "'", null, function(jsonApplication) {
// var localApp = jsonApplication.d.results;
// var localAppType;
// var localSiteCount = 0;
//
// for (var j = 0; j < localApp.length; j++) {
// localAppType = localApp[j].Title.split("- ")[1];
// if (localAppType == "Local Site Application") {
// localSiteCount++;
// }
// }
// // var localSiteCount = localApp.length + "";
//
// data["Projected # of Sites"] = localSiteCount.toString();
// data["Actual Number of Sites"] = localSiteCount.toString();
//
// // render tracking & reporting interface with data
// generateForm(data);
//
// });
// });
// });
// });
// }
function sendResponses() {

   getRAMSUsers(function() {

      getStudyMembers(
         assignedID,
         function() {
            // updating the facility
            if ((window.location.href.split("?")[1] != "1")
               && (currentFormType == "Parent")
               && (changedQuestions
                  .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[6].question.questionsID))) {
               var facilityQID = study.containedFormCollection[0].containedForm.formQuestionCollection[6].question.questionsID;

               // Lookup Prior Facility and Id - the true facility is only stored on application list
               var priorFacilityId = spAppData[RAMS.Environment.SharePoint.List.Application.FacilitiesLookupId];
               var priorFacilityName;
               // Lookup newFacilityLookupId
               var newFacilityName = $('[name="QID_' + facilityQID + '"]').val();
               var newFacilityId;
               sendREST("GET", siteUrl.concat("_api/web/lists/getbytitle('Facilities')/items" + "?$filter=(Title eq '"
                  + newFacilityName + "') or (Id eq " + priorFacilityId + ")&$select=Title,Id"), null, function(
                  jsonFacility) {

                  var returnedFacilities = jsonFacility.d.results;
                  for (var i = 0; i < returnedFacilities.length; i++) {
                     if (returnedFacilities[i].Title == newFacilityName) {
                        newFacilityId = returnedFacilities[i].Id;
                     } else if (returnedFacilities[i].Id == priorFacilityId) {
                        priorFacilityName = returnedFacilities[i].Title;
                     }
                  }

                  if (priorFacilityId != newFacilityId) {
                     // See if there are any Local Site Applications for the new facility
                     sendREST("GET", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items"
                        + "?$filter=(ParentStudyID eq '" + assignedID + "') and (ParticipatingFacilitiesLookup eq "
                        + priorFacilityId + ")", null, function(jsonApplication) {
                        var spData = {
                           "__metadata" : spAppData.__metadata,
                           "FacilitiesLookupId" : newFacilityId,
                           "ParticipatingFacilitiesLookupId" : newFacilityId
                        };
                        var localApp = jsonApplication.d.results;
                        var localAppId;
                        var localSiteCount = 0;
                        var sendData = false;
                        for (var j = 0; j < localApp.length; j++) {
                           // if there is a Local Site Application for this site, there should only be one
                           localSiteCount++;
                           localAppId = localApp[j].Id;
                           break;
                        }
                        if (localSiteCount > 0) {
                           // Tell user that the local site application for the new facility will be deleted and
                           // ask
                           // if
                           // they want to
                           // continue
                           var msg = "You are changing the facility from "
                              + (priorFacilityName ? priorFacilityName : "Id = " + priorFacilityId) + " to "
                              + newFacilityName + ". There is a Local Site Application placeholder established for "
                              + newFacilityName
                              + ". If you choose to continue with this change, the Local Site Application "
                              + "placeholder will be deleted. Do you want to continue?";
                           sendData = confirm(msg
                              + "\n\nPress [OK] to change the facility and delete the Local Site Application.");
                           if (sendData) {
                              // assuming yes - Lookup the Participating Site Investigator and Participating Site
                              // Liaison remove the Participating Site Investigator and Participating Site Liaison
                              // from parent study delete Local Site application for the new facility
                              removeStudyMember(localApp[0].ParticipatingSiteInvestigatorId, LOCALSITEINVESTIGATOR,
                                 true, assignedID, null);
                              removeStudyMember(localApp[0].ParticipatingSiteLiaisonId, LIAISONREVIEWER, true,
                                 assignedID, null);
                              // spLocalAppData is never used.
                              // var spLocalAppData = {
                              // "__metadata" : spAppData.__metadata
                              // };
                              sendREST("POST", siteUrl + "_api/Web/Lists/GetByTitle('Application')/Items(" + localAppId
                                 + ")", null, function() {
                                 // update the Application List
                                 spAppData.FacilitiesLookupId = newFacilityId;
                                 sendREST("POST", spAppData.__metadata.uri, spData, function() {
                                 }, "Merge");
                              }, "Delete");
                           } else {
                              // if no, reselect the prior facility and stop this portion of update (need to test
                              // if other updates still occur.)
                              if (priorFacilityName) {
                                 $('[name="QID_' + facilityQID + '"]').val(priorFacilityName);
                                 getFacilityAnswer(function(facAns) {
                                    /* jshint unused: false */
                                    // Do Nothing
                                 });
                              } else {
                                 alert("Unable to reset the Facility to the prior value.");
                              }
                           }
                        } else {
                           // update the Application List
                           spAppData.FacilitiesLookupId = newFacilityId;
                           sendREST("POST", spAppData.__metadata.uri, spData, function() {
                           }, "Merge");

                        }
                        continueSendResponses();
                     });
                  } else {
                     continueSendResponses();
                  }
               });
            } else {
               continueSendResponses();
            }
         });

   });
}

function continueSendResponses() {
   var changedPackage, fileList;
   var qrData = [];
   var response = {};
   var uploaders;
   var i, j;
   var allowedExtensions = /(\.pdf|\.doc|\.docx|\.xls|\.xlsx|\.ppt|\.pptx|\.vdx|\.vsd|\.vsdx)$/i;

   // looking for uploaded files to send to the cache
   // Sustainment Defect #608335 looks for form file input except modal window
   // uploaders = $('[type="file"]');
   // Sustainment Defect #608335 End
   uploaders = $('[type=file]').not("#uploadDoc");
   for (i = 0; i < uploaders.length; i++) {
      if (uploaders[i].value) {
         if(!allowedExtensions.exec(uploaders[i].value)){
            alert('Please upload file having extensions .pdf/.doc/.docx/.xls/.xlsx/.ppt/.pptx/.vdx/.vsd/.vsdx only.');
         }else{
            
         if ((typeof window.FileReader !== "undefined") && (window.FileReader !== null)) {
            fileList = uploaders[i].files;

            for (j = 0; j < fileList.length; j++) {
               // console.log("uploading: " + uploaders[i].name.replace('QID_', '') + "-RAMS-" +fileList[j].name);

               sendFile(fileList[j], uploaders[i].name.replace('QID_', '') + "-RAMS-" + fileList[j].name,
                  fileList[j].size, "Study%20Team", function() {
                     var sidebar = $('.list-group-item.sectionLink');
                     var current = $('.list-group-item.active');
                     for (var k = 0; k < sidebar.length; k++) {
                        if (sidebar.get(k) == current.get(0)) {
                           var targetSection = k;
                           // updating sidebar icons now that we have the section that
                           // should show up as activated.
                           sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/"
                              + assignedID + "/Study%20Team')/Files"), null, function(returnedFiles) {
                              // JSHINT buildSelecterHelper is not defined - it is defined in RAMS_Tools
                              // jshint -W117
                              var attachments = filterChecklists(returnedFiles.d.results, "-RAMS-");
                              wellSidebarBuilder(targetSection, attachments.length);
                           });
                        }
                     }
                  });
            }

            response = {
               "questionsID" : (uploaders[i].name).replace('QID_', ''),
               "response" : fileList[0].name
            };
            qrData.push(response);

            if (changedQuestions.contains(parseInt((uploaders[i].name).replace('QID_', '')))) {
               changedQuestions.remove(parseInt((uploaders[i].name).replace('QID_', '')));
            }
         } else {
            alert('Please upgrade your browser to upload files.');
            i = uploaders.length;
         }
      }
      }
   }

   changedPackage = changedQuestions.values();

   // updating the study title in the database
   if ((window.location.href.split("?")[1] != "1")
      && (currentFormType == "Parent")
      && (changedQuestions
         .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID))) {
      studyTitleID = study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID;

      studyTitle = $('[name="QID_' + studyTitleID + '"]').val();

      var studyPackage = {
         "title" : studyTitle
      };

      var spdata = {
         "__metadata" : spAppData.__metadata,
         "Title" : studyTitle,
         "StudyTitle" : studyTitle
      };

      sendREST("POST", spAppData.__metadata.uri, spdata, function() {
         var headerArea = $('div.page-header');
         var updateStudyInService = function() {
            sendREST("POST", restPath + "study/" + assignedID, studyPackage, function() {
               headerArea.empty();
               headerBuilder(0, headerArea);
            });
         };
         updateStudyInService();
      }, "Merge");
   }

   // if the PI or CoPI changed update them in RAMS DB
   if ((window.location.href.split("?")[1] != "1")
      && (((currentFormType == "Parent") && ((changedQuestions
         .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[1].question.questionsID)) || (changedQuestions
         .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[3].question.questionsID)))) || ((currentFormType == "Local") && ((changedQuestions
         .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.questionsID)) || (changedQuestions
         .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[8].question.questionsID)))))) {
      sendREST("GET", restPath.concat("user"), null, function(returnedUsers) {
         console.log("getUsers: 2. after call REST user. # returned: " + returnedUsers.length);
         for (var i = 0; i < returnedUsers.length; i++) {
            // add each user the the users associative array
            users[returnedUsers[i].SPUser] = returnedUsers[i];
         }

         var pid = grabPI();
         var copid = grabCoPI();
         // var priorPI = grabPriorPI();
         var priorCoPI = grabPriorCoPI();

         // updating the PI in the database
         // if ( (window.location.href.split("?")[1] != "1")
         // && (currentFormType == "Parent")
         // && (changedQuestions
         // .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[1].question.questionsID)))
         // {
         var spData = {
            "__metadata" : spAppData.__metadata
         };
         spData[RAMS.Environment.SharePoint.List.Application.PrincipalInvestigatorId] = pid;
         spData[RAMS.Environment.SharePoint.List.Application.CoPrincipalInvestigatorId] = (copid == "none" ? null
            : copid);

         if ((pid != currentUser.Id) && (copid != currentUser.Id)) {
            var currentSM = null;
            var cuMemberRole;

            for ( var sm in smcPlusIncumbents.smc) {
               if (smcPlusIncumbents.smc[sm].memberRole == STUDYMEMBER) {
                  // Found the study member
                  currentSM = smcPlusIncumbents.smc[sm];
                  break;
               }
            }

            if ((smcPlusIncumbents.smc[currentUser.Id]) && (currentSM)
               && (smcPlusIncumbents.smc[currentUser.Id].user.SPUser != currentSM.user.SPUser)) {
               // the currentUser is in the smc, but is not in the study member role
               cuMemberRole = smcPlusIncumbents.smc[currentUser.Id].memberRole;
               if (Roles.indexOf(cuMemberRole) > -1) {
                  /* jshint noempty: false */
                  // The current user has a role on the manage personnal page, do not change the role.
               } else {
                  // The user is not the PI or CoPI and doesn't have a manage personnel role, make them the study member
                  // but how were they able to edit the study?
                  insertStudyMember(currentUser.Id, STUDYMEMBER, true, assignedID, null);
               }
            } else if ((smcPlusIncumbents.smc[currentUser.Id]) && !(currentSM)) {
               // the currentUser is in the smc and there is no one in the Study Member Role - add the current user to
               // the Study Member Role
               if (Roles.indexOf(cuMemberRole) > -1) {
                  /* jshint noempty: false */
                  // The current user has a role on the manage personnal page, do not change the role.
               } else {
                  // The user is not the PI or CoPI and doesn't have a manage personnel role, make them the study member
                  // but how were they able to edit the study?
                  insertStudyMember(currentUser.Id, STUDYMEMBER, true, assignedID, null);
               }
            }
         }

         if (smcPlusIncumbents.currentPI.user.SPUser != pid) {
            insertStudyMember(pid, PRINCIPALINVESTIGATOR, true, assignedID, null);
         }

         if ((typeof copid === "undefined") || (copid === null) || (copid == "none")) {
            if ((smcPlusIncumbents.currentCoPI) && (smcPlusIncumbents.currentCoPI.user.SPUser)) {
               // remove the priorCoPI
               removeStudyMember(priorCoPI, COPRINCIPALINVESTIGATOR, true, assignedID, null);
            }
         } else {
            // prior CoPI exists - check if it changed and if so call insertStudyMember.
            if (((smcPlusIncumbents.currentCoPI) && (smcPlusIncumbents.currentCoPI.user.SPUser != copid))
               || (!(smcPlusIncumbents.currentCoPI) && (copid))) {
               // add CoPI to study member
               insertStudyMember(copid, COPRINCIPALINVESTIGATOR, true, assignedID, null);
            }
         }

         sendREST("POST", spAppData.__metadata.uri, spData, function() {
         }, "Merge");
      });
   }
   // changed questions keeps track of what questions the user
   // has modified while filling out the current section. we only
   // want to package up and send these questions
   changedPackage = changedQuestions.values();
   // JSHINT HashSet is not defined - it is expected to be included in the page
   // jshint -W117
   changedQuestions = new HashSet();

   // Filtering through each changed question and packaging it up based on what
   // type of input it is.
   for (i = 0; i < changedPackage.length; i++) {
      // var toPackage = $('[name="QID_'+changedPackage[i]+'"], #AID_'+changedPackage[i]+'');
      var toPackage = $('[name="QID_' + changedPackage[i] + '"], [name="QID_' + changedPackage[i] + '_multi"]');
      for (j = 0; j < toPackage.length; j++) {
         if ((toPackage[j].type == "radio")) {
            // have to grab the set of radios assigned to a questions and see
            // which one is actually selected to send to the database
            if (toPackage[j].checked === true) {
               response = {
                  "answersID" : (toPackage[j].id).replace('AID_', ''),
                  "questionsID" : (toPackage[j].name).replace('QID_', '').replace('_multi', '')
               };
               qrData.push(response);
            }
         } else if ((toPackage[j].type == "checkbox")) {
            // there's a difference in how checked checkboxes
            // are sent vs how unchecked checkboxes are sent
            if (toPackage[j].checked === true) {
               response = {
                  "answersID" : (toPackage[j].id).replace('AID_', ''),
                  "checkboxIndex" : toPackage[j].title,
                  "questionsID" : (toPackage[j].name).replace('QID_', '').replace('_multi', '')
               };
               qrData.push(response);
            } else if (toPackage[j].checked === false) {
               response = {
                  "checkboxIndex" : toPackage[j].title,
                  "questionsID" : (toPackage[j].name).replace('QID_', '').replace('_multi', '')
               };
               qrData.push(response);
            }
         } else if ((toPackage[j].type == "text") || (toPackage[j].type == "textarea")
            || (toPackage[j].type == "select-one")) {
            // everything else is pretty straightforward to send
            response = {
               "questionsID" : (toPackage[j].name).replace('QID_', ''),
               "response" : toPackage[j].value
            };
            qrData.push(response);
         }
      }
   }

   if (qrData.length === 0) {
      insertWarning("There are no modified responses to be saved.", "alert-warning");
   } else {
      sendREST(
         "POST",
         restPath.concat("study/" + assignedID + "/responses/" + targetStage),
         qrData,
         function() {
            // indicates that the application is out of sync
            if (window.location.href.split("?")[1] == "1") {
               window.location.assign("../SiteAssets/irb-applications-general.html?" + assignedID);
               // alert('kishore, you commented out 2727');
            }
            sendREST(
               "GET",
               restPath.concat("study/" + assignedID + "/responses/" + targetStage + "/rollup"),
               null,
               function(json) {
                  rollup = json;
                  rollupAnswers = new HashSet();
                  rollupQuestions = new HashSet();

                  for (var j = 0; j < rollup.responses.length; j++) {
                     // adding selected radios and checkboxes
                     if (typeof rollup.responses[j].answersID !== "undefined") {
                        rollupAnswers.add(rollup.responses[j].answersID);
                     }
                     // adding questions regardless
                     rollupQuestions.add(rollup.responses[j].questionsID);
                  }

                  // getting current target section to pass
                  // //console.log("the amount of data");
                  // if(data.length > 0){
                  var sidebar = $('.list-group-item.sectionLink');
                  var current = $('.list-group-item.active');
                  for (i = 0; i < sidebar.length; i++) {
                     // if(sidebar.get(i) == current.get(0)){
                     if (sidebar[i] == current[0]) {
                        targetSection = i;

                        var requiredQuestions = remainingQuestions[targetSection];
                        // Get list of questions for current section, filter out changed ones
                        for (j in changedPackage) {
                           if (changedPackage.hasOwnProperty(j)) {
                              var index = requiredQuestions.indexOf(changedPackage[j]);
                              if (index > -1) {
                                 $('#QID_' + changedPackage[j]).parent().css({
                                    "backgroundColor" : "#fff"
                                 });
                                 requiredQuestions.splice(index, 1);
                              }
                           }
                        }
                        // Filter out disabled questions
                        var requiredQuestionsT = [];
                        for ( var q in requiredQuestions) {
                           if (requiredQuestions.hasOwnProperty(q)) {
                              if (!($('#QID_' + requiredQuestions[q]).prop('disabled'))) {
                                 requiredQuestionsT.push(requiredQuestions[q]);
                              } else {
                                 $('#QID_' + requiredQuestions[q]).parent().css({
                                    "backgroundColor" : "#fff"
                                 });
                              }
                           }
                        }
                        // Map QID to question and highlight box in red
                        for (j = 0; j < requiredQuestionsT.length; j++) {
                           $('#QID_' + requiredQuestionsT[j]).parent().css({
                              "backgroundColor" : "#fcc",
                              "border-radius" : "8px"
                           });
                        }

                        // updating sidebar icons now that we have the section that
                        // should show up as activated.
                        // updateSidebarIcons(i);
                        sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/"
                           + assignedID + "/Study%20Team')"), null, function(returnedFolder) {
                           wellSidebarBuilder(targetSection, returnedFolder.d.ItemCount);
                        });

                        // alerting users of exempt question constraints
                        if ((currentFormType == "Parent") && (i == 2)) {
                           // if a user is submitting an application as an expedited study, then they
                           // need to meet the three requirements, which the below logic looks for
                           if (rollupAnswers
                              .contains(study.containedFormCollection[0].containedForm.formQuestionCollection[5].question.answerCollection[1].answersID)) {
                              if (!(rollupAnswers
                                 .contains(study.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[0].answersID))) {
                                 insertWarning(
                                    "You have selected that your application is expedited, but you haven't checked all three expedited categories",
                                    "alert-warning");
                              } else if (!(rollupAnswers
                                 .contains(study.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[1].answersID))) {
                                 insertWarning(
                                    "You have selected that your application is expedited, but you haven't checked all three expedited categories",
                                    "alert-warning");
                              } else if (!(rollupAnswers
                                 .contains(study.containedFormCollection[2].containedForm.formQuestionCollection[1].question.answerCollection[2].answersID))) {
                                 insertWarning(
                                    "You have selected that your application is expedited, but you haven't checked all three expedited categories",
                                    "alert-warning");
                              }
                           }
                        }
                     }
                  }
                  // }
               });
         });
   }

}

// // 552083: REFACTOR: Update Study Application submit process to populate the new CommiteeTopics /Application CT at
// the
// // Site Collection level
// function getCommitteeTypeItemID(committeeName) {
// var committeeTypesJson;
// var committeeTypeID;
// $.ajax({
// async : false,
// type : 'GET',
// url : siteUrl.concat("_api/Web/lists/getbytitle('Committee Types')/items" + "?$filter=CommitteesName eq '"
// + committeeName + "'"),
// headers : {
// "accept" : "application/json;odata=verbose"
// },
// dataType : 'json',
// success : function(json) {
// committeeTypesJson = json.d.results;
// committeeTypeID = committeeTypesJson[0].CommitteeID;
//
// }
// });
// // JSHINT NOTE: committeeTypeId may not be initialized at this point. It is up to the caller to verify or handle
// // the
// // result
// return committeeTypeID;
// }

// 682562: Add a column to the Application List to store the data for the
// Off-site waiver request.
function getOffsiteWaiverRequests() {
   var waiverDetails = '';

   if (currentFormType == "Parent") {
      var questionOffsiteYesID = study.containedFormCollection[2].containedForm.formQuestionCollection[27].question.answerCollection[0].answersID;
      var wid;
      if (rollupAnswers.contains(questionOffsiteYesID)) {
         wid = study.containedFormCollection[2].containedForm.formQuestionCollection[28].question.questionsID;
         if (wid) {
            var widVal = $('[name="QID_' + wid + '"]').val();
            if ((typeof widVal !== "undefined") && (widVal !== null)) {
               waiverDetails = $('[name="QID_' + wid + '"]').val();
            } else {
               for (var i = 0; i < rollup.responses.length; i++) {
                  if (rollup.responses && (rollup.responses[i].questionsID == wid)) {
                     waiverDetails = rollup.responses[i].response;
                  }
               }
            }
         }
      }
   }
   // Return the waiverDetails (text box of question 29 in section C).
   return waiverDetails;
}

function getAdverseReportableEventAnswers() {
   var adverseReportableEventAnswers = {};
   adverseReportableEventAnswers.AdverseReportableEvent = "";
   adverseReportableEventAnswers.DeviationReportableEvent = "";

   if (studyTitle.indexOf("Adverse/Reportable Event") !== -1) {
      // NOT USED
      // var adverseReportableEventQuestionID =
      // study.containedFormCollection[0].containedForm.formQuestionCollection[0].formsQuestionsID;
      // var deviationReportableEventQuestionID =
      // study.containedFormCollection[0].containedForm.formQuestionCollection[1].formsQuestionsID;
      var adverseReportableEventYesID = study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.answerCollection[0].answersID;
      var adverseReportableEventNoID = study.containedFormCollection[0].containedForm.formQuestionCollection[0].question.answerCollection[1].answersID;
      var deviationReportableEventYesID = study.containedFormCollection[0].containedForm.formQuestionCollection[1].question.answerCollection[0].answersID;
      var deviationReportableEventNoID = study.containedFormCollection[0].containedForm.formQuestionCollection[1].question.answerCollection[1].answersID;

      if (rollupAnswers.contains(adverseReportableEventYesID)) {
         adverseReportableEventAnswers.AdverseReportableEvent = "Yes";
      } else if (rollupAnswers.contains(adverseReportableEventNoID)) {
         adverseReportableEventAnswers.AdverseReportableEvent = "No";
      }

      if (rollupAnswers.contains(deviationReportableEventYesID)) {
         adverseReportableEventAnswers.DeviationReportableEvent = "Yes";
      } else if (rollupAnswers.contains(deviationReportableEventNoID)) {
         adverseReportableEventAnswers.DeviationReportableEvent = "No";
      }
   }

   return adverseReportableEventAnswers;

}

/**
 * Create a html link tag with onclick event.
 * 
 * @param {linkName} name of the hyperlink
 * @param {functionName} function binds to onclick event.
 * @param {params} function parameters.
 */
function createHyperlink(linkName, functionName, params) {
   var functionNameWithParms = "";
   functionNameWithParms = functionNameWithParms.concat(functionName);
   functionNameWithParms = functionNameWithParms.concat("(");
   for (var i = 0; i < params.length; i++) {
      functionNameWithParms = functionNameWithParms.concat("'" + params[i] + "'");
      functionNameWithParms = functionNameWithParms.concat(",");
   }
   functionNameWithParms = functionNameWithParms.slice(0, -1);
   functionNameWithParms = functionNameWithParms.concat(");return false;");

   var link = document.createElement('a');
   link.setAttribute('onclick', functionNameWithParms);
   link.setAttribute('href', '#');
   link.innerHTML = linkName;
   return link;

}
/* exported doCreateTemplate */
function doCreateTemplate(linkName, questionID) {
   var templateName = "";
   switch (linkName) {
   case "ACORPmainbody":
      templateName = "ACORP_Main_Body";
      break;
   case "ACORPAppendix1":
      templateName = "ACORP_Appendix1";
      break;
   case "ACORPAppendix2":
      templateName = "ACORP_Appendix2";
      break;
   case "ACORPAppendix3":
      templateName = "ACORP_Appendix3";
      break;
   case "ACORPAppendix4":
      templateName = "ACORP_Appendix4";
      break;
   case "ACORPAppendix5":
      templateName = "ACORP_Appendix5";
      break;
   case "ACORPAppendix6":
      templateName = "ACORP_Appendix6";
      break;
   case "ACORPAppendix7":
      templateName = "ACORP_Appendix7";
      break;
   case "ACORPAppendix8":
      templateName = "ACORP_Appendix8";
      break;
   case "ACORPAppendix9":
      templateName = "ACORP_Appendix9";
      break;
   case "Form10-0398":
      templateName = "Form_10-0398_RPSS";
      break;
   case "Form10-9012":
      templateName = "Form_10-9012";
      break;
   case "IACUCunanticipatedeventform":
      templateName = "IACUC-adverse-Event-Form";
      break;
   case "IACUCDeviationform":
      templateName = "IACUC-Deviations-form";
      break;
   case "renewalform":
      templateName = "IACUC-renewal-form";
      break;
   default:
      templateName = "";
   }

   sendREST("GET", siteUrl.concat("_api/web/GetFolderByServerRelativeUrl('Shared%20Documents/" + assignedID
      + "/Study%20Team')/Files?$expand=ListItemAllFields"), null, function(returnedFiles) {
      var fileFound = false;
      if (returnedFiles) {
         var attachments = returnedFiles.d.results;
         if (attachments) {
            for (var i = 0; i < attachments.length; i++) {
               if (attachments[i].Name == templateName + ".docx") {
                  fileFound = true;
                  triggerWarningModal(templateName, questionID);
               }
            }
         }
      }
      if (!fileFound) {
         generateDocument(templateName, questionID);
      }
   });

}

/*
 * This function generate the document calling ServiceREST API, then calls addFileNameToRollup function.
 * addFileNameToRollup function calls ServiceREST setStudyResponses method to store the filename associated to question.
 */
function generateDocument(templateName, questionID) {

   var documentSection = 1;

   $('[name="QID_' + questionID + '"]').attr("disabled", true);
   var response = {
      "questionsID" : questionID,
      "response" : templateName
   };

   var ajaxUrl = restPath.concat("study/" + assignedID + "/documentGeneration/" + targetState + "/" + documentSection
      + "/" + templateName);

   $.ajax({
      type : 'GET',
      url : ajaxUrl,
      data : {},
      dataType : "text",
      success : function(data) {
         addFileNameToRollup(response, data);

      },
      // JSHINT {var} is defined but never used.
      // jshint -W098
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError("Unable to generate template. " + JSON.stringify(XMLHttpRequest.responseJSON.error.message));
      }
   });

}

/*
 * This function calls ServiceREST setStudyResponses method to store the filename associated to question.
 */
function addFileNameToRollup(response, url) {

   var qrData = [];
   qrData.push(response);
   var rollupUrl = restPath.concat("study/" + assignedID + "/responses/" + targetStage);
   $.ajax({
      type : 'POST',
      url : rollupUrl,
      data : JSON.stringify(qrData),
      dataType : "json",
      headers : {
         "X-RequestDigest" : requestDigest,
         "Content-Type" : "application/json;odata=verbose",
         "IF-MATCH" : "*",
         "X-HTTP-Method" : "MERGE"
      },
      success : function() {
         window.open(url);
      },
      // JSHINT {var} is defined but never used.
      // jshint -W098
      error : function(XMLHttpRequest, textStatus, errorThrown) {
         logError("Unable to add file name to rollup. " + JSON.stringify(XMLHttpRequest.responseJSON.error.message));
      }
   });

}

/*
 * open window displays warning message to user if the file already exists in the attachement section.
 */
function triggerWarningModal(templateName, questionID) {
   var modal, modalDialog, modalContent, modalHeader;
   var header, heading1, lineBreak, yesButton, closeButton;
   var paragraph, text, modalFooter;
   var container, textDiv;
   // NOT USED
   // var modalTitle, modalBody;

   // remove the div otherwise "Yes" button will hold previous URL.
   $("#warningModel").remove();

   modal = document.createElement("div");
   modal.setAttribute("class", "modal");
   modal.setAttribute("id", "warningModel");
   modal.setAttribute("tabindex", "-1");
   modal.setAttribute("role", "dialog");
   modal.setAttribute("aria-labelledby", "warningModel");

   modalDialog = document.createElement("div");
   modalDialog.setAttribute("role", "document");

   modalContent = document.createElement("div");
   modalContent.setAttribute("class", "modal-content");
   modalHeader = document.createElement("div");
   modalHeader.setAttribute("class", "modal-header");
   modalHeader.setAttribute("tabindex", "0"); // start tabbing on modal from header

   header = document.createElement("header");
   header.setAttribute("class", "modal-title");
   header.setAttribute("id", "modalTitle");

   heading1 = document.createElement("H2");
   heading1.setAttribute("class", "modal-title");
   text = document.createTextNode("Warning");
   heading1.appendChild(text);
   header.appendChild(heading1);
   modalHeader.appendChild(header);

   textDiv = document.createElement("div");
   textDiv.setAttribute("class", "col-xs-12");
   paragraph = document.createElement("p");
   lineBreak = document.createElement("br");
   text = document.createTextNode("Warning! The file already exists in the attachement section, "
      + "any changes to the existing file will be lost. Would you like to continue?. "
      + "Click the [Yes] button to continue. Click the [No] button to return.");

   paragraph.appendChild(lineBreak);
   paragraph.appendChild(text);
   textDiv.appendChild(paragraph);
   modalContent.appendChild(textDiv);

   modalContent.appendChild(modalHeader);
   modalContent.appendChild(lineBreak);
   modalContent.appendChild(text);
   modalContent.appendChild(lineBreak);

   container = document.createElement("div");
   container.setAttribute("class", "form-content");

   modalFooter = document.createElement("div");
   modalFooter.setAttribute("class", "modal-footer");

   /*Add Yes Button*/
   yesButton = document.createElement("button");
   yesButton.setAttribute("class", "btn btn-sm modal-button");
   yesButton.setAttribute("aria-label", "Yes");

   yesButton.setAttribute("title", "Continue generation"); // hover tip
   yesButton.setAttribute("type", "button");
   yesButton.setAttribute("role", "button");
   yesButton.setAttribute("id", "saveLocalSite");

   var functionName = "continueDocumentGeneration(" + "'" + templateName + "'" + "," + questionID + ")";
   yesButton.setAttribute("onclick", functionName);
   text = document.createTextNode("Yes");
   yesButton.appendChild(text);

   /*Add Close Window Button*/
   closeButton = document.createElement("button");
   closeButton.setAttribute("class", "btn btn-sm modal-button modal-button-right");
   closeButton.setAttribute("data-dismiss", "modal");
   closeButton.setAttribute("aria-label", "Close");

   closeButton.setAttribute("title", "Close this window and return"); // hover tip
   closeButton.setAttribute("type", "button");
   closeButton.setAttribute("role", "button");
   closeButton.setAttribute("id", "closeModal");

   closeButton.setAttribute("onclick", "closeWarningModal()");
   text = document.createTextNode("No");
   closeButton.appendChild(text);

   /*Combine Modal Elements*/
   modalContent.appendChild(container);
   modalFooter.appendChild(yesButton);
   modalFooter.appendChild(closeButton);
   modalContent.appendChild(modalFooter);
   modalDialog.appendChild(modalContent);
   modal.appendChild(modalDialog);
   $('body').append(modal);
   $('#warningModel').popup('show');
}

/* close warning window
 */
/* exported closeWarningModal */
function closeWarningModal() {
   $('#warningModel').popup('toggle');
}

/* Continue form generation
 */
/* exported continueDocumentGeneration */
function continueDocumentGeneration(templateName, questionID) {
   $('#warningModel').popup('toggle');
   generateDocument(templateName, questionID);
}

/* exported displayOtherDocType */
function displayOtherDocType() {
   var documentType = $('#docType').find(':selected').val();
   if ("Other" == documentType) {
      if (document.getElementById('docTypeNameDiv') === undefined) {
         var textDiv = document.createElement("div");
         textDiv.setAttribute("class", "col-xs-12");
         textDiv.setAttribute("id", "docTypeNameDiv");
         var label = document.createElement("label");
         label.setAttribute("for", "Document Name");
         label.setAttribute("class", "control-label");
         var text = document.createTextNode("Document Name:");
         label.appendChild(text);
         textDiv.appendChild(label);
         var inputElement = document.createElement("INPUT");
         inputElement.setAttribute("type", "text");
         inputElement.setAttribute("id", "docTypeName");
         inputElement.setAttribute("class", "form-control normal");
         textDiv.appendChild(inputElement);
         var para = document.createElement("p");
         $('#docType').parent().parent().append(para);
         $('#docType').parent().parent().append(textDiv);
      } else if ($('#docTypeNameDiv').css('display') == 'none') {
         $('#docTypeNameDiv').show();
      }

   } else {
      $('#docTypeNameDiv').hide();
   }
}

/* exported closeUploadDocumentModel */
function closeUploadDocumentModel() {
   if ($("#docTypeNameDiv").length) {
      $('#docTypeNameDiv').hide();
   }
   $("#docType").prop('selectedIndex', 0);
   $('#uploadModal').popup('toggle');
}
